Package: eq5dsuite
Type: Package
Title: Handling and Analysing EQ-5d Data
Version: 2.0.0
Authors@R: 
  c(person(given = "Kim",
           family = "Rand",
           role = c("aut", "cre"),
           email = "krand@mathsinhealth.com",
           comment = c(ORCID = "0000-0001-7692-4099")),
    person(given = "Oliver",
           family = "Rivero-Arias",
           role = "aut",
           comment = c(ORCID = "0000-0003-2233-6544")),
    person(given = "Iryna",
           family = "Schlackow",
           role = "aut",
           comment = c(ORCID = "0000-0002-4154-1431")),
    person(given = "Anabel",
           family = "Estévez-Carrillo",
           role = "aut",
           comment = c(ORCID = "0000-0001-8778-5055")))
Maintainer: Kim Rand <krand@mathsinhealth.com>
Description: The EQ-5D is a widely-used standarized instrument for measuring Health Related Quality Of Life (HRQOL), 
    developed by the EuroQol group <https://euroqol.org/>. It assesses five dimensions; mobility, self-care, 
    usual activities, pain/discomfort, and anxiety/depression, using either a three-level (EQ-5D-3L) or five-level (EQ-5D-5L) scale.
    Scores from these dimensions are commonly converted into a single utility index using country-specific value sets, 
    which are critical in clinical and economic evaluations of healthcare and in population health surveys. 
    The eq5dsuite package enables users to calculate utility index values for the EQ-5D instruments, 
    including crosswalk utilities using the original crosswalk developed by van Hout et al. (2012) <doi:10.1016/j.jval.2012.02.008> 
    (mapping EQ-5D-5L responses to EQ-5D-3L index values), or the recently developed reverse crosswalk 
    by van Hout et al. (2021) <doi:10.1016/j.jval.2021.03.009> (mapping EQ-5D-3L responses
    to EQ-5D-5L index values). Users are allowed to add and/or remove user-defined value sets. 
    Additionally, the package provides tools to analyze EQ-5D data according to the recommended 
    guidelines outlined in "Methods for Analyzing and Reporting EQ-5D data" by Devlin et al. (2020) <doi:10.1007/978-3-030-47622-9>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 8.0.0
Depends: R (>= 3.5)
Imports: curl, ggplot2, moments, RColorBrewer, rlang, scales, rappdirs
Suggests: shiny, DT, bslib, readxl, spelling, testthat (>= 3.0.0),
        withr, knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-14 08:22:36 UTC; aeste
Author: Kim Rand [aut, cre] (ORCID: <https://orcid.org/0000-0001-7692-4099>),
  Oliver Rivero-Arias [aut] (ORCID:
    <https://orcid.org/0000-0003-2233-6544>),
  Iryna Schlackow [aut] (ORCID: <https://orcid.org/0000-0002-4154-1431>),
  Anabel Estévez-Carrillo [aut] (ORCID:
    <https://orcid.org/0000-0001-8778-5055>)
Repository: CRAN
Date/Publication: 2026-05-14 10:30:02 UTC
Built: R 4.5.2; ; 2026-05-14 14:19:38 UTC; unix

Package: eq5dsuite
Type: Package
Title: Handling and Analysing EQ-5d Data
Version: 2.0.0
Authors@R: 
  c(person(given = "Kim",
           family = "Rand",
           role = c("aut", "cre"),
           email = "krand@mathsinhealth.com",
           comment = c(ORCID = "0000-0001-7692-4099")),
    person(given = "Oliver",
           family = "Rivero-Arias",
           role = "aut",
           comment = c(ORCID = "0000-0003-2233-6544")),
    person(given = "Iryna",
           family = "Schlackow",
           role = "aut",
           comment = c(ORCID = "0000-0002-4154-1431")),
    person(given = "Anabel",
           family = "Estévez-Carrillo",
           role = "aut",
           comment = c(ORCID = "0000-0001-8778-5055")))
Maintainer: Kim Rand <krand@mathsinhealth.com>
Description: The EQ-5D is a widely-used standarized instrument for measuring Health Related Quality Of Life (HRQOL), 
    developed by the EuroQol group <https://euroqol.org/>. It assesses five dimensions; mobility, self-care, 
    usual activities, pain/discomfort, and anxiety/depression, using either a three-level (EQ-5D-3L) or five-level (EQ-5D-5L) scale.
    Scores from these dimensions are commonly converted into a single utility index using country-specific value sets, 
    which are critical in clinical and economic evaluations of healthcare and in population health surveys. 
    The eq5dsuite package enables users to calculate utility index values for the EQ-5D instruments, 
    including crosswalk utilities using the original crosswalk developed by van Hout et al. (2012) <doi:10.1016/j.jval.2012.02.008> 
    (mapping EQ-5D-5L responses to EQ-5D-3L index values), or the recently developed reverse crosswalk 
    by van Hout et al. (2021) <doi:10.1016/j.jval.2021.03.009> (mapping EQ-5D-3L responses
    to EQ-5D-5L index values). Users are allowed to add and/or remove user-defined value sets. 
    Additionally, the package provides tools to analyze EQ-5D data according to the recommended 
    guidelines outlined in "Methods for Analyzing and Reporting EQ-5D data" by Devlin et al. (2020) <doi:10.1007/978-3-030-47622-9>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 8.0.0
Depends: R (>= 3.5)
Imports: curl, ggplot2, moments, RColorBrewer, rlang, scales, rappdirs
Suggests: shiny, DT, bslib, readxl, spelling, testthat (>= 3.0.0),
        withr, knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-14 08:22:36 UTC; aeste
Author: Kim Rand [aut, cre] (ORCID: <https://orcid.org/0000-0001-7692-4099>),
  Oliver Rivero-Arias [aut] (ORCID:
    <https://orcid.org/0000-0003-2233-6544>),
  Iryna Schlackow [aut] (ORCID: <https://orcid.org/0000-0002-4154-1431>),
  Anabel Estévez-Carrillo [aut] (ORCID:
    <https://orcid.org/0000-0001-8778-5055>)
Repository: CRAN
Date/Publication: 2026-05-14 10:30:02 UTC
Built: R 4.6.0; ; 2026-05-14 12:40:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(.add_utility)
export(.check_uniqueness)
export(.gen_colours)
export(.get_lfs)
export(.get_names)
export(.getmode)
export(.modify_ggplot_theme)
export(.pchc)
export(.pchc_plot_by_dim)
export(.pchctab)
export(.prep_eq5d)
export(.prep_fu)
export(.prep_vas)
export(.summary_cts_by_fu)
export(.summary_mean_ci)
export(.summary_table_2_1)
export(.summary_table_4_3)
export(.summary_table_4_4)
export(eq5d)
export(eq5d3l)
export(eq5d5l)
export(eq5d_profile_better_dimensions_by_group_plot)
export(eq5d_profile_change_summary)
export(eq5d_profile_density_curve)
export(eq5d_profile_dimension_change_table)
export(eq5d_profile_health_profile_grid)
export(eq5d_profile_level_summary)
export(eq5d_profile_level_summary_by_group)
export(eq5d_profile_lfs_distribution)
export(eq5d_profile_lfs_mean_utility)
export(eq5d_profile_lfs_utility_plot)
export(eq5d_profile_lfs_utility_summary)
export(eq5d_profile_lss_utility_plot)
export(eq5d_profile_lss_utility_summary)
export(eq5d_profile_mixed_dimensions_by_group_plot)
export(eq5d_profile_pchc_by_group_plot)
export(eq5d_profile_pchc_table)
export(eq5d_profile_pchc_with_no_problems_table)
export(eq5d_profile_top_states)
export(eq5d_profile_worse_dimensions_by_group_plot)
export(eq5d_utility_by_group_plot)
export(eq5d_utility_change_by_group_plot)
export(eq5d_utility_distribution_plot)
export(eq5d_utility_norms_comparison)
export(eq5d_utility_over_time_plot)
export(eq5d_utility_summary)
export(eq5d_utility_summary_by_group)
export(eq5d_utility_vas_scatter_plot)
export(eq5d_vas_distribution_table)
export(eq5d_vas_grouped_distribution_plot)
export(eq5d_vas_histogram)
export(eq5d_vas_summary)
export(eq5dy3l)
export(eqvs_add)
export(eqvs_display)
export(eqvs_drop)
export(eqvs_load)
export(eqxw)
export(eqxw_UK)
export(eqxwr)
export(make_all_EQ_indexes)
export(make_all_EQ_states)
export(make_dummies)
export(run_app)
export(toEQ5Ddims)
export(toEQ5Dindex)
export(update_value_sets)
import(RColorBrewer)
import(ggplot2)
import(moments)
import(scales)
importFrom(grDevices,colorRampPalette)
importFrom(moments,kurtosis)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
importFrom(utils,head)
importFrom(utils,read.csv)

# eq5dsuite <img src="man/figures/logo.png" align="right" height="139" alt="" />

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/eq5dsuite)](https://CRAN.R-project.org/package=eq5dsuite)
[![R-CMD-check](https://github.com/MathsInHealth/eq5dsuite-r/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/MathsInHealth/eq5dsuite-r/actions/workflows/R-CMD-check.yaml)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

R implementation of [eq5dsuite](https://github.com/MathsInHealth/eq5dsuite) 
— a standardised suite of tools for EQ-5D analysis across R, Stata, and 
Excel. This package provides a comprehensive set of functions for 
calculating EQ-5D values and analysing EQ-5D data following the 
recommendations of Devlin et al. (2020).

## Features

- EQ-5D value calculation for the **EQ-5D-3L**, **EQ-5D-5L**, and 
  **EQ-5D-Y-3L** instruments
- **39+ published national value sets** for the EQ-5D-3L, plus a 
  growing library of EQ-5D-5L and EQ-5D-Y-3L sets
- Three **crosswalk methods**: original (Van Hout et al., 2012), 
  reverse (Van Hout & Shaw, 2021), and UK-specific NICE-recommended 
  mapping (Hernández Alava et al., 2023)
- Support for **user-defined custom value sets**
- **Automatic value set updates** via `update_value_sets()` — install 
  newly published value sets without waiting for a CRAN update
- **35+ analysis functions** for profile analysis, EQ-5D value 
  analysis, and EQ-VAS analysis
- An interactive **Shiny application** for point-and-click access 
  to the same analytical workflow
- Integration with `dplyr` and `ggplot2` for reproducible, 
  script-based analyses
- Publication-ready outputs in both PDF and HTML formats

## Installation

Install the released version from CRAN:

```r
install.packages("eq5dsuite")
```

Install the development version from GitHub:

```r
# install.packages("remotes")
remotes::install_github("MathsInHealth/eq5dsuite-r")
```

## Quick start

```r
library(eq5dsuite)

# Calculate EQ-5D-3L values using the UK value set
eq5d3l(
  data.frame(mo = 1, sc = 2, ua = 3, pd = 2, ad = 1),
  country   = "UK",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

# View available value sets
eqvs_display(version = "3L")

# Check for newly published value sets
update_value_sets()

# Launch the Shiny application
run_app()
```

## Documentation

- Full function reference: `?eq5dsuite`
- Vignettes:

```r
browseVignettes("eq5dsuite")
```

Available vignettes:
- *Getting started* — installation, value calculation, value set management
- *Analysing EQ-5D data* — complete workflow using NHS PROMs data
- *Crosswalk methods* — when and how to use each crosswalk method
- *Custom value sets* — adding, saving, and managing custom value sets
- *Keeping value sets up to date* — using the online update system

## Value sets repository

Country-specific value sets are maintained at:

**[eq5dsuite-value-sets](https://github.com/MathsInHealth/eq5dsuite-value-sets)**

New value sets are added as they are published in the literature.
Use `update_value_sets()` to install them without waiting for a 
CRAN package update.

## Cross-platform suite

eq5dsuite is available across three platforms:

| Platform | Repository | Status |
|---|---|---|
| R | [eq5dsuite-r](https://github.com/MathsInHealth/eq5dsuite-r) | Available on CRAN |
| Stata | [eq5dsuite-stata](https://github.com/MathsInHealth/eq5dsuite-stata) | Available |
| Excel | [eq5dsuite-excel](https://github.com/MathsInHealth/eq5dsuite-excel) | Available |

## Issues and feedback

For R-specific issues please use the 
[issue tracker](https://github.com/MathsInHealth/eq5dsuite-r/issues) 
in this repository.

For cross-platform concerns please use the 
[umbrella issue tracker](https://github.com/MathsInHealth/eq5dsuite/issues).

For general enquiries contact 
[info@mathsinhealth.com](mailto:info@mathsinhealth.com).

## Citation

If you use eq5dsuite in your research, please cite:

> Estévez-Carrillo A, Rivero-Arias O, Schlackow I, Rand K.
> eq5dsuite: An R Package for Describing and Analysing EQ-5D Data.
> *The R Journal* (forthcoming).

## License

This package is licensed under the GPL (>= 2).

# mod_export.R — Export module

# ── UI ────────────────────────────────────────────────────────────────────────

mod_export_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    bslib::layout_columns(
      col_widths = c(8, 4),
      
      # Main: export list
      bslib::card(
        bslib::card_header("Export Results"),
        bslib::card_body(
          shiny::uiOutput(ns("export_list"))
        )
      ),
      
      # Sidebar: bulk export
      bslib::card(
        bslib::card_header("Bulk Export"),
        bslib::card_body(
          shiny::p("Download all tables (CSV) and figures (PNG) as a ZIP file."),
          shiny::downloadButton(ns("download_all_zip"),
                                "All results (.zip)",
                                class = "btn-outline-primary w-100 mb-2"),
          shiny::hr()
          # shiny::p("Export all results to a Word document."),
          # shiny::downloadButton(ns("download_word"),
          #                       "Word report (.docx)",
          #                       class = "btn-outline-primary w-100 mb-2")
        )
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_export_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns
    
    # ── Export list ───────────────────────────────────────────────────────────
    output$export_list <- shiny::renderUI({
      results <- rv$results
      if (length(results) == 0L) {
        return(shiny::p("No results to export. Run analyses first.",
                        class = "text-muted small"))
      }
      
      rows <- lapply(seq_along(results), function(i) {
        r    <- results[[i]]
        dl_buttons <- list()
        
        if (r$result_type %in% c("table", "both") && !is.null(r$data)) {
          dl_buttons <- c(dl_buttons, list(
            shiny::downloadButton(
              ns(paste0("dl_csv_", i)),
              label = "CSV",
              class = "btn-sm btn-outline-secondary me-1"
            )
          ))
        }
        
        if (r$result_type %in% c("plot", "both") && !is.null(r$plot)) {
          dl_buttons <- c(dl_buttons, list(
            shiny::downloadButton(
              ns(paste0("dl_png_", i)),
              label = "PNG",
              class = "btn-sm btn-outline-secondary me-1"
            ),
            shiny::downloadButton(
              ns(paste0("dl_pdf_", i)),
              label = "PDF",
              class = "btn-sm btn-outline-secondary"
            )
          ))
        }
        
        shiny::div(
          class = "d-flex align-items-center justify-content-between border-bottom py-2",
          shiny::div(
            shiny::strong(r$label),
            shiny::tags$br(),
            shiny::tags$small(
              class = "text-muted",
              format(r$timestamp, "%d %b %Y %H:%M"),
              " \u2014 ", r$result_type
            )
          ),
          shiny::div(dl_buttons)
        )
      })
      
      shiny::tagList(rows)
    })
    
    # ── Dynamic download handlers ─────────────────────────────────────────────
    # We observe rv$results and register handlers each time it changes.
    shiny::observe({
      results <- rv$results
      lapply(seq_along(results), function(i) {
        r <- results[[i]]
        
        # CSV download
        if (r$result_type %in% c("table", "both") && !is.null(r$data)) {
          local({
            local_r <- r
            local_i <- i
            output[[paste0("dl_csv_", local_i)]] <- shiny::downloadHandler(
              filename = function() {
                paste0(safe_filename(local_r$label), "_",
                       format(local_r$timestamp, "%Y%m%d"), ".csv")
              },
              content = function(file) {
                utils::write.csv(local_r$data, file, row.names = FALSE)
              }
            )
          })
        }
        
        # PNG download
        if (r$result_type %in% c("plot", "both") && !is.null(r$plot)) {
          local({
            local_r <- r
            local_i <- i
            output[[paste0("dl_png_", local_i)]] <- shiny::downloadHandler(
              filename = function() {
                paste0(safe_filename(local_r$label), "_",
                       format(local_r$timestamp, "%Y%m%d"), ".png")
              },
              content = function(file) {
                ggplot2::ggsave(file, plot = local_r$plot,
                                width = 8, height = 5, dpi = 150)
              }
            )
          })
          
          # PDF download
          local({
            local_r <- r
            local_i <- i
            output[[paste0("dl_pdf_", local_i)]] <- shiny::downloadHandler(
              filename = function() {
                paste0(safe_filename(local_r$label), "_",
                       format(local_r$timestamp, "%Y%m%d"), ".pdf")
              },
              content = function(file) {
                ggplot2::ggsave(file, plot = local_r$plot,
                                width = 8, height = 5, device = "pdf")
              }
            )
          })
        }
      })
    })
    
    # ── Bulk ZIP download (tables + plots) ────────────────────────────────────
    output$download_all_zip <- shiny::downloadHandler(
      filename = function() {
        paste0("eq5dsuite_results_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".zip")
      },
      content = function(file) {
        results  <- rv$results
        tmp_dir  <- file.path(tempdir(), paste0("eq5dzip_", format(Sys.time(), "%H%M%S%OS3")))
        dir.create(tmp_dir, recursive = TRUE, showWarnings = FALSE)
        
        out_files <- character(0L)
        
        # Tables → CSV
        for (r in results) {
          if (r$result_type %in% c("table", "both") && !is.null(r$data)) {
            fname <- file.path(tmp_dir, paste0(safe_filename(r$label), ".csv"))
            utils::write.csv(r$data, fname, row.names = FALSE)
            out_files <- c(out_files, fname)
          }
        }
        
        # Plots → PNG
        for (r in results) {
          if (r$result_type %in% c("plot", "both") && !is.null(r$plot)) {
            fname <- file.path(tmp_dir, paste0(safe_filename(r$label), ".png"))
            tryCatch(
              ggplot2::ggsave(fname, plot = r$plot, width = 8, height = 5, dpi = 150),
              error = function(e) NULL
            )
            if (file.exists(fname) && file.size(fname) > 0L) {
              out_files <- c(out_files, fname)
            }
          }
        }
        
        if (length(out_files) == 0L) {
          placeholder <- file.path(tmp_dir, "no_results.txt")
          writeLines("No results to export.", placeholder)
          out_files <- placeholder
        }
        
        utils::zip(file, files = out_files, flags = "-j")
      }
    )
    
  })
}

# ── Helper ────────────────────────────────────────────────────────────────────

safe_filename <- function(label) {
  gsub("[^A-Za-z0-9_-]", "_", label)
}

# mod_analysis_utility.R — EQ-5D Utility (index) analysis module

# ── UI ────────────────────────────────────────────────────────────────────────

mod_analysis_utility_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::uiOutput(ns("guard")),
    shiny::conditionalPanel(
      condition = "output.app_has_data",

      # Default country card
      bslib::card(
        bslib::card_header(
          shiny::tags$span(shiny::icon("globe"), " Default Country / Value Set")
        ),
        bslib::card_body(
          shiny::p(
            class = "text-muted small",
            "Select a default country to pre-fill all analysis country pickers below."
          ),
          shiny::uiOutput(ns("default_country_ui"))
        )
      ),
      shiny::br(),

      bslib::accordion(
        open = "acc_u31",

        # Table 3.1 — summary stats (uses ensure_fu)
        bslib::accordion_panel(
          title = "Table 3.1 \u2014 Utility summary statistics",
          value = "acc_u31",
          shiny::p("Descriptive statistics for the EQ-5D utility index, grouped by timepoint."),
          shiny::uiOutput(ns("note_fu_31")),
          shiny::uiOutput(ns("cpick_31")),
          shiny::actionButton(ns("run_31"), "Run", class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_31"))
        ),

        # Figure 3.4 — bar chart (no fu/group required)
        bslib::accordion_panel(
          title = "Figure 3.4 \u2014 Utility distribution (bar chart)",
          value = "acc_ufig34",
          shiny::p("Bar chart of the EQ-5D utility index distribution."),
          shiny::uiOutput(ns("cpick_fig34")),
          shiny::actionButton(ns("run_fig34"), "Run", class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_fig34"))
        ),

        # Table 3.2 — group required (body rendered conditionally)
        bslib::accordion_panel(
          title = "Table 3.2 \u2014 Mean utility by group",
          value = "acc_u32",
          shiny::uiOutput(ns("body_32"))
        ),

        # Figure 3.2 — group required (body rendered conditionally)
        bslib::accordion_panel(
          title = "Figure 3.2 \u2014 Mean utility and 95% CI by group (bar chart)",
          value = "acc_ufig32",
          shiny::uiOutput(ns("body_fig32"))
        ),

        # Figure 3.1 — fu required (body rendered conditionally)
        bslib::accordion_panel(
          title = "Figure 3.1 \u2014 Utility by timepoint (box plots)",
          value = "acc_ufig31",
          shiny::uiOutput(ns("body_fig31"))
        ),

        # Figure 3.3 — group + fu required (body rendered conditionally)
        bslib::accordion_panel(
          title = "Figure 3.3 \u2014 Utility by timepoint and group",
          value = "acc_ufig33",
          shiny::uiOutput(ns("body_fig33"))
        ),

        # Figure 3.5 — VAS required (body rendered conditionally)
        bslib::accordion_panel(
          title = "Figure 3.5 \u2014 Utility vs EQ-VAS",
          value = "acc_ufig35",
          shiny::uiOutput(ns("body_fig35"))
        )
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_analysis_utility_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    output$guard <- shiny::renderUI({
      if (is.null(rv$processed_data)) {
        bslib::card(bslib::card_body(
          shiny::p(shiny::icon("circle-info"),
                   " Please complete data upload and validation first.",
                   class = "text-muted")
        ))
      }
    })

    # ── Country helpers ───────────────────────────────────────────────────────
    country_choices <- shiny::reactive({
      shiny::req(rv$mapping$eq5d_version)
      get_country_choices(rv$mapping$eq5d_version)
    })

    # reactiveVal holds the currently selected default country
    user_default_country <- shiny::reactiveVal("")

    # Seed from mapping country when mapping is first confirmed
    shiny::observe({
      shiny::req(rv$mapping)
      cty <- rv$mapping$country
      if (!is.null(cty) && nzchar(cty) && !nzchar(user_default_country())) {
        user_default_country(cty)
      }
    })

    # Default picker UI
    output$default_country_ui <- shiny::renderUI({
      ch <- country_choices()
      if (length(ch) == 0L) {
        return(shiny::p("No value sets available for this version.", class = "text-muted small"))
      }
      shiny::selectizeInput(
        ns("default_country"), NULL,
        choices  = c("(select)" = "", ch),
        selected = user_default_country()
      )
    })

    # When default picker changes, propagate to reactiveVal
    shiny::observeEvent(input$default_country, {
      user_default_country(input$default_country)
    })

    # Factory: per-analysis country picker; re-renders when default changes
    make_cpick <- function(picker_id, label = "Country / Value set") {
      shiny::renderUI({
        ch  <- country_choices()
        sel <- user_default_country()   # reactive dependency — re-renders on default change
        if (length(ch) == 0L) {
          return(shiny::p("No value sets found.", class = "text-danger small"))
        }
        shiny::selectizeInput(
          ns(picker_id), label,
          choices  = c("(select)" = "", ch),
          selected = sel
        )
      })
    }

    output$cpick_31    <- make_cpick("country_31")
    output$cpick_fig34 <- make_cpick("country_fig34")
    output$cpick_32    <- make_cpick("country_32")
    output$cpick_fig32 <- make_cpick("country_fig32")
    output$cpick_fig31 <- make_cpick("country_fig31")
    output$cpick_fig33 <- make_cpick("country_fig33")
    output$cpick_fig35 <- make_cpick("country_fig35")

    # ── Base args reactive ────────────────────────────────────────────────────
    base_args <- shiny::reactive({
      shiny::req(rv$mapping)
      list(names_eq5d = DIMS_STD, eq5d_version = rv$mapping$eq5d_version)
    })

    # ── Reusable warning notes ────────────────────────────────────────────────
    no_group_note <- shiny::div(
      class = "alert alert-warning small py-2",
      shiny::icon("triangle-exclamation"), " ",
      "This analysis requires a Group variable. ",
      "Please map a group column on the Data tab."
    )

    no_fu_note <- shiny::div(
      class = "alert alert-info small py-2",
      shiny::icon("circle-info"), " ",
      "No timepoint variable mapped. ",
      "Results will be shown for all observations combined."
    )

    no_fu_required_note <- shiny::div(
      class = "alert alert-warning small py-2",
      shiny::icon("triangle-exclamation"), " ",
      "This analysis requires a Timepoint variable. ",
      "Please map a follow-up column on the Data tab."
    )

    no_vas_note <- shiny::div(
      class = "alert alert-warning small py-2",
      shiny::icon("triangle-exclamation"), " ",
      "This analysis requires an EQ-VAS variable. ",
      "Please map a VAS column on the Data tab."
    )

    # ── Table 3.1 ─────────────────────────────────────────────────────────────
    r31 <- shiny::reactiveValues(data = NULL, call = NULL)

    output$note_fu_31 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_fu <- !is.null(rv$mapping$name_fu) && nzchar(rv$mapping$name_fu)
      if (!has_fu) no_fu_note
    })

    shiny::observeEvent(input$run_31, {
      shiny::req(rv$processed_data, nzchar(input$country_31 %||% ""))
      fu_info  <- ensure_fu(rv$processed_data, rv$mapping)
      args     <- c(base_args(),
                    list(name_fu = fu_info$name_fu, country = input$country_31))
      call_str <- format_call("eq5d_utility_summary",
                              c(list(df = "data"), base_args(),
                                list(country = input$country_31)))
      tryCatch({
        r31$data <- do.call(eq5d_utility_summary, c(list(df = fu_info$df), args))
        r31$call <- call_str
        save_result(rv, "Utility summary stats (3.1)", call_str, "table", data = r31$data)
      }, error = function(e) err_notify(e))
    })

    output$out_31 <- shiny::renderUI({
      if (is.null(r31$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_31")), call_display(ns("call_31")))
    })
    output$tbl_31  <- render_analysis_table(shiny::reactive(r31$data))
    output$call_31 <- shiny::renderText(r31$call)

    # ── Figure 3.4 ────────────────────────────────────────────────────────────
    rfig34 <- shiny::reactiveValues(plot = NULL, call = NULL)

    shiny::observeEvent(input$run_fig34, {
      shiny::req(rv$processed_data, nzchar(input$country_fig34 %||% ""))
      args     <- c(base_args(), list(country = input$country_fig34))
      call_str <- format_call("eq5d_utility_distribution_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_utility_distribution_plot, c(list(df = rv$processed_data), args))
        rfig34$plot  <- res$p
        rfig34$call  <- call_str
        save_result(rv, "Utility bar chart (fig. 3.4)", call_str, "plot", plot = rfig34$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig34 <- shiny::renderUI({
      if (is.null(rfig34$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig34"), height = "420px"),
        call_display(ns("call_fig34"))
      )
    })
    output$plt_fig34  <- shiny::renderPlot(rfig34$plot)
    output$call_fig34 <- shiny::renderText(rfig34$call)

    # ── Table 3.2 (group required) ────────────────────────────────────────────
    r32 <- shiny::reactiveValues(data = NULL, call = NULL)

    output$body_32 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_group <- !is.null(rv$mapping$name_groupvar) &&
                   nzchar(rv$mapping$name_groupvar)
      if (!has_group) return(shiny::tagList(shiny::br(), no_group_note))
      shiny::tagList(
        shiny::p("Mean utility (with 95% CI) overall and by group."),
        shiny::uiOutput(ns("cpick_32")),
        shiny::actionButton(ns("run_32"), "Run", class = "btn-sm btn-outline-primary"),
        shiny::br(), shiny::br(),
        shiny::uiOutput(ns("out_32"))
      )
    })

    shiny::observeEvent(input$run_32, {
      shiny::req(rv$processed_data, rv$mapping$name_groupvar,
                 nzchar(input$country_32 %||% ""))
      args     <- c(base_args(),
                    list(name_groupvar = "groupvar", country = input$country_32))
      call_str <- format_call("eq5d_utility_summary_by_group", c(list(df = "data"), args))
      tryCatch({
        r32$data <- do.call(eq5d_utility_summary_by_group, c(list(df = rv$processed_data), args))
        r32$call <- call_str
        save_result(rv, "Utility by group (3.2)", call_str, "table", data = r32$data)
      }, error = function(e) err_notify(e))
    })

    output$out_32 <- shiny::renderUI({
      if (is.null(r32$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_32")), call_display(ns("call_32")))
    })
    output$tbl_32  <- render_analysis_table(shiny::reactive(r32$data))
    output$call_32 <- shiny::renderText(r32$call)

    # ── Figure 3.2 (group required) ───────────────────────────────────────────
    rfig32 <- shiny::reactiveValues(plot = NULL, call = NULL)

    output$body_fig32 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_group <- !is.null(rv$mapping$name_groupvar) &&
                   nzchar(rv$mapping$name_groupvar)
      if (!has_group) return(shiny::tagList(shiny::br(), no_group_note))
      shiny::tagList(
        shiny::p("Bar chart of mean utility with 95% confidence intervals by group."),
        shiny::uiOutput(ns("cpick_fig32")),
        shiny::actionButton(ns("run_fig32"), "Run", class = "btn-sm btn-outline-primary"),
        shiny::br(), shiny::br(),
        shiny::uiOutput(ns("out_fig32"))
      )
    })

    shiny::observeEvent(input$run_fig32, {
      shiny::req(rv$processed_data, rv$mapping$name_groupvar,
                 nzchar(input$country_fig32 %||% ""))
      args     <- c(base_args(),
                    list(name_groupvar = "groupvar", country = input$country_fig32))
      call_str <- format_call("eq5d_utility_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_utility_by_group_plot, c(list(df = rv$processed_data), args))
        rfig32$plot  <- res$p
        rfig32$call  <- call_str
        save_result(rv, "Utility CI bar chart (fig. 3.2)", call_str, "plot", plot = rfig32$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig32 <- shiny::renderUI({
      if (is.null(rfig32$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig32"), height = "420px"),
        call_display(ns("call_fig32"))
      )
    })
    output$plt_fig32  <- shiny::renderPlot(rfig32$plot)
    output$call_fig32 <- shiny::renderText(rfig32$call)

    # ── Figure 3.1 (fu required) ──────────────────────────────────────────────
    rfig31 <- shiny::reactiveValues(plot = NULL, call = NULL)

    output$body_fig31 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_fu <- !is.null(rv$mapping$name_fu) && nzchar(rv$mapping$name_fu)
      if (!has_fu) return(shiny::tagList(shiny::br(), no_fu_required_note))
      shiny::tagList(
        shiny::p("Box plots of the utility index at each timepoint."),
        shiny::uiOutput(ns("cpick_fig31")),
        shiny::actionButton(ns("run_fig31"), "Run", class = "btn-sm btn-outline-primary"),
        shiny::br(), shiny::br(),
        shiny::uiOutput(ns("out_fig31"))
      )
    })

    shiny::observeEvent(input$run_fig31, {
      shiny::req(rv$processed_data, rv$mapping$name_fu,
                 nzchar(input$country_fig31 %||% ""))
      args     <- c(base_args(), list(name_fu = "fu", country = input$country_fig31))
      call_str <- format_call("eq5d_utility_over_time_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_utility_over_time_plot, c(list(df = rv$processed_data), args))
        rfig31$plot  <- res$p
        rfig31$call  <- call_str
        save_result(rv, "Utility box plots (fig. 3.1)", call_str, "plot", plot = rfig31$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig31 <- shiny::renderUI({
      if (is.null(rfig31$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig31"), height = "420px"),
        call_display(ns("call_fig31"))
      )
    })
    output$plt_fig31  <- shiny::renderPlot(rfig31$plot)
    output$call_fig31 <- shiny::renderText(rfig31$call)

    # ── Figure 3.3 (group + fu required) ─────────────────────────────────────
    rfig33 <- shiny::reactiveValues(plot = NULL, call = NULL)

    output$body_fig33 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_fu    <- !is.null(rv$mapping$name_fu)       && nzchar(rv$mapping$name_fu)
      has_group <- !is.null(rv$mapping$name_groupvar) && nzchar(rv$mapping$name_groupvar)
      if (!has_fu || !has_group) {
        missing <- c(if (!has_fu) "Timepoint", if (!has_group) "Group")
        return(shiny::tagList(
          shiny::br(),
          shiny::div(
            class = "alert alert-warning small py-2",
            shiny::icon("triangle-exclamation"), " ",
            paste("This analysis requires:",
                  paste(missing, collapse = " and "),
                  "variable(s). Please map them on the Data tab.")
          )
        ))
      }
      shiny::tagList(
        shiny::p("Mean utility with 95% CI by timepoint, coloured by group."),
        shiny::uiOutput(ns("cpick_fig33")),
        shiny::actionButton(ns("run_fig33"), "Run", class = "btn-sm btn-outline-primary"),
        shiny::br(), shiny::br(),
        shiny::uiOutput(ns("out_fig33"))
      )
    })

    shiny::observeEvent(input$run_fig33, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_groupvar,
                 nzchar(input$country_fig33 %||% ""))
      args     <- c(base_args(),
                    list(name_fu       = "fu",
                         name_groupvar = "groupvar",
                         country       = input$country_fig33))
      call_str <- format_call("eq5d_utility_change_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_utility_change_by_group_plot, c(list(df = rv$processed_data), args))
        rfig33$plot  <- res$p
        rfig33$call  <- call_str
        save_result(rv, "Utility by timepoint & group (fig. 3.3)", call_str,
                    "plot", plot = rfig33$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig33 <- shiny::renderUI({
      if (is.null(rfig33$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig33"), height = "420px"),
        call_display(ns("call_fig33"))
      )
    })
    output$plt_fig33  <- shiny::renderPlot(rfig33$plot)
    output$call_fig33 <- shiny::renderText(rfig33$call)

    # ── Figure 3.5 (VAS required) ─────────────────────────────────────────────
    rfig35 <- shiny::reactiveValues(plot = NULL, call = NULL)

    output$body_fig35 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_vas <- !is.null(rv$mapping$name_vas) && nzchar(rv$mapping$name_vas)
      if (!has_vas) return(shiny::tagList(shiny::br(), no_vas_note))
      shiny::tagList(
        shiny::p("Scatter plot of EQ-5D utility index values against EQ-VAS scores."),
        shiny::uiOutput(ns("cpick_fig35")),
        shiny::actionButton(ns("run_fig35"), "Run", class = "btn-sm btn-outline-primary"),
        shiny::br(), shiny::br(),
        shiny::uiOutput(ns("out_fig35"))
      )
    })

    shiny::observeEvent(input$run_fig35, {
      shiny::req(rv$processed_data, rv$mapping$name_vas,
                 nzchar(input$country_fig35 %||% ""))
      args     <- c(base_args(), list(name_vas = "vas", country = input$country_fig35))
      call_str <- format_call("eq5d_utility_vas_scatter_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_utility_vas_scatter_plot, c(list(df = rv$processed_data), args))
        rfig35$plot  <- res$p
        rfig35$call  <- call_str
        save_result(rv, "Utility vs VAS (fig. 3.5)", call_str, "plot", plot = rfig35$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig35 <- shiny::renderUI({
      if (is.null(rfig35$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig35"), height = "420px"),
        call_display(ns("call_fig35"))
      )
    })
    output$plt_fig35  <- shiny::renderPlot(rfig35$plot)
    output$call_fig35 <- shiny::renderText(rfig35$call)
  })
}

# Null-coalescing operator
`%||%` <- function(a, b) if (!is.null(a) && length(a) > 0L) a else b

# ui.R — main UI definition

# ── Shared footer (defined once) ──────────────────────────────────────────────
mih_footer <- shiny::tags$div(
  class = "mih-footer",
  style = "
    display:flex;
    flex-direction:column;
    align-items:center;
    justify-content:center;
    text-align:center;
    gap:0.4rem;
  ",
  
  # # Logo (big and centered)
  # shiny::tags$a(
  #   href   = "https://www.mathsinhealth.com",
  #   target = "_blank",
  #   shiny::tags$img(
  #     src = "logo.png",
  #     height = "20px",
  #     style = "margin-bottom:4px;"
  #   )
  # ),
  
  # Text below logo
  shiny::tags$div(
    style = "font-size:0.9rem;",
    "Developed by ",
    shiny::tags$a(
      href   = "https://www.mathsinhealth.com",
      target = "_blank",
      shiny::tags$strong("Maths in Health")
    )
  )
)

# ── Navbar brand with MiH logo ────────────────────────────────────────────────
navbar_brand <- shiny::tags$span(
  class = "mih-brand-logo",
  shiny::tags$img(
    src = "logo.png",
    height = "28px",
    style = "margin-right: 8px;"
  ),
  shiny::tags$span(
    shiny::tags$strong("eq5dsuite"),
    shiny::tags$span(class = "mih-label", "\u00a0|\u00a0EQ-5D Analysis")
  )
)

# ── UI ────────────────────────────────────────────────────────────────────────
ui <- shiny::tagList(
  bslib::page_navbar(
    title = navbar_brand,
    theme = bslib::bs_theme(
      version    = 5,
      bootswatch = "flatly",
      primary    = "#142036", #"#4f6f9f",
      success    = "#5c8f6f"
    ),
    # Load custom stylesheet
    shiny::tags$head(
      shiny::tags$link(rel = "stylesheet", type = "text/css", href = "styles.css")
    ),
    window_title = "eq5dsuite | Maths in Health",

    # ── Home ────────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("house"), " Home"),
      value = "home",
      bslib::layout_columns(
        col_widths = c(7, 5),

        # Left: welcome + workflow
        bslib::card(
          bslib::card_header(
            shiny::tags$span(shiny::icon("chart-line"), "Welcome to eq5dsuite")
          ),
          bslib::card_body(
            shiny::p(
              style = "font-size:2.88rem; color:#344055; margin-bottom:0.6rem;",
              "Welcome to eq5dsuite"
            ),
            shiny::p(
              style = "font-size:0.88rem; color:#344055; margin-bottom:0.6rem;",
              "Upload, validate, analyse, and export EQ-5D data using the ",
              shiny::code("eq5dsuite"), " R package. ",
              "Analyses follow ",
              shiny::tags$em("Devlin et al. (2020) — Methods for Analyzing and Reporting EQ-5D Data.")
            ),
            shiny::tags$div(
              style = "font-size:0.8rem; font-weight:600; color:#6b7280;
                       text-transform:uppercase; letter-spacing:0.05em;
                       margin-bottom:0.4rem;",
              "Workflow"
            ),
            shiny::tags$div(
              class = "workflow-steps",
              # Step 1
              shiny::tags$div(
                class = "workflow-step",
                shiny::tags$div(class = "workflow-step-num", "1"),
                shiny::tags$div(
                  class = "workflow-step-body",
                  shiny::tags$strong(shiny::icon("upload"), " Data"),
                  shiny::tags$span("Upload file & map columns")
                )
              ),
              # Step 2
              shiny::tags$div(
                class = "workflow-step",
                shiny::tags$div(class = "workflow-step-num", "2"),
                shiny::tags$div(
                  class = "workflow-step-body",
                  shiny::tags$strong(shiny::icon("shield-check"), " Validation"),
                  shiny::tags$span("Review data quality")
                )
              ),
              # Step 3
              shiny::tags$div(
                class = "workflow-step",
                shiny::tags$div(class = "workflow-step-num", "3"),
                shiny::tags$div(
                  class = "workflow-step-body",
                  shiny::tags$strong(shiny::icon("magnifying-glass-chart"), " Analysis"),
                  shiny::tags$span("Profile, VAS & utility")
                )
              ),
              # Step 4
              shiny::tags$div(
                class = "workflow-step",
                shiny::tags$div(class = "workflow-step-num", "4"),
                shiny::tags$div(
                  class = "workflow-step-body",
                  shiny::tags$strong(shiny::icon("table-list"), " Results"),
                  shiny::tags$span("Review saved outputs")
                )
              ),
              # Step 5
              shiny::tags$div(
                class = "workflow-step",
                shiny::tags$div(class = "workflow-step-num", "5"),
                shiny::tags$div(
                  class = "workflow-step-body",
                  shiny::tags$strong(shiny::icon("download"), " Export"),
                  shiny::tags$span("Download tables & plots")
                )
              )
            )
          )
        ),

        # Right: quick start
        bslib::card(
          bslib::card_header(
            shiny::tags$span(shiny::icon("circle-info"), " Quick start")
          ),
          bslib::card_body(
            shiny::tags$div(
              style = "font-size:0.8rem; font-weight:600; color:#6b7280;
                       text-transform:uppercase; letter-spacing:0.05em;
                       margin-bottom:0.35rem;",
              "Accepted formats"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$span(
                shiny::code(".csv"), ", ", shiny::code(".xlsx"), ", ",
                shiny::code(".rds")
              )
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              "EQ-5D-3L and EQ-5D-5L"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              "Cross-sectional and longitudinal"
            ),
            shiny::tags$hr(style = "margin: 0.55rem 0;"),
            shiny::tags$div(
              style = "font-size:0.8rem; font-weight:600; color:#6b7280;
                       text-transform:uppercase; letter-spacing:0.05em;
                       margin-bottom:0.35rem;",
              "Required"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              "Five EQ-5D dimensions: MO, SC, UA, PD, AD"
            ),
            shiny::tags$hr(style = "margin: 0.55rem 0;"),
            shiny::tags$div(
              style = "font-size:0.8rem; font-weight:600; color:#6b7280;
                       text-transform:uppercase; letter-spacing:0.05em;
                       margin-bottom:0.35rem;",
              "Optional"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$strong("Timepoint"), " — unlocks longitudinal analyses"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$strong("Group"), " — enables stratified analyses"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$strong("Patient ID"), " — matched analyses (PCHC)"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$strong("VAS score"), " — EQ-VAS analyses"
            ),
            shiny::tags$div(
              class = "qs-item",
              shiny::tags$span(class = "qs-dot", "\u25cf"),
              shiny::tags$strong("Utility / index"), " — or calculate on Data page"
            )
          )
        )
      )
    ),

    # ── Data ──────────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("upload"), " Data"),
      value = "data",
      mod_data_ui("data")
    ),

    # ── Validation ────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("shield-check"), " Validation"),
      value = "validation",
      mod_validation_ui("validation")
    ),

    # ── Analysis ──────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("chart-bar"), " Analysis"),
      value = "analysis",
      shiny::tabsetPanel(
        id   = "analysis_main_tabs",
        type = "tabs",
        shiny::tabPanel(
          title = "EQ-5D Profile",
          value = "tab_profile",
          mod_analysis_profile_ui("profile")
        ),
        shiny::tabPanel(
          title = "EQ-VAS",
          value = "tab_vas",
          mod_analysis_vas_ui("vas")
        ),
        shiny::tabPanel(
          title = "EQ-5D Utility",
          value = "tab_utility",
          mod_analysis_utility_ui("utility")
        )
      )
    ),

    # ── Results ───────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("table-list"), " Results"),
      value = "results",
      mod_results_ui("results")
    ),

    # ── Export ────────────────────────────────────────────────────────────────
    bslib::nav_panel(
      title = shiny::tagList(shiny::icon("download"), " Export"),
      value = "export",
      mod_export_ui("export")
    ),

    # Right-aligned nav items
    bslib::nav_spacer(),
    bslib::nav_item(
      shiny::tags$small(
        shiny::tags$a(
          href   = "https://github.com/MathsInHealth/eq5dsuite",
          target = "_blank",
          style  = "color: rgba(255,255,255,0.5); font-size:0.75rem;",
          "eq5dsuite package"
        )
      )
    )
  )

  # ── Global footer (defined once, visible on all pages) ────────────────────
  # mih_footer
)

# mod_results.R — Results explorer module

# ── UI ────────────────────────────────────────────────────────────────────────

mod_results_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    bslib::layout_columns(
      col_widths = c(4, 8),

      # Left: results list
      bslib::card(
        bslib::card_header("Saved Results"),
        bslib::card_body(
          shiny::uiOutput(ns("results_list"))
        )
      ),

      # Right: result viewer
      bslib::card(
        bslib::card_header(shiny::uiOutput(ns("viewer_title"))),
        bslib::card_body(
          shiny::uiOutput(ns("result_viewer"))
        )
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_results_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    # Currently selected result id
    selected_id <- shiny::reactiveVal(NULL)

    # ── Results list ──────────────────────────────────────────────────────────
    output$results_list <- shiny::renderUI({
      results <- rv$results
      if (length(results) == 0L) {
        return(shiny::p("No results saved yet. Run analyses and they will appear here.",
                        class = "text-muted small"))
      }

      # Build a button list (most recent first)
      btns <- lapply(rev(results), function(r) {
        icon_name <- switch(r$result_type,
                            table = "table",
                            plot  = "chart-bar",
                            both  = "layer-group",
                            "file")
        shiny::actionButton(
          ns(paste0("view_", r$id)),
          label = shiny::tagList(
            shiny::icon(icon_name), " ",
            r$label,
            shiny::tags$br(),
            shiny::tags$small(
              class = "text-muted",
              format(r$timestamp, "%H:%M:%S %d %b %Y")
            )
          ),
          class = "btn btn-outline-secondary w-100 text-start mb-1",
          style = "white-space: normal;"
        )
      })
      shiny::div(btns)
    })

    # Observe click on any result button
    shiny::observe({
      results <- rv$results
      lapply(results, function(r) {
        shiny::observeEvent(input[[paste0("view_", r$id)]], {
          selected_id(r$id)
        }, ignoreInit = TRUE)
      })
    })

    # ── Viewer title ──────────────────────────────────────────────────────────
    output$viewer_title <- shiny::renderUI({
      id  <- selected_id()
      res <- find_result(rv$results, id)
      if (is.null(res)) return("Result Viewer")
      shiny::tagList(res$label,
                     shiny::tags$small(
                       class = "text-muted ms-2",
                       format(res$timestamp, "%d %b %Y %H:%M")
                     ))
    })

    # ── Result viewer ─────────────────────────────────────────────────────────
    output$result_viewer <- shiny::renderUI({
      id  <- selected_id()
      res <- find_result(rv$results, id)

      if (is.null(res)) {
        return(shiny::p("Select a result from the list on the left.",
                        class = "text-muted"))
      }

      content <- list()

      # Show table
      if (res$result_type %in% c("table", "both") && !is.null(res$data)) {
        content <- c(content, list(
          shiny::h6("Table"),
          DT::DTOutput(ns("viewer_table"))
        ))
      }

      # Show plot
      if (res$result_type %in% c("plot", "both") && !is.null(res$plot)) {
        content <- c(content, list(
          shiny::h6("Plot"),
          shiny::plotOutput(ns("viewer_plot"), height = "420px")
        ))
      }

      # Show R call
      content <- c(content, list(
        shiny::hr(),
        shiny::tags$details(
          shiny::tags$summary(shiny::tags$small("R code used", class = "text-muted")),
          shiny::verbatimTextOutput(ns("viewer_call"), placeholder = TRUE)
        )
      ))

      shiny::tagList(content)
    })

    # Render selected result table
    output$viewer_table <- DT::renderDT({
      id  <- selected_id()
      res <- find_result(rv$results, id)
      shiny::req(res, !is.null(res$data))
      DT::datatable(res$data,
                    options  = list(pageLength = 15L, scrollX = TRUE, dom = "tip"),
                    rownames = FALSE,
                    class    = "table-sm table-striped")
    })

    # Render selected result plot
    output$viewer_plot <- shiny::renderPlot({
      id  <- selected_id()
      res <- find_result(rv$results, id)
      shiny::req(res, !is.null(res$plot))
      res$plot
    })

    # Render selected result R call
    output$viewer_call <- shiny::renderText({
      id  <- selected_id()
      res <- find_result(rv$results, id)
      shiny::req(res)
      res$fn_call
    })
  })
}

# ── Helper ────────────────────────────────────────────────────────────────────

find_result <- function(results, id) {
  if (is.null(id) || length(results) == 0L) return(NULL)
  idx <- which(vapply(results, function(r) identical(r$id, id), logical(1L)))
  if (length(idx) == 0L) return(NULL)
  results[[idx[1L]]]
}

# global.R — loaded once by Shiny before ui.R and server.R
# Loads packages, sources modules, and defines shared helpers.

library(shiny)
library(bslib)
library(DT)

# Source all modules
for (.f in list.files("modules", pattern = "\\.R$", full.names = TRUE)) {
  source(.f, local = FALSE)
}
rm(.f)

# ── Shared helper functions ───────────────────────────────────────────────────

#' Read an uploaded file (CSV, XLSX, or RDS)
read_uploaded_file <- function(path, ext) {
  ext <- tolower(ext)
  switch(ext,
    csv  = utils::read.csv(path, stringsAsFactors = FALSE, check.names = FALSE),
    xlsx = readxl::read_excel(path) |> as.data.frame(),
    rds  = readRDS(path),
    stop("Unsupported file type: ", ext, call. = FALSE)
  )
}

#' Suggest column mapping from a data frame's names
suggest_mapping <- function(col_names) {
  lower <- tolower(col_names)

  match_first <- function(patterns) {
    for (p in patterns) {
      idx <- grep(p, lower)
      if (length(idx) > 0L) return(col_names[idx[1L]])
    }
    NULL
  }

  list(
    mo      = match_first(c("^mo$", "mobil")),
    sc      = match_first(c("^sc$", "self.?care", "selfcare")),
    ua      = match_first(c("^ua$", "usual.?act")),
    pd      = match_first(c("^pd$", "pain")),
    ad      = match_first(c("^ad$", "anxiet")),
    name_fu = match_first(c("^fu$", "^time$", "timepoint", "follow")),
    name_groupvar = match_first(c("^group$", "groupvar", "procedure", "category", "cohort")),
    name_id = match_first(c("^id$", "patient.?id", "subject.?id", "person.?id")),
    name_vas = match_first(c("^vas$", "eq.?vas", "visual"))
  )
}

#' Apply confirmed mapping: rename columns to standard names and cast types
apply_mapping <- function(raw_data, mapping) {
  df <- raw_data

  # Rename EQ-5D dimension columns to standard names
  std_dims <- c("mo", "sc", "ua", "pd", "ad")
  for (i in seq_along(std_dims)) {
    orig <- mapping$names_eq5d[i]
    std  <- std_dims[i]
    if (!is.null(orig) && nzchar(orig) && orig != std && orig %in% names(df)) {
      names(df)[names(df) == orig] <- std
    }
  }

  # Cast dimensions to integer
  for (d in std_dims) {
    if (d %in% names(df)) df[[d]] <- suppressWarnings(as.integer(df[[d]]))
  }

  # Rename optional columns
  rename_col <- function(df, from, to) {
    if (!is.null(from) && nzchar(from) && from != to && from %in% names(df)) {
      names(df)[names(df) == from] <- to
    }
    df
  }

  df <- rename_col(df, mapping$name_fu,       "fu")
  df <- rename_col(df, mapping$name_groupvar,  "groupvar")
  df <- rename_col(df, mapping$name_id,        "id")
  df <- rename_col(df, mapping$name_vas,       "vas")
  df <- rename_col(df, mapping$name_utility,   "utility")

  if ("vas" %in% names(df)) df$vas <- suppressWarnings(as.numeric(df$vas))

  df
}

#' Build a human-readable R function call string for display
format_call <- function(fn_name, args) {
  fmt_val <- function(v) {
    if (is.null(v)) return("NULL")
    if (is.character(v) && length(v) > 1L) {
      paste0('c("', paste(v, collapse = '", "'), '")')
    } else if (is.character(v)) {
      paste0('"', v, '"')
    } else if (is.logical(v)) {
      toupper(as.character(v))
    } else {
      as.character(v)
    }
  }
  pairs <- mapply(
    function(nm, val) paste0("  ", nm, " = ", fmt_val(val)),
    names(args), args,
    SIMPLIFY = FALSE
  )
  paste0(fn_name, "(\n", paste(pairs, collapse = ",\n"), "\n)")
}

#' Save a result to rv$results
save_result <- function(rv, label, fn_call, result_type, data = NULL, plot = NULL) {
  entry <- list(
    id          = paste0("r", format(Sys.time(), "%Y%m%d%H%M%S%OS3")),
    timestamp   = Sys.time(),
    label       = label,
    fn_call     = fn_call,
    result_type = result_type,   # "table", "plot", or "both"
    data        = data,
    plot        = plot
  )
  rv$results <- c(rv$results, list(entry))
  invisible(entry$id)
}

#' Get version-aware utility method choices for the UI
#'
#' Returns a named character vector suitable for selectInput choices,
#' filtered to only include methods valid for the given EQ-5D version:
#' - 3L: direct 3L value sets + reverse crosswalk (XWR)
#' - 5L: direct 5L value sets + crosswalk (XW)
get_utility_method_choices <- function(eq5d_version) {
  if (identical(eq5d_version, "3L")) {
    c(
      "Direct (3L value set)"              = "direct",
      "Reverse crosswalk 3L\u21925L (XWR)" = "xwr"
    )
  } else {
    c(
      "Direct (5L value set)"        = "direct",
      "Crosswalk 5L\u21923L (XW)"   = "xw"
    )
  }
}

#' Get available value-set country codes for a given EQ-5D version
get_country_choices <- function(eq5d_version) {
  tryCatch({
    vs_df <- eqvs_display(version = eq5d_version, return_df = TRUE)
    codes <- vs_df[["VS_code"]]
    names_col <- if ("Name" %in% names(vs_df)) vs_df[["Name"]] else codes
    stats::setNames(codes, paste0(names_col, " (", codes, ")"))
  }, error = function(e) character(0L))
}

# ── Expose internal package helpers in Shiny scope ────────────────────────────
# These are defined in R/shiny_helpers.R as non-exported functions.

DIM_LABELS          <- eq5dsuite:::DIM_LABELS
ensure_fu           <- eq5dsuite:::ensure_fu
compute_utility_col <- eq5dsuite:::compute_utility_col

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4
)
library(eq5dsuite)

## ----load-data----------------------------------------------------------------
# The example dataset is bundled with the package
data(example_data)
head(example_data)

## ----wex1-prep----------------------------------------------------------------
dim_names <- c("mo", "sc", "ua", "pd", "ad")

# Subset to hip replacement patients
hip_data <- example_data[example_data$procedure == "Hip Replacement", ]

# Add profile code and EQ-5D value
hip_data$profile_code <- toEQ5Dindex(
  x         = hip_data,
  dim.names = dim_names
)
hip_data$value <- eq5d3l(
  hip_data[, dim_names],
  country   = "UK",
  dim.names = dim_names
)

# Pre-operative subset
hip_preop <- hip_data[hip_data$time == "Pre-op", ]

## ----wex1-profile-------------------------------------------------------------
eq5d_profile_level_summary(
  df           = hip_preop,
  names_eq5d   = dim_names,
  eq5d_version = "3L"
)

## ----wex1-top-states----------------------------------------------------------
eq5d_profile_top_states(
  df           = hip_preop,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  n            = 5
)

## ----wex1-pchc, fig.cap = "PCHC classification for hip replacement patients."----
eq5d_profile_pchc_by_group_plot(
  df         = hip_data,
  name_id    = "id",
  names_eq5d = dim_names,
  name_fu    = "time",
  levels_fu  = c("Pre-op", "Post-op")
)$p +
  ggplot2::labs(
    title = "PCHC: hip replacement patients",
    x     = NULL,
    y     = "Percentage of respondents"
  ) +
  ggplot2::theme_minimal()

## ----wex1-better, fig.cap = "Dimensions improved among patients classified as Better."----
eq5d_profile_better_dimensions_by_group_plot(
  df         = hip_data,
  name_id    = "id",
  names_eq5d = dim_names,
  name_fu    = "time",
  levels_fu  = c("Pre-op", "Post-op")
)$p +
  ggplot2::theme_minimal()

## ----wex1-utility-------------------------------------------------------------
eq5d_utility_summary(
  df           = hip_data,
  name_fu      = "time",
  levels_fu    = c("Pre-op", "Post-op"),
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)

## ----wex1-vas-----------------------------------------------------------------
eq5d_vas_summary(
  df        = hip_data,
  name_vas  = "vas",
  name_fu   = "time",
  levels_fu = c("Pre-op", "Post-op")
)

## ----wex2-prep----------------------------------------------------------------
# Subset to two procedures, pre-operative only
procs <- c("Knee Replacement", "Groin Hernia")
comparison_data <- example_data[
  example_data$procedure %in% procs &
  example_data$time == "Pre-op", ]

# Add profile code and EQ-5D value
comparison_data$profile_code <- toEQ5Dindex(
  x         = comparison_data,
  dim.names = dim_names
)
comparison_data$value <- eq5d3l(
  comparison_data[, dim_names],
  country   = "UK",
  dim.names = dim_names
)

## ----wex2-profile-------------------------------------------------------------
eq5d_profile_level_summary_by_group(
  df           = comparison_data,
  names_eq5d   = dim_names,
  name_cat     = "procedure",
  eq5d_version = "3L"
)

## ----wex2-utility-summary-----------------------------------------------------
eq5d_utility_summary_by_group(
  df            = comparison_data,
  names_eq5d    = dim_names,
  name_groupvar = "procedure",
  eq5d_version  = "3L",
  country       = "UK"
)

## ----wex2-utility-plot, fig.cap = "Mean pre-operative EQ-5D values by procedure group."----
eq5d_utility_by_group_plot(
  df            = comparison_data,
  names_eq5d    = dim_names,
  name_groupvar = "procedure",
  eq5d_version  = "3L",
  country       = "UK"
)$p +
  ggplot2::theme_minimal()

## ----wex2-lss-hernia, fig.cap = "EQ-5D values by LSS — Groin Hernia."---------
hernia_data <- comparison_data[
  comparison_data$procedure == "Groin Hernia", ]

eq5d_profile_lss_utility_plot(
  hernia_data,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)$p +
  ggplot2::labs(
    title = "Groin Hernia",
    x     = "Level Sum Score (LSS)",
    y     = "EQ-5D value"
  ) +
  ggplot2::scale_x_continuous(limits = c(5, 15),
                               breaks = seq(5, 15, 2)) +
  ggplot2::theme_minimal()

## ----wex2-lss-knee, fig.cap = "EQ-5D values by LSS — Knee Replacement."-------
knee_data <- comparison_data[
  comparison_data$procedure == "Knee Replacement", ]

eq5d_profile_lss_utility_plot(
  knee_data,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)$p +
  ggplot2::labs(
    title = "Knee Replacement",
    x     = "Level Sum Score (LSS)",
    y     = "EQ-5D value"
  ) +
  ggplot2::scale_x_continuous(limits = c(5, 15),
                               breaks = seq(5, 15, 2)) +
  ggplot2::theme_minimal()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)

## ----update-basic, eval = FALSE-----------------------------------------------
# update_value_sets()

## ----update-specific, eval = FALSE--------------------------------------------
# # Check only EQ-5D-5L value sets
# update_value_sets(versions = "5L")
# 
# # Check 3L and Y3L only
# update_value_sets(versions = c("3L", "Y3L"))

## ----update-noninteractive, eval = FALSE--------------------------------------
# # Install without asking for confirmation
# update_value_sets(ask = FALSE)

# mod_analysis_vas.R — EQ-VAS analysis module

# ── UI ────────────────────────────────────────────────────────────────────────

mod_analysis_vas_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::uiOutput(ns("guard")),
    shiny::conditionalPanel(
      condition = "output.app_has_data",
      bslib::accordion(
        open = "acc_21",

        # Table 2.1 — summary stats by timepoint
        bslib::accordion_panel(
          title = "Table 2.1 \u2014 VAS summary statistics by timepoint",
          value = "acc_21",
          shiny::p("Descriptive statistics for the EQ-VAS score, grouped by timepoint."),
          shiny::uiOutput(ns("note_fu_21")),
          shiny::uiOutput(ns("grp_sel_ui_21")),
          shiny::actionButton(ns("run_21"), "Run",
                              class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_21"))
        ),

        # Table 2.2 — frequency of mid-points
        bslib::accordion_panel(
          title = "Table 2.2 \u2014 VAS frequency of mid-points",
          value = "acc_22",
          shiny::p("Frequency distribution of EQ-VAS scores by mid-point intervals."),
          shiny::actionButton(ns("run_22"), "Run",
                              class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_22"))
        ),

        # Figure 2.1 — histogram
        bslib::accordion_panel(
          title = "Figure 2.1 \u2014 VAS distribution (histogram)",
          value = "acc_fig21",
          shiny::p("Histogram of the EQ-VAS distribution."),
          shiny::actionButton(ns("run_fig21"), "Run",
                              class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_fig21"))
        ),

        # Figure 2.2 — box plot by midpoints
        bslib::accordion_panel(
          title = "Figure 2.2 \u2014 VAS distribution by midpoints (box plot)",
          value = "acc_fig22",
          shiny::p("Box plots of EQ-VAS score distribution by midpoint intervals."),
          shiny::actionButton(ns("run_fig22"), "Run",
                              class = "btn-sm btn-outline-primary"),
          shiny::br(), shiny::br(),
          shiny::uiOutput(ns("out_fig22"))
        )
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_analysis_vas_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    output$guard <- shiny::renderUI({
      if (is.null(rv$processed_data)) {
        bslib::card(bslib::card_body(
          shiny::p(shiny::icon("circle-info"),
                   " Please complete data upload and validation first.",
                   class = "text-muted")
        ))
      }
    })

    # ── Info note: no timepoint mapped ────────────────────────────────────────
    output$note_fu_21 <- shiny::renderUI({
      shiny::req(rv$mapping)
      has_fu <- !is.null(rv$mapping$name_fu) && nzchar(rv$mapping$name_fu)
      if (!has_fu) {
        shiny::div(
          class = "alert alert-info small py-2",
          shiny::icon("circle-info"), " ",
          "No timepoint variable mapped. ",
          "Results will be shown for all observations combined."
        )
      }
    })

    # ── Optional group filter for Table 2.1 ───────────────────────────────────
    output$grp_sel_ui_21 <- shiny::renderUI({
      shiny::req(rv$processed_data, rv$mapping)
      has_group <- !is.null(rv$mapping$name_groupvar) &&
                   nzchar(rv$mapping$name_groupvar)
      if (!has_group) return(NULL)
      df       <- rv$processed_data
      grp_vals <- if ("groupvar" %in% names(df)) {
        sort(unique(as.character(df[["groupvar"]])))
      } else {
        character(0L)
      }
      if (length(grp_vals) == 0L) return(NULL)
      shiny::selectInput(
        ns("grp_sel_21"), "Filter by group",
        choices  = c("All" = "All", stats::setNames(grp_vals, grp_vals)),
        selected = "All"
      )
    })

    # ── Table 2.1 ─────────────────────────────────────────────────────────────
    r21 <- shiny::reactiveValues(data = NULL, call = NULL)

    shiny::observeEvent(input$run_21, {
      shiny::req(rv$processed_data)
      df      <- rv$processed_data
      # Optional group filter
      grp_sel <- input$grp_sel_21
      if (!is.null(grp_sel) && grp_sel != "All" && "groupvar" %in% names(df)) {
        df <- df[df[["groupvar"]] == grp_sel, , drop = FALSE]
      }
      fu_info  <- ensure_fu(df, rv$mapping)
      args     <- list(name_vas = "vas", name_fu = fu_info$name_fu)
      call_str <- format_call("eq5d_vas_summary", list(df = "data", name_vas = "vas"))
      tryCatch({
        r21$data <- do.call(eq5d_vas_summary, c(list(df = fu_info$df), args))
        r21$call <- call_str
        save_result(rv, "VAS summary stats (2.1)", call_str, "table", data = r21$data)
      }, error = function(e) err_notify(e))
    })

    output$out_21 <- shiny::renderUI({
      if (is.null(r21$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_21")), call_display(ns("call_21")))
    })
    output$tbl_21  <- render_analysis_table(shiny::reactive(r21$data))
    output$call_21 <- shiny::renderText(r21$call)

    # ── Table 2.2 ─────────────────────────────────────────────────────────────
    r22 <- shiny::reactiveValues(data = NULL, call = NULL)

    shiny::observeEvent(input$run_22, {
      shiny::req(rv$processed_data)
      args     <- list(name_vas = "vas")
      call_str <- format_call("eq5d_vas_distribution_table", c(list(df = "data"), args))
      tryCatch({
        r22$data <- do.call(eq5d_vas_distribution_table, c(list(df = rv$processed_data), args))
        r22$call <- call_str
        save_result(rv, "VAS mid-points (2.2)", call_str, "table", data = r22$data)
      }, error = function(e) err_notify(e))
    })

    output$out_22 <- shiny::renderUI({
      if (is.null(r22$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_22")), call_display(ns("call_22")))
    })
    output$tbl_22  <- render_analysis_table(shiny::reactive(r22$data))
    output$call_22 <- shiny::renderText(r22$call)

    # ── Figure 2.1 ────────────────────────────────────────────────────────────
    rfig21 <- shiny::reactiveValues(plot = NULL, call = NULL)

    shiny::observeEvent(input$run_fig21, {
      shiny::req(rv$processed_data)
      args     <- list(name_vas = "vas")
      call_str <- format_call("eq5d_vas_histogram", c(list(df = "data"), args))
      tryCatch({
        res         <- do.call(eq5d_vas_histogram, c(list(df = rv$processed_data), args))
        rfig21$plot <- res$p
        rfig21$call <- call_str
        save_result(rv, "VAS histogram (fig. 2.1)", call_str, "plot", plot = rfig21$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig21 <- shiny::renderUI({
      if (is.null(rfig21$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig21"), height = "400px"),
        call_display(ns("call_fig21"))
      )
    })
    output$plt_fig21  <- shiny::renderPlot(rfig21$plot)
    output$call_fig21 <- shiny::renderText(rfig21$call)

    # ── Figure 2.2 ────────────────────────────────────────────────────────────
    rfig22 <- shiny::reactiveValues(plot = NULL, call = NULL)

    shiny::observeEvent(input$run_fig22, {
      shiny::req(rv$processed_data)
      args     <- list(name_vas = "vas")
      call_str <- format_call("eq5d_vas_grouped_distribution_plot", c(list(df = "data"), args))
      tryCatch({
        res         <- do.call(eq5d_vas_grouped_distribution_plot, c(list(df = rv$processed_data), args))
        rfig22$plot <- res$p
        rfig22$call <- call_str
        save_result(rv, "VAS box plot (fig. 2.2)", call_str, "plot", plot = rfig22$plot)
      }, error = function(e) err_notify(e))
    })

    output$out_fig22 <- shiny::renderUI({
      if (is.null(rfig22$plot)) return(NULL)
      shiny::tagList(
        shiny::plotOutput(ns("plt_fig22"), height = "400px"),
        call_display(ns("call_fig22"))
      )
    })
    output$plt_fig22  <- shiny::renderPlot(rfig22$plot)
    output$call_fig22 <- shiny::renderText(rfig22$call)
  })
}

# server.R — main server function

server <- function(input, output, session) {

  # ── Shared reactive state ─────────────────────────────────────────────────
  rv <- shiny::reactiveValues(
    raw_data       = NULL,   # data.frame as uploaded
    mapping        = NULL,   # named list of column mapping + version + country
    processed_data = NULL,   # data.frame with standardised column names
    results        = list()  # list of saved analysis results
  )

  # ── Initialise: hide conditional analysis tabs ─────────────────────────────
  shiny::hideTab("analysis_main_tabs", "tab_vas",     session = session)
  shiny::hideTab("analysis_main_tabs", "tab_utility", session = session)

  # ── Module servers ─────────────────────────────────────────────────────────
  mod_data_server("data",             rv)
  mod_validation_server("validation", rv)
  mod_analysis_profile_server("profile", rv)
  mod_analysis_vas_server("vas",         rv)
  mod_analysis_utility_server("utility", rv)
  mod_results_server("results", rv)
  mod_export_server("export",   rv)

  # ── Conditional analysis tab visibility ────────────────────────────────────
  shiny::observe({
    mapping <- rv$mapping
    if (is.null(mapping)) return()

    # EQ-VAS tab: show when VAS is mapped
    has_vas <- !is.null(mapping$name_vas) && nzchar(mapping$name_vas)
    if (has_vas) {
      shiny::showTab("analysis_main_tabs", "tab_vas", session = session)
    } else {
      shiny::hideTab("analysis_main_tabs", "tab_vas", session = session)
    }

    # Utility tab: show when utility column is mapped OR was calculated on
    # the Data page (rv$mapping$name_utility updated by mod_data_server).
    has_utility <- !is.null(mapping$name_utility) && nzchar(mapping$name_utility)
    if (has_utility) {
      shiny::showTab("analysis_main_tabs", "tab_utility", session = session)
    } else {
      shiny::hideTab("analysis_main_tabs", "tab_utility", session = session)
    }
  })

  # ── Provide flags to conditionalPanel (suspendWhenHidden = FALSE required) ─
  output$app_has_data      <- shiny::reactive(!is.null(rv$processed_data))
  output$app_has_timepoint <- shiny::reactive(
    !is.null(rv$mapping) && !is.null(rv$mapping$name_fu) && nzchar(rv$mapping$name_fu)
  )
  output$app_has_group <- shiny::reactive(
    !is.null(rv$mapping) &&
      !is.null(rv$mapping$name_groupvar) && nzchar(rv$mapping$name_groupvar)
  )
  output$app_has_id <- shiny::reactive(
    !is.null(rv$mapping) &&
      !is.null(rv$mapping$name_id) && nzchar(rv$mapping$name_id)
  )
  shiny::outputOptions(output, "app_has_data",      suspendWhenHidden = FALSE)
  shiny::outputOptions(output, "app_has_timepoint", suspendWhenHidden = FALSE)
  shiny::outputOptions(output, "app_has_group",     suspendWhenHidden = FALSE)
  shiny::outputOptions(output, "app_has_id",        suspendWhenHidden = FALSE)
}

# mod_data.R — Data upload, variable mapping, utility calculation, and download

# ── UI ────────────────────────────────────────────────────────────────────────

mod_data_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    # Row 1: Upload + Preview
    bslib::layout_columns(
      col_widths = c(5, 7),
      
      # Left panel: upload + mapping controls
      bslib::card(
        bslib::card_header(
          shiny::tags$span(shiny::icon("upload"), " Upload & Variable Mapping")
        ),
        bslib::card_body(
          
          # File upload
          shiny::fileInput(
            ns("file"), "Choose file",
            accept      = c(".csv", ".xlsx", ".rds"),
            placeholder = "CSV, XLSX, or RDS"
          ),
          shiny::div(
            class = "d-flex align-items-center my-2",
            shiny::tags$span(class = "text-muted small me-2", "or"),
            shiny::actionButton(
              ns("use_example"), "Try example data",
              icon  = shiny::icon("database"),
              class = "btn-outline-secondary btn-sm"
            )
          ),

          # EQ-5D version
          shiny::selectInput(
            ns("version"), "EQ-5D version",
            choices  = c("3L", "5L"),
            selected = "3L"
          ),
          
          shiny::hr(style = "margin: 0.6rem 0;"),
          
          # Column mapping (rendered after upload)
          # NOTE: Country selector has been removed from here.
          # It now lives exclusively in the Calculate Utility section.
          shiny::uiOutput(ns("mapping_ui")),
          
          shiny::hr(style = "margin: 0.6rem 0;"),
          shiny::actionButton(
            ns("confirm"), "Confirm Mapping",
            class = "btn-primary w-100",
            icon  = shiny::icon("check")
          )
        )
      ),
      
      # Right panel: data preview
      bslib::card(
        bslib::card_header(
          shiny::tags$span(shiny::icon("table"), " Data Preview")
        ),
        bslib::card_body(
          shiny::uiOutput(ns("preview_info")),
          DT::DTOutput(ns("preview_table"))
        )
      )
    ),
    
    # Row 2: Utility calculation (shown after mapping confirmed)
    shiny::uiOutput(ns("utility_section_ui")),
    
    # Row 3: Download (shown when data is ready)
    shiny::uiOutput(ns("download_section_ui"))
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_data_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns
    
    # ── Reactive: read uploaded file ─────────────────────────────────────────
    active_data <- shiny::reactiveVal(NULL)

    shiny::observeEvent(input$file, {
      ext <- tools::file_ext(input$file$name)
      tryCatch(
        active_data(read_uploaded_file(input$file$datapath, ext)),
        error = function(e) {
          shiny::showNotification(paste("Error reading file:", conditionMessage(e)),
                                  type = "error", duration = 8)
        }
      )
    })

    shiny::observeEvent(input$use_example, {
      active_data(eq5dsuite::example_data)
      shiny::showNotification("Example dataset loaded (10 000 rows).",
                              type = "message", duration = 3)
    })

    uploaded <- shiny::reactive(active_data())
    
    # ── Reactive: smart column suggestions ───────────────────────────────────
    suggestions <- shiny::reactive({
      df <- uploaded()
      if (is.null(df)) return(list())
      suggest_mapping(names(df))
    })
    
    # ── Render: column mapping selectors ─────────────────────────────────────
    output$mapping_ui <- shiny::renderUI({
      df <- uploaded()
      if (is.null(df)) {
        return(shiny::p(shiny::em("Upload a file to map columns."),
                        class = "text-muted"))
      }
      cols      <- names(df)
      cols_none <- c("(none)" = "")
      sug       <- suggestions()
      
      sel <- function(label, inputId, choices, selected, ...) {
        shiny::selectInput(
          ns(inputId), label,
          choices  = choices,
          selected = selected %||% "",
          ...
        )
      }
      
      shiny::tagList(
        shiny::h6("EQ-5D Dimensions", class = "text-muted fw-semibold mt-2"),
        shiny::p(shiny::em("All five dimensions are required."),
                 class = "small text-muted"),
        bslib::layout_columns(
          col_widths = c(6, 6),
          sel("Mobility (MO)",           "col_mo", cols, sug$mo),
          sel("Self-care (SC)",          "col_sc", cols, sug$sc),
          sel("Usual activities (UA)",   "col_ua", cols, sug$ua),
          sel("Pain/discomfort (PD)",    "col_pd", cols, sug$pd),
          sel("Anxiety/depression (AD)", "col_ad", cols, sug$ad),
          shiny::div()   # placeholder to keep 2-col layout tidy
        ),
        
        shiny::h6("Optional Variables", class = "text-muted fw-semibold mt-3"),
        bslib::layout_columns(
          col_widths = c(6, 6),
          sel("Timepoint / Follow-up", "col_fu",
              c(cols_none, cols), sug$name_fu),
          sel("Group / Category",      "col_groupvar",
              c(cols_none, cols), sug$name_groupvar),
          sel("Patient ID",            "col_id",
              c(cols_none, cols), sug$name_id),
          sel("EQ-VAS score",          "col_vas",
              c(cols_none, cols), sug$name_vas),
          sel("Utility / Index column", "col_utility",
              c(cols_none, cols), sug$name_utility)
        )
      )
    })
    
    # ── Render: data preview ──────────────────────────────────────────────────
    # FIXED: preview now uses local_data() (which reflects processed_data if
    # available) rather than the raw uploaded() reactive.  This means the
    # table refreshes immediately whenever a utility column is added.
    output$preview_info <- shiny::renderUI({
      df <- local_data()
      if (is.null(df)) {
        # Fall back to raw upload count before mapping is confirmed
        df_raw <- uploaded()
        if (is.null(df_raw)) return(NULL)
        df <- df_raw
      }
      shiny::p(
        shiny::strong(format(nrow(df), big.mark = ",")), " rows \u00d7 ",
        shiny::strong(ncol(df)), " columns",
        class = "text-muted small mb-2"
      )
    })
    
    output$preview_table <- DT::renderDT({
      # Use local_data() when mapping is confirmed (includes utility cols);
      # fall back to raw uploaded data while mapping is still being set up.
      df <- tryCatch(local_data(), error = function(e) NULL)
      if (is.null(df)) df <- uploaded()
      shiny::req(df)
      DT::datatable(
        head(df, 200L),
        options  = list(pageLength = 10L, scrollX = TRUE, dom = "tip"),
        rownames = FALSE,
        class    = "table-sm table-striped"
      )
    })
    
    # ── Validation helpers ────────────────────────────────────────────────────
    mapping_complete <- shiny::reactive({
      if (is.null(uploaded())) return(FALSE)
      dims <- c(input$col_mo, input$col_sc, input$col_ua, input$col_pd, input$col_ad)
      all(nzchar(dims)) && length(unique(dims)) == 5L
    })
    
    # ── Confirm mapping ───────────────────────────────────────────────────────
    shiny::observeEvent(input$confirm, {
      if (!isTRUE(mapping_complete())) {
        shiny::showNotification(
          "Please map all five EQ-5D dimension columns (MO, SC, UA, PD, AD) \
           to distinct columns before confirming.",
          type = "warning", duration = 5
        )
        return()
      }
      
      rv$raw_data <- uploaded()
      rv$mapping  <- list(
        eq5d_version  = input$version,
        names_eq5d    = c(input$col_mo, input$col_sc,
                          input$col_ua, input$col_pd, input$col_ad),
        name_fu       = if (nzchar(input$col_fu))       input$col_fu       else NULL,
        name_groupvar = if (nzchar(input$col_groupvar)) input$col_groupvar else NULL,
        name_id       = if (nzchar(input$col_id))       input$col_id       else NULL,
        name_vas      = if (nzchar(input$col_vas))      input$col_vas      else NULL,
        name_utility  = if (nzchar(input$col_utility))  input$col_utility  else NULL,
        country       = ""   # country now lives only in the utility section
      )
      # Reset downstream state when mapping changes
      rv$processed_data <- NULL
      
      shiny::showNotification(
        "Mapping confirmed. Proceed to the Validation tab or calculate utility below.",
        type = "message", duration = 4
      )
    })
    
    # ── Local working dataset (mapped, may include added utility cols) ─────────
    local_data <- shiny::reactive({
      shiny::req(rv$raw_data, rv$mapping)
      if (!is.null(rv$processed_data)) rv$processed_data else apply_mapping(rv$raw_data, rv$mapping)
    })
    
    # ── Render: utility section (after mapping confirmed) ─────────────────────
    # REDESIGNED: stacked layout with 2-per-row controls at wider widths,
    # no cramped 3-column layout.  Country selector is now here exclusively.
    output$utility_section_ui <- shiny::renderUI({
      shiny::req(rv$mapping)
      
      version         <- rv$mapping$eq5d_version
      has_utility_col <- !is.null(rv$mapping$name_utility) &&
        nzchar(rv$mapping$name_utility %||% "")
      
      # Status banner: no utility column detected vs already present
      status_banner <- if (!has_utility_col) {
        shiny::tags$div(
          class = "alert alert-warning small py-2 mb-3",
          shiny::icon("triangle-exclamation"), " ",
          "No utility column detected. You can calculate utility values below."
        )
      } else {
        shiny::tags$div(
          class = "alert alert-success small py-2 mb-3",
          shiny::icon("circle-check"), " ",
          "Utility column mapped: ",
          shiny::strong(rv$mapping$name_utility), ". ",
          "You can add more utility columns with different value sets below."
        )
      }
      
      method_choices <- get_utility_method_choices(version)
      
      bslib::card(
        bslib::card_header(
          shiny::tags$span(shiny::icon("calculator"), " Calculate Utility")
        ),
        bslib::card_body(
          style = "padding: 1.25rem;",
          
          status_banner,
          
          shiny::p(
            class = "text-muted small mb-3",
            "Compute a utility index from the EQ-5D dimensions and append it as ",
            "a new column. You can add multiple utility columns with different ",
            "value sets or methods."
          ),
          
          # Controls row: Method + Country + Output column name
          # Using fluidRow/column which resolves reliably inside renderUI
          # (bslib::layout_columns can produce overlapping labels there)
          shiny::fluidRow(
            shiny::column(
              4,
              shiny::selectInput(
                ns("utility_method"), "Method",
                choices  = method_choices,
                selected = "direct"
              )
            ),
            shiny::column(
              4,
              shiny::uiOutput(ns("utility_country_picker"))
            ),
            shiny::column(
              4,
              shiny::textInput(
                ns("utility_col_name"), "Output column name",
                value       = "utility",
                placeholder = "e.g. utility_uk, utility_es"
              )
            )
          ),
          
          # Action button
          shiny::actionButton(
            ns("add_utility"), "Add utility column",
            class = "btn-primary",
            icon  = shiny::icon("plus")
          ),
          
          # Status output (appears after column is added)
          shiny::div(
            class = "mt-3",
            shiny::uiOutput(ns("utility_status"))
          )
        )
      )
    })
    
    # ── Render: dynamic method choices (re-render when version changes) ───────
    shiny::observe({
      shiny::req(rv$mapping)
      choices <- get_utility_method_choices(rv$mapping$eq5d_version)
      shiny::updateSelectInput(session, "utility_method", choices = choices,
                               selected = "direct")
    })
    
    # ── Render: utility country picker ────────────────────────────────────────
    output$utility_country_picker <- shiny::renderUI({
      shiny::req(rv$mapping)
      method <- input$utility_method %||% "direct"
      ver <- switch(method,
                    direct = rv$mapping$eq5d_version,
                    xw     = "3L",
                    xwr    = "5L",
                    rv$mapping$eq5d_version
      )
      ch <- get_country_choices(ver)
      if (length(ch) == 0L) {
        return(shiny::p("No value sets found for this method.",
                        class = "text-danger small"))
      }
      shiny::selectizeInput(
        ns("utility_country"), "Country / Value set",
        choices  = c("(select)" = "", ch),
        selected = rv$mapping$country %||% ""
      )
    })
    
    # ── Add utility column ────────────────────────────────────────────────────
    shiny::observeEvent(input$add_utility, {
      shiny::req(
        rv$mapping,
        rv$raw_data,
        nzchar(input$utility_country %||% ""),
        nzchar(trimws(input$utility_col_name %||% ""))
      )
      
      col_name <- trimws(input$utility_col_name)
      method   <- input$utility_method
      country  <- input$utility_country
      df       <- local_data()
      
      if (col_name %in% names(df)) {
        shiny::showNotification(
          paste0('Column "', col_name, '" already exists. ',
                 "Rename the output column or it will be overwritten."),
          type = "warning", duration = 4
        )
      }
      
      tryCatch({
        util_vals <- compute_utility_col(df, method, country, rv$mapping$eq5d_version)
        df[[col_name]] <- util_vals
        
        # FIXED: update processed_data and mapping atomically so that:
        #   (a) preview_table re-renders with the new column immediately
        #   (b) downstream modules see the correct utility column name
        #   (c) if no utility column existed before, this one becomes the default
        new_mapping <- rv$mapping
        new_mapping$name_utility <- col_name
        rv$processed_data <- df
        rv$mapping        <- new_mapping
        
        # Also refresh the utility column selector in the mapping panel so the
        # user sees the newly created column if they revisit the mapping UI.
        shiny::updateSelectInput(
          session, "col_utility",
          choices  = c("(none)" = "", names(df)),
          selected = col_name
        )
        
        shiny::showNotification(
          paste0(
            'Column "', col_name, '" added (',
            sum(!is.na(util_vals)), "/", nrow(df), " non-missing values)."
          ),
          type = "message", duration = 4
        )
      }, error = function(e) {
        shiny::showNotification(
          paste("Error computing utility:", conditionMessage(e)),
          type = "error", duration = 8
        )
      })
    })
    
    # ── Render: utility status (shows after column is added) ──────────────────
    output$utility_status <- shiny::renderUI({
      shiny::req(rv$processed_data)
      col_name <- trimws(input$utility_col_name %||% "")
      if (!nzchar(col_name) || !col_name %in% names(rv$processed_data)) return(NULL)
      vals  <- rv$processed_data[[col_name]]
      n_ok  <- sum(!is.na(vals))
      n_tot <- length(vals)
      mn    <- round(min(vals, na.rm = TRUE), 3L)
      mx    <- round(max(vals, na.rm = TRUE), 3L)
      shiny::div(
        class = "alert alert-success small py-2 mb-0",
        shiny::icon("circle-check"), " ",
        shiny::strong(col_name), ": ",
        n_ok, "/", n_tot, " non-missing. Range: ", mn, " to ", mx, "."
      )
    })
    
    # ── Render: download section ──────────────────────────────────────────────
    # REDESIGNED: uses a flex-wrap row so buttons never overlap on narrow widths.
    output$download_section_ui <- shiny::renderUI({
      shiny::req(rv$processed_data)
      bslib::card(
        bslib::card_header(
          shiny::tags$span(shiny::icon("file-arrow-down"), " Download Dataset")
        ),
        bslib::card_body(
          style = "padding: 1.25rem;",
          shiny::p(
            class = "text-muted small mb-3",
            "Download the current working dataset, including any calculated ",
            "utility columns."
          ),
          # d-flex + flex-wrap + gap-2 ensures buttons wrap cleanly at any width
          shiny::div(
            class = "d-flex flex-wrap gap-2",
            shiny::downloadButton(
              ns("download_csv"),  "Download CSV",
              class = "btn-outline-secondary",
              icon  = shiny::icon("file-csv")
            ),
            shiny::downloadButton(
              ns("download_xlsx"), "Download XLSX",
              class = "btn-outline-secondary",
              icon  = shiny::icon("file-excel")
            ),
            shiny::downloadButton(
              ns("download_rds"),  "Download RDS",
              class = "btn-outline-secondary",
              icon  = shiny::icon("r-project")
            )
          )
        )
      )
    })
    
    # ── Download handlers ─────────────────────────────────────────────────────
    
    output$download_csv <- shiny::downloadHandler(
      filename = function() {
        paste0("eq5dsuite_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".csv")
      },
      content = function(file) {
        shiny::req(rv$processed_data)
        utils::write.csv(rv$processed_data, file, row.names = FALSE)
      }
    )
    
    output$download_xlsx <- shiny::downloadHandler(
      filename = function() {
        paste0("eq5dsuite_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
      },
      content = function(file) {
        shiny::req(rv$processed_data)
        if (!requireNamespace("writexl", quietly = TRUE)) {
          stop("Package 'writexl' is required for XLSX export. ",
               "Install it with: install.packages('writexl')", call. = FALSE)
        }
        writexl::write_xlsx(rv$processed_data, file)
      }
    )
    
    output$download_rds <- shiny::downloadHandler(
      filename = function() {
        paste0("eq5dsuite_data_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".rds")
      },
      content = function(file) {
        shiny::req(rv$processed_data)
        saveRDS(rv$processed_data, file)
      }
    )
  })
}

# Null-coalescing operator (safe for module scope)
`%||%` <- function(a, b) if (!is.null(a) && length(a) > 0L) a else b

# mod_validation.R — Data validation and preprocessing module

# ── UI ────────────────────────────────────────────────────────────────────────

mod_validation_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::uiOutput(ns("main_ui"))
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_validation_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    # ── Reactive: run validation when mapping is confirmed ───────────────────
    validation <- shiny::reactive({
      shiny::req(rv$raw_data, rv$mapping)
      run_validation(rv$raw_data, rv$mapping)
    })

    # ── Render: main UI (shown only when mapping exists) ─────────────────────
    output$main_ui <- shiny::renderUI({
      if (is.null(rv$mapping)) {
        return(bslib::card(
          bslib::card_body(shiny::p(
            shiny::icon("circle-info"),
            " Please upload data and confirm mapping on the Data tab first.",
            class = "text-muted"
          ))
        ))
      }

      bslib::layout_columns(
        col_widths = c(5, 7),

        # Left: validation summary
        bslib::card(
          bslib::card_header(
            shiny::tags$span(shiny::icon("clipboard-check"), " Validation Summary")
          ),
          bslib::card_body(
            shiny::uiOutput(ns("validation_msgs")),
            shiny::hr(),
            shiny::h6("Mapping confirmed"),
            shiny::uiOutput(ns("mapping_summary")),
            shiny::hr(),
            shiny::actionButton(
              ns("proceed"), "Proceed to Analysis",
              class = "btn-success w-100",
              icon  = shiny::icon("arrow-right")
            )
          )
        ),

        # Right: processed data preview
        bslib::card(
          bslib::card_header(
            shiny::tags$span(shiny::icon("table"), " Processed Data Preview")
          ),
          bslib::card_body(
            shiny::uiOutput(ns("processed_info")),
            DT::DTOutput(ns("processed_table"))
          )
        )
      )
    })

    # ── Render: validation messages ───────────────────────────────────────────
    output$validation_msgs <- shiny::renderUI({
      v <- validation()
      shiny::tagList(
        lapply(v$messages, function(msg) {
          cls <- switch(msg$type,
            ok      = "alert alert-success",
            warning = "alert alert-warning",
            error   = "alert alert-danger",
            "alert alert-info"
          )
          shiny::div(class = cls, role = "alert", shiny::HTML(msg$text))
        })
      )
    })

    # ── Render: mapping summary ───────────────────────────────────────────────
    output$mapping_summary <- shiny::renderUI({
      m <- rv$mapping
      shiny::req(m)
      items <- list(
        list("EQ-5D version", m$eq5d_version),
        list("Dimensions",    paste(m$names_eq5d, collapse = ", ")),
        list("Timepoint",     m$name_fu       %||% "(not mapped)"),
        list("Group",         m$name_groupvar %||% "(not mapped)"),
        list("Patient ID",    m$name_id       %||% "(not mapped)"),
        list("EQ-VAS",        m$name_vas      %||% "(not mapped)"),
        list("Utility",       m$name_utility  %||% "(not mapped)"),
        list("Country",       if (nzchar(m$country)) m$country else "(not selected)")
      )
      shiny::tags$dl(
        class = "row small mb-0",
        lapply(items, function(x) {
          shiny::tagList(
            shiny::tags$dt(class = "col-sm-5 text-muted", x[[1]]),
            shiny::tags$dd(class = "col-sm-7", x[[2]])
          )
        })
      )
    })

    # ── Reactive: apply mapping to produce processed_data preview ─────────────
    preview_data <- shiny::reactive({
      shiny::req(rv$raw_data, rv$mapping)
      apply_mapping(rv$raw_data, rv$mapping)
    })

    # Use rv$processed_data when available (preserves utility columns added on
    # the Data page), otherwise fall back to the freshly-mapped preview.
    current_data <- shiny::reactive({
      if (!is.null(rv$processed_data)) rv$processed_data else preview_data()
    })

    output$processed_info <- shiny::renderUI({
      df <- current_data()
      shiny::p(
        shiny::strong(format(nrow(df), big.mark = ",")), " rows \u00d7 ",
        shiny::strong(ncol(df)), " columns",
        class = "text-muted small mb-2"
      )
    })

    output$processed_table <- DT::renderDT({
      df <- current_data()
      shiny::req(df)
      DT::datatable(
        head(df, 200L),
        options  = list(pageLength = 10L, scrollX = TRUE, dom = "tip"),
        rownames = FALSE,
        class    = "table-sm table-striped"
      )
    })

    # ── Proceed: finalise processed_data and store in rv ──────────────────────
    shiny::observeEvent(input$proceed, {
      v <- validation()
      has_error <- any(vapply(v$messages, function(m) m$type == "error", logical(1L)))
      if (has_error) {
        shiny::showNotification(
          "Please resolve data errors before proceeding.",
          type = "error", duration = 5
        )
        return()
      }
      # Preserve any utility columns already added on the Data page.
      # Only set from raw mapping if processed_data not yet initialised.
      if (is.null(rv$processed_data)) {
        rv$processed_data <- preview_data()
      }
      shiny::showNotification(
        "Dataset validated. You can now run analyses.",
        type = "message", duration = 4
      )
    })
  })
}

# Null-coalescing operator
`%||%` <- function(a, b) if (!is.null(a) && length(a) > 0L) a else b

# ── Validation logic (pure function, not reactive) ────────────────────────────

run_validation <- function(df, mapping) {
  msgs <- list()

  add_msg <- function(type, text) {
    msgs[[length(msgs) + 1L]] <<- list(type = type, text = text)
  }

  n_rows <- nrow(df)
  add_msg("ok", paste0("<strong>", format(n_rows, big.mark = ","),
                       "</strong> rows loaded."))

  # Check EQ-5D column existence
  eq5d_cols   <- mapping$names_eq5d
  missing_cols <- eq5d_cols[!eq5d_cols %in% names(df)]
  if (length(missing_cols) > 0L) {
    add_msg("error",
            paste("EQ-5D columns not found in data:",
                  paste(missing_cols, collapse = ", ")))
  } else {
    # Check value range
    max_level <- if (mapping$eq5d_version == "3L") 3L else 5L
    out_of_range <- vapply(eq5d_cols, function(col) {
      vals <- suppressWarnings(as.integer(df[[col]]))
      any(!is.na(vals) & (vals < 1L | vals > max_level))
    }, logical(1L))

    if (any(out_of_range)) {
      add_msg("warning",
              paste0(
                "Some values in [",
                paste(eq5d_cols[out_of_range], collapse = ", "),
                "] are outside the expected range (1\u2013", max_level,
                ") and will be set to NA."
              ))
    } else {
      add_msg("ok", paste0(
        "All EQ-5D values within expected range (1\u2013", max_level, ")."
      ))
    }

    # Missing EQ-5D values
    n_complete <- sum(complete.cases(
      lapply(eq5d_cols, function(col) suppressWarnings(as.integer(df[[col]])))
    ))
    n_miss <- n_rows - n_complete
    if (n_miss > 0L) {
      pct <- round(100 * n_miss / n_rows, 1)
      add_msg("warning",
              paste0("<strong>", n_miss, "</strong> rows (",
                     pct, "%) have missing EQ-5D values."))
    } else {
      add_msg("ok", "No missing EQ-5D values.")
    }
  }

  # Duplicate ID check
  if (!is.null(mapping$name_id) && nzchar(mapping$name_id)) {
    if (mapping$name_id %in% names(df)) {
      ids   <- df[[mapping$name_id]]
      n_dup <- sum(duplicated(ids))
      if (n_dup == 0L) {
        add_msg("ok", "No duplicate patient IDs.")
      } else {
        has_fu  <- !is.null(mapping$name_fu) && nzchar(mapping$name_fu) &&
                   mapping$name_fu %in% names(df)
        if (has_fu) {
          fu_vals  <- df[[mapping$name_fu]]
          pairs    <- paste(ids, fu_vals, sep = "\u00b7")
          n_dup_pairs <- sum(duplicated(pairs))
          n_tp     <- length(unique(fu_vals))
          if (n_dup_pairs == 0L) {
            add_msg("ok",
                    paste0("Repeated patient IDs detected across ",
                           "<strong>", n_tp, "</strong> timepoints \u2014 ",
                           "this is expected for longitudinal data. ",
                           "All ID\u2013timepoint combinations are unique."))
          } else {
            add_msg("warning",
                    paste0("<strong>", n_dup_pairs, "</strong> ID\u2013timepoint ",
                           "combinations are duplicated. Check for duplicate records."))
          }
        } else {
          add_msg("warning",
                  paste0("<strong>", n_dup, "</strong> repeated patient IDs. ",
                         "For cross-sectional data each row should have a unique ID. ",
                         "If this is longitudinal data, map a Timepoint variable."))
        }
      }
    }
  }

  # VAS range check
  if (!is.null(mapping$name_vas) && nzchar(mapping$name_vas) &&
      mapping$name_vas %in% names(df)) {
    vas_vals <- suppressWarnings(as.numeric(df[[mapping$name_vas]]))
    out_vas  <- any(!is.na(vas_vals) & (vas_vals < 0 | vas_vals > 100))
    if (out_vas) {
      add_msg("warning", "Some VAS values are outside the expected range (0\u2013100).")
    } else {
      add_msg("ok", "VAS values within expected range (0\u2013100).")
    }
  }

  list(messages = msgs)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)

## ----add-custom-vs------------------------------------------------------------
# Generate all valid 3L health state codes
states <- make_all_EQ_indexes(version = "3L")

# Create a custom value set (random values for illustration)
set.seed(42)
custom_vs <- data.frame(
  state  = states,
  MY_VS  = runif(243, min = -0.5, max = 1.0)
)

head(custom_vs)

## ----register-vs, eval = FALSE------------------------------------------------
# # Register for current session only (saveOption = 1)
# eqvs_add(
#   custom_vs,
#   version     = "3L",
#   country     = "My Country",
#   countryCode = "MC",
#   VSCode      = "MY_VS",
#   description = "Custom value set — demonstration",
#   saveOption  = 1
# )
# 
# # Verify it appears in the list
# eqvs_display(version = "3L")
# 
# # Use it for value calculation
# eq5d3l(c(11111, 12321), country = "MY_VS")

## ----save-vs, eval = FALSE----------------------------------------------------
# eqvs_add(
#   custom_vs,
#   version     = "3L",
#   country     = "My Country",
#   countryCode = "MC",
#   VSCode      = "MY_VS",
#   description = "Custom value set — permanent",
#   saveOption  = 2  # saves to user cache
# )

## ----load-vs, eval = FALSE----------------------------------------------------
# eqvs_load(country = "MY_VS", version = "3L")

## ----drop-vs, eval = FALSE----------------------------------------------------
# eqvs_drop(
#   country    = "MY_VS",
#   version    = "3L",
#   saveOption = 1  # remove from current session only
# )

## ----custom-xw, eval = FALSE--------------------------------------------------
# # Use custom 3L value set with original crosswalk
# eqxw(
#   eq5d5l_data,
#   country   = "MY_VS",
#   dim.names = c("mo", "sc", "ua", "pd", "ad")
# )

## ----update-vs, eval = FALSE--------------------------------------------------
# update_value_sets()

# mod_analysis_profile.R — EQ-5D Profile analysis module

DIMS_STD <- c("mo", "sc", "ua", "pd", "ad")

# ── UI ────────────────────────────────────────────────────────────────────────

mod_analysis_profile_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::uiOutput(ns("guard")),
    shiny::conditionalPanel(
      condition = "output.app_has_data",
      shiny::tabsetPanel(
        id       = ns("profile_tabs"),
        type     = "pills",
        selected = "cs",
        
        # ── 1. Cross-sectional ───────────────────────────────────────────────
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("table"), " Cross-sectional"),
          value = "cs",
          shiny::br(),
          bslib::accordion(
            open     = "acc_111",
            multiple = TRUE,
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.1.1 \u2014 Level frequencies by dimension"),
              value = "acc_111",
              shiny::p(class = "text-muted small",
                       "Frequency (n and %) of each response level across all five EQ-5D dimensions."),
              shiny::actionButton(ns("run_111"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_111"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.1.2 \u2014 Level frequencies by group"),
              value = "acc_112",
              shiny::p(class = "text-muted small",
                       "Frequency of response levels stratified by group. Requires a Group variable."),
              shiny::conditionalPanel(
                condition = "!output.app_has_group",
                shiny::div(class = "alert alert-warning small py-2",
                           shiny::icon("triangle-exclamation"),
                           " Group variable not mapped. Please add one on the Data tab.")
              ),
              shiny::conditionalPanel(
                condition = "output.app_has_group",
                shiny::actionButton(ns("run_112"), "Run",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play"))
              ),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_112"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("star"),
                                     " Table 1.1.3 \u2014 Most common health states"),
              value = "acc_113",
              shiny::p(class = "text-muted small",
                       "The most frequently observed EQ-5D health states in the dataset."),
              bslib::layout_columns(
                col_widths = c(3, 9),
                shiny::numericInput(ns("n_113"), "Top N states",
                                    value = 10L, min = 1L, max = 50L, step = 1L),
                shiny::div()
              ),
              shiny::actionButton(ns("run_113"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_113"))
            )
          )
        ),
        
        # ── 2. Longitudinal ──────────────────────────────────────────────────
        # NOTE: this tab is hidden by default on module init (see server below)
        # and shown only when rv$mapping$name_fu is a non-empty string.
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("clock"), " Longitudinal"),
          value = "long",
          shiny::br(),
          bslib::accordion(
            open     = "acc_121",
            multiple = TRUE,
            
            # Table 1.2.1
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.2.1 \u2014 Level frequencies by timepoint"),
              value = "acc_121",
              shiny::p(class = "text-muted small",
                       "Frequency of response levels at each timepoint."),
              shiny::actionButton(ns("run_121"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_121"))
            ),
            
            # Table 1.2.2
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-bar"),
                                     " Table 1.2.2 \u2014 PCHC health change"),
              value = "acc_122",
              shiny::p(class = "text-muted small",
                       "Paretian Classification of Health Change (PCHC). Requires Patient ID. ",
                       "If no group variable is mapped, analysis runs on the full population."),
              shiny::conditionalPanel(
                condition = "!output.app_has_id",
                shiny::div(class = "alert alert-warning small py-2",
                           shiny::icon("triangle-exclamation"), " Patient ID not mapped.")
              ),
              shiny::conditionalPanel(
                condition = "output.app_has_id",
                shiny::uiOutput(ns("group_note_122")),
                shiny::actionButton(ns("run_122"), "Run",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play"))
              ),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_122"))
            ),
            
            # Table 1.2.3 (id gated — body rendered conditionally by server)
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-bar"),
                                     " Table 1.2.3 \u2014 PCHC (accounting for no problems)"),
              value = "acc_123",
              shiny::p(class = "text-muted small",
                       "PCHC accounting for those reporting no problems at baseline. ",
                       "Requires Patient ID."),
              shiny::uiOutput(ns("panel_123_body")),
              shiny::br(),
              shiny::uiOutput(ns("out_123"))
            ),
            
            # Figures 1.2.1–1.2.4 (grouped in one panel)
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-bar"),
                                     " Figures 1.2.1\u20131.2.4 \u2014 PCHC visualisations"),
              value = "acc_12figs",
              shiny::p(class = "text-muted small",
                       "Bar charts of PCHC results by group and dimension. Requires Patient ID. ",
                       "If no group variable is mapped, uses the full population."),
              shiny::conditionalPanel(
                condition = "!output.app_has_id",
                shiny::div(class = "alert alert-warning small py-2",
                           shiny::icon("triangle-exclamation"), " Patient ID not mapped.")
              ),
              shiny::conditionalPanel(
                condition = "output.app_has_id",
                shiny::uiOutput(ns("group_note_12figs")),
                shiny::tags$div(style = "margin-top:0.5rem;"),
                shiny::h6("Figure 1.2.1 \u2014 PCHC categories by group",
                          class = "fw-semibold mt-2"),
                shiny::actionButton(ns("run_121fig"), "Run Fig. 1.2.1",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
                shiny::br(), shiny::br(),
                shiny::uiOutput(ns("out_121fig")),
                shiny::hr(style = "margin: 0.5rem 0;"),
                shiny::h6("Figure 1.2.2 \u2014 Improvements by group and dimension",
                          class = "fw-semibold"),
                shiny::actionButton(ns("run_122fig"), "Run Fig. 1.2.2",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
                shiny::br(), shiny::br(),
                shiny::uiOutput(ns("out_122fig")),
                shiny::hr(style = "margin: 0.5rem 0;"),
                shiny::h6("Figure 1.2.3 \u2014 Worsenings by group and dimension",
                          class = "fw-semibold"),
                shiny::actionButton(ns("run_123fig"), "Run Fig. 1.2.3",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
                shiny::br(), shiny::br(),
                shiny::uiOutput(ns("out_123fig")),
                shiny::hr(style = "margin: 0.5rem 0;"),
                shiny::h6("Figure 1.2.4 \u2014 Mixed changes by group and dimension",
                          class = "fw-semibold"),
                shiny::actionButton(ns("run_124fig"), "Run Fig. 1.2.4",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
                shiny::br(), shiny::br(),
                shiny::uiOutput(ns("out_124fig"))
              )
            ),
            
            # Figure 1.2.5 — HPG
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-line"),
                                     " Figure 1.2.5 \u2014 Health Profile Grid (HPG)"),
              value = "acc_125fig",
              shiny::p(class = "text-muted small",
                       "HPG scatter plot comparing health profiles between two timepoints. ",
                       "Requires Patient ID and a country value set. Select exactly two timepoints."),
              shiny::conditionalPanel(
                condition = "!output.app_has_id",
                shiny::div(class = "alert alert-warning small py-2",
                           shiny::icon("triangle-exclamation"), " Patient ID not mapped.")
              ),
              shiny::conditionalPanel(
                condition = "output.app_has_id",
                bslib::layout_columns(
                  col_widths = c(6, 6),
                  shiny::uiOutput(ns("fu_picker_125")),
                  shiny::uiOutput(ns("country_picker_125"))
                ),
                shiny::actionButton(ns("run_125fig"), "Run",
                                    class = "btn-sm btn-outline-primary", icon = shiny::icon("play"))
              ),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_125fig"))
            ),
            
            # Table 1.2.4 (3L only)
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("arrows-left-right"),
                                     " Table 1.2.4 \u2014 Level change between timepoints [3L only]"),
              value = "acc_124",
              shiny::p(class = "text-muted small",
                       "Proportion improving, stable, or worsening per dimension. ",
                       "Only available for EQ-5D-3L. Requires Patient ID."),
              shiny::uiOutput(ns("panel_124_body")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_124"))
            )
          )
        ),
        
        # ── 3. Summarizing Severity ──────────────────────────────────────────
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("layer-group"),
                                 " Summarizing Severity"),
          value = "severity",
          shiny::br(),
          shiny::p(shiny::em("Summarizing severity of EQ-5D profiles"),
                   class = "text-muted mb-3"),
          bslib::accordion(
            open     = "acc_131",
            multiple = TRUE,
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.3.1 \u2014 Summary statistics by Level Sum Score (LSS)"),
              value = "acc_131",
              shiny::p(class = "text-muted small",
                       "Summary statistics for EQ-5D utility values at each Level Sum Score."),
              shiny::uiOutput(ns("country_picker_131")),
              shiny::actionButton(ns("run_131"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_131"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-line"),
                                     " Figure 1.3.1 \u2014 EQ-5D values vs. Level Sum Score"),
              value = "acc_131fig",
              shiny::p(class = "text-muted small",
                       "Plot of EQ-5D utility values against the Level Sum Score (LSS)."),
              shiny::uiOutput(ns("country_picker_131fig")),
              shiny::actionButton(ns("run_131fig"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_131fig"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.3.2 \u2014 Distribution by Level Frequency Score (LFS)"),
              value = "acc_132",
              shiny::p(class = "text-muted small",
                       "Distribution of EQ-5D health states by Level Frequency Score."),
              shiny::actionButton(ns("run_132"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_132"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("table"),
                                     " Table 1.3.4 \u2014 Summary statistics by Level Frequency Score (LFS)"),
              value = "acc_134",
              shiny::p(class = "text-muted small",
                       "Summary statistics of EQ-5D utility values by Level Frequency Score."),
              shiny::uiOutput(ns("country_picker_134")),
              shiny::actionButton(ns("run_134"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_134"))
            ),
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-line"),
                                     " Figure 1.3.2 \u2014 EQ-5D values vs. Level Frequency Score"),
              value = "acc_132fig",
              shiny::p(class = "text-muted small",
                       "Plot of EQ-5D utility values against the Level Frequency Score (LFS)."),
              shiny::uiOutput(ns("country_picker_132fig")),
              shiny::actionButton(ns("run_132fig"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_132fig"))
            )
          )
        ),
        
        # ── 4. Informativity ─────────────────────────────────────────────────
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("chart-bar"), " Informativity"),
          value = "informativity",
          shiny::br(),
          shiny::p(shiny::em("Informativity of EQ-5D profile data"),
                   class = "text-muted mb-3"),
          bslib::accordion(
            open = "acc_141",
            
            bslib::accordion_panel(
              title = shiny::tagList(shiny::icon("chart-bar"),
                                     " Figure 1.4.1 \u2014 Health State Density Index (HSDI)"),
              value = "acc_141",
              shiny::p(class = "text-muted small",
                       "Scatter plot of the cumulative distribution of observed health profiles."),
              shiny::actionButton(ns("run_141"), "Run",
                                  class = "btn-sm btn-outline-primary", icon = shiny::icon("play")),
              shiny::br(), shiny::br(),
              shiny::uiOutput(ns("out_141"))
            )
          )
        )
      )
    )
  )
}

# ── Server ────────────────────────────────────────────────────────────────────

mod_analysis_profile_server <- function(id, rv) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns
    
    # ── Guard ─────────────────────────────────────────────────────────────────
    output$guard <- shiny::renderUI({
      if (is.null(rv$processed_data)) {
        bslib::card(bslib::card_body(
          shiny::p(shiny::icon("circle-info"),
                   " Please complete data upload and validation first.",
                   class = "text-muted")
        ))
      }
    })
    
    # ── Show/hide Longitudinal tab ────────────────────────────────────────────
    # Hide immediately on module init so the tab is never visible before
    # rv$mapping is set (prevents flash of the tab on first render).
    shiny::hideTab(inputId = "profile_tabs", target = "long", session = session)
    
    # React to rv$mapping changes (ignoreNULL = FALSE so hideTab fires
    # when mapping is cleared / reset, not just when it first appears).
    shiny::observeEvent(rv$mapping, {
      has_fu <- !is.null(rv$mapping$name_fu) &&
        nzchar(rv$mapping$name_fu %||% "")
      if (has_fu) {
        shiny::showTab(inputId = "profile_tabs", target = "long", session = session)
      } else {
        shiny::hideTab(inputId = "profile_tabs", target = "long", session = session)
      }
    }, ignoreNULL = FALSE)
    
    # ── Shared helpers ────────────────────────────────────────────────────────
    
    # Version shortcut
    eq5d_ver <- shiny::reactive({
      shiny::req(rv$mapping)
      rv$mapping$eq5d_version
    })
    
    is_3L <- shiny::reactive({
      identical(eq5d_ver(), "3L")
    })
    
    # Country choices for value-set analyses
    country_choices <- shiny::reactive({
      get_country_choices(eq5d_ver())
    })
    
    # Factory: render a country selectizeInput for a given namespaced id
    make_country_picker <- function(picker_id) {
      shiny::renderUI({
        ch <- country_choices()
        if (length(ch) == 0L) {
          return(shiny::p("No value sets available.", class = "text-muted small"))
        }
        default_c <- if (!is.null(rv$mapping) && nzchar(rv$mapping$country %||% ""))
          rv$mapping$country else ""
        shiny::selectizeInput(
          ns(picker_id), "Country / Value set",
          choices  = c("(select)" = "", ch),
          selected = default_c
        )
      })
    }
    
    output$country_picker_125    <- make_country_picker("country_125")
    output$country_picker_131    <- make_country_picker("country_131")
    output$country_picker_131fig <- make_country_picker("country_131fig")
    output$country_picker_134    <- make_country_picker("country_134")
    output$country_picker_132fig <- make_country_picker("country_132fig")
    
    # Factory: info note shown when no group variable is mapped
    make_group_note <- function() {
      shiny::renderUI({
        has_gv <- !is.null(rv$mapping) &&
          !is.null(rv$mapping$name_groupvar) &&
          nzchar(rv$mapping$name_groupvar %||% "")
        if (!has_gv) {
          shiny::div(class = "alert alert-info small py-2 mb-2",
                     shiny::icon("circle-info"),
                     " No group variable mapped. Analysis will run on the full population.")
        }
      })
    }
    
    output$group_note_122    <- make_group_note()
    output$group_note_12figs <- make_group_note()
    output$group_note_123    <- make_group_note()
    
    # Add synthetic "groupvar" = "All" when group not mapped
    ensure_groupvar <- function(df) {
      if (!"groupvar" %in% names(df)) df[["groupvar"]] <- "All"
      df
    }
    
    # Timepoints available in processed data (for Fig 1.2.5 picker)
    timepoints_avail <- shiny::reactive({
      df <- rv$processed_data
      if (is.null(df) || !"fu" %in% names(df)) return(character(0L))
      sort(as.character(unique(df[["fu"]])))
    })
    
    # ── Table 1.2.4 conditional body (3L + id gated) ──────────────────────────
    output$panel_124_body <- shiny::renderUI({
      if (!isTRUE(is_3L())) {
        shiny::div(class = "alert alert-info small py-2",
                   shiny::icon("circle-info"),
                   " This analysis is only available for EQ-5D-3L data.")
      } else if (is.null(rv$mapping$name_id) || !nzchar(rv$mapping$name_id %||% "")) {
        shiny::div(class = "alert alert-warning small py-2",
                   shiny::icon("triangle-exclamation"), " Patient ID not mapped.")
      } else {
        shiny::actionButton(ns("run_124"), "Run",
                            class = "btn-sm btn-outline-primary", icon = shiny::icon("play"))
      }
    })
    
    # ── Table 1.2.3 conditional body (id gated only) ──────────────────────────
    output$panel_123_body <- shiny::renderUI({
      if (is.null(rv$mapping$name_id) || !nzchar(rv$mapping$name_id %||% "")) {
        shiny::div(class = "alert alert-warning small py-2",
                   shiny::icon("triangle-exclamation"), " Patient ID not mapped.")
      } else {
        shiny::tagList(
          shiny::uiOutput(ns("group_note_123")),
          shiny::actionButton(ns("run_123"), "Run",
                              class = "btn-sm btn-outline-primary", icon = shiny::icon("play"))
        )
      }
    })
    
    # ── Figure 1.2.5 timepoint picker ─────────────────────────────────────────
    output$fu_picker_125 <- shiny::renderUI({
      tp <- timepoints_avail()
      if (length(tp) < 2L) {
        return(shiny::p("At least 2 timepoints required.", class = "text-warning small"))
      }
      shiny::selectizeInput(
        ns("fu_levels_125"), "Select exactly two timepoints",
        choices  = tp,
        selected = tp[seq_len(min(2L, length(tp)))],
        multiple = TRUE,
        options  = list(maxItems = 2L, placeholder = "Choose 2 timepoints...")
      )
    })
    
    # ═══════════════════════════════════════════════════════════════════════════
    # CROSS-SECTIONAL
    # ═══════════════════════════════════════════════════════════════════════════
    
    # ── Table 1.1.1 ───────────────────────────────────────────────────────────
    r111 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_111, {
      shiny::req(rv$processed_data)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver)
      cs   <- format_call("eq5d_profile_level_summary", c(list(df = "data"), args))
      tryCatch({
        r111$data <- do.call(eq5d_profile_level_summary, c(list(df = rv$processed_data), args))
        r111$call <- cs
        save_result(rv, "Level frequencies (1.1.1)", cs, "table", data = r111$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_111 <- shiny::renderUI({
      if (is.null(r111$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_111")), call_display(ns("call_111")))
    })
    output$tbl_111  <- render_analysis_table(shiny::reactive(r111$data))
    output$call_111 <- shiny::renderText(r111$call)
    
    # ── Table 1.1.2 ───────────────────────────────────────────────────────────
    r112 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_112, {
      shiny::req(rv$processed_data, rv$mapping$name_groupvar)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, name_cat = "groupvar", eq5d_version = ver)
      cs   <- format_call("eq5d_profile_level_summary_by_group", c(list(df = "data"), args))
      tryCatch({
        r112$data <- do.call(eq5d_profile_level_summary_by_group, c(list(df = rv$processed_data), args))
        r112$call <- cs
        save_result(rv, "Level freq. by group (1.1.2)", cs, "table", data = r112$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_112 <- shiny::renderUI({
      if (is.null(r112$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_112")), call_display(ns("call_112")))
    })
    output$tbl_112  <- render_analysis_table(shiny::reactive(r112$data))
    output$call_112 <- shiny::renderText(r112$call)
    
    # ── Table 1.1.3 ───────────────────────────────────────────────────────────
    r113 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_113, {
      shiny::req(rv$processed_data, input$n_113)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver, n = as.integer(input$n_113))
      cs   <- format_call("eq5d_profile_top_states", c(list(df = "data"), args))
      tryCatch({
        r113$data <- do.call(eq5d_profile_top_states, c(list(df = rv$processed_data), args))
        r113$call <- cs
        save_result(rv, "Most common states (1.1.3)", cs, "table", data = r113$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_113 <- shiny::renderUI({
      if (is.null(r113$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_113")), call_display(ns("call_113")))
    })
    output$tbl_113  <- render_analysis_table(shiny::reactive(r113$data))
    output$call_113 <- shiny::renderText(r113$call)
    
    # ═══════════════════════════════════════════════════════════════════════════
    # LONGITUDINAL
    # ═══════════════════════════════════════════════════════════════════════════
    
    # ── Table 1.2.1 ───────────────────────────────────────────────────────────
    r121 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_121, {
      shiny::req(rv$processed_data, rv$mapping$name_fu)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, name_fu = "fu", eq5d_version = ver)
      cs   <- format_call("eq5d_profile_change_summary", c(list(df = "data"), args))
      tryCatch({
        r121$data <- do.call(eq5d_profile_change_summary, c(list(df = rv$processed_data), args))
        r121$call <- cs
        save_result(rv, "Level freq. by timepoint (1.2.1)", cs, "table", data = r121$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_121 <- shiny::renderUI({
      if (is.null(r121$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_121")), call_display(ns("call_121")))
    })
    output$tbl_121  <- render_analysis_table(shiny::reactive(r121$data))
    output$call_121 <- shiny::renderText(r121$call)
    
    # ── Table 1.2.4 ───────────────────────────────────────────────────────────
    r124 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_124, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id, is_3L())
      args <- list(name_id = "id", names_eq5d = DIMS_STD, name_fu = "fu")
      cs   <- format_call("eq5d_profile_dimension_change_table", c(list(df = "data"), args))
      tryCatch({
        r124$data <- do.call(eq5d_profile_dimension_change_table, c(list(df = rv$processed_data), args))
        r124$call <- cs
        save_result(rv, "Level change (1.2.4)", cs, "table", data = r124$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_124 <- shiny::renderUI({
      if (is.null(r124$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_124")), call_display(ns("call_124")))
    })
    output$tbl_124  <- render_analysis_table(shiny::reactive(r124$data))
    output$call_124 <- shiny::renderText(r124$call)
    
    # ── Table 1.2.2 ───────────────────────────────────────────────────────────
    r122 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_122, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id)
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_pchc_table", c(list(df = "data"), args))
      tryCatch({
        r122$data <- do.call(eq5d_profile_pchc_table, c(list(df = df_gv), args))
        r122$call <- cs
        save_result(rv, "PCHC (1.2.2)", cs, "table", data = r122$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_122 <- shiny::renderUI({
      if (is.null(r122$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_122")), call_display(ns("call_122")))
    })
    output$tbl_122  <- render_analysis_table(shiny::reactive(r122$data))
    output$call_122 <- shiny::renderText(r122$call)
    
    # ── Table 1.2.3 (3L only) ────────────────────────────────────────────────
    r123 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_123, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id, is_3L())
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_pchc_with_no_problems_table", c(list(df = "data"), args))
      tryCatch({
        r123$data <- do.call(eq5d_profile_pchc_with_no_problems_table, c(list(df = df_gv), args))
        r123$call <- cs
        save_result(rv, "PCHC no-problems (1.2.3)", cs, "table", data = r123$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_123 <- shiny::renderUI({
      if (is.null(r123$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_123")), call_display(ns("call_123")))
    })
    output$tbl_123  <- render_analysis_table(shiny::reactive(r123$data))
    output$call_123 <- shiny::renderText(r123$call)
    
    # ── Figure 1.2.1 ─────────────────────────────────────────────────────────
    r121fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_121fig, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id)
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_pchc_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_pchc_by_group_plot, c(list(df = df_gv), args))
        r121fig$plot <- res$p
        r121fig$call <- cs
        save_result(rv, "PCHC bar chart (fig. 1.2.1)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_121fig <- shiny::renderUI({
      if (is.null(r121fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_121fig"), height = "420px"),
                     call_display(ns("call_121fig")))
    })
    output$plt_121fig  <- shiny::renderPlot(r121fig$plot)
    output$call_121fig <- shiny::renderText(r121fig$call)
    
    # ── Figure 1.2.2 ─────────────────────────────────────────────────────────
    r122fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_122fig, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id)
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_better_dimensions_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_better_dimensions_by_group_plot, c(list(df = df_gv), args))
        r122fig$plot <- res$p
        r122fig$call <- cs
        save_result(rv, "Improvements chart (fig. 1.2.2)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_122fig <- shiny::renderUI({
      if (is.null(r122fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_122fig"), height = "420px"),
                     call_display(ns("call_122fig")))
    })
    output$plt_122fig  <- shiny::renderPlot(r122fig$plot)
    output$call_122fig <- shiny::renderText(r122fig$call)
    
    # ── Figure 1.2.3 ─────────────────────────────────────────────────────────
    r123fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_123fig, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id)
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_worse_dimensions_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_worse_dimensions_by_group_plot, c(list(df = df_gv), args))
        r123fig$plot <- res$p
        r123fig$call <- cs
        save_result(rv, "Worsenings chart (fig. 1.2.3)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_123fig <- shiny::renderUI({
      if (is.null(r123fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_123fig"), height = "420px"),
                     call_display(ns("call_123fig")))
    })
    output$plt_123fig  <- shiny::renderPlot(r123fig$plot)
    output$call_123fig <- shiny::renderText(r123fig$call)
    
    # ── Figure 1.2.4 ─────────────────────────────────────────────────────────
    r124fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_124fig, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id)
      df_gv <- ensure_groupvar(rv$processed_data)
      args  <- list(name_id = "id", name_groupvar = "groupvar",
                    names_eq5d = DIMS_STD, name_fu = "fu")
      cs    <- format_call("eq5d_profile_mixed_dimensions_by_group_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_mixed_dimensions_by_group_plot, c(list(df = df_gv), args))
        r124fig$plot <- res$p
        r124fig$call <- cs
        save_result(rv, "Mixed change chart (fig. 1.2.4)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_124fig <- shiny::renderUI({
      if (is.null(r124fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_124fig"), height = "420px"),
                     call_display(ns("call_124fig")))
    })
    output$plt_124fig  <- shiny::renderPlot(r124fig$plot)
    output$call_124fig <- shiny::renderText(r124fig$call)
    
    # ── Figure 1.2.5 — HPG ───────────────────────────────────────────────────
    r125fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_125fig, {
      shiny::req(rv$processed_data, rv$mapping$name_fu, rv$mapping$name_id,
                 nzchar(input$country_125 %||% ""))
      tp_sel <- input$fu_levels_125
      if (is.null(tp_sel) || length(tp_sel) != 2L) {
        shiny::showNotification(
          "Please select exactly two timepoints for the HPG plot.",
          type = "warning", duration = 5
        )
        return()
      }
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, name_fu = "fu", levels_fu = tp_sel,
                   name_id = "id", eq5d_version = ver, country = input$country_125)
      cs   <- format_call("eq5d_profile_health_profile_grid", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_health_profile_grid, c(list(df = rv$processed_data), args))
        r125fig$plot <- res$p
        r125fig$call <- cs
        save_result(rv, "HPG scatter (fig. 1.2.5)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_125fig <- shiny::renderUI({
      if (is.null(r125fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_125fig"), height = "500px"),
                     call_display(ns("call_125fig")))
    })
    output$plt_125fig  <- shiny::renderPlot(r125fig$plot)
    output$call_125fig <- shiny::renderText(r125fig$call)
    
    # ═══════════════════════════════════════════════════════════════════════════
    # SUMMARIZING SEVERITY
    # ═══════════════════════════════════════════════════════════════════════════
    
    # ── Table 1.3.1 ───────────────────────────────────────────────────────────
    r131 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_131, {
      shiny::req(rv$processed_data, nzchar(input$country_131 %||% ""))
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver, country = input$country_131)
      cs   <- format_call("eq5d_profile_lss_utility_summary", c(list(df = "data"), args))
      tryCatch({
        r131$data <- do.call(eq5d_profile_lss_utility_summary, c(list(df = rv$processed_data), args))
        r131$call <- cs
        save_result(rv, "LSS summary stats (1.3.1)", cs, "table", data = r131$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_131 <- shiny::renderUI({
      if (is.null(r131$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_131")), call_display(ns("call_131")))
    })
    output$tbl_131  <- render_analysis_table(shiny::reactive(r131$data))
    output$call_131 <- shiny::renderText(r131$call)
    
    # ── Figure 1.3.1 ─────────────────────────────────────────────────────────
    r131fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_131fig, {
      shiny::req(rv$processed_data, nzchar(input$country_131fig %||% ""))
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver, country = input$country_131fig)
      cs   <- format_call("eq5d_profile_lss_utility_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_lss_utility_plot, c(list(df = rv$processed_data), args))
        r131fig$plot <- res$p
        r131fig$call <- cs
        save_result(rv, "LSS scatter (fig. 1.3.1)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_131fig <- shiny::renderUI({
      if (is.null(r131fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_131fig"), height = "420px"),
                     call_display(ns("call_131fig")))
    })
    output$plt_131fig  <- shiny::renderPlot(r131fig$plot)
    output$call_131fig <- shiny::renderText(r131fig$call)
    
    # ── Table 1.3.2 ───────────────────────────────────────────────────────────
    r132 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_132, {
      shiny::req(rv$processed_data)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver)
      cs   <- format_call("eq5d_profile_lfs_distribution", c(list(df = "data"), args))
      tryCatch({
        r132$data <- do.call(eq5d_profile_lfs_distribution, c(list(df = rv$processed_data), args))
        r132$call <- cs
        save_result(rv, "LFS distribution (1.3.2)", cs, "table", data = r132$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_132 <- shiny::renderUI({
      if (is.null(r132$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_132")), call_display(ns("call_132")))
    })
    output$tbl_132  <- render_analysis_table(shiny::reactive(r132$data))
    output$call_132 <- shiny::renderText(r132$call)
    
    # ── Table 1.3.4 ───────────────────────────────────────────────────────────
    r134 <- shiny::reactiveValues(data = NULL, call = NULL)
    
    shiny::observeEvent(input$run_134, {
      shiny::req(rv$processed_data, nzchar(input$country_134 %||% ""))
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver, country = input$country_134)
      cs   <- format_call("eq5d_profile_lfs_utility_summary", c(list(df = "data"), args))
      tryCatch({
        r134$data <- do.call(eq5d_profile_lfs_utility_summary, c(list(df = rv$processed_data), args))
        r134$call <- cs
        save_result(rv, "LFS summary stats (1.3.4)", cs, "table", data = r134$data)
      }, error = function(e) err_notify(e))
    })
    
    output$out_134 <- shiny::renderUI({
      if (is.null(r134$data)) return(NULL)
      shiny::tagList(analysis_table_output(ns("tbl_134")), call_display(ns("call_134")))
    })
    output$tbl_134  <- render_analysis_table(shiny::reactive(r134$data))
    output$call_134 <- shiny::renderText(r134$call)
    
    # ── Figure 1.3.2 ─────────────────────────────────────────────────────────
    r132fig <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_132fig, {
      shiny::req(rv$processed_data, nzchar(input$country_132fig %||% ""))
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver, country = input$country_132fig)
      cs   <- format_call("eq5d_profile_lfs_utility_plot", c(list(df = "data"), args))
      tryCatch({
        res          <- do.call(eq5d_profile_lfs_utility_plot, c(list(df = rv$processed_data), args))
        r132fig$plot <- res$p
        r132fig$call <- cs
        save_result(rv, "LFS scatter (fig. 1.3.2)", cs, "plot", plot = res$p)
      }, error = function(e) err_notify(e))
    })
    
    output$out_132fig <- shiny::renderUI({
      if (is.null(r132fig$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_132fig"), height = "420px"),
                     call_display(ns("call_132fig")))
    })
    output$plt_132fig  <- shiny::renderPlot(r132fig$plot)
    output$call_132fig <- shiny::renderText(r132fig$call)
    
    # ═══════════════════════════════════════════════════════════════════════════
    # INFORMATIVITY
    # ═══════════════════════════════════════════════════════════════════════════
    
    # ── Figure 1.4.1 ─────────────────────────────────────────────────────────
    r141 <- shiny::reactiveValues(plot = NULL, call = NULL)
    
    shiny::observeEvent(input$run_141, {
      shiny::req(rv$processed_data)
      ver  <- eq5d_ver()
      args <- list(names_eq5d = DIMS_STD, eq5d_version = ver)
      cs   <- format_call("eq5d_profile_density_curve", c(list(df = "data"), args))
      tryCatch({
        res       <- do.call(eq5d_profile_density_curve, c(list(df = rv$processed_data), args))
        r141$plot <- res$p
        r141$call <- cs
        save_result(rv, "HSDI (fig. 1.4.1)", cs, "plot", plot = r141$plot)
      }, error = function(e) err_notify(e))
    })
    
    output$out_141 <- shiny::renderUI({
      if (is.null(r141$plot)) return(NULL)
      shiny::tagList(shiny::plotOutput(ns("plt_141"), height = "450px"),
                     call_display(ns("call_141")))
    })
    output$plt_141  <- shiny::renderPlot(r141$plot)
    output$call_141 <- shiny::renderText(r141$call)
  })
}

# ── Shared rendering helpers (module-local) ───────────────────────────────────

analysis_table_output <- function(output_id) {
  shiny::tagList(DT::DTOutput(output_id), shiny::br())
}

build_profile_container <- function(df) {
  col_names <- names(df)
  if (length(col_names) < 2L || col_names[1L] != "level") return(NULL)
  
  rest <- col_names[-1L]
  if (!all(grepl("^(n|freq)_", rest))) return(NULL)
  
  parsed <- lapply(rest, function(cn) {
    parts <- strsplit(cn, "_")[[1L]]
    if (length(parts) < 3L) return(NULL)
    list(
      metric = parts[1L],
      dim    = parts[length(parts)],
      fu     = paste(parts[2L:(length(parts) - 1L)], collapse = "_")
    )
  })
  if (any(vapply(parsed, is.null, logical(1L)))) return(NULL)
  
  dims <- vapply(parsed, `[[`, character(1L), "dim")
  if (!all(dims %in% names(DIM_LABELS))) return(NULL)
  
  fu_vals   <- vapply(parsed, `[[`, character(1L), "fu")
  multi_fu  <- length(unique(fu_vals)) > 1L
  dim_order <- unique(dims)
  
  th_level <- shiny::tags$th(rowspan = 2L, "Level")
  th_dims  <- lapply(dim_order, function(d) {
    shiny::tags$th(
      colspan = sum(dims == d),
      style   = "text-align: center; border-bottom: 0;",
      DIM_LABELS[[d]]
    )
  })
  th_sub <- lapply(seq_along(rest), function(i) {
    p      <- parsed[[i]]
    symbol <- if (p$metric == "freq") "%" else "n"
    label  <- if (multi_fu) paste0(p$fu, " ", symbol) else symbol
    shiny::tags$th(label, style = "text-align: right;")
  })
  
  shiny::tags$table(
    class = "table table-sm table-striped display",
    shiny::tags$thead(
      shiny::tags$tr(th_level, th_dims),
      shiny::tags$tr(th_sub)
    )
  )
}

render_analysis_table <- function(data_reactive) {
  DT::renderDT({
    df <- data_reactive()
    shiny::req(df)
    
    container      <- build_profile_container(df)
    freq_col_names <- names(df)[grepl("^freq_", names(df))]
    # Bare proportion columns produced by eq5d_profile_top_states / eq5d_profile_*_table functions
    pct_col_names  <- intersect(c("p", "cum_p"), names(df))
    all_pct        <- c(freq_col_names, pct_col_names)
    # Round remaining double columns (e.g. mean, sd) to 2 decimal places
    dbl_col_names  <- setdiff(
      names(df)[vapply(df, is.double, logical(1L))],
      all_pct
    )
    
    opts <- list(pageLength = 15L, scrollX = TRUE, dom = "tip")
    
    dt <- if (!is.null(container)) {
      DT::datatable(df, container = container, options = opts, rownames = FALSE)
    } else {
      DT::datatable(df, options = opts, rownames = FALSE,
                    class = "table-sm table-striped")
    }
    
    if (length(freq_col_names) > 0L)
      dt <- DT::formatPercentage(dt, columns = freq_col_names, digits = 1L)
    if (length(pct_col_names) > 0L)
      dt <- DT::formatPercentage(dt, columns = pct_col_names, digits = 1L)
    if (length(dbl_col_names) > 0L)
      dt <- DT::formatRound(dt, columns = dbl_col_names, digits = 2L)
    
    dt
  })
}

call_display <- function(output_id) {
  shiny::tagList(
    shiny::tags$details(
      shiny::tags$summary(shiny::tags$small("Show R code", class = "text-muted")),
      shiny::verbatimTextOutput(output_id, placeholder = TRUE)
    ),
    shiny::br()
  )
}

err_notify <- function(e) {
  shiny::showNotification(
    paste("Analysis error:", conditionMessage(e)),
    type = "error", duration = 8
  )
}

#' @keywords internal
.onAttach <- function(libname, pkgname) {

  # Check if a value set update is due
  # Uses cached date — no internet request is made here
  tryCatch({
    if (is_update_due(threshold_days = 60)) {
      packageStartupMessage(
        "eq5dsuite: Value sets were last checked more than ",
        "2 months ago.\n",
        "Run eq5dsuite::update_value_sets() to check for ",
        "new value sets."
      )
    }
  },
  error = function(e) {
    # Fail silently — never block package loading
    invisible(NULL)
  })
}

#' eq5dsuite: Standardised Tools for EQ-5D Analysis
#'
#' @description
#' The \pkg{eq5dsuite} package provides a comprehensive set of
#' functions for calculating EQ-5D preference-based values and
#' analysing EQ-5D data following the recommendations of
#' Devlin et al. (2020).
#'
#' The package supports three EQ-5D instruments:
#' \itemize{
#'   \item EQ-5D-3L
#'   \item EQ-5D-5L
#'   \item EQ-5D-Y-3L
#' }
#'
#' @section Value calculation:
#' Functions for calculating EQ-5D preference-based values:
#' \itemize{
#'   \item \code{\link{eq5d}} — Generic calculator for all
#'     instruments
#'   \item \code{\link{eq5d3l}} — EQ-5D-3L value calculation
#'   \item \code{\link{eq5d5l}} — EQ-5D-5L value calculation
#'   \item \code{\link{eq5dy3l}} — EQ-5D-Y-3L value calculation
#'   \item \code{\link{eqxw}} — Original crosswalk method
#'     (van Hout et al., 2012)
#'   \item \code{\link{eqxwr}} — Reverse crosswalk method
#'     (van Hout and Shaw, 2021)
#'   \item \code{\link{eqxw_UK}} — UK-specific NICE-recommended
#'     mapping (Hernandez Alava et al., 2023)
#' }
#'
#' @section Value set management:
#' Functions for managing country-specific value sets:
#' \itemize{
#'   \item \code{\link{eqvs_display}} — List all available
#'     value sets
#'   \item \code{\link{eqvs_add}} — Add a custom value set
#'   \item \code{\link{eqvs_drop}} — Remove a custom value set
#'   \item \code{\link{eqvs_load}} — Load a saved custom value
#'     set into the current session
#'   \item \code{\link{update_value_sets}} — Check for and
#'     install new value sets from the online repository
#' }
#'
#' @section Profile analysis:
#' Functions for analysing EQ-5D health state profiles,
#' including cross-sectional summaries, longitudinal change
#' analyses, and severity metrics:
#' \itemize{
#'   \item \code{\link{eq5d_profile_level_summary}} —
#'     Cross-sectional frequency summary by dimension
#'   \item \code{\link{eq5d_profile_level_summary_by_group}} —
#'     Stratified frequency summary by subgroup
#'   \item \code{\link{eq5d_profile_top_states}} —
#'     Most frequently observed health state profiles
#'   \item \code{\link{eq5d_profile_change_summary}} —
#'     Dimension-level frequencies at two time points
#'   \item \code{\link{eq5d_profile_dimension_change_table}} —
#'     Changes in levels per dimension as percentages of total
#'     and of type of change
#'   \item \code{\link{eq5d_profile_pchc_table}} —
#'     Paretian Classification of Health Change table
#'   \item \code{\link{eq5d_profile_pchc_with_no_problems_table}}
#'     — PCHC table with no-problems category
#'   \item \code{\link{eq5d_profile_pchc_by_group_plot}} —
#'     PCHC bar chart by subgroup
#'   \item \code{\link{eq5d_profile_better_dimensions_by_group_plot}}
#'     — Dimensions improved among Better patients
#'   \item \code{\link{eq5d_profile_worse_dimensions_by_group_plot}}
#'     — Dimensions worsened among Worse patients
#'   \item \code{\link{eq5d_profile_mixed_dimensions_by_group_plot}}
#'     — Dimensions changed among Mixed patients
#'   \item \code{\link{eq5d_profile_health_profile_grid}} —
#'     Health Profile Grid of individual transitions
#'   \item \code{\link{eq5d_profile_lss_utility_summary}} —
#'     Descriptive statistics by Level Sum Score
#'   \item \code{\link{eq5d_profile_lss_utility_plot}} —
#'     EQ-5D values plotted against LSS
#'   \item \code{\link{eq5d_profile_lfs_distribution}} —
#'     Level Frequency Score distribution table
#'   \item \code{\link{eq5d_profile_lfs_mean_utility}} —
#'     Mean EQ-5D values by LFS category
#'   \item \code{\link{eq5d_profile_lfs_utility_summary}} —
#'     Descriptive statistics by Level Frequency Score
#'   \item \code{\link{eq5d_profile_lfs_utility_plot}} —
#'     EQ-5D values plotted against LFS
#'   \item \code{\link{eq5d_profile_density_curve}} —
#'     Health State Density Curve
#' }
#'
#' @section EQ-5D value analysis:
#' Functions for analysing EQ-5D preference-based values
#' across time points, subgroups, and population norms:
#' \itemize{
#'   \item \code{\link{eq5d_utility_summary}} —
#'     Descriptive statistics at each time point
#'   \item \code{\link{eq5d_utility_summary_by_group}} —
#'     Descriptive statistics stratified by subgroup
#'   \item \code{\link{eq5d_utility_norms_comparison}} —
#'     Comparison with population norms by age and sex
#'   \item \code{\link{eq5d_utility_over_time_plot}} —
#'     Mean values with confidence intervals over time
#'   \item \code{\link{eq5d_utility_by_group_plot}} —
#'     Mean values with confidence intervals by subgroup
#'   \item \code{\link{eq5d_utility_change_by_group_plot}} —
#'     Longitudinal change by subgroup
#'   \item \code{\link{eq5d_utility_distribution_plot}} —
#'     Histogram of EQ-5D value distribution
#'   \item \code{\link{eq5d_utility_vas_scatter_plot}} —
#'     Scatter plot of EQ-5D values against EQ-VAS
#' }
#'
#' @section EQ-VAS analysis:
#' Functions for analysing EQ-VAS self-rated health scores:
#' \itemize{
#'   \item \code{\link{eq5d_vas_summary}} —
#'     Descriptive statistics at each time point
#'   \item \code{\link{eq5d_vas_distribution_table}} —
#'     Frequency distribution in pre-defined ranges
#'   \item \code{\link{eq5d_vas_histogram}} —
#'     Histogram of EQ-VAS scores
#'   \item \code{\link{eq5d_vas_grouped_distribution_plot}} —
#'     Bar chart of grouped EQ-VAS frequencies
#' }
#'
#' @section Helper functions:
#' Utility functions for preparing EQ-5D data:
#' \itemize{
#'   \item \code{\link{toEQ5Dindex}} — Combine dimension
#'     columns into a 5-digit profile code
#'   \item \code{\link{toEQ5Ddims}} — Split a 5-digit profile
#'     code into separate dimension columns
#'   \item \code{\link{make_all_EQ_indexes}} — Generate all
#'     valid EQ-5D state codes as a vector
#'   \item \code{\link{make_all_EQ_states}} — Generate all
#'     valid EQ-5D states as a data frame
#'   \item \code{\link{make_dummies}} — Create dummy variables
#'     from EQ-5D dimension columns
#' }
#'
#' @section Interactive application:
#' A Shiny application providing point-and-click access to
#' the same analytical workflow without requiring R coding:
#' \itemize{
#'   \item \code{\link{run_app}} — Launch the Shiny application
#' }
#'
#' @section Cross-platform suite:
#' eq5dsuite is part of a cross-platform suite also available
#' in Stata and Excel:
#' \itemize{
#'   \item Full suite:
#'     \url{https://github.com/MathsInHealth/eq5dsuite}
#'   \item R package:
#'     \url{https://github.com/MathsInHealth/eq5dsuite-r}
#'   \item Value sets repository:
#'     \url{https://github.com/MathsInHealth/eq5dsuite-value-sets}
#' }
#'
#' @references
#' Devlin N, Parkin D, Janssen B (2020).
#' \emph{Methods for Analysing and Reporting EQ-5D Data}.
#' Springer, Cham.
#' \doi{10.1007/978-3-030-47622-9}
#'
#' van Hout B, Janssen MF, Feng YS, et al. (2012).
#' Interim scoring for the EQ-5D-5L: mapping the EQ-5D-5L
#' to EQ-5D-3L value sets.
#' \emph{Value in Health}, 15(5), 708--715.
#' \doi{10.1016/j.jval.2012.02.008}
#'
#' @author
#' Anabel Estévez-Carrillo, Oliver Rivero-Arias,
#' Iryna Schlackow, Kim Rand
#'
#' Maintainer: Anabel Estévez-Carrillo
#' \email{aestevez@@mathsinhealth.com}
#'
#' @docType package
#' @name eq5dsuite-package
#' @aliases eq5dsuite
"_PACKAGE"

#' Fetch the migrations index from the repository
#'
#' Downloads the migrations.csv file from the eq5dsuite-value-sets
#' repository, which records all historical value set renames.
#' Returns NULL silently if the fetch fails.
#'
#' @return A data frame with columns version, old_VS_code,
#'   new_VS_code, reason, date. Returns NULL if fetch fails.
#' @keywords internal
fetch_migrations <- function() {
  url <- paste0(.vs_base_url, "migrations.csv")
  tryCatch({
    utils::read.csv(
      curl::curl(url),
      stringsAsFactors = FALSE
    )
  },
  error = function(e) {
    message("eq5dsuite: Could not fetch migrations: ",
            conditionMessage(e))
    NULL
  })
}

#' Get the list of already-applied migrations
#'
#' Reads the locally cached list of migration IDs that have
#' already been applied. Returns an empty character vector if
#' no migrations have been applied yet.
#'
#' @return A character vector of migration IDs.
#' @keywords internal
get_applied_migrations <- function() {
  path <- file.path(get_cache_dir(), "applied_migrations.rds")
  if (file.exists(path)) {
    tryCatch(
      readRDS(path),
      error = function(e) character(0)
    )
  } else {
    character(0)
  }
}

#' Save the list of applied migrations to cache
#'
#' @param migrations Character vector of migration IDs to save.
#' @return Invisibly returns the saved migrations.
#' @keywords internal
set_applied_migrations <- function(migrations) {
  path <- file.path(get_cache_dir(), "applied_migrations.rds")
  tryCatch({
    saveRDS(migrations, path)
    invisible(migrations)
  },
  error = function(e) {
    warning("eq5dsuite: Could not save applied migrations: ",
            conditionMessage(e))
    invisible(NULL)
  })
}

#' Apply pending value set migrations
#'
#' Fetches the migrations index from the repository and applies
#' any renames that have not yet been applied locally. This
#' ensures that value set codes remain consistent when a country
#' publishes a second value set and the original code needs to
#' be disambiguated with a year suffix.
#'
#' Migrations are applied before checking for new value sets in
#' \code{update_value_sets()}, ensuring the local state is
#' consistent before any new installations.
#'
#' @param ask Logical. Whether to ask for confirmation before
#'   applying migrations. Defaults to TRUE.
#' @return Invisibly returns a character vector of migration IDs
#'   that were applied in this session.
#' @keywords internal
apply_pending_migrations <- function(ask = TRUE) {

  # Fetch migrations from repository
  migrations <- fetch_migrations()
  if (is.null(migrations)) return(invisible(character(0)))

  # Validate structure (date column not required — not used in logic)
  required_cols <- c("version", "old_VS_code",
                     "new_VS_code", "reason")
  if (!all(required_cols %in% colnames(migrations))) {
    message("eq5dsuite: migrations.csv has unexpected format.")
    return(invisible(character(0)))
  }

  # Create unique migration IDs
  migrations$id <- paste0(
    migrations$version, ":",
    migrations$old_VS_code, "->",
    migrations$new_VS_code
  )

  # Find pending migrations
  applied <- get_applied_migrations()
  pending <- migrations[!migrations$id %in% applied, ]

  if (nrow(pending) == 0) {
    return(invisible(character(0)))
  }

  # Report pending migrations
  message("eq5dsuite: ", nrow(pending),
          " value set rename(s) to apply:")
  for (i in seq_len(nrow(pending))) {
    message("  - Renaming EQ-5D-", pending$version[i],
            " ", pending$old_VS_code[i],
            " -> ", pending$new_VS_code[i],
            ": ", pending$reason[i])
  }

  # Ask for confirmation
  if (ask && interactive()) {
    response <- readline(
      "Apply these renames now? [y/n]: "
    )
    if (tolower(trimws(response)) != "y") {
      message("eq5dsuite: Renames postponed. Run ",
              "update_value_sets() again to apply later.")
      return(invisible(character(0)))
    }
  }

  # Apply each pending migration
  newly_applied <- character(0)

  for (i in seq_len(nrow(pending))) {
    row <- pending[i, ]

    # Check if old code is installed
    installed <- get_installed_vs_codes(row$version)

    if (!row$old_VS_code %in% installed) {
      # Old code not installed — mark as applied and skip
      message("eq5dsuite: ", row$old_VS_code,
              " not installed \u2014 skipping rename.")
      newly_applied <- c(newly_applied, row$id)
      next
    }

    # Check if new code already exists
    if (row$new_VS_code %in% installed) {
      # Already renamed — mark as applied and skip
      message("eq5dsuite: ", row$new_VS_code,
              " already exists \u2014 skipping rename.")
      newly_applied <- c(newly_applied, row$id)
      next
    }

    # Apply the rename
    success <- rename_value_set(
      old_vs_code = row$old_VS_code,
      new_vs_code = row$new_VS_code,
      version     = row$version,
      ask         = FALSE
    )

    if (success) {
      newly_applied <- c(newly_applied, row$id)
      message("\u2705 ", row$old_VS_code, " renamed to ",
              row$new_VS_code)
    } else {
      message("\u274c Could not rename ", row$old_VS_code,
              " to ", row$new_VS_code)
    }
  }

  # Save updated list of applied migrations
  set_applied_migrations(c(applied, newly_applied))

  invisible(newly_applied)
}

#' Launch the eq5dsuite Shiny Application
#'
#' Opens an interactive Shiny application for uploading, processing,
#' analysing, and exporting EQ-5D data using the eq5dsuite package.
#'
#' @param ... Additional arguments passed to \code{\link[shiny]{runApp}},
#'   such as \code{port} or \code{launch.browser}.
#' @return Called for its side effect of launching a Shiny application.
#'   Returns invisibly.
#' @export
#' @examples
#' \dontrun{
#'   eq5dsuite::run_app()
#' }
run_app <- function(...) {
  app_packages <- c("shiny", "bslib", "DT", "readxl")
  
  missing_packages <- app_packages[
    !vapply(app_packages, requireNamespace, logical(1), quietly = TRUE)
  ]
  
  if (length(missing_packages) > 0) {
    stop(
      paste0(
        "To run the eq5dsuite Shiny app, please install the following package",
        if (length(missing_packages) > 1) "s" else "",
        ":\n\n",
        paste0("  - ", missing_packages, collapse = "\n"),
        "\n\nYou can install them with:\n\n",
        "install.packages(c(",
        paste(sprintf('"%s"', missing_packages), collapse = ", "),
        "))"
      ),
      call. = FALSE
    )
  }
  
  app_dir <- system.file("shiny", package = "eq5dsuite")
  
  if (!nzchar(app_dir)) {
    stop(
      "Could not find the bundled Shiny app in the installed eq5dsuite package. ",
      "Please reinstall eq5dsuite and try again.",
      call. = FALSE
    )
  }
  
  shiny::runApp(app_dir, ...)
}

#' Get the eq5dsuite cache directory
#'
#' Returns the path to the user-specific cache directory used by
#' eq5dsuite to store the date of the last value set check.
#' The directory is created if it does not exist.
#'
#' @return A character string giving the path to the cache directory.
#' @keywords internal
get_cache_dir <- function() {
  dir <- find_cache_dir("eq5dsuite")
  if (!dir.exists(dir)) dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  dir
}

#' Get the date of the last value set check
#'
#' Reads the cached date of the last value set update check from the
#' user cache directory. Returns \code{1970-01-01} if no cache file
#' exists (i.e., the check has never been run).
#'
#' @return A \code{Date} object.
#' @keywords internal
get_last_checked <- function() {
  cache_file <- file.path(get_cache_dir(), "last_checked.txt")
  if (!file.exists(cache_file)) return(as.Date("1970-01-01"))
  as.Date(readLines(cache_file, warn = FALSE)[1])
}

#' Set the date of the last value set check
#'
#' Writes the given date to the user cache directory so that future
#' calls to \code{get_last_checked()} and \code{is_update_due()} can
#' determine how long ago the check was performed.
#'
#' @param date A \code{Date} object. Defaults to \code{Sys.Date()}.
#' @return Invisibly returns \code{date}.
#' @keywords internal
set_last_checked <- function(date = Sys.Date()) {
  cache_file <- file.path(get_cache_dir(), "last_checked.txt")
  writeLines(format(date, "%Y-%m-%d"), cache_file)
  invisible(date)
}

#' Check whether a value set update check is due
#'
#' Compares the date of the last value set check (from the user cache)
#' against \code{Sys.Date()}. Returns \code{TRUE} if more than
#' \code{threshold_days} days have elapsed, indicating that the user
#' should be prompted to run \code{update_value_sets()}.
#'
#' @param threshold_days A single positive integer giving the number of
#'   days after which an update check is considered overdue. Defaults
#'   to \code{60}.
#' @return A single logical value.
#' @keywords internal
is_update_due <- function(threshold_days = 60) {
  as.numeric(Sys.Date() - get_last_checked()) > threshold_days
}

# Wrapper for file.copy that throws if any files fail to copy. Use name
# file.copy2 to make clear that it's a wrapper for a base function, and not for
# fs::file_copy.
file.copy2 <- function(from, to, ...) {
  res <- file.copy(from, to, ...)
  if (!all(res)) {
    stop("Error copying files: ", paste0(from[!res], collapse = ", "))
  }
}

dir.create2 <- function(path, ...) {
  res <- dir.create(path, ...)
  if (!res) {
    stop("Error creating directory: ", path)
  }
}

# .fixCountries <- function(countries, EQvariant = '5L') {
#   pkgenv <- getOption("eq.env")
#   cntrs <- rbind(pkgenv$country_codes[[EQvariant]], pkgenv[[paste0("user_defined_", EQvariant)]])
#   
#   sapply(countries, function(country) {
#     tmp <- which(toupper(as.matrix(cntrs)) == toupper(country), arr.ind = T)
#     if(nrow(tmp))  return(cntrs$ISO3166Alpha2[tmp[1,1]])
#     NA
#   })
# }

.fixCountries <- function(countries, EQvariant = '5L') {
  pkgenv <- getOption("eq.env")
  cc <- pkgenv$country_codes[[EQvariant]]
  ud <- pkgenv[[paste0("user_defined_", EQvariant)]]
  if (!is.null(ud) && nrow(ud) > 0) {
    common_cols <- intersect(colnames(cc), colnames(ud))
    cntrs <- rbind(cc[, common_cols, drop = FALSE],
                   ud[, common_cols, drop = FALSE])
  } else {
    cntrs <- cc
  }
  
  result <- sapply(countries, function(country) {
    matched_rows <- which(toupper(cntrs$Country_code) == toupper(country) | 
                            toupper(cntrs$VS_code) == toupper(country), arr.ind = TRUE)
    
    if (length(matched_rows) == 0) {
      return(NA)
    } else if (length(matched_rows) == 1) {
      return(cntrs$VS_code[matched_rows])
    } else {
      # If multiple value sets exist for the country, ask the user to specify
      message(paste0("There are ", length(matched_rows), " value sets available for country code '", country, "'."))
      options_table <- cntrs[matched_rows, c("Version", "Name", "Country_code", "VS_code", "doi")]
      print(options_table)
      
      # Ask user to input the VS_code
      repeat {
        user_input <- readline(prompt = "Please enter the VS_code you would like to use: ")
        if (user_input %in% cntrs$VS_code[matched_rows]) {
          return(user_input)
        } else {
          message("Invalid VS_code. Please enter one from the table above.")
        }
      }
    }
  }, USE.NAMES = TRUE)
  
  return(result)
}


find_cache_dir <- function(pkg) {
  # In R 4.0 and above, CRAN wants us to use the new tools::R_user_dir().
  # If not present, fall back to rappdirs::user_cache_dir().
  R_user_dir <- getNamespace('tools')$R_user_dir
  if (!is.null(R_user_dir)) {
    R_user_dir(pkg, which = "cache")
  } else {
    rappdirs::user_cache_dir(pkg, "R")
  }
}

.prettyPrint <- function(df, justify = 'r') {
  cnames <- colnames(df)
  if(length(justify)<length(cnames)) justify <- rep(justify, length(cnames))
  n      <- as.matrix(nchar(cnames))
  
  d <- as.matrix(as.data.frame(apply(df, 2, format, simplify = F)))
  d[which(trimws(d) == "NA", arr.ind = T)] <- ""
  n <- apply(cbind(n, nchar(d[1,])), 1, max)
  
  fmts <- vapply(1:length(justify), FUN.VALUE = 'aha', function(i) paste0("%",ifelse(tolower(justify[[i]]) == 'l','-', ''), n[[i]], "s"))
  for(i in 1:length(cnames)) {
    cnames[i] <- sprintf(fmts[i], cnames[i])
    d[,i] <- sprintf(fmts[i], trimws(d[,i]))
  }
  d <- rbind(cnames, d)
  
  for(i in 1:NROW(d)) cat(d[i,], '\r\n')
}

#' @title .pstate3t5
#' @description Takes a N x 25 matrix with probabilities per level/dimension, and creates an N * 3125 matrix with probabilities per state
#' @param PPP N x 25 matrix with probabilities per level/dimension created by EQrxwprobs
#' @return An N * 3125 matrix with probabilities per state
.pstate3t5 <-function(PPP) {
  vallst <- as.vector(as.matrix(expand.grid(21:25, 16:20, 11:15, 6:10, 1:5))[, 5:1])
  t1l<- lapply(seq_len(dim(PPP)[2]), function(x) PPP[, x])
  do.call(cbind, lapply(seq_len(3125), function(i) {
    t1l[[vallst[[i]]]]*
      t1l[[vallst[[i+3125]]]]*
      t1l[[vallst[[i+6250]]]]*
      t1l[[vallst[[i+9375]]]]*
      t1l[[vallst[[i+12500]]]]
  }))
}

#' @title .EQxwrprob
#' @description Takes a matrix of parameters for reverse crosswalk model, returns 243 x 25 matrix of state/level transition probabilities.
#' @param par Matrix of model parameters 
#' @return An 243 * 25 matrix with probabilities for state level transitions.
.EQxwrprob <- function(par=NULL) {
  if(is.null(par)) stop('No parameters provided.')
  par <- as.matrix(par)
  
  # Logistic function by way of hyperbolic tangent
  lp <- function(eta) 0.5+0.5*tanh(eta*0.5)
  
  # Parameters for transitions between levels 1-->2, 2-->3, 3-->4, and 4-->5, per dimension
  Zx<-t(par[1:4,])            
  # Parameters for adjacent dimensions (10), age, age^2, and gender
  Bx<-par[5:14,]
  
  tmp <- diag(3)[,2:3]
  Y <- - do.call(cbind, lapply(as.list(expand.grid(1:3, 1:3, 1:3, 1:3, 1:3)[,5:1]), function(x) tmp[x,]))
  
  # Model matrix: dummies for EQ-5D-3L (i.e. mo2, mo3, sc2 ...ad2, ad3), age, age^2, gender
  # Y <- - as.matrix(cbind(inp[, 1:10], inp[,11]/10, (inp[,11]/10)^2, inp[,12])) # Note the "-" first; these are negatives
  do.call(cbind, lapply(1:5, function(i) {
    # inverse logit, first the level transitions
    # t(Zx[, rep(i, NROW(inp)])) creates a N * 4 matrix with the corresponding level transition parameters in the four columns
    tmp <- lp(Zx[rep.int(i,243),] + 
                # Y %*% Bx[,i] is a matrix multiplication of the model matrix and the corresponding coefficients
                (Y %*% Bx[,i])[,1])
    # subtract previous level from next, creating a matrix with column 1 representing p for level 1, column 2 for level 2, etc.
    cbind(tmp,1)-cbind(0, tmp)
  }))
}

#' @title eqxwr
#' @description Get reverse crosswalk values
#' @param x A vector of 5-digit EQ-5D-3L state indexes or a matrix/data.frame with columns corresponding to EQ-5D state dimensions
#' @param country String vector indicating country names or  ISO3166 Alpha 2 / 3 country codes.
#' @param dim.names A vector of dimension names to identify dimension columns
#' @return A vector of reverse crosswalk values or data.frame with one column per reverse crosswalk set requested.
#' @examples 
#' eqxwr(c(11111, 12321, 32123, 33333), 'US')
#' eqxwr(make_all_EQ_states('3L'), c('DK', 'US'))
#' @export
eqxwr <- function(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad")) {
  pkgenv <- getOption("eq.env")
  if(!length(dim.names) == 5) stop("Argument dim.names not of length 5.")
  if(length(dim(x)) == 2) {
    colnames(x) <- tolower(colnames(x))
    if(is.null(colnames(x))) {
      message("No column names")
      if(NCOL(x) == 5) {
        message("Given 5 columns, will assume dimensions in conventional order: MO, SC, UA, PD, AD.")
        colnames(x) <- dim.names
      } 
    }
    if(!all(dim.names %in% colnames(x))) stop("Provided dimension names not available in matrix/data.frame.")
    x <- toEQ5Dindex(x = x, dim.names = dim.names)
  }
  
  country <- .fixCountries(country)
  if(any(is.na(country))) {
    isnas <- which(is.na(country))
    for(i in isnas)  warning('Country ', names(country)[i], ' not found. Dropped.')
    country <- country[!is.na(country)]
  }
  
  if(length(country)==0) {
    message('No valid countries listed. These value sets are currently available.')
    eqvs_display(version = "5L")
    stop('No valid countries listed.')
  }
  
  x <- as.integer(x)
  x[!regexpr("^[1-3]{5}$", x)==1] <- NA
  
  if(length(country)>1) {
    names(country) <- country
    return(do.call(cbind, lapply(country, function(count) eqxwr(x, count, dim.names))))
  }
  
  xout <- rep(NA, length(x))
  
  xout[!is.na(x)] <- pkgenv$xwrsets[match(x[!is.na(x)], pkgenv$states_3L$state), country]
  xout
  
}

#' @title .pstate5t3
#' @description Takes a 15 x 5 matrix with probabilities per level/dimension, and creates an 3125x243 matrix with probabilities per state
#' @param probs 15 x 5 matrix with probabilities per level/dimension, typically saved in .EQxwprob
#' @return An 3125x243 matrix with probabilities per state
.pstate5t3 <-  function(probs = .EQxwprob) {
  
  allst5l <- make_all_EQ_states()
  allst3l <- make_all_EQ_states(version = '3L')
  
  t(Reduce('*', lapply(0:4, function(i)    probs[i*3+allst3l[, i+1],allst5l[,i+1]])))
}



#' @title eqxw
#' @description Get crosswalk values
#' @param x A vector of 5-digit EQ-5D-5L state indexes or a matrix/data.frame with columns corresponding to EQ-5D state dimensions
#' @param country String vector indicating country names or  ISO3166 Alpha 2 / 3 country codes.
#' @param dim.names A vector of dimension names to identify dimension columns
#' @return A vector of reverse crosswalk values or data.frame with one column per reverse crosswalk set requested.
#' @examples 
#' eqxw(c(11111, 12521, 32123, 55555), 'US')
#' eqxw(make_all_EQ_states('5L'), c('DK', 'US'))
#' @export
eqxw <- function(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad")) {
  eq5d(x = x, country = country, version = "xw", dim.names = dim.names)
}

#' @title eqxw_UK
#' @description Crosswalks EQ-5D-5L responses to EQ-5D-3L utilities using NICE's mapping.
#' @param x A vector of 5-digit EQ-5D-5L states (domain scores) or a summary score.
#' @param age  A numeric vector or column name (if `x` is a data frame). Can be either:
#'   (1) a numeric age between 18 and 100, which will be automatically grouped into NICE-defined age bands (18-35, 35-45, 45-55, 55-65, +65), or
#'   (2) a factor/character/numeric vector already representing the NICE age bands with values 1-5 indicating age bands (18-35, 35-45, 45-55, 55-65, +65).
#' @param male A numeric vector (1=male, 0=female) or column name indicating gender.
#' @param dim.names A vector of dimension names for EQ-5D states (default: c("mo", "sc", "ua", "pd", "ad")).
#' @param bwidth Numeric. Bandwidth for kernel smoothing when using summary scores.
#' @return A vector or data frame with crosswalked EQ-5D-3L utilities.
#' @examples 
#' eqxw_UK(c(11111, 12345, 32423, 55555), age = c(30, 40, 55, 70), male = c(1, 0, 1, 0))
#' @importFrom stats dnorm weighted.mean
#' @export

eqxw_UK <- function(x, age, male, dim.names = c("mo", "sc", "ua", "pd", "ad"), bwidth = 0) {
 
  # Load crosswalk data
  pkgenv <- getOption("eq.env")
  if (is.null(pkgenv) || is.null(pkgenv$crosswalk_NICE[["5L"]])) {
    stop("Missing NICE crosswalk data in `options(eq.env)`. Please set the environment.")
  }
  cw <- pkgenv$crosswalk_NICE[["5L"]]
  names(cw)[match(c("X_age",  "X_male"),  names(cw))] <- c("age",  "male")
  
  # Determine input type and construct data
  if (is.data.frame(x)) {
    if (!all(dim.names %in% names(x))) 
      stop("Provided dimension names not found in data.")
    if (!(age %in% names(x)) || !(male %in% names(x))) 
      stop("Age and male must be column names in `x`.")
    df <- data.frame(
      Domain = toEQ5Dindex(as.matrix(x[ , dim.names])),
      age    = x[[age]],
      male   = as.numeric(x[[male]]),
      stringsAsFactors = FALSE
    )
    
  } else if (is.vector(x)) {
    df <- data.frame(
      Domain = as.character(x),
      age = age,
      male = male,
      stringsAsFactors = FALSE
    )
  } else {
    stop("`x` must be either a data frame or a character/numeric vector of EQ-5D states.")
  }
  
  # Process age into age bands
  df$age_band <- if (is.numeric(df$age)) {
    as.numeric(cut(df$age,
                   breaks = c(1, 18, 35, 45, 55, 65, 100),
                   labels = c("1", "1", "2", "3", "4", "5"),
                   right  = FALSE))
  } else {
    as.numeric(as.character(df$age))
  }

  is_domain <- all(grepl("^[1-5]{5}$", df$Domain))
  
  if (is_domain) {
    res <- merge(df,
                 cw[ , c("Domain", "age", "male", "Output")],
                 by.x = c("Domain", "age_band", "male"),
                 by.y = c("Domain", "age",      "male"),
                 all.x = TRUE, sort = FALSE)
  } else {
    
    df$score <- as.numeric(df$Domain)
    cw$score   <- as.numeric(cw$Domain)
    
    if (bwidth == 0) {
      res <- merge(df,
                   cw[ , c("score", "age", "male", "Output")],
                   by.x = c("score", "age_band", "male"),
                   by.y = c("score", "age",      "male"),
                   all.x = TRUE, sort = FALSE)
      
    } else {
      
      # kernel-weighted estimate
      res <- df
      res$Output <- mapply(
        function(a, m, s) {
          sub <- cw[cw$age == a & cw$male == m, ]
          w   <- stats::dnorm((sub$score - s) / bwidth)
          if (length(w)) stats::weighted.mean(sub$Output, w) else NA_real_
        },
        df$age_band, df$male, df$score, SIMPLIFY = TRUE)
    }
  }

  return(res$Output)
}

#' @title example_data
#' @description A dataset containing patient-level data in a long format.
#' @format A data frame with 10000 rows and 14 variables:
#' \describe{
#'   \item{\code{id}}{double Patient id}
#'   \item{\code{time}}{character Follow-up (Pre-op / Post-op)}
#'   \item{\code{mo}}{double EQ-5D-5L Mobility dimension}
#'   \item{\code{sc}}{double EQ-5D-5L Self-care dimension}
#'   \item{\code{ua}}{double EQ-5D-5L Usual activities dimension}
#'   \item{\code{pd}}{double EQ-5D-5L Pain / discomfort dimension}
#'   \item{\code{ad}}{double EQ-5D-5L Anxiety/depression dimension}
#'   \item{\code{vas}}{double Value of the VAS scale measurememnt}
#'   \item{\code{providercode}}{character Provider code}
#'   \item{\code{procedure}}{character Type of surgery}
#'   \item{\code{year}}{character Year of intervention}
#'   \item{\code{ageband}}{character Age in pre-defined ranges} 
#'   \item{\code{gender}}{character Patient's gender (Female / Male)}
#'}
#' @docType data
#' @keywords datasets
#' @name example_data
#' @usage data(example_data)
NULL

# Internal helpers used by the package's Shiny application.
# None of these functions are part of the public API.

# ── EQ-5D full dimension labels ───────────────────────────────────────────────

DIM_LABELS <- c(
  mo = "Mobility",
  sc = "Self care",
  ua = "Usual activities",
  pd = "Pain/Discomfort",
  ad = "Anxiety/Depression"
)

# ── ensure_fu ─────────────────────────────────────────────────────────────────

#' Ensure a follow-up column exists (adding a synthetic one when absent)
#'
#' Package functions such as \code{eq5d_vas_summary} and \code{eq5d_utility_summary} internally
#' require a follow-up column.  When the user has not mapped a timepoint
#' variable, this helper adds a single-level factor column \code{".fu_all."}
#' (value \code{"All"}) so those functions can run cross-sectionally without
#' error.
#'
#' @param df A data frame (the processed Shiny dataset).
#' @param mapping A named list; \code{mapping$name_fu} should be non-\code{NULL}
#'   and non-empty when a follow-up variable was mapped.
#' @return A named list with elements \code{df} (possibly augmented with
#'   \code{".fu_all."}) and \code{name_fu} (the column name to pass to package
#'   functions).
#' @keywords internal
#' @noRd
ensure_fu <- function(df, mapping) {
  has_fu <- !is.null(mapping) &&
            !is.null(mapping[["name_fu"]]) &&
            nzchar(mapping[["name_fu"]]) &&
            "fu" %in% names(df)
  if (has_fu) {
    list(df = df, name_fu = "fu")
  } else {
    df[[".fu_all."]] <- factor("All")
    list(df = df, name_fu = ".fu_all.")
  }
}

# ── prettify_table_columns ────────────────────────────────────────────────────

#' Rename profile-table columns using full EQ-5D dimension labels
#'
#' Renames columns of a \code{.freqtab()}-style wide data frame.
#' Columns named \code{n_\{fu\}_\{dim\}} become \code{"\{DIM_LABEL\} n"} and
#' \code{freq_\{fu\}_\{dim\}} columns become \code{"\{DIM_LABEL\} \%"}.
#' When multiple follow-up levels exist the fu label is prepended to avoid
#' duplicate column names.
#'
#' @param df A data frame as returned by \code{eq5d_profile_level_summary} etc.
#' @return The same data frame with renamed columns (original order preserved).
#' @keywords internal
#' @noRd
prettify_table_columns <- function(df) {
  col_names <- names(df)
  if (!"level" %in% col_names) return(df)

  rest <- col_names[col_names != "level"]
  if (length(rest) == 0L || !all(grepl("^(n|freq)_", rest))) return(df)

  parsed <- lapply(rest, function(cn) {
    parts <- strsplit(cn, "_")[[1L]]
    if (length(parts) < 3L) return(NULL)
    list(
      metric = parts[1L],
      dim    = parts[length(parts)],
      fu     = paste(parts[2L:(length(parts) - 1L)], collapse = "_")
    )
  })
  if (any(vapply(parsed, is.null, logical(1L)))) return(df)

  fu_vals  <- vapply(parsed, `[[`, character(1L), "fu")
  multi_fu <- length(unique(fu_vals)) > 1L

  new_names <- vapply(parsed, function(p) {
    dim_label <- if (p$dim %in% names(DIM_LABELS)) DIM_LABELS[[p$dim]] else p$dim
    symbol    <- if (p$metric == "freq") "%" else "n"
    if (multi_fu) {
      paste0(p$fu, " \u2014 ", dim_label, " ", symbol)
    } else {
      paste0(dim_label, " ", symbol)
    }
  }, character(1L))

  names(df)[match(rest, names(df))] <- new_names
  df
}

# ── compute_utility_col ───────────────────────────────────────────────────────

#' Compute an EQ-5D utility index column
#'
#' @param df A data frame containing columns \code{mo}, \code{sc}, \code{ua},
#'   \code{pd}, \code{ad}.
#' @param method One of \code{"direct"} (use the instrument's own value set),
#'   \code{"xw"} (crosswalk 5L\eqn{\to}3L), or \code{"xwr"} (reverse crosswalk
#'   3L\eqn{\to}5L).
#' @param country Country/value-set code passed to \code{\link{eq5d}}.
#' @param eq5d_version \code{"3L"} or \code{"5L"} (used only when
#'   \code{method = "direct"}).
#' @return Numeric vector of utility index values, length \code{nrow(df)}.
#' @keywords internal
#' @noRd
compute_utility_col <- function(df, method, country, eq5d_version) {
  version <- switch(method,
    direct = eq5d_version,
    xw     = "XW",
    xwr    = "XWR",
    stop("Unknown utility method: ", method, call. = FALSE)
  )
  x <- df[, c("mo", "sc", "ua", "pd", "ad"), drop = FALSE]
  eq5d(x, country = country, version = version)
}

#' Replace NULL names with default values
#'
#' This function takes in a list of parameters, which would be column names of the input data frame, and checks if they are null. Any nulls are replaced with default values, and the updated list of parameters is returned.
#'
#' @param df a data frame; only used/supplied if levels_fu needs to be defined
#' @param ... a list of parameters consisting of any/all of `names_eq5d`, `name_fu`, `levels_fu`, `eq5d_version`, and `name_vas`.
#' @return a list of parameters with null entries replaced with default values.
#' @examples
#' .get_names(names_eq5d = c("mo", "sc", "ua", "pd", "ad"))
#' .get_names(names_eq5d = NULL, eq5d_version = NULL, name_vas = NULL)
#' @export
#' 
.get_names <- function(df = NULL, ...) {
  
  # read off input parameters and their supplied names
  args <- list(...)
  names_list <- names(args)
  
  # check names_eq5d
  if ("names_eq5d" %in% names_list) {
    names_eq5d <- args$names_eq5d
    if (is.null(names_eq5d)) {
      message("Argument `names_eq5d` not supplied. Default column names will be used: mo, sc, ua, pd, ad")
      names_eq5d <- c("mo", "sc", "ua", "pd", "ad")
    }
    args[["names_eq5d"]] <- names_eq5d
  }
  # check name_fu
  # if name_fu is specified, so must be levels_fu
  if ("name_fu" %in% names_list) {
    name_fu <- args$name_fu
    levels_fu <- args$levels_fu
    if (is.null(name_fu)) {
      message("Argument `name_fu` not supplied. Default column name will be used: fu")
      name_fu <- "fu"
    }
    args[["name_fu"]] <- name_fu
    # check also levels of fu
    if (is.null(levels_fu)) {
      message("No ordering of time suppled. The time variable will be factorised according to the order in the data frame.")
      levels_fu <- unique(df[[name_fu]])
    }
    args[["levels_fu"]] <- levels_fu
  }
  # check eq5d_version
  if ("eq5d_version" %in% names_list) {
    eq5d_version <- args$eq5d_version
    if (is.null(eq5d_version)) {
      message("No EQ-5D version was provided. 5L version will be used.")
      eq5d_version <- "5L"
    }
    args[["eq5d_version"]] <- eq5d_version
  }
  # check name_vas
  if ("name_vas" %in% names_list) {
    name_vas <- args$name_vas
    if (is.null(name_vas)) {
      message("Argument `name_vas` not supplied. Default column name will be used: vas")
      name_vas <- "vas"
    }
    args[["name_vas"]] <- name_vas
  }
  
  return(args)
}

#' Calculate the Level Frequency Score (LFS)
#'
#' This function calculates the Level Frequency Score (LFS) for a given EQ-5D state and a specified version of EQ-5D.
#' If at least one domain contains a missing entry, the whole LFS is set to be NA.
#'
#' @param s A character vector representing the EQ-5D state, e.g. 11123.
#' @param eq5d_version A character string specifying the version of EQ-5D, i.e. 3L or 5L.
#' @return A character vector representing the calculated LFS.
#' @examples
#' .get_lfs("333", "3L") # returns 003
#' .get_lfs("333", "5L") # returns 00300
#' .get_lfs("12345", "5L") # returns 11111
#' @export
#' 
.get_lfs <- function(s, eq5d_version) {
  
  # count occurrences of each digit; nchar(s) - nchar(gsub(...)) preserves NAs
  cnt <- function(s, ch) nchar(s) - nchar(gsub(ch, "", s, fixed = TRUE))
  # for any eq5d version need to count 1s, 2s and 3s
  lfs <- paste0(cnt(s, "1"), cnt(s, "2"), cnt(s, "3"))
  lfs[is.na(s)] <- NA_character_
  # if 5L, also add count of 4s and 5s
  if (eq5d_version == "5L") {
    lfs <- paste0(lfs, cnt(s, "4"), cnt(s, "5"))
    lfs[is.na(s)] <- NA_character_
  }
  
  return(lfs)
}

#' Add utility values to a data frame
#'
#' This function adds utility values to a data frame based on a specified version of EQ-5D and a country name.
#'
#' @param df A data frame containing the state data. The state must be included in the data frame as a character vector under the column named `state`.
#' @param eq5d_version A character string specifying the version of EQ-5D, i.e. 3L or 5L.
#' @param country A character string representing the name of the country. This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return A data frame with an additional column named `utility` containing the calculated utility values. If the input country name is not found in the country_codes dataset, a list of available codes is printed, and subsequentyl an error message is displayed and the function stops.
#' @examples
#' df <- data.frame(state = c("11111", "11123", "32541"))
#' .add_utility(df, "5L", "DK")
#' @export
#' 
.add_utility <- function(df, eq5d_version, country) {
  
  pkgenv <- getOption("eq.env")
  
  # check whether the valuation for this country code exists
  country_code <- .fixCountries(countries = country, EQvariant = eq5d_version)
  if (is.na(country_code)) {
    message('No valid countries listed. These value sets are currently available.')
    eqvs_display(version = eq5d_version)
    stop('Stopping.')
  }
  
  df$utility <- eq5d(x = df$state, country = country_code, version = eq5d_version)
  # 
  # # country identifiable; proceed to extract the data
  # vs <- pkgenv[[paste0("vsets", eq5d_version, "_combined")]] %>%
  #   select(state, !!(sym(country_code))) %>%
  #   rename(utility = !!quo_name(country_code))
  # 
  # # merge with df
  # df <- merge(df, vs, all.x = TRUE) 
  
  return(df)
}

#' Data checking/preparation: EQ-5D variables
#'
#' This function prepares a data frame for analysis by extracting, processing, and adding columns for EQ-5D variables, including state, LSS (Level Sum Score), LFS (Level Frequency Score) and utility.
#' 
#' @param df a data frame of EQ-5D scores
#' @param names character vector of length 5 with names of EQ-5D variables in the data frame. The variables should be in an integer format.
#' @param add_state logical indicating whether the EQ-5D state should be added
#' @param add_lss logical indicating whether the LSS (Level Sum Score) should be added
#' @param add_lfs logical indicating whether the LFS (Level Frequency Score) should be added
#' @param add_utility logical indicating whether the utility should be added
#' @param eq5d_version character indicating the version of the EQ-5D questionnaire to use (either "3L" or "5L")
#' @param country character indicating the country to retrieve the quality of life score for
#' @return a modified data frame with EQ-5D domain columns renamed to default names, and, if necessary, with added columns for state, LSS, LFS, and/or utility. If any of the checks fail (e.g. EQ-5D columns are not in an integer format), an error message is displayed and the function is stopping.
#' @examples
#' set.seed(1234)
#' df <- data.frame(mo = sample(1:5, 3), sc = sample(1:5, 3), 
#'   ua = sample(1:5, 3), pd = sample(1:5, 3), ad = sample(1:5, 3))
#' .prep_eq5d(df, names = c("mo", "sc", "ua", "pd", "ad"), 
#'   add_state = TRUE, add_lss = TRUE)
#' .prep_eq5d(df, names = c("mo", "sc", "ua", "pd", "ad"),
#'   add_state = TRUE, add_lss = TRUE, add_lfs = TRUE, add_utility = TRUE,
#'   eq5d_version = "5L", country = "ES")
#' @export

.prep_eq5d <- function(df, names,
                       add_state = FALSE,
                       add_lss = FALSE,
                       add_lfs = FALSE,
                       add_utility = FALSE,
                       eq5d_version = NULL,
                       country = NULL) {

  # confirm correct length
  if (length(names) != 5)
    stop("Argument dim_names not of length 5. Stopping.")

  # confirm numeric format
  df_eq5d <- df[, names, drop = FALSE]

  x <- as.matrix(df_eq5d)
  xorig <- x
  x[,] <- as.integer(x)
  if(c(eq5d_version %in% c('3L', 'Y3L', 'XWR'),0)[1]) {
    x[!x %in% 1:3] <- NA
  } else {
    x[!x %in% 1:5] <- NA
  }
  if(sum(is.na(as.vector(x)))>sum(is.na(as.vector(xorig)))) warning(paste0(sum(is.na(as.vector(x)))-sum(is.na(as.vector(xorig))), " observations were coerced to NAs as they were not interpretable as integer values in the range allowed by the EQ-5D descriptive system."))
  df_eq5d[,] <- x

  # confirm EQ-5D version if required
  if (!is.null(eq5d_version))
    if (!(tolower(eq5d_version) %in% c("3l", "5l", 'y3l')))
      stop("EQ-5D version can only be 3L, 3l, Y3L, y3l, 5L or 5l. Stopping.")

  df[, names] <- df_eq5d

  # rename EQ-5D columns to standard names
  std_names <- c("mo", "sc", "ua", "pd", "ad")
  names(df)[match(names, names(df))] <- std_names

  # add additional columns if required
  if (add_state)
    df$state <- ifelse(
      is.na(df$mo) | is.na(df$sc) | is.na(df$ua) | is.na(df$pd) | is.na(df$ad),
      NA_character_,
      paste0(df$mo, df$sc, df$ua, df$pd, df$ad))
  if (add_lss)
    df$lss <- df$mo + df$sc + df$ua + df$pd + df$ad
  if (add_lfs)
    df$lfs <- .get_lfs(s = df$state, eq5d_version = eq5d_version)
  if (add_utility)
    df <- .add_utility(df = df, eq5d_version = eq5d_version, country = country)

  return(df)
}

#' Data checking/preparation: follow-up variable
#' 
#' This function prepares the follow-up (FU) variable for analysis by giving it a default name (`fu`) and factorising
#'
#' @param df A data frame.
#' @param name Column name in the data frame that contains follow-up information.
#' @param levels Levels to factorise the FU variable into.
#' @return A data frame with the follow-up variable renamed as "fu" and factorised.
#' @examples
#' df <- data.frame(id = c(1, 1, 2, 2),
#'   visit = c("baseline", "follow-up", "baseline", "follow-up"))
#' .prep_fu(df = df, name = "visit", levels = c("baseline", "follow-up"))
#' @export

.prep_fu <- function(df, name = NULL, levels = NULL) {

  names(df)[names(df) == name] <- "fu"
  df$fu <- factor(df$fu, levels = levels)

  return(df = df)
}

#' Data checking/preparation: VAS variable
#' 
#' The function prepares the data for VAS (Visual Analogue Scale) analyses. 
#' 
#' @param df A data frame.
#' @param name Column name in the data frame that holds the VAS score. The column can only contain integers or NAs
#' @return A modified data frame with the VAS score renamed to "vas". If any checks fail (e.g. column is not numeric), an error message is displayed and the function is stopping.
#' @examples
#' df <- data.frame(vas_score = c(20, 50, 80, NA, 100))
#' .prep_vas(df = df, name = "vas_score")
#' df <- data.frame(vas_score = c(20.5, 50, 80, NA, 100))
#' .prep_vas(df = df, name = "vas_score")
#' @export
#' 
.prep_vas <- function(df, name) {
  
  # extract data
  x <- as.vector(as.data.frame(df)[, name])
  
  
  xorig <- x <- as.integer(x)
  x[!x %in% 0:100] <- NA
  if(sum(is.na(x))>sum(is.na(xorig))) warning(paste0(sum(is.na(x))>sum(is.na(xorig)), " observations were coerced to NAs as they were not interpretable as integer values in the range allowed by the EQ-5D descriptive system."))
  df[,name] <- x
  
  # # remove NAs
  # v <- v[!is.na(v)]
  # # confirm numeric format
  # if (!is.numeric(v))
  #   stop("VAS column must be in a numeric format. Stopping.")
  # # confirm integers only
  # if (!all(floor(v) == v))
  #   stop("VAS column can only contain integers or NAs. Stopping.")
  
  # all checks passed; rename column
  names(df)[names(df) == name] <- "vas"

  # return value
  return(df)
}

#' Check the uniqueness of groups
#' This function takes a data frame `df` and a vector of columns `group_by`, and checks whether the combinations of values in the columns specified by `group_by` are unique. If the combinations are not unique, a warning message is printed.
#' @param df A data frame.
#' @param group_by A character vector of column names in `df` that specify the groups to check for uniqueness.
#' @return No return value, called for side effects: it will stop with an error if any group combinations are not unique.
#' @examples
#' df <- data.frame(id = c(1, 1, 1, 1, 2, 2),
#'                  fu = rep(c("baseline", "follow-up"), 3),
#'                  value = rnorm(6))
#' .check_uniqueness(df, c("id", "fu"))
#' @export
.check_uniqueness <- function(df, group_by) {

  keys <- do.call(paste, c(df[group_by], list(sep = "\001")))
  counts <- tabulate(match(keys, unique(keys)))

  if (!all(counts == 1L))
    message(paste0("Warning: there are non-unique ",
                   paste(group_by, collapse = "-"),
                   " combinations."))
}

#' Get the mode of a vector.
#'
#' This function calculates the mode of a numeric or character vector. 
#' If there are multiple modes, the first one is returned. 
#' The code is taken from an \href{https://www.tutorialspoint.com/r/r_mean_median_mode.htm}{R help page}.
#'
#' @param v A numeric or character vector.
#' @return The mode of `v`.
#' @examples
#' .getmode(c(1, 2, 3, 3))
#' .getmode(c("a", "b", "b", "c"))
#' @export
#' 
.getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

#' Wrapper for the repetitive code in function_table_2_1. Data frame summary
#' 
#' This internal function summarises a data frame by grouping it based on the variables specified in the 'group_by' argument and calculates the frequency of each group. The output is used in Table 2.1
#' 
#' @param df A data frame
#' @param group_by A character vector of variables in `df` to group by. Should contain 'eq5d' and 'fu'.
#' @return A summarised data frame with groups defined by `eq5d` and `fu` variables, the count of observations in each group, and the frequency of each group.
#' @examples
#' set.seed(1234)
#' df <- data.frame(eq5d = rep(rnorm(5), 2),
#'                  fu = rep(c(1, 0, 1, 0, 1), 2))
#' .summary_table_2_1(df, c("eq5d", "fu"))
#' @export

.summary_table_2_1 <- function(df, group_by) {

  # count occurrences per group
  counts <- aggregate(rep(1L, nrow(df)), by = df[group_by], FUN = sum)
  names(counts)[ncol(counts)] <- "n"

  # freq: n / sum(n) within each (eq5d, fu) combination
  key <- paste(counts$eq5d, counts$fu, sep = "\001")
  totals <- tapply(counts$n, key, sum)
  counts$freq <- counts$n / totals[key]

  return(counts)
}

#' Helper function for frequency of levels by dimensions tables
#' 
#' @param df Data frame with the EQ-5D and follow-up columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column. If NULL, no grouping is used, and the table reports for the total population.
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param add_summary_problems_change If set to false, the resulting dataframe does not include a row on problems change.
#' @param eq5d_version Version of the EQ-5D instrument
#' @return Summary data frame.

.freqtab<- function(df,
                    names_eq5d = NULL,
                    name_fu = NULL,
                    levels_fu = NULL,
                    eq5d_version = NULL,
                    add_summary_problems_change = TRUE) {
  
  ### data preparation ###
  
  if(is.null(name_fu)) {
    df$`_all` <- "All"
    name_fu <- "_all"
  }
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d, eq5d_version = eq5d_version)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  ### analysis ###

  # reshape to long format (replace pivot_longer)
  levels_eq5d <- c("mo", "sc", "ua", "pd", "ad")
  df_long <- do.call(rbind, lapply(levels_eq5d, function(d) {
    data.frame(fu = df$fu, eq5d = d, value = df[[d]], stringsAsFactors = FALSE)
  }))

  # complete case dataset: remove NA values
  df_cc <- df_long[!is.na(df_long$value), , drop = FALSE]

  # summary: individual levels
  summary_dim <- .summary_table_2_1(df = df_cc, group_by = c("eq5d", "value", "fu"))
  names(summary_dim)[names(summary_dim) == "value"] <- "level"
  summary_dim$level <- as.character(summary_dim$level)

  # summary: total
  summary_total <- .summary_table_2_1(df = df_cc, group_by = c("eq5d", "fu"))
  summary_total$level <- "Total"

  # summary: some problems and change
  df_probs <- df_cc[df_cc$value != 1, , drop = FALSE]
  summary_problems <- aggregate(rep(1L, nrow(df_probs)),
                                by = list(eq5d = df_probs$eq5d, fu = df_probs$fu),
                                FUN = sum)
  names(summary_problems)[names(summary_problems) == "x"] <- "n"
  # merge with totals
  summary_problems <- merge(summary_problems,
                            summary_total[, c("eq5d", "fu", "n")],
                            by = c("eq5d", "fu"), suffixes = c("", "_total"))
  summary_problems$freq <- summary_problems$n / summary_problems$n_total
  summary_problems$n_total <- NULL
  suffix <- if (eq5d_version == "3L") "2+3" else "2+3+4+5"
  summary_problems$level <- paste0("Number reporting any problems (levels ", suffix, ")")

  # change in numbers reporting problems since previous time point
  # sort by (eq5d, fu) with fu ordered by levels_fu
  sp_sub <- summary_problems[, c("eq5d", "fu", "n")]
  sp_sub$fu_ord <- match(as.character(sp_sub$fu), as.character(levels_fu))
  sp_sub <- sp_sub[order(sp_sub$eq5d, sp_sub$fu_ord), ]
  sp_sub$fu_ord <- NULL
  # lag n within each eq5d group
  sp_split <- split(sp_sub, sp_sub$eq5d)
  change_list <- lapply(sp_split, function(g) {
    g$n_prev <- c(NA_real_, head(g$n, -1))
    g$n_change <- g$n - g$n_prev
    g$freq <- g$n_change / g$n_prev
    g$n <- g$n_change
    g$n_prev <- NULL
    g$n_change <- NULL
    g
  })
  summary_problems_change <- do.call(rbind, change_list)
  summary_problems_change$level <- "Change in numbers reporting problems"
  rownames(summary_problems_change) <- NULL

  # summary: rankings
  if (add_summary_problems_change) {
    sr <- summary_problems_change[!is.na(summary_problems_change$freq),
                                  c("eq5d", "fu", "freq"), drop = FALSE]
    fu_grps <- unique(as.character(sr$fu))
    rank_list <- lapply(fu_grps, function(f) {
      g <- sr[as.character(sr$fu) == f, , drop = FALSE]
      g$n <- rank(g$freq)
      g
    })
    if (length(rank_list) > 0L) {
      summary_rank <- do.call(rbind, rank_list)
      summary_rank$freq <- NA_real_
      summary_rank$level <- "Rank of dimensions in terms of % changes"
      rownames(summary_rank) <- NULL
    } else {
      summary_rank <- NULL
    }
  } else {
    summary_rank <- NULL
  }

  # summary: missing data
  na_agg <- aggregate(
    cbind(n_miss = is.na(df_long$value), n_total = rep(1L, nrow(df_long))),
    by = list(eq5d = df_long$eq5d, fu = df_long$fu),
    FUN = sum)
  summary_na <- data.frame(
    eq5d = na_agg$eq5d,
    fu = na_agg$fu,
    n = na_agg$n_miss,
    freq = na_agg$n_miss / na_agg$n_total,
    level = "Missing data",
    stringsAsFactors = FALSE)

  # combine all summaries
  all_pieces <- list(summary_dim, summary_total, summary_problems,
                     if (add_summary_problems_change && any(!is.na(summary_problems_change$n))) summary_problems_change,
                     summary_rank, summary_na)
  all_pieces <- Filter(Negate(is.null), all_pieces)

  # ensure all pieces have the same columns: level, eq5d, fu, n, freq
  std_cols <- c("level", "eq5d", "fu", "n", "freq")
  all_pieces <- lapply(all_pieces, function(p) {
    p$eq5d <- as.character(p$eq5d)
    p$fu   <- as.character(p$fu)
    p[, std_cols, drop = FALSE]
  })
  combined <- do.call(rbind, all_pieces)

  # define row order
  row_order_levels <- c(
    sort(unique(summary_dim$level)),
    unique(summary_total$level),
    unique(summary_problems$level),
    if (add_summary_problems_change) unique(summary_problems_change$level),
    if (!is.null(summary_rank)) unique(summary_rank$level),
    unique(summary_na$level))

  # define column order: expand.grid(c("n","freq"), levels_fu, levels_eq5d)
  col_grid <- expand.grid(c("n", "freq"), as.character(levels_fu), levels_eq5d,
                          stringsAsFactors = FALSE)
  col_order <- apply(col_grid, 1, paste, collapse = "_")

  # build wide output: rows = levels, cols = n/freq per (fu, eq5d)
  all_levels <- row_order_levels
  retval <- data.frame(level = all_levels, stringsAsFactors = FALSE)

  for (cn in col_order) {
    parts_cn <- strsplit(cn, "_")[[1]]
    val_type <- parts_cn[1]                              # "n" or "freq"
    eq5d_val <- parts_cn[length(parts_cn)]               # e.g. "mo"
    fu_val   <- paste(parts_cn[2:(length(parts_cn)-1)], collapse = "_")  # fu (may contain _)

    sub <- combined[combined$eq5d == eq5d_val & combined$fu == fu_val, ,
                    drop = FALSE]
    vals <- setNames(sub[[val_type]], sub$level)
    retval[[cn]] <- vals[retval$level]
  }

  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' .pchctab: Changes in health according to the PCHC (Paretian Classification of Health Change)
#' 
#' @param df Data frame with the EQ-5D, grouping, id and follow-up columns
#' @param name_id Character string for the patient id column
#' @param name_groupvar Character string for the grouping column
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param add_noprobs if set to TRUE, level corresponding to "no problems" will be added to the table
#' @return Summary data frame
#' @examples
#' .pchctab(df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   name_fu = "time",
#'   levels_fu = c('Pre-op', 'Post-op')
#' )
#' @export
.pchctab <- function(df,
                  name_id,
                  name_groupvar,
                  names_eq5d = NULL,
                  name_fu = NULL, 
                  levels_fu = NULL,
                  add_noprobs = FALSE) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  # check existence of columns 
  names_all <- c(name_groupvar, name_id, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  
  
  # Enforce EQ-5D levels
  tmp <- as.matrix(df[, names_eq5d])
  tmp[,] <- as.integer(tmp)
  tmp[!tmp %in% 1:5] <- NA
  df[, names_eq5d] <- tmp
  rm(tmp)
  
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  names(df)[names(df) == name_id]       <- "id"
  names(df)[names(df) == name_groupvar] <- "groupvar"
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  # sort by id - groupvar - time
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]
  # check uniqueness of id-groupvar-fu combinations
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))
  
  
  ### analysis ###
  
  uids <- data.frame(id = unique(df$id))
  uids$groupvar <- df$groupvar[match(uids$id, df$id)]
  lvls <- structure(.Data = levels(df$fu), .Names = levels(df$fu))
  grps <- structure(.Data = unique(df$groupvar), .Names = unique(df$groupvar))
  grpr <- lapply(grps, function(grp) which(uids$groupvar == grp))
  
  idstates <- lapply(lvls, FUN = function(thisfu) {
    tmp <- df[df$fu == thisfu, c("id", names_eq5d)]
    as.matrix(tmp[match(uids$id, tmp$id),2:6])
  })
  
  progress <- as.data.frame(lapply(structure(.Data = 1:length(lvls), .Names = lvls), FUN = function(thisfu) {
    if(thisfu == 1) return(rep(NA, NROW(uids)))
    tmp <- sign(idstates[[thisfu]]-idstates[[thisfu-1]])
    factor(1*(rowSums(1*(tmp == 1))>0)-1*(rowSums(1*(tmp == -1))>0)+2*(rowSums(tmp == 0)==5), levels = c(2, -1, 1, 0), labels = c("No change", "Improve", "Worsen", "Mixed change"))   
    
  }))
  
  
  problems <- as.data.frame(lapply(idstates, function(idstate) {
    factor(2-1*(rowSums(idstate>1)>0), labels = c("Total with problems", "No problems"))
  }))
  
  colnames(progress) <- colnames(problems)<- lvls
  
  tmp <- do.call(rbind,lapply(grps, FUN = function(grp) {
    cbind(groupvar = grp, do.call(rbind, lapply(lvls[-1], function(lvl) {
      tmp <- cbind(fu = lvl, 
                   rbind(as.data.frame(table(progress[grpr[[grp]],lvl], useNA = 'always')),
                         if(add_noprobs) as.data.frame(table(problems[grpr[[grp]],lvl], useNA = 'no'))))
      colnames(tmp) <- c('fu', 'state', 'n')
      
      tmp$p <- c(tmp$n/sum(tmp$n[-which(is.na(tmp$state))]))
      tmp$p[which(is.na(tmp$state))] <- NA
      tmp <- rbind(tmp, data.frame(fu = lvl, state = 'Grand total', n = sum(tmp$n[1:4]), p = 1))
      tmp
    })))
  }))
  
  tmp$state <- factor(tmp$state, levels = c(levels(tmp$state), "Missing/NA"))
  tmp$state[is.na(tmp$state)] <- 'Missing/NA'
  
  
  # combine & tidy up: spread (groupvar, fu, n/p) into wide columns
  all_states <- levels(tmp$state)
  retval <- data.frame(state = all_states, stringsAsFactors = FALSE)

  # build column names in order: for each (groupvar, fu), add _n and _p columns
  for (grp in unique(tmp$groupvar)) {
    for (f in levels_fu[-1]) {
      sub <- tmp[tmp$groupvar == grp & tmp$fu == f, , drop = FALSE]
      vals_n <- setNames(as.numeric(sub$n), as.character(sub$state))
      vals_p <- setNames(as.numeric(sub$p), as.character(sub$state))
      cn <- paste(grp, f, "n", sep = "_")
      cp <- paste(grp, f, "p", sep = "_")
      retval[[cn]] <- ifelse(is.na(vals_n[retval$state]), 0, vals_n[retval$state])
      retval[[cp]] <- ifelse(is.na(vals_p[retval$state]), 0, vals_p[retval$state])
    }
  }

  rownames(retval) <- NULL
  return(retval)
}


#' Wrapper to determine Paretian Classification of Health Change
#' 
#' This internal function determines Paretian Classification of Health Change (PCHC) for each combination of the variables specified in the `group_by` argument. 
#' It is used in the code for eq5d_profile_pchc_table, eq5d_profile_pchc_with_no_problems_table, eq5d_profile_dimension_change_table, and the eq5d_profile_*_by_group_plot functions.
#' An EQ-5D health state is deemed to be `better` than another if it is better on at least one dimension and is no worse on any other dimension.
#' An EQ-5D health state is deemed to be `worse` than another if it is worse in at least one dimension and is no better in any other dimension.
#' @param df A data frame with EQ-5D states and follow-up variable. The dataset is assumed to be have been ordered correctly.
#' @param level_fu_1 Value of the first (i.e. earliest) follow-up. Would normally be defined as levels_fu[1].
#' @param add_noprobs Logical value indicating whether to include a separate classification for those without problems (default is FALSE)
#' @return A data frame with PCHC value for each combination of the grouping variables. 
#' If 'add_noprobs' is TRUE, a separate classification for those without problems is also included.
#' @examples
#' df <- data.frame(id = c(1, 1, 2, 2),
#'                  fu = c(1, 2, 1, 2),
#'                  mo = c(1, 1, 1, 1),
#'                  sc = c(1, 1, 5, 1),
#'                  ua = c(1, 1, 4, 3),
#'                  pd = c(1, 1, 1, 3),
#'                  ad = c(1, 1, 1, 1))
#' .pchc(df, level_fu_1 = 1, add_noprobs = TRUE)
#' @export

.pchc <- function(df, level_fu_1, add_noprobs = FALSE) {

  levels_eq5d <- c("mo", "sc", "ua", "pd", "ad")

  # initialise positive, negative & zero difference counts
  df$better <- 0L
  df$worse  <- 0L

  for (dom in levels_eq5d) {
    dom_diff <- paste0(dom, "_diff")

    # lag shift: previous row's value minus current (dplyr::lag equivalent)
    df[[dom_diff]] <- c(NA_real_, head(df[[dom]], -1)) - df[[dom]]
    # baseline rows: set diff to NA
    df[[dom_diff]][df$fu == level_fu_1] <- NA_real_

    # accumulate improvement/worsening counts (NA propagates for baseline rows)
    df$better <- df$better + (df[[dom_diff]] > 0)
    df$worse  <- df$worse  + (df[[dom_diff]] < 0)
  }

  # classify each row (NA when better/worse are NA, i.e. baseline rows)
  df$state <- ifelse(
    df$better == 0 & df$worse == 0, "No change",
    ifelse(df$better > 0 & df$worse == 0, "Improve",
      ifelse(df$worse > 0 & df$better == 0, "Worsen",
        ifelse(df$better > 0 & df$worse > 0, "Mixed change", NA_character_)
      )
    )
  )

  # separate classification for those without problems if required
  if (add_noprobs) {
    # no change & 11111 at second timepoint means 11111 at first timepoint
    noprobs <- df$mo == 1 & df$sc == 1 & df$ua == 1 & df$pd == 1 & df$ad == 1
    df$noprobs <- noprobs
    df$state_noprobs <- ifelse(!is.na(df$state) & df$state == "No change" & !is.na(noprobs) & noprobs,
                               "No problems", df$state)
  }

  return(df)
}

#' Wrapper to summarise a continuous variable by follow-up (FU) 
#' 
#' This function summarizes a continuous variable for each follow-up (FU) and calculates various statistics such as mean, standard deviation, median, mode, kurtosis, skewness, minimum, maximum, range, and number of observations. It also reports the total sample size and the number (and proportion) of missing values for each FU. 
#' The input `df` must contain an ordered FU variable and the continuous variable of interest. 
#' The name of the continuous variable must be specified using `name_v`. 
#' The wrapper is used in Table 3.1 (for VAS) or Table 4.2 (for EQ-5D utility)
#'
#' @param df A data frame containing the FU and continuous variable of interest. The dataset must contain an ordered `fu` variable.
#' @param name_v A character string with the name of the continuous variable in `df` to be summarised.
#' @return Data frame with one row for each statistic and one column for each FU. 
#' @examples
#' df <- data.frame(fu = c(1,1,2,2,3,3), 
#'                  vas = c(7,8,9,NA,7,6))
#' .summary_cts_by_fu(df, name_v = "vas")
#' @importFrom stats median quantile sd
#' @importFrom moments kurtosis
#' @export

.summary_cts_by_fu <- function(df, name_v) {

  ### prepare dataset ###

  names(df)[names(df) == name_v] <- "v"

  fu_levels <- if (is.factor(df$fu)) levels(df$fu) else sort(unique(df$fu))

  # summarise non-NA values per fu level
  stats_list <- lapply(fu_levels, function(f) {
    v <- df$v[df$fu == f & !is.na(df$v)]
    data.frame(
      fu = f,
      Mean = mean(v),
      `Standard error` = sd(v) / sqrt(length(v)),
      Median = median(v),
      Mode = .getmode(v),
      `Standard deviation` = sd(v),
      Kurtosis = kurtosis(v),
      Skewness = skewness(v),
      Minimum = min(v),
      Maximum = max(v),
      Range = max(v) - min(v),
      Observations = length(v),
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  })
  summary <- do.call(rbind, stats_list)

  # summarise total and NA values per fu level
  total_na_list <- lapply(fu_levels, function(f) {
    v <- df$v[df$fu == f]
    miss_n <- sum(is.na(v))
    tot <- length(v)
    data.frame(
      fu = f,
      `Missing (n)` = miss_n,
      `Total sample` = tot,
      `Missing (%)` = miss_n / tot,
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
  })
  summary_total_na <- do.call(rbind, total_na_list)

  # combine
  combined <- merge(summary, summary_total_na, by = "fu")

  # transpose: rows become stat names, columns become fu levels
  stat_cols <- setdiff(names(combined), "fu")
  mat <- as.matrix(combined[, stat_cols, drop = FALSE])
  rownames(mat) <- as.character(combined$fu)
  t_mat <- t(mat)

  retval <- as.data.frame(t_mat, stringsAsFactors = FALSE)
  for (cn in names(retval)) retval[[cn]] <- as.numeric(retval[[cn]])
  retval <- data.frame(name = rownames(retval), retval, check.names = FALSE,
                       stringsAsFactors = FALSE, row.names = NULL)

  return(retval)
}

#' Summary wrapper for Table 4.3
#' 
#' This internal function creates a summary of the data frame for Table 4.3. 
#' It groups the data by the variables specified in `group_by` and calculates various summary statistics.
#' 
#' @param df A data frame.
#' @param group_by A character vector of names of variables by which to group the data.
#' @return A data frame with the summary statistics.
#' @examples
#' df <- data.frame(group = c("A", "A", "B", "B"), 
#'                  utility = c(0.5, 0.7, 0.8, 0.9))
#' .summary_table_4_3(df, group_by = "group")
#' @export

.summary_table_4_3 <- function(df, group_by) {

  grp_key <- do.call(paste, c(df[group_by], list(sep = "\001")))
  parts <- split(df$utility, grp_key)

  result_list <- lapply(names(parts), function(k) {
    u <- parts[[k]]
    group_vals <- df[match(k, grp_key), group_by, drop = FALSE]
    row.names(group_vals) <- NULL
    cbind(group_vals, data.frame(
      Mean = mean(u, na.rm = TRUE),
      `Standard error` = sd(u, na.rm = TRUE) / sqrt(sum(!is.na(u))),
      Median = median(u, na.rm = TRUE),
      `25th` = quantile(u, probs = 0.25, na.rm = TRUE, names = FALSE),
      `75th` = quantile(u, probs = 0.75, na.rm = TRUE, names = FALSE),
      N = sum(!is.na(u)),
      Missing = sum(is.na(u)),
      check.names = FALSE,
      stringsAsFactors = FALSE
    ))
  })

  retval <- do.call(rbind, result_list)
  rownames(retval) <- NULL
  return(retval)
}

#' Summary wrapper for Table 4.4
#' 
#' This internal function creates a summary of the data frame for Table 4.4. 
#' It groups the data by the variables specified in `group_by` and calculates various summary statistics.
#' 
#' @param df A data frame.
#' @param group_by A character vector of names of variables by which to group the data.
#' @return A data frame with the summary statistics.
#' @examples
#' df <- data.frame(group = c("A", "A", "B", "B"), 
#'                  utility = c(0.5, 0.7, 0.8, 0.9))
#' .summary_table_4_4(df, group_by = "group")
#' @export

.summary_table_4_4 <- function(df, group_by) {

  grp_key <- do.call(paste, c(df[group_by], list(sep = "\001")))
  parts <- split(df$utility, grp_key)

  result_list <- lapply(names(parts), function(k) {
    u <- parts[[k]]
    group_vals <- df[match(k, grp_key), group_by, drop = FALSE]
    row.names(group_vals) <- NULL
    cbind(group_vals, data.frame(
      Mean = mean(u, na.rm = TRUE),
      `Standard error` = sd(u, na.rm = TRUE) / sqrt(sum(!is.na(u))),
      `25th Percentile` = quantile(u, probs = 0.25, na.rm = TRUE, names = FALSE),
      `50th Percentile (median)` = median(u, na.rm = TRUE),
      `75th Percentile` = quantile(u, probs = 0.75, na.rm = TRUE, names = FALSE),
      n = sum(!is.na(u)),
      Missing = sum(is.na(u)),
      check.names = FALSE,
      stringsAsFactors = FALSE
    ))
  })

  retval <- do.call(rbind, result_list)
  rownames(retval) <- NULL
  return(retval)
}

#' Wrapper to calculate summary mean with 95\% confidence interval
#' 
#' This internal function calculates summary mean and 95\% confidence interval of the utility variable, which can also be grouped.
#' The function is used in Figures 4.2-4.4.
#'
#' @param df A data frame containing a `utility` column.
#' @param group_by A character vector of column names to group by.
#' @return A data frame with the mean, lower bound, and upper bound of the 95% confidence interval of `utility` grouped by the `group_by` variables.
#' @examples
#' df <- data.frame(group = c("A", "A", "B", "B"), 
#'                  utility = c(0.5, 0.7, 0.8, 0.9))
#' .summary_mean_ci(df, group_by = "group")
#' @export
#'
.summary_mean_ci <- function(df, group_by) {

  df <- df[!is.na(df$utility), , drop = FALSE]

  if (is.null(group_by) || length(group_by) == 0L) {
    u <- df$utility
    m <- mean(u)
    se <- sd(u) / sqrt(length(u))
    retval <- data.frame(
      mean = m,
      ci_lb = m - 1.96 * se,
      ci_ub = m + 1.96 * se,
      stringsAsFactors = FALSE
    )
    return(retval)
  }

  grp_key <- do.call(paste, c(df[group_by], list(sep = "\001")))
  parts <- split(df$utility, grp_key)

  result_list <- lapply(names(parts), function(k) {
    u <- parts[[k]]
    group_vals <- df[match(k, grp_key), group_by, drop = FALSE]
    row.names(group_vals) <- NULL
    m <- mean(u)
    se <- sd(u) / sqrt(length(u))
    cbind(group_vals, data.frame(
      mean = m,
      ci_lb = m - 1.96 * se,
      ci_ub = m + 1.96 * se,
      stringsAsFactors = FALSE
    ))
  })

  retval <- do.call(rbind, result_list)
  rownames(retval) <- NULL
  return(retval)
}

#' Generate colours for PCHC figures
#'
#' This internal function generates a vector of colours based on the specified base colour. 
#' Currently only green and orange colours are implemented. 
#' The wrapper is used in Figures 2.2-2.4.
#'
#' @param col A character string specifying the base colour. Only "green" or "orange" is accepted.
#' @param n A positive integer specifying the number of colours to generate.
#' @return A vector of colours generated based on the specified base colour and number of colours.
#' @examples
#' # generate 10 colours for base colour "green"
#' .gen_colours("green", 10)
#' # generate 7 colours for base colour "orange"
#' .gen_colours("orange", 7)
#' @importFrom grDevices colorRampPalette
#' @export
#'
.gen_colours <- function(col, n) {
  retval <- if (col == "green")
    colorRampPalette(c("#99FF99", "#006600"))(n) else 
      if (col == "orange")
        colorRampPalette(c("#FFCC99", "#663300"))(n)
  
  return(retval)
}

#' Modify ggplot2 theme
#'
#' @param p ggplot2 plot
#' @return ggplot2 plot with modified theme
#' @export
.modify_ggplot_theme <- function(p) {
  # set ggplot2 theme
  p <- p + theme_bw() + theme(
    # remove vertical gridlines
    panel.grid.minor.x = element_blank(),
    panel.grid.major.x = element_blank(),
    # remove horisontal minor gridlines
    panel.grid.minor.y = element_blank(),
    # remove x-axis ticks
    axis.ticks.x = element_blank(),
    # centre plot title
    plot.title = element_text(hjust = 0.5),
    # remove legend title
    legend.title = element_blank(),
    # move legend to bottom
    legend.position = "bottom")

return(p)

}

#' Wrapper to generate Paretian Classification of Health Change plot by dimension
#'
#' This internal function plots Paretian Classification of Health Change (PCHC) by dimension. 
#' The input is a data frame containing the information to plot, and the plot will contain bars representing 
#' the proportion of the total data that falls into each dimension, stacked by covariate.
#' The wrapper is used in Figures 2.2-2.4.
#'
#' @param plot_data A data frame containing information to plot, with columns for name (the dimensions to plot), p (the proportion of the total data falling into each dimension), and fu (the follow-up).
#' @param ylab The label for the y-axis.
#' @param title The plot title.
#' @param cols A vector of colors to use for the bars.
#' @param text_rotate A logical indicating whether to rotate the text labels for the bars.
#' @return A ggplot object containing the PCHC plot.
#' @examples
#' df <- data.frame(
#'   name = rep(c("Dim1", "Dim2"), each = 2),
#'   p = c(0.6, 0.4, 0.7, 0.3),
#'   groupvar = rep(c("Group A", "Group B"), 2)
#' )
#' colors <- c("Group A" = "#1b9e77", "Group B" = "#d95f02")
#' .pchc_plot_by_dim(df, ylab = "Proportion", title = "Example Plot", cols = colors)
#' @export

.pchc_plot_by_dim <- function(plot_data, ylab, title, cols, text_rotate = FALSE) {
  
  p <- ggplot(plot_data, aes(x = .data$name, y = p, fill = .data$groupvar)) + 
    # bar chart
    geom_bar(stat = "identity", position = "dodge") + 
    # manipuilate x-axis
    scale_x_discrete(name = "") + 
    # manipulate y-axis
    scale_y_continuous(name = ylab,
                       expand = expansion(mult = c(0, 0.2)),
                       labels = scales::percent_format()) +
    # title
    ggtitle(title) +
    # manipulate legend
    scale_fill_manual(values = cols)
  
  # add percentages
  if (text_rotate) { 
    p <- p + geom_text(aes(label = scales::percent(p, accuracy = 0.1)), 
                       position = position_dodge(width = 0.9),
                       hjust = -0.1, angle = 90)
  } else {
   p <- p + geom_text(aes(label = scales::percent(p, accuracy = 0.1)), 
                      position = position_dodge(width = 0.9),
                      vjust = -0.5)
  }
  return(p)
}

#' eq5d_profile_level_summary: Frequency of levels by dimensions, cross-sectional
#'
#' @param df Data frame with the EQ-5D and follow-up columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @return Summary data frame.
#' @examples
#' eq5d_profile_level_summary(
#'  df = example_data[example_data$time == "Pre-op",],
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'  eq5d_version = "3L"
#' )
#' @export

eq5d_profile_level_summary <- function(df, 
                      names_eq5d = NULL,
                      eq5d_version = NULL) {
  do.call(.freqtab, list(
    df = df,
    names_eq5d = names_eq5d,
    eq5d_version = eq5d_version,
    add_summary_problems_change = FALSE
  ))
}


#' eq5d_profile_level_summary_by_group: Frequency of levels by dimensions, separated by category
#' 
#' @param df Data frame with the EQ-5D and follow-up columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_cat Character string for the category  column. If NULL, no grouping is used, and the table reports for the total population, i.e. equal to table 1.1.1.
#' @param levels_cat Character vector containing the order of the values in the category column, if the wish is to have these presented in a particular order. 
#' If NULL (default value), unless the variable is a factor, the levels will be ordered in the order of appearance in df.
#' @param eq5d_version Version of the EQ-5D instrument
#' @return Summary data frame.
#' @examples
#' eq5d_profile_level_summary_by_group(
#'  df = example_data[example_data$time == "Pre-op",],
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'  name_cat = "procedure",
#'  levels_cat = c("Hip Replacement", "Knee Replacement"),
#'  eq5d_version = "3L"
#' )
#' @export
#' 
eq5d_profile_level_summary_by_group <- function(df, 
                       names_eq5d = NULL,
                       name_cat = NULL,
                       levels_cat = NULL,
                       eq5d_version = NULL) {
  tmp <- .freqtab(df, names_eq5d, name_cat, levels_cat, eq5d_version)
  tmp[-(NROW(tmp)-(1:2)),]
}

#' eq5d_profile_change_summary: Frequency of levels by dimensions, by follow-up
#' 
#' @param df Data frame with the EQ-5D and follow-up columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column. If NULL, the function will check if there is a column named "follow-up" or "fu", in which case the first of those will be used.
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param eq5d_version Version of the EQ-5D instrument
#' @return Summary data frame.
#' @examples
#' eq5d_profile_change_summary(
#'   df = example_data,
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op" , "Post-op"),
#'   eq5d_version = "3L"
#' )
#' @export

eq5d_profile_change_summary <- function(df, 
                       names_eq5d = NULL,
                       name_fu = NULL,
                       levels_fu = NULL,
                       eq5d_version = NULL) {
  mc <- as.list(match.call()[-1])
  
  if(is.null(name_fu)){
    if("follow-up" %in% colnames(df)) mc$name_fu <- "follow_up"
    if("fu" %in% colnames(df)) mc$name_fu <- "fu"
  }
  do.call(.freqtab, mc)
}

#' eq5d_profile_top_states: Prevalence of the 10 most frequently observed self-reported health states
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param n Number of most frequently observed states to display (default 10)
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_top_states(
#'   df = example_data[example_data$time == "Pre-op",],
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   eq5d_version = "3L",
#'   n = 10
#' )

eq5d_profile_top_states <- function(df, 
                      names_eq5d = NULL,
                      eq5d_version = NULL,
                      n = 10) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, 
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- names_eq5d
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d, eq5d_version = eq5d_version, add_state = TRUE)
  df <- df[, "state", drop = FALSE]

  ### analysis ####

  # all states (non-NA only), aggregated and sorted
  df_cc <- df[!is.na(df$state), , drop = FALSE]
  states_agg <- aggregate(rep(1L, nrow(df_cc)), by = list(state = df_cc$state), FUN = sum)
  names(states_agg)[2] <- "n"
  states_agg <- states_agg[order(-states_agg$n), , drop = FALSE]
  states_agg$p <- states_agg$n / sum(states_agg$n)
  states_agg$cum_p <- cumsum(states_agg$p)
  rownames(states_agg) <- NULL

  # most frequent n non-NA states
  states_top <- states_agg[seq_len(min(n, nrow(states_agg))), , drop = FALSE]

  # worst state
  state_worst <- if (eq5d_version == "5L") "55555" else "33333"

  if (!state_worst %in% states_top[, 1]) {
    idx <- match(state_worst, states_agg$state)
    if (!is.na(idx)) {
      states_worst <- states_agg[idx, , drop = FALSE]
      states_worst$cum_p <- 1
    } else {
      states_worst <- states_agg[0, , drop = FALSE]
    }
    states_worst <- states_worst[c(1, 1), , drop = FALSE]
    states_worst[1, ] <- NA
    states_worst[1, 1] <- '...'
  } else {
    states_worst <- states_agg[0, , drop = FALSE]
  }

  # missing data
  n_miss <- sum(is.na(df$state))
  state_na <- data.frame(state = "Missing", n = n_miss,
                         p = n_miss / nrow(df), cum_p = NA_real_,
                         stringsAsFactors = FALSE)

  # combine and tidy up
  retval <- rbind(states_top, states_worst, state_na)
  names(retval) <- c("Health state", "Frequency", "Percentage", "Cumulative percentage")
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_pchc_table: Changes in health according to the PCHC (Paretian Classification of Health Change)
#' 
#' @param df Data frame with the EQ-5D, grouping, id and follow-up columns
#' @param name_id Character string for the patient id column
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column.
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_pchc_table(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op" , "Post-op")
#' )

eq5d_profile_pchc_table <- function(df,
                      name_id,
                      name_groupvar = NULL,
                      names_eq5d = NULL,
                      name_fu = NULL,
                      levels_fu = NULL) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  # check existence of columns 
  names_all <- c(name_groupvar, name_id, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  # sort by id - groupvar - time

  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]
  # check uniqueness of id-groupvar-fu combinations
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))

  ### analysis ###

  # calculate change
  df <- .pchc(df = df, level_fu_1 = levels_fu[1])
  df <- df[!is.na(df$state), , drop = FALSE]
  df$state <- factor(df$state, levels = c("No change", "Improve", "Worsen", "Mixed change"))

  # summarise by groupvar, fu & state
  summary_dim <- aggregate(rep(1L, nrow(df)),
                           by = list(groupvar = df$groupvar, fu = df$fu, state = df$state),
                           FUN = sum)
  names(summary_dim)[names(summary_dim) == "x"] <- "n"
  key_gf <- paste(summary_dim$groupvar, summary_dim$fu, sep = "\001")
  summary_dim$p <- summary_dim$n / tapply(summary_dim$n, key_gf, sum)[key_gf]

  # summarise totals
  agg_tot <- aggregate(cbind(n = summary_dim$n, p = summary_dim$p),
                       by = list(groupvar = summary_dim$groupvar, fu = summary_dim$fu),
                       FUN = sum)
  agg_tot$state <- "Grand Total"
  summary_total <- agg_tot

  # combine & tidy up: pivot n/p to wide columns {groupvar}_{fu}_n / {groupvar}_{fu}_p
  combined <- rbind(summary_dim, summary_total)
  state_order <- c(levels(df$state), "Grand Total")
  all_states <- state_order[state_order %in% unique(as.character(combined$state))]
  retval <- data.frame(state = all_states, stringsAsFactors = FALSE)
  grp_vals <- unique(as.character(combined$groupvar))
  fu_vals  <- as.character(levels_fu[-1])
  for (gv in grp_vals) {
    for (f in fu_vals) {
      sub <- combined[combined$groupvar == gv & as.character(combined$fu) == f, , drop = FALSE]
      vals_n <- setNames(sub$n, as.character(sub$state))
      vals_p <- setNames(sub$p, as.character(sub$state))
      retval[[paste(gv, f, "n", sep = "_")]] <- ifelse(is.na(vals_n[retval$state]), 0, vals_n[retval$state])
      retval[[paste(gv, f, "p", sep = "_")]] <- ifelse(is.na(vals_p[retval$state]), 0, vals_p[retval$state])
    }
  }
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_pchc_with_no_problems_table: Changes in health according to the PCHC, taking account of those with no problems
#' 
#' @param df Data frame with the EQ-5D, grouping, id and follow-up columns
#' @param name_id Character string for the patient id column
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column.
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_pchc_with_no_problems_table(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op" , "Post-op")
#' )

eq5d_profile_pchc_with_no_problems_table <- function(df,
                      name_id,
                      name_groupvar = NULL,
                      names_eq5d = NULL,
                      name_fu = NULL,
                      levels_fu = NULL) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  # check existence of columns 
  names_all <- c(name_groupvar, name_id, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  # sort by id - groupvar - time
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]
  # check uniqueness of id-groupvar-fu combinations
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))
  
  ### analysis ###
  
  # calculate change
  df <- .pchc(df = df, level_fu_1 = levels_fu[1], add_noprobs = TRUE)
  df <- df[!is.na(df$state), , drop = FALSE]

  # separate out those with problems & calculate percentages
  df_status <- df
  df_status$state_noprobs <- ifelse(df_status$state_noprobs == "No problems",
                                    "No problems", "Total with problems")
  summary_by_probs_status <- aggregate(rep(1L, nrow(df_status)),
    by = list(groupvar = df_status$groupvar, fu = df_status$fu,
              state_noprobs = df_status$state_noprobs), FUN = sum)
  names(summary_by_probs_status)[names(summary_by_probs_status) == "x"] <- "n"
  key_bp <- paste(summary_by_probs_status$groupvar, summary_by_probs_status$fu, sep = "\001")
  summary_by_probs_status$p <- summary_by_probs_status$n /
    tapply(summary_by_probs_status$n, key_bp, sum)[key_bp]

  # summarise classes within those with problems
  df_probs <- df[df$state_noprobs != "No problems", , drop = FALSE]
  summary_with_probs <- aggregate(rep(1L, nrow(df_probs)),
    by = list(groupvar = df_probs$groupvar, fu = df_probs$fu,
              state_noprobs = df_probs$state_noprobs), FUN = sum)
  names(summary_with_probs)[names(summary_with_probs) == "x"] <- "n"
  key_wp <- paste(summary_with_probs$groupvar, summary_with_probs$fu, sep = "\001")
  summary_with_probs$p <- summary_with_probs$n /
    tapply(summary_with_probs$n, key_wp, sum)[key_wp]

  # combine & tidy up
  combined <- rbind(summary_with_probs, summary_by_probs_status)
  state_order <- c("No change", "Improve", "Worsen", "Mixed change",
                   "Total with problems", "No problems")
  combined$state_noprobs <- factor(combined$state_noprobs, levels = state_order)
  combined <- combined[order(combined$state_noprobs), , drop = FALSE]
  all_states <- levels(combined$state_noprobs)[levels(combined$state_noprobs) %in%
                                                unique(as.character(combined$state_noprobs))]
  retval <- data.frame(state_noprobs = all_states, stringsAsFactors = FALSE)
  grp_vals <- unique(as.character(combined$groupvar))
  fu_vals  <- as.character(levels_fu[-1])
  for (gv in grp_vals) {
    for (f in fu_vals) {
      sub <- combined[combined$groupvar == gv & as.character(combined$fu) == f, , drop = FALSE]
      vals_n <- setNames(sub$n, as.character(sub$state_noprobs))
      vals_p <- setNames(sub$p, as.character(sub$state_noprobs))
      retval[[paste(gv, f, "n", sep = "_")]] <- ifelse(is.na(vals_n[retval$state_noprobs]), 0, vals_n[retval$state_noprobs])
      retval[[paste(gv, f, "p", sep = "_")]] <- ifelse(is.na(vals_p[retval$state_noprobs]), 0, vals_p[retval$state_noprobs])
    }
  }
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_dimension_change_table: Changes in levels in each dimension, percentages of total and of type of change
#' 
#' @param df Data frame with the EQ-5D, id and follow-up columns
#' @param name_id Character string for the patient id column
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_dimension_change_table(
#'   df = example_data,
#'   name_id = "id",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op" , "Post-op")
#' )

eq5d_profile_dimension_change_table <- function(df, 
                      name_id,
                      names_eq5d = NULL,
                      name_fu = NULL,
                      levels_fu = NULL) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  # check existence of columns 
  names_all <- c(name_id, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  names(df)[names(df) == name_id] <- "id"
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  # sort by id - time

  df <- df[order(df$id, df$fu), , drop = FALSE]
  # check uniqueness of id-fu combinations
  .check_uniqueness(df, group_by = c("id", "fu"))
  
  ### analysis ###
  
  # calculate change
  levels_eq5d <- c("mo", "sc", "ua", "pd", "ad")
  level_fu_1 <- levels_fu[1]

  # reshape to long format
  df_long <- do.call(rbind, lapply(levels_eq5d, function(d) {
    data.frame(domain = d, id = df$id, fu = df$fu, value = df[[d]],
               stringsAsFactors = FALSE)
  }))
  # sort by domain, id, fu
  df_long <- df_long[order(df_long$domain, df_long$id, df_long$fu), , drop = FALSE]
  rownames(df_long) <- NULL

  # compute lagged difference (dplyr::lag equivalent)
  prev_val <- c(NA_real_, head(df_long$value, -1))
  df_long$diff <- prev_val - df_long$value
  df_long$level_change <- ifelse(is.na(prev_val), NA_character_,
                                 paste0(prev_val, "-", df_long$value))
  # set baseline to NA
  df_long$diff[as.character(df_long$fu) == as.character(level_fu_1)] <- NA_real_
  df_long$level_change[as.character(df_long$fu) == as.character(level_fu_1)] <- NA_character_

  # remove NAs (baseline rows) and classify difference
  df_diff <- df_long[!is.na(df_long$diff), , drop = FALSE]
  df_diff$diff <- ifelse(df_diff$diff > 0, "Better",
                   ifelse(df_diff$diff < 0, "Worse", "No change"))
  df_diff$domain <- factor(df_diff$domain, levels = levels_eq5d)
  df_diff <- df_diff[order(df_diff$domain), , drop = FALSE]

  # % total: n per (domain, diff, level_change), p = n / sum(n) within domain
  p_agg <- aggregate(rep(1L, nrow(df_diff)),
                     by = list(domain = df_diff$domain, diff = df_diff$diff,
                                level_change = df_diff$level_change), FUN = sum)
  names(p_agg)[names(p_agg) == "x"] <- "n"
  p_total <- p_agg
  p_total$p <- p_total$n / tapply(p_total$n, p_total$domain, sum)[p_total$domain]
  p_total$type <- "% Total"

  # % type: p = n / sum(n) within (domain, diff)
  p_type <- p_agg
  key_dt <- paste(p_type$domain, p_type$diff, sep = "\001")
  p_type$p <- p_type$n / tapply(p_type$n, key_dt, sum)[key_dt]
  p_type$type <- "% Type"

  # combine and build wide output
  combined_p <- rbind(p_total[, c("domain", "type", "diff", "level_change", "p")],
                      p_type[,  c("domain", "type", "diff", "level_change", "p")])

  # ordered (diff, level_change) row keys
  all_rows <- unique(combined_p[, c("diff", "level_change")])
  all_rows$diff <- factor(all_rows$diff, levels = c("No change", "Better", "Worse"))
  all_rows <- all_rows[order(all_rows$diff, all_rows$level_change), , drop = FALSE]
  rownames(all_rows) <- NULL
  retval <- all_rows

  # column order: {domain}_{type} as produced by expand_grid(levels_eq5d, c("% Total","% Type"))
  col_names <- as.vector(outer(levels_eq5d, c("% Total", "% Type"), paste, sep = "_"))
  for (cn in col_names) {
    sep_idx <- regexpr("_% ", cn)
    dom  <- substr(cn, 1, sep_idx - 1)
    type <- substr(cn, sep_idx + 1, nchar(cn))
    sub  <- combined_p[as.character(combined_p$domain) == dom & combined_p$type == type, , drop = FALSE]
    vals <- setNames(sub$p, paste(sub$diff, sub$level_change, sep = "\001"))
    row_key <- paste(as.character(retval$diff), retval$level_change, sep = "\001")
    retval[[cn]] <- ifelse(is.na(vals[row_key]), 0, vals[row_key])
  }
  retval <- retval[, c("diff", "level_change", col_names), drop = FALSE]
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_lss_utility_summary: Summary statistics for the EQ-5D values by all the different LSSs (Level Sum Scores)
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary data frame
#' @export
#' @examples
#' df <- data.frame(make_all_EQ_states(version = "5L"))
#' eq5d_profile_lss_utility_summary(
#'   df, 
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'   eq5d_version = "3L", 
#'   country = "US"
#' )

eq5d_profile_lss_utility_summary <- function(df, 
                      names_eq5d = NULL,
                      eq5d_version = NULL,
                      country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_lss = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("lss", "utility"), drop = FALSE]

  ### analysis ###

  # summarise non-NA values
  df_cc <- df[!is.na(df$lss), , drop = FALSE]
  grp_key <- df_cc$lss
  parts <- split(df_cc$utility, grp_key)
  summary_list <- lapply(sort(unique(grp_key)), function(k) {
    u <- parts[[as.character(k)]]
    data.frame(lss = k, Number = length(u), Mean = mean(u),
               `Standard Deviation` = sd(u), Median = median(u),
               Minimum = min(u), Maximum = max(u), Range = max(u) - min(u),
               check.names = FALSE, stringsAsFactors = FALSE)
  })
  summary <- do.call(rbind, summary_list)

  # missing data
  summary_na <- data.frame(lss = "Missing", Number = sum(is.na(df$lss)),
                            Mean = NA_real_, `Standard Deviation` = NA_real_,
                            Median = NA_real_, Minimum = NA_real_,
                            Maximum = NA_real_, Range = NA_real_,
                            check.names = FALSE, stringsAsFactors = FALSE)

  # combine and tidy up
  summary$lss <- as.character(summary$lss)
  retval <- rbind(summary, summary_na)
  names(retval)[names(retval) == "lss"] <- "LSS"
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_lfs_distribution: Distribution of the EQ-5D states by LFS (Level Frequency Score)
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_lfs_distribution(
#'   example_data, 
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'   eq5d_version = "3L"
#' )

eq5d_profile_lfs_distribution <- function(df, 
                      names_eq5d = NULL, 
                      eq5d_version = NULL){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_lfs = TRUE, eq5d_version = eq5d_version)
  df <- df[, "lfs", drop = FALSE]

  ### analysis ###

  # summarise non-NA values
  df_cc <- df[!is.na(df$lfs), , drop = FALSE]
  counts <- aggregate(rep(1L, nrow(df_cc)), by = list(lfs = df_cc$lfs), FUN = sum)
  names(counts)[2] <- "n"
  counts <- counts[order(-counts$n), , drop = FALSE]
  counts$p <- counts$n / sum(counts$n)
  counts$cum_p <- cumsum(counts$p)
  names(counts) <- c("LFS", "Frequency", "%", "Cum (%)")

  # missing data
  summary_na <- data.frame(LFS = "Missing", Frequency = sum(is.na(df$lfs)),
                            `%` = NA_real_, `Cum (%)` = NA_real_,
                            check.names = FALSE, stringsAsFactors = FALSE)

  # combine
  retval <- rbind(counts, summary_na)
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_lfs_mean_utility: Number of observations in the LFS (Level Frequency Score) according to the EQ-5D values
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_lfs_mean_utility(
#'   example_data, 
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'   eq5d_version = "3L",
#'   country = "UK"
#' )

eq5d_profile_lfs_mean_utility <- function(df, 
                      names_eq5d = NULL,
                      eq5d_version = NULL,
                      country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_lfs = TRUE, eq5d_version = eq5d_version,
                   add_utility = TRUE, country = country)
  df <- df[, c("lfs", "utility"), drop = FALSE]

  ### analysis ###

  # exclude NAs and count per (utility, lfs) combination
  df_cc <- df[!is.na(df$lfs), , drop = FALSE]
  counts <- aggregate(rep(1L, nrow(df_cc)),
                      by = list(utility = df_cc$utility, lfs = df_cc$lfs), FUN = sum)
  names(counts)[3] <- "n"

  # all lfs levels (sorted)
  all_lfs <- sort(unique(as.character(df_cc$lfs)))
  all_utils <- sort(unique(df_cc$utility))

  # build wide matrix: rows = utility, cols = lfs
  retval <- data.frame(utility = all_utils, stringsAsFactors = FALSE)
  for (lf in all_lfs) {
    sub <- counts[as.character(counts$lfs) == lf, , drop = FALSE]
    vals <- setNames(sub$n, sub$utility)
    retval[[lf]] <- ifelse(is.na(vals[as.character(retval$utility)]), 0L,
                           vals[as.character(retval$utility)])
  }
  # add row totals
  retval$Total <- rowSums(retval[, all_lfs, drop = FALSE])
  # tidy up
  names(retval)[names(retval) == "utility"] <- "EQ-5D Value"
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_lfs_utility_summary: Summary statistics of EQ-5D values by LFS (Level Frequency Score)
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_profile_lfs_utility_summary(
#'   example_data, 
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'   eq5d_version = "3L",
#'   country = "UK"
#' )

eq5d_profile_lfs_utility_summary <- function(df, 
                       names_eq5d = NULL,
                       eq5d_version = NULL,
                       country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d, 
                   add_state = TRUE, 
                   add_lfs = TRUE, eq5d_version = eq5d_version,
                   add_utility = TRUE, country = country) 
  
  ### analysis ###
  
  # summarise non-NA values
  df_cc <- df[!is.na(df$utility), , drop = FALSE]
  parts <- split(df_cc$utility, df_cc$lfs)
  result_list <- lapply(sort(names(parts)), function(k) {
    u <- parts[[k]]
    data.frame(LFS = k, Frequency = length(u), Mean = mean(u), SD = sd(u),
               Median = median(u), Minimum = min(u), Maximum = max(u),
               Range = max(u) - min(u), stringsAsFactors = FALSE)
  })
  retval <- do.call(rbind, result_list)
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_profile_density_curve: Generate a Health State Density Curve (HSDC) for EQ-5D Data
#'
#' This function calculates and plots the Health State Density Curve (HSDC) for a given
#' EQ-5D dataset. It concatenates dimension values to form health state profiles, filters
#' out invalid states based on the specified EQ-5D version, then computes the cumulative
#' distribution of profiles (profiles vs. observations). A diagonal reference line
#' indicates a perfectly even distribution. The function also calculates the Health State
#' Density Index (HSDI), representing how sharply the observed distribution deviates from
#' the diagonal.
#'
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @return A list containing:
#'   \item{plot_data}{A data frame with the cumulative distribution of profiles}
#'   \item{p}{A ggplot2 object showing the Health State Density Index}
#' @export
#' @examples
#' figure <- eq5d_profile_density_curve(
#'             df = example_data, 
#'             names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'             eq5d_version = "3L"
#'           )
#' figure$plot_data
#' figure$p
#' @importFrom utils head

eq5d_profile_density_curve <- function(df, names_eq5d, eq5d_version) {
  
  # Retrieve validated names & version from helper function
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d   <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  
  # Check existence of columns
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in the data frame. Stopping.")
  }
  
  # Subset to relevant columns
  df <- df[, names_all, drop = FALSE]

  # Prepare data (adds 'state' column via .prep_eq5d)
  df <- .prep_eq5d(
    df           = df,
    names        = names_eq5d,
    add_state    = TRUE,
    add_lfs      = FALSE,
    eq5d_version = eq5d_version
  )

  # Check for NA in 'state', show warning and filter
  n_na <- sum(is.na(df$state))
  if (n_na > 0) {
    warning(n_na, " excluded observations with invalid EQ-5D profiles.")
    df <- df[!is.na(df$state), , drop = FALSE]
  }

  # Calculate frequencies & cumulative proportions
  freq_agg <- aggregate(rep(1L, nrow(df)), by = list(state = df$state), FUN = sum)
  names(freq_agg)[2] <- "Frequency"
  freq_agg <- freq_agg[order(-freq_agg$Frequency), , drop = FALSE]
  rownames(freq_agg) <- NULL
  freq_agg$CumPropObservations <- cumsum(freq_agg$Frequency) / sum(freq_agg$Frequency)
  freq_agg$CumPropStates       <- seq_len(nrow(freq_agg)) / nrow(freq_agg)
  profile_freq <- freq_agg
  
  # Compute the Health State Density Index (HSDI)
  hsdi_area <- sum(
    diff(c(0, profile_freq$CumPropObservations)) * 
      (head(c(0, profile_freq$CumPropStates), -1) + profile_freq$CumPropStates) / 2
  )
  hsdi <- round(2 * hsdi_area, 3)
  
  # Plot Health State Density Curve
  p <- ggplot2::ggplot(profile_freq, ggplot2::aes(
    x = .data$CumPropObservations, 
    y = .data$CumPropStates
  )) +
    ggplot2::geom_line(color = "blue", linewidth = 1.2) +
    ggplot2::geom_abline(
      intercept = 0, 
      slope = 1, 
      linetype = "solid", 
      linewidth = 1.2, 
      color = "orange"
    ) +
    ggplot2::labs(
      title    = paste0("Health State Density Curve (HSDC)"),
      subtitle = paste0("HSDI = ", hsdi),
      x        = "Cumulative proportion of observations",
      y        = "Cumulative proportion of profiles"
    ) +
    # Force the aspect ratio to 1:1
    ggplot2::coord_fixed(ratio = 1) +
    # Limit x,y range from 0 to 1
    ggplot2::scale_x_continuous(limits = c(0, 1), expand = c(0, 0)) +
    ggplot2::scale_y_continuous(limits = c(0, 1), expand = c(0, 0)) +
    ggplot2::theme_minimal()
  
  return(list(plot_data = profile_freq, p = p))
}

#' eq5d_vas_summary: EQ VAS Score by timepoints
#' 
#' @param df Data frame with the VAS and the follow-up columns
#' @param name_vas Character string for the VAS column
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_vas_summary(
#'   example_data,
#'   name_vas = 'vas', 
#'   name_fu = 'time', 
#'   levels_fu = c('Pre-op', 'Post-op')
#' )

eq5d_vas_summary <- function(df, 
                      name_vas = NULL,
                      name_fu = NULL,
                      levels_fu = NULL){
  
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, name_fu = name_fu, levels_fu = levels_fu, name_vas = name_vas)
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  name_vas <- temp$name_vas
  # check existence of columns 
  names_all <- c(name_vas, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_vas(df = df, name = name_vas)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  
  ### analysis ###
  
  # values calculated in the .summary_cts_by_fu wrapper
  retval <- .summary_cts_by_fu(df = df, name_v = "vas")
  
  # return value
  return(retval)
}

#' eq5d_vas_distribution_table: EQ VAS Scores frequency of mid-points
#' 
#' @param df Data frame with the VAS column
#' @param name_vas Character string for the VAS column
#' @param add_na_total Logical, whether to add summary of the missing, and across the Total, data
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_vas_distribution_table(
#'   example_data,
#'   name_vas = 'vas', 
#'   add_na_total =  TRUE
#'   )

eq5d_vas_distribution_table <- function(df, 
                      name_vas = NULL, 
                      add_na_total = TRUE){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(name_vas = name_vas)
  name_vas <- temp$name_vas
  # check existence of columns 
  names_all <- c(name_vas)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_vas(df = df, name = name_vas)
  
  ### analysis ###
  
  # define ranges and midpoints
  l_vals <- c(0:2, seq(from = 3, to = 93, by = 5), 98:100)
  u_vals <- c(0:2, seq(from = 7, to = 97, by = 5), 98:100)
  mid_vals <- c(0:2, seq(from = 5, to = 95, by = 5), 98:100)
  retval <- data.frame(l = l_vals, u = u_vals, Midpoint = mid_vals, Frequency = NA_integer_)

  # populate the table
  for (i in seq_len(nrow(retval))) {
    retval[i, "Frequency"] <- sum(!is.na(df$vas) & df$vas >= retval$l[i] & df$vas <= retval$u[i])
  }

  # add range column and tidy up
  retval$Range <- ifelse(retval$l == retval$u, as.character(retval$l),
                         paste(retval$l, retval$u, sep = "-"))
  retval <- retval[, c("Range", "Midpoint", "Frequency")]

  # add totals & missing data if needed
  if (add_na_total) {
    n_total <- nrow(df)
    n_na    <- sum(is.na(df$vas))
    retval  <- rbind(retval,
                     data.frame(Range = c("Total observed", "Missing", "Total sample"),
                                Midpoint = rep(NA_real_, 3),
                                Frequency = c(n_total - n_na, n_na, n_total)))
  }
  
  # return value
  return(retval)
}

#' eq5d_utility_summary: EQ-5D values: by timepoints
#' 
#' @param df Data frame with the EQ-5D and follow-up columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_utility_summary(
#'   example_data,
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'   name_fu = "time",
#'   levels_fu = c('Pre-op', 'Post-op'),
#'   eq5d_version = "3L",
#'   country = "UK"
#' )

eq5d_utility_summary <- function(df, 
                      names_eq5d = NULL,
                      name_fu = NULL,
                      levels_fu = NULL,
                      eq5d_version = NULL,
                      country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  df <- df[, c("fu", "utility"), drop = FALSE]

  ### analysis ###

  # summarise
  retval <- .summary_cts_by_fu(df = df, name_v = "utility")
  
  # return value
  return(retval)
}

#' eq5d_utility_summary_by_group:EQ-5D values: by groupvar
#' 
#' @param df Data frame with the EQ-5D, follow-up and grouping columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country.
#' @return Summary data frame
#' @export
#' @examples
#' eq5d_utility_summary_by_group(
#'   example_data,
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_groupvar = "procedure",
#'   eq5d_version = "3L",
#'   country = "UK"
#' )

eq5d_utility_summary_by_group <- function(df,
                      names_eq5d = NULL,
                      name_groupvar = NULL,
                      eq5d_version = NULL,
                      country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_groupvar)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("groupvar", "utility"), drop = FALSE]

  ### analysis ###

  # summary by category
  summary_by <- .summary_table_4_3(df = df, group_by = "groupvar")
  # summary_total (all groups combined)
  u_all <- df$utility
  summary_total <- data.frame(
    groupvar = "All groups",
    Mean = mean(u_all, na.rm = TRUE),
    `Standard error` = sd(u_all, na.rm = TRUE) / sqrt(sum(!is.na(u_all))),
    Median = median(u_all, na.rm = TRUE),
    `25th` = quantile(u_all, probs = 0.25, na.rm = TRUE, names = FALSE),
    `75th` = quantile(u_all, probs = 0.75, na.rm = TRUE, names = FALSE),
    N = sum(!is.na(u_all)),
    Missing = sum(is.na(u_all)),
    check.names = FALSE, stringsAsFactors = FALSE
  )

  # combine: pivot longer then wider (rows = stat names, cols = groups)
  combined <- rbind(summary_by, summary_total)
  stat_cols <- setdiff(names(combined), "groupvar")
  grp_vals  <- unique(combined$groupvar)
  retval <- data.frame(name = stat_cols, stringsAsFactors = FALSE)
  for (gv in grp_vals) {
    sub <- combined[combined$groupvar == gv, stat_cols, drop = FALSE]
    retval[[gv]] <- as.numeric(sub[1, ])
  }
  rownames(retval) <- NULL

  # return value
  return(retval)
}

#' eq5d_utility_norms_comparison:EQ-5D values: by age and groupvar
#' 
#' @param df Data frame with the EQ-5D, age, follow-up and grouping columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column.
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param name_age Character string for the age column
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary data frame
#' @export
#' @examples
#' example_data$ageband <- factor(
#'   example_data$ageband,
#'   levels = c("20 to 29", "30 to 39", "40 to 49", "50 to 59", "60 to 69", "70 to 79", "80 to 89")
#' )
#' example_data <- example_data[example_data$gender %in% c("Male", "Female"),]
#' eq5d_utility_norms_comparison(
#'   example_data,
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c('Pre-op', 'Post-op'),
#'   name_groupvar = "gender",
#'   name_age = "ageband",
#'   eq5d_version = "3L",
#'   country = "UK"
#' )

eq5d_utility_norms_comparison <- function(df,
                      names_eq5d = NULL,
                      name_fu = NULL,
                      levels_fu = NULL,
                      name_groupvar = NULL,
                      name_age,
                      eq5d_version = NULL,
                      country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df,
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_fu, name_groupvar, name_age)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }
  names(df)[names(df) == name_age]      <- "age"
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  df <- df[, c("groupvar", "age", "fu", "utility"), drop = FALSE]

  # split age into categories
  if (is.numeric(df$age)) {
    age_breaks <- c(0, 18, seq(from = 25, to = 75, by = 10), Inf)
    df$age_cat <- cut(df$age, breaks = age_breaks, right = FALSE)
    df$age_cat <- factor(df$age_cat,
      levels = c("[0,18)", "[18,25)", "[25,35)", "[35,45)", "[45,55)", "[55,65)", "[65,75)", "[75,Inf)"),
      labels = c("0-17", "18-24", "25-34", "35-44", "45-54", "55-64", "65-74", "75+"))
    df$age <- NULL
  } else if (is.factor(df$age)) {
    names(df)[names(df) == "age"] <- "age_cat"
  } else {
    stop("The 'age' column must be either numeric or factor. Stopping.")
  }

  ### analysis ###

  # helper for overall summary without grouping
  .total_summary_4_4 <- function(u) {
    data.frame(
      Mean = mean(u, na.rm = TRUE),
      `Standard error` = sd(u, na.rm = TRUE) / sqrt(sum(!is.na(u))),
      `25th Percentile` = quantile(u, probs = 0.25, na.rm = TRUE, names = FALSE),
      `50th Percentile (median)` = median(u, na.rm = TRUE),
      `75th Percentile` = quantile(u, probs = 0.75, na.rm = TRUE, names = FALSE),
      n = sum(!is.na(u)),
      Missing = sum(is.na(u)),
      check.names = FALSE, stringsAsFactors = FALSE
    )
  }

  # summary by category
  summary_by <- .summary_table_4_4(df = df, group_by = c("groupvar", "age_cat"))
  # summary of totals by groupvar (age_cat = "Total")
  summary_total_age <- .summary_table_4_4(df = df, group_by = "groupvar")
  summary_total_age$age_cat <- "Total"
  # summary of totals by age_cat (groupvar = "Total")
  summary_total_by <- .summary_table_4_4(df = df, group_by = "age_cat")
  summary_total_by$groupvar <- "Total"
  # overall total
  st <- .total_summary_4_4(df$utility)
  st$groupvar <- "Total"
  st$age_cat  <- "Total"
  summary_total_by_age <- st

  # combine all
  combined <- rbind(summary_by, summary_total_age, summary_total_by, summary_total_by_age)

  # pivot: rows = (groupvar, stat_name), cols = age_cat values
  stat_cols <- setdiff(names(combined), c("groupvar", "age_cat"))
  all_gc <- unique(combined[, c("groupvar", "age_cat"), drop = FALSE])
  grp_vals <- unique(combined$groupvar)
  age_vals <- unique(combined$age_cat)

  # build result: for each (groupvar, stat), create row; cols = age_cat
  result_list <- lapply(grp_vals, function(gv) {
    sub <- combined[combined$groupvar == gv, , drop = FALSE]
    do.call(rbind, lapply(stat_cols, function(sc) {
      row <- data.frame(groupvar = gv, name = sc, stringsAsFactors = FALSE)
      for (ac in age_vals) {
        s2 <- sub[sub$age_cat == ac, sc, drop = TRUE]
        row[[ac]] <- if (length(s2) == 1) s2 else NA_real_
      }
      row
    }))
  })
  retval <- do.call(rbind, result_list)
  rownames(retval) <- NULL

  # return value
  return(retval)
}


#' eq5d_profile_pchc_by_group_plot: Paretian Classification of Health Change (PCHC) by Group
#' This function computes PCHC categories between two time points for each subject, 
#' stratifies them by a grouping variable, and produces
#' a single bar chart with side-by-side bars showing the distribution of PCHC categories.
#' @param df Data frame containing EQ-5D dimensions, a grouping variable, patient ID, and follow-up columns
#' @param name_id Character string for the patient ID column
#' @param name_groupvar Character string for the grouping column (e.g., procedure). If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector of length 2 indicating the order of follow-up time points (e.g., c("Pre-op", "Post-op"))
#' @return A list with two elements:
#'   \item{plot_data}{A tibble of PCHC percentages by group}
#'   \item{p}{A ggplot2 object showing a bar chart with side-by-side bars for each PCHC category}
#' @export
#' @examples
#' result <- eq5d_profile_pchc_by_group_plot(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op", "Post-op")
#' )
#' result$p        # shows the plot
#' result$plot_data  # shows the summary table

eq5d_profile_pchc_by_group_plot <- function(df,
                         name_id,
                         name_groupvar = NULL,
                         names_eq5d = NULL,
                         name_fu = NULL,
                         levels_fu = NULL) {
  ### 1) Data Preparation ###

  # Replace NULL names with defaults (helper function that sets names_eq5d, name_fu, etc.)
  temp <- .get_names(df = df,
                     names_eq5d = names_eq5d,
                     name_fu = name_fu,
                     levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu    <- temp$name_fu
  levels_fu  <- temp$levels_fu

  # Check columns exist
  names_all <- c(name_id, name_groupvar, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in dataframe. Stopping.")
  }

  # Keep only relevant columns
  df <- df[, names_all, drop = FALSE]

  # Rename for internal use
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }

  # Prepare EQ-5D & Follow-up columns
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  # Sort by (id, groupvar, fu)
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]

  # Check uniqueness: each (id, groupvar, fu) should appear at most once
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))

  ### 2) Calculate PCHC (Paretian Classification) ###
  # .pchc() compares the first level in levels_fu (e.g. "Pre-op") vs. the second (e.g. "Post-op")
  df <- .pchc(df = df, level_fu_1 = levels_fu[1], add_noprobs = TRUE)
  df <- df[!is.na(df$state), , drop = FALSE]
  # The factor levels for PCHC categories can be: "No problems", "No change", "Improve", "Worsen", "Mixed change"
  df$state_noprobs <- factor(
    df$state_noprobs,
    levels = c("No problems", "No change", "Improve", "Worsen", "Mixed change")
  )

  ### 3) Summarize for Single Bar Chart ###
  # We want one bar per groupvar for each state_noprobs
  plot_data <- aggregate(rep(1L, nrow(df)),
                         by = list(groupvar = df$groupvar, state_noprobs = df$state_noprobs),
                         FUN = sum)
  names(plot_data)[names(plot_data) == "x"] <- "n"
  key_gv <- plot_data$groupvar
  plot_data$p <- plot_data$n / tapply(plot_data$n, key_gv, sum)[key_gv]

  ### 4) Build the Plot ###
  p <- ggplot2::ggplot(plot_data,
                       ggplot2::aes(x = .data$state_noprobs,
                                    y = .data$p,
                                    fill = .data$groupvar)) +
    ggplot2::geom_bar(stat = "identity", position = ggplot2::position_dodge(width = 0.9)) +
    ggplot2::geom_text(
      ggplot2::aes(label = scales::percent(.data$p, accuracy = 0.1)),
      position = ggplot2::position_dodge(width = 0.9),
      vjust = -0.5
    ) +
    ggplot2::scale_x_discrete(name = "Pareto classification") +
    ggplot2::scale_y_continuous(
      name    = "Percentage of respondents",
      labels  = scales::percent_format(accuracy = 1),
      expand  = ggplot2::expansion(mult = c(0, 0.1))
    ) +
    ggplot2::labs(fill = "Group") +
    ggplot2::ggtitle("Paretian Classification of Health Change (PCHC) by group over time") +
    ggplot2::scale_fill_brewer(palette = "Blues") +
    ggplot2::theme_minimal(base_size = 14)

  ### 5) Reshape Plot Data for Return ###
  # Convert from long to wide so each group appears as a column
  plot_data_wide <- stats::reshape(
    plot_data[, c("state_noprobs", "groupvar", "p")],
    idvar     = "state_noprobs",
    timevar   = "groupvar",
    direction = "wide"
  )
  names(plot_data_wide) <- sub("^p\\.", "", names(plot_data_wide))
  plot_data_wide[is.na(plot_data_wide)] <- 0
  names(plot_data_wide)[names(plot_data_wide) == "state_noprobs"] <- "Change category"

  ### 6) Return ###
  # Return list with summarized data & the ggplot object (with optional theme modifications)
  return(list(
    plot_data = plot_data_wide,
    p = .modify_ggplot_theme(p = p)  # if .modify_ggplot_theme is available
  ))
}

#' eq5d_profile_better_dimensions_by_group_plot: Percentage of Respondents Who Improved in Each EQ-5D Dimension, by Group
#' This function calculates how many respondents improved in each dimension between
#' two time points and summarizes the results for each group. The, it prodcuces 
#' a dimension-focused chart illustrating improvement percentages by dimension.
#' @param df Data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column
#' @param name_id Character string for the patient ID column
#' @param name_groupvar Character string for the grouping column (e.g. procedure). If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d Character vector of EQ-5D dimension names
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector of length 2, specifying the order of the follow-up levels (e.g. c("Pre-op","Post-op"))
#' @return A list containing:
#'   \item{plot_data}{A data frame of improvements by group and dimension}
#'   \item{p}{A ggplot2 object produced by `.pchc_plot_by_dim()`}
#' @export
#'
#' @examples
#' result <- eq5d_profile_better_dimensions_by_group_plot(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op","Post-op")
#' )
#' result$p
#' result$plot_data
#'
eq5d_profile_better_dimensions_by_group_plot <- function(df,
                         name_id,
                         name_groupvar = NULL,
                         names_eq5d = NULL,
                         name_fu = NULL,
                         levels_fu = NULL) {
  ### 1) Data Preparation ###
  
  # Replace NULL names with defaults
  temp <- .get_names(df = df, 
                     names_eq5d = names_eq5d, 
                     name_fu = name_fu, 
                     levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu    <- temp$name_fu
  levels_fu  <- temp$levels_fu
  
  # Check columns exist
  names_all <- c(name_id, name_groupvar, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in dataframe. Stopping.")
  }
  
  # Keep only relevant columns
  df <- df[, names_all, drop = FALSE]

  # Rename internal columns
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }

  # Prepare EQ-5D & Follow-up columns
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  # Sort by (id, groupvar, fu)
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]

  # Ensure uniqueness of (id, groupvar, fu)
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))

  ### 2) Analysis ###
  # .pchc() calculates difference columns like mo_diff, sc_diff, etc.
  df <- .pchc(df = df, level_fu_1 = levels_fu[1])
  df <- df[!is.na(df$state), , drop = FALSE]

  # Focus on those whose overall 'state' == "Improve"
  # then check dimension-specific improvements (e.g. mo_diff > 0)
  dimension_names <- c("mo_diff", "sc_diff", "ua_diff", "pd_diff", "ad_diff")

  df_imp <- df[df$state == "Improve", c("groupvar", "fu", dimension_names), drop = FALSE]
  # pivot longer manually
  df_long <- do.call(rbind, lapply(dimension_names, function(dn) {
    data.frame(groupvar = df_imp$groupvar,
               fu       = df_imp$fu,
               name     = dn,
               value    = df_imp[[dn]],
               stringsAsFactors = FALSE)
  }))
  # group_by(groupvar, fu, name) summarise
  plot_data <- do.call(rbind, lapply(
    split(df_long, list(df_long$groupvar, df_long$fu, df_long$name), drop = TRUE),
    function(sub) {
      data.frame(groupvar = sub$groupvar[1],
                 fu       = sub$fu[1],
                 name     = sub$name[1],
                 n        = sum(sub$value > 0),
                 n_total  = nrow(sub),
                 stringsAsFactors = FALSE)
    }
  ))
  rownames(plot_data) <- NULL
  plot_data$p <- plot_data$n / plot_data$n_total
  # Convert dimension code to a nice label
  plot_data$name <- factor(
    plot_data$name,
    levels = dimension_names,
    labels = c("Mobility", "Self-care", "Usual activities", "Pain & Discomfort", "Anxiety & Depression")
  )
  
  ### 3) Plot ###
  # We assume .pchc_plot_by_dim() is a helper function that
  # takes a data frame w/ columns: groupvar, fu, name, p
  # and returns a ggplot object. Alternatively, you can build your own chart here.
  
  p <- .pchc_plot_by_dim(
    plot_data = plot_data,
    ylab      = "Percentage of respondents who improved overall",
    title     = "Percentage of respondents who improved overall \nby the dimensions in which they improved (%)",
    cols      = .gen_colours("green", length(unique(plot_data$groupvar)))
  )
  
  ### 4) Tidy Up & Return ###
  # Reshape from long to wide for final table
  # e.g. columns for each fu within each groupvar
  # If you want separate columns by groupvar as well, you can unify them in a single pivot.
  
  # Pivot only by fu, keeping groupvar as id col
  plot_data_wide <- stats::reshape(
    plot_data[, c("groupvar", "name", "fu", "p")],
    idvar     = c("groupvar", "name"),
    timevar   = "fu",
    direction = "wide"
  )
  names(plot_data_wide) <- sub("^p\\.", "", names(plot_data_wide))
  plot_data_wide[is.na(plot_data_wide)] <- 0
  names(plot_data_wide)[names(plot_data_wide) == "groupvar"] <- "Group"
  names(plot_data_wide)[names(plot_data_wide) == "name"]     <- "Values"

  return(list(
    plot_data = plot_data_wide,
    p         = .modify_ggplot_theme(p = p) # if you have a custom theme function
  ))
}


#’ Figure 1.2.3: Percentage of Respondents Who Worsened in Each EQ-5D Dimension, by Group
#'
#' This function identifies respondents with a "Worsen" PCHC state (i.e., overall
#' health state got worse between levels_fu[1] and levels_fu[2]), checks
#' dimension-specific changes (e.g., mo_diff < 0), and summarizes by a grouping variable
#' (e.g., procedure) and time points. It returns a data table and a ggplot object.
#'
#' @param df A data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column
#' @param name_id A character string for the patient ID column
#' @param name_groupvar A character string for the grouping column (e.g., procedure). If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d A character vector of EQ-5D dimension names
#' @param name_fu A character string for the follow-up column
#' @param levels_fu A character vector of length 2, specifying the order of the follow-up levels (e.g., c("Pre-op","Post-op"))
#'
#' @return A list containing:
#'   \item{plot_data}{A data frame of "Worsen" percentages by group and dimension}
#'   \item{p}{A ggplot2 object produced by `.pchc_plot_by_dim()`}
#' @export
#'
#' @examples
#' result <- eq5d_profile_worse_dimensions_by_group_plot(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op","Post-op")
#' )
#' result$p        # shows the plot
#' result$plot_data  # shows the summary table
#'
eq5d_profile_worse_dimensions_by_group_plot <- function(df,
                         name_id,
                         name_groupvar = NULL,
                         names_eq5d = NULL,
                         name_fu = NULL,
                         levels_fu = NULL) {
  
  ### 1) Data Preparation ###
  
  # Replace NULL names with defaults (helper function that sets names_eq5d, name_fu, etc.)
  temp <- .get_names(df = df,
                     names_eq5d = names_eq5d,
                     name_fu = name_fu,
                     levels_fu = levels_fu)
  
  names_eq5d <- temp$names_eq5d
  name_fu    <- temp$name_fu
  levels_fu  <- temp$levels_fu
  
  # Check columns exist
  names_all <- c(name_id, name_groupvar, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in dataframe. Stopping.")
  }
  
  # Keep only relevant columns
  df <- df[, names_all, drop = FALSE]

  # Rename internal columns
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }

  # Prepare EQ-5D & Follow-up columns
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  # Sort by (id, groupvar, fu)
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]

  # Ensure uniqueness of (id, groupvar, fu)
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))

  ### 2) Analysis: PCHC ###

  # .pchc() calculates difference columns like mo_diff, sc_diff, etc.
  # level_fu_1 is the 'baseline' time point.
  df <- .pchc(df = df, level_fu_1 = levels_fu[1])
  df <- df[!is.na(df$state), , drop = FALSE]

  # We focus on those whose overall state == "Worsen"
  # then check dimension-specific changes, e.g., mo_diff < 0
  dimension_names <- c("mo_diff", "sc_diff", "ua_diff", "pd_diff", "ad_diff")

  df_wor <- df[df$state == "Worsen", c("groupvar", "fu", dimension_names), drop = FALSE]
  # pivot longer manually
  df_long <- do.call(rbind, lapply(dimension_names, function(dn) {
    data.frame(groupvar = df_wor$groupvar,
               fu       = df_wor$fu,
               name     = dn,
               value    = df_wor[[dn]],
               stringsAsFactors = FALSE)
  }))
  # group_by(groupvar, fu, name) summarise
  plot_data <- do.call(rbind, lapply(
    split(df_long, list(df_long$groupvar, df_long$fu, df_long$name), drop = TRUE),
    function(sub) {
      data.frame(groupvar = sub$groupvar[1],
                 fu       = sub$fu[1],
                 name     = sub$name[1],
                 n        = sum(sub$value < 0),
                 n_total  = nrow(sub),
                 stringsAsFactors = FALSE)
    }
  ))
  rownames(plot_data) <- NULL
  plot_data$p <- plot_data$n / plot_data$n_total
  # Convert dimension code to a nice label
  plot_data$name <- factor(
    plot_data$name,
    levels = dimension_names,
    labels = c("Mobility", "Self-care", "Usual activities", "Pain & Discomfort", "Anxiety & Depression")
  )
  
  ### 3) Plot ###
  
  # We assume .pchc_plot_by_dim() is a helper function that
  # creates a bar chart from plot_data with columns:
  # groupvar, fu, name, and p
  # The color palette is "orange" for Worsen by dimension
  p <- .pchc_plot_by_dim(
    plot_data = plot_data,
    ylab      = "Percentage of respondents who worsened overall",
    title     = "Percentage of respondents who worsened overall \nby the dimensions in which they got worse (%)",
    cols      = .gen_colours("orange", length(unique(plot_data$groupvar)))
  )
  
  ### 4) Tidy Up & Return ###

  # Reshape from long to wide for final table
  plot_data_wide <- stats::reshape(
    plot_data[, c("groupvar", "name", "fu", "p")],
    idvar     = c("groupvar", "name"),
    timevar   = "fu",
    direction = "wide"
  )
  names(plot_data_wide) <- sub("^p\\.", "", names(plot_data_wide))
  plot_data_wide[is.na(plot_data_wide)] <- 0
  names(plot_data_wide)[names(plot_data_wide) == "groupvar"] <- "Group"
  names(plot_data_wide)[names(plot_data_wide) == "name"]     <- "Values"

  return(list(
    plot_data = plot_data_wide,
    p         = .modify_ggplot_theme(p = p) # if you have a custom theme function
  ))
}


#' eq5d_profile_mixed_dimensions_by_group_plot: Percentage of Respondents Who Had a Mixed Change Overall,
#' by Dimension Improved or Worsened, Grouped by Procedure (or Other Grouping)
#'
#' This function focuses on patients classified as having "Mixed change" overall
#' (i.e., some dimensions improved, others worsened). It then examines which dimensions
#' improved vs. worsened for each subject. Results are summarized by a grouping variable
#' (e.g., procedure) and time points. The final output is a table plus a ggplot object.
#'
#' @param df Data frame containing columns for EQ-5D dimensions, a grouping variable,
#' a patient ID, and a follow-up variable
#' @param name_id Character string indicating the patient ID column
#' @param name_groupvar Character string for the grouping column (e.g. "procedure"). If NULL (default), the analysis is performed on the full population.
#' @param names_eq5d Character vector naming the EQ-5D dimensions (e.g. c("mo","sc","ua","pd","ad"))
#' @param name_fu Character string for the follow-up column (e.g. "time")
#' @param levels_fu Character vector of length 2 specifying the time order (e.g. c("Pre-op","Post-op"))
#'
#' @return A list with two elements:
#'   \item{plot_data}{A wide-format data frame of dimension-specific improvements/worsenings for "Mixed change"}
#'   \item{p}{A ggplot2 object showing a dimension-level bar chart from .pchc_plot_by_dim}
#'
#' @export
#'
#' @examples
#' result <- eq5d_profile_mixed_dimensions_by_group_plot(
#'   df = example_data,
#'   name_id = "id",
#'   name_groupvar = "procedure",
#'   names_eq5d = c("mo","sc","ua","pd","ad"),
#'   name_fu = "time",
#'   levels_fu = c("Pre-op","Post-op")
#' )
#' result$plot_data
#' result$p
#'
eq5d_profile_mixed_dimensions_by_group_plot <- function(df,
                         name_id,
                         name_groupvar = NULL,
                         names_eq5d = NULL,
                         name_fu = NULL,
                         levels_fu = NULL) {
  
  ### 1) Data Preparation ###
  
  # Replace NULL names with defaults
  temp <- .get_names(df = df,
                     names_eq5d = names_eq5d,
                     name_fu = name_fu,
                     levels_fu = levels_fu)
  names_eq5d <- temp$names_eq5d
  name_fu    <- temp$name_fu
  levels_fu  <- temp$levels_fu
  
  # Check columns exist
  names_all <- c(name_id, name_groupvar, names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in dataframe. Stopping.")
  }
  
  # Keep only relevant columns
  df <- df[, names_all, drop = FALSE]

  # Rename for internal use
  names(df)[names(df) == name_id] <- "id"
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }

  # Prepare EQ-5D & Follow-up columns
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  # Sort by (id, groupvar, fu)
  df <- df[order(df$id, df$groupvar, df$fu), , drop = FALSE]

  # Check uniqueness
  .check_uniqueness(df, group_by = c("id", "groupvar", "fu"))

  ### 2) Analysis: Identify Mixed Change ###

  # .pchc() adds difference columns like mo_diff, sc_diff, etc.
  # level_fu_1 is the 'baseline' time point
  df <- .pchc(df = df, level_fu_1 = levels_fu[1])
  df <- df[!is.na(df$state), , drop = FALSE]

  # We'll focus on those with "Mixed change" overall
  dimension_names <- c("mo_diff", "sc_diff", "ua_diff", "pd_diff", "ad_diff")

  # We also define the fu values and directions for labeling
  n_time <- length(levels_fu)
  # For example: "Improve: Pre-op", "Improve: Post-op", "Worsen: Pre-op", "Worsen: Post-op"
  levels_fu_change <- c(
    paste0("Improve: ", levels_fu),
    paste0("Worsen: ", levels_fu)
  )

  # 2a) Summarize dimension-level improvements & worsenings among "Mixed change" patients
  df_mix <- df[df$state == "Mixed change", c("groupvar", "fu", dimension_names), drop = FALSE]
  # pivot longer manually
  df_long <- do.call(rbind, lapply(dimension_names, function(dn) {
    data.frame(groupvar = df_mix$groupvar,
               fu       = df_mix$fu,
               name     = dn,
               value    = df_mix[[dn]],
               stringsAsFactors = FALSE)
  }))
  # group_by(groupvar, fu, name) summarise
  agg <- do.call(rbind, lapply(
    split(df_long, list(df_long$groupvar, df_long$fu, df_long$name), drop = TRUE),
    function(sub) {
      data.frame(groupvar  = sub$groupvar[1],
                 fu        = as.character(sub$fu[1]),
                 name      = sub$name[1],
                 n_improve = sum(sub$value > 0),
                 n_worsen  = sum(sub$value < 0),
                 n_total   = nrow(sub),
                 stringsAsFactors = FALSE)
    }
  ))
  rownames(agg) <- NULL
  agg$Improve <- agg$n_improve / agg$n_total
  agg$Worsen  <- agg$n_worsen  / agg$n_total
  # Convert e.g. "mo_diff" -> "Mobility"
  agg$name <- factor(
    agg$name,
    levels = dimension_names,
    labels = c("Mobility", "Self-care", "Usual activities", "Pain & Discomfort", "Anxiety & Depression")
  )
  # pivot Improve & Worsen into single column
  plot_data_imp <- data.frame(groupvar = agg$groupvar, fu = agg$fu, name = agg$name,
                               change = "Improve", p = agg$Improve, stringsAsFactors = FALSE)
  plot_data_wor <- data.frame(groupvar = agg$groupvar, fu = agg$fu, name = agg$name,
                               change = "Worsen",  p = agg$Worsen,  stringsAsFactors = FALSE)
  plot_data <- rbind(plot_data_imp, plot_data_wor)
  # Build combined factor "Improve: Pre-op" / "Worsen: Pre-op" etc.
  plot_data$fu <- factor(paste0(plot_data$change, ": ", plot_data$fu), levels = levels_fu_change)
  
  ### 3) Plot ###
  
  plot_data2 <- plot_data
  plot_data2$groupvar <- paste(plot_data$groupvar, plot_data$fu)
  
  plot_data2$name   <- factor(plot_data2$name,   levels = c("Mobility", "Self-care", "Usual activities", "Pain & Discomfort", "Anxiety & Depression"))
  plot_data2$change <- factor(plot_data2$change, levels = c("Improve", "Worsen"))
  plot_data2 <- plot_data2[order(plot_data2$name, plot_data2$change), , drop = FALSE]
  
  plot_data2$groupvar <- factor(plot_data2$groupvar, levels = unique(plot_data2$groupvar))
  
  p <- .pchc_plot_by_dim(
    plot_data = plot_data2,
    ylab      = "Percentage of respondents who had mixed change",
    title     = "Percentage of respondents who had a mixed change overall \nby the dimensions in which they improved and worsened (%)",
    cols      = c(
      .gen_colours("green", n_time*2),  # for "Improve: ...",
      .gen_colours("orange", n_time*2)  # for "Worsen: ..."
    ),
    text_rotate = TRUE
  )
  
  ### 4) Reshape & Return ###

  # Pivot wide by the combined fu factor
  plot_data_wide <- stats::reshape(
    plot_data[, c("groupvar", "name", "fu", "p")],
    idvar     = c("groupvar", "name"),
    timevar   = "fu",
    direction = "wide"
  )
  names(plot_data_wide) <- sub("^p\\.", "", names(plot_data_wide))
  plot_data_wide[is.na(plot_data_wide)] <- 0
  names(plot_data_wide)[names(plot_data_wide) == "groupvar"] <- "Group"
  names(plot_data_wide)[names(plot_data_wide) == "name"]     <- "Values"

  return(list(
    plot_data = plot_data_wide,
    p         = .modify_ggplot_theme(p = p)
  ))
}

#' eq5d_profile_health_profile_grid: Health Profile Grid (HPG) for Two Time Points
#'
#' This function creates a Health Profile Grid (HPG) for EQ-5D data, plotting each
#' individual's change in health states (ranked from best to worst) between two time
#' points. A diagonal reference line indicates no change; points above the line reflect
#' improvement, and points below indicate deterioration.
#'
#' @param df A data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column
#' @param names_eq5d A character vector of EQ-5D dimension names
#' @param name_fu A character string for the follow-up column
#' @param levels_fu A character vector of length 2, specifying the order of the follow-up levels (e.g., c("Pre-op","Post-op"))
#' @param name_id A character string for the patient ID column
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' @return A list with components:
#'   \item{plot_data}{The plot data with ranks and classification.}
#'   \item{p}{A \code{ggplot2} object displaying the HPG scatter plot.}
#' @export
#' @examples
#' tmp <- eq5d_profile_health_profile_grid(
#'            df = example_data, 
#'            names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'            name_fu = "time", 
#'            levels_fu = c("Pre-op", "Post-op"), 
#'            name_id = "id", 
#'            eq5d_version = "3L", 
#'            country = "UK"
#'        )

eq5d_profile_health_profile_grid <- function(df,
                         names_eq5d,
                         name_fu,
                         levels_fu = NULL,
                         name_id,
                         eq5d_version,
                         country) {
  ### 1) Data Preparation ###
  # Replace NULL names with defaults (helper function that sets names_eq5d, name_fu, etc.)
  temp <- .get_names(df = df,
                     names_eq5d = names_eq5d,
                     name_fu = name_fu,
                     levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu    <- temp$name_fu
  levels_fu  <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  
  # Check columns exist
  names_all <- c(name_id,  names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df))) {
    stop("Provided column names not in dataframe. Stopping.")
  }
  
  # Keep only relevant columns
  df <- df[, names_all, drop = FALSE]

  # Rename for internal use
  names(df)[names(df) == name_id] <- "id"

  # Prepare EQ-5D & Follow-up columns
  df <- .prep_eq5d(df = df, names = names_eq5d)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)

  # Sort by (id, fu)
  df <- df[order(df$id, df$fu), , drop = FALSE]

  # Check uniqueness: each (id, fu) should appear at most once
  .check_uniqueness(df, group_by = c("id", "fu"))

  ### 2) Calculate PCHC (Paretian Classification) ###
  # .pchc() compares the first level in levels_fu (e.g. "Pre-op") vs. the second (e.g. "Post-op")
  df <- .pchc(df = df, level_fu_1 = levels_fu[1], add_noprobs = TRUE)
  df <- df[!is.na(df$state), , drop = FALSE]
  # The factor levels for PCHC categories can be: "No problems", "No change", "Improve", "Worsen", "Mixed change"
  df$state_noprobs <- factor(
    df$state_noprobs,
    levels = c("No problems", "No change", "Improve", "Worsen", "Mixed change")
  )

  ### 3) Summarize for HPG ###
  # Create value set
  vs <- data.frame(profile = make_all_EQ_indexes(version = eq5d_version))
  vs$utility <- eq5d(vs$profile, country = country, version = eq5d_version)
  vs <- vs[order(-vs$utility), , drop = FALSE]
  vs$rank <- seq_len(nrow(vs))
  
  # Build profiles
  eqdims <- names_eq5d
  for (d in eqdims) {
    df[[paste0(d, "_t2")]] <- df[[d]]
    df[[paste0(d, "_t1")]] <- df[[d]] + df[[paste0(d, "_diff")]]
  }
  # Build profile codes row-by-row using apply
  t1_cols <- paste0(eqdims, "_t1")
  t2_cols <- paste0(eqdims, "_t2")
  df$profile_t1 <- as.integer(apply(df[, t1_cols, drop = FALSE], 1,
                                     function(r) paste0(r, collapse = "")))
  df$profile_t2 <- as.integer(apply(df[, t2_cols, drop = FALSE], 1,
                                     function(r) paste0(r, collapse = "")))

  # Get rank via match
  df$rank_t1 <- vs$rank[match(df$profile_t1, vs$profile)]
  df$rank_t2 <- vs$rank[match(df$profile_t2, vs$profile)]
  
  # Cteate plot
  max_rank <- nrow(vs)
  p <- ggplot(df, aes(x = .data$rank_t2, y = .data$rank_t1, color = .data$state, shape = .data$state)) +
    geom_point(size = 3) +
    geom_abline(intercept = 0, slope = 1, linetype = "solid", color = "black") +
    scale_x_continuous(limits = c(1, max_rank), expand = c(0,0)) +
    scale_y_continuous(limits = c(1, max_rank), expand = c(0,0)) +
    labs(
      title    = "Health Profile Grid (HPG)",
      x        = paste0(levels_fu[1], " rank"),
      y        = paste0(levels_fu[2], " rank"),
      color    = "Classification",  # Legend title for color
      shape    = "Classification"   # Legend title for shape
    ) +
    scale_shape_manual(values = c(
      "Improve"    = 3,  
      "No change"  = 16,  
      "Worsen"     = 4,   
      "Mixed change" = 17  
    )) +
    scale_color_manual(values = c(
      "Improve"      = "#69AB3E",
      "No change"    = "#FDC020",
      "Worsen"       = "#BF2518",
      "Mixed change" = "#5782BF"
    )) +
    
    theme_minimal(base_size = 14) + 
    theme(
      panel.grid = element_blank(),
      axis.line  = element_line(color = "black"),
      legend.position  = "bottom",
      legend.direction = "horizontal",
      legend.box       = "horizontal"
    ) +
    coord_fixed(ratio = 1)
  
  return(list(plot_data = df, p = p))
}



#' eq5d_profile_lss_utility_plot: EQ-5D values plotted against LSS
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' df <- data.frame(make_all_EQ_states(version = "5L"))
#' tmp <- eq5d_profile_lss_utility_plot(
#'  df, 
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  eq5d_version = "5L", 
#'  country = "US"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_profile_lss_utility_plot <- function(df,
                       names_eq5d = NULL,
                       eq5d_version = NULL,
                       country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_lss = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("lss", "utility"), drop = FALSE]

  ### analysis ###

  # prepare data for plotting: remove NAs, group by lss, summarise
  df2 <- df[!is.na(df$lss) & !is.na(df$utility), , drop = FALSE]
  plot_data <- do.call(rbind, lapply(split(df2$utility, df2$lss), function(vals) {
    data.frame(median = median(vals), min = min(vals), max = max(vals))
  }))
  plot_data$lss <- as.integer(rownames(plot_data))
  rownames(plot_data) <- NULL
  plot_data <- plot_data[order(plot_data$lss), , drop = FALSE]

  # graphical parameters
  x_breaks <- 1:100
  y_breaks <- seq(from = -1, to = 1, by = 0.2)

  # pivot longer for segment plot
  seg_data <- do.call(rbind, list(
    data.frame(lss = plot_data$lss, name = "Median", value = plot_data$median),
    data.frame(lss = plot_data$lss, name = "Lowest",  value = plot_data$min),
    data.frame(lss = plot_data$lss, name = "Highest", value = plot_data$max)
  ))
  seg_data$name <- factor(seg_data$name, levels = c("Median", "Lowest", "Highest"))

  # plot
  p <- ggplot() +
    # plot median, min and max
    geom_segment(data = seg_data,
                 aes(x = .data$lss - 0.2, xend = .data$lss + 0.2,
                     y = .data$value, yend = .data$value,
                     colour = .data$name)) +
    # connect values with a line segment
    geom_segment(data = plot_data, aes(x = .data$lss, xend = .data$lss, y = min, yend = max)) +
    # plot title
    ggtitle("EQ-5D values plotted against the LSS (Level Sum Score)") +
    # manipulate x-axis
    scale_x_continuous(name = "Level Sum Score",
                       breaks = x_breaks,
                       expand = expansion(add = c(0.5, 0.5))) +
    # manipulate y-axis
    scale_y_continuous(name = "EQ-5D value",
                       breaks = y_breaks)

  # tidy up summary
  names(plot_data)[names(plot_data) == "lss"]    <- "LSS"
  names(plot_data)[names(plot_data) == "median"] <- "Median"
  names(plot_data)[names(plot_data) == "min"]    <- "Minimum"
  names(plot_data)[names(plot_data) == "max"]    <- "Maximum"
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_profile_lfs_utility_plot: EQ-5D values plotted against LFS
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_profile_lfs_utility_plot(
#'  example_data, 
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  eq5d_version = "3L",
#'  country = "UK"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_profile_lfs_utility_plot <- function(df,
                        names_eq5d = NULL,
                        eq5d_version = NULL,
                        country){
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_lfs = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("lfs", "utility"), drop = FALSE]

  ### analysis ###

  # prepare data for plotting: remove NAs, group by lfs, summarise
  df2 <- df[!is.na(df$lfs) & !is.na(df$utility), , drop = FALSE]
  plot_data <- do.call(rbind, lapply(split(df2$utility, df2$lfs), function(vals) {
    data.frame(median = median(vals), min = min(vals), max = max(vals))
  }))
  plot_data$lfs <- rownames(plot_data)
  rownames(plot_data) <- NULL
  # order by descending median
  plot_data <- plot_data[order(-plot_data$median), , drop = FALSE]

  # impose order on the x-axis
  lfs_levels <- plot_data$lfs
  plot_data$lfs_f <- factor(plot_data$lfs, levels = lfs_levels)
  
  # axis breaks
  n <- length(lfs_levels)
  i <- seq(from = 1, to = n, by = 3)
  x_breaks <- (1:n)[i]
  x_labels <- lfs_levels[i]
  y_breaks <- seq(from = -1, to = 1, by = 0.2)
  
  # pivot longer for segment plot
  seg_data2 <- do.call(rbind, list(
    data.frame(lfs = plot_data$lfs, lfs_f = plot_data$lfs_f, name = "Median", value = plot_data$median),
    data.frame(lfs = plot_data$lfs, lfs_f = plot_data$lfs_f, name = "Lowest",  value = plot_data$min),
    data.frame(lfs = plot_data$lfs, lfs_f = plot_data$lfs_f, name = "Highest", value = plot_data$max)
  ))
  seg_data2$name <- factor(seg_data2$name, levels = c("Median", "Lowest", "Highest"))

  # plot
  p <- ggplot() +
    # plot median, min and max
    geom_segment(data = seg_data2,
                 aes(x = as.numeric(.data$lfs_f) - 0.5, xend = as.numeric(.data$lfs_f) + 0.5,
                     y = .data$value, yend = .data$value,
                     colour = .data$name)) +
    # connect values with a line segment
    geom_segment(data = plot_data, aes(x = as.numeric(.data$lfs_f), xend = as.numeric(.data$lfs_f), y = min, yend = max)) +
    # plot title
    ggtitle("EQ-5D values plotted against the LFS (Level Frequency Score)") +
    # manipulate x-axis
    scale_x_continuous(name = "Level Frequency Score", 
                       breaks = x_breaks,
                       labels = x_labels,
                       expand = expansion(add = c(0.5, 0.5))) +
    # manipulate y-axis
    scale_y_continuous(name = "EQ-5D value",
                       breaks = y_breaks)
  p <- .modify_ggplot_theme(p = p) + 
    # tidy up the graph
    theme(
      # rotate labels in the x-axis
      axis.text.x = element_text(angle = 90))
  
  # tidy up summary
  plot_data <- plot_data[, setdiff(names(plot_data), "lfs_f"), drop = FALSE]
  names(plot_data)[names(plot_data) == "lfs"]    <- "LFS"
  names(plot_data)[names(plot_data) == "median"] <- "Median"
  names(plot_data)[names(plot_data) == "min"]    <- "Minimum"
  names(plot_data)[names(plot_data) == "max"]    <- "Maximum"
  
  return(list(plot_data = plot_data, p = p))
}

#' eq5d_vas_histogram: EQ VAS scores
#' 
#' @param df Data frame with the VAS column
#' @param name_vas Character string for the VAS column
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_vas_histogram(example_data, name_vas = 'vas')
#' tmp$p
#' tmp$plot_data

eq5d_vas_histogram <- function(df, name_vas = NULL){
  
  ### data preparation ###
  # replace NULL names with defaults
  temp <- .get_names(name_vas = name_vas)
  name_vas <- temp$name_vas
  # check existence of columns 
  names_all <- c(name_vas)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_vas(df = df, name = name_vas)
  
  ### analysis ###
  
  # plot
  p <- ggplot(df, aes(x = .data$vas)) +
    geom_bar(width = 0.1, fill = "blue") + 
    scale_x_continuous(name = "EQ VAS score", 
                       breaks = seq(from = 0, to = 100, by = 5),
                       limits = c(-3, 103),
                       expand = c(0, 0)) +
    scale_y_continuous(name = "Frequency",
                       expand = expansion(mult = c(0, 0.05))) + 
    ggtitle("EQ VAS Scores") + 
    # tidy up the graph
    theme(
      # reinstate x-axis ticks
      axis.ticks.x = element_line(colour = "grey50"))
  
  # output plotting data: count for each value between 0 & 100
  vas_factor <- factor(df$vas, levels = 1:100)
  tbl <- tabulate(vas_factor, nbins = 100L)
  plot_data <- data.frame(vas = factor(1:100, levels = 1:100), count = tbl,
                          stringsAsFactors = FALSE)
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_vas_grouped_distribution_plot: Mid-point EQ VAS scores
#' 
#' @param df Data frame with the VAS column
#' @param name_vas Character string for the VAS column
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_vas_grouped_distribution_plot(example_data, name_vas = 'vas')
#' tmp$p
#' tmp$plot_data

eq5d_vas_grouped_distribution_plot <- function(df, name_vas = NULL){
  
  # produce output to be plotted (Table 3.2)
  # do not add totals
  plot_data <- eq5d_vas_distribution_table(df = df, name_vas = name_vas, add_na_total = FALSE)
  # remove Range column, not needed
  plot_data <- plot_data[, setdiff(names(plot_data), "Range"), drop = FALSE]
  
  # plot
  p <- ggplot(plot_data, aes(x = .data$Midpoint, y = .data$Frequency)) +
    geom_bar(width = 0.7, fill = "blue", stat = "identity") + coord_flip() +
    scale_x_continuous(name = "EQ VAS score (Midpoint)", 
                       breaks = seq(from = 0, to = 100, by = 10),
                       limits = c(-3, 103),
                       expand = c(0, 0)) +
    scale_y_continuous(name = "Frequency",
                       expand = expansion(mult = c(0, 0.05))) + 
    ggtitle("EQ VAS Scores (Midpoint)") +
    # tidy up the graph
    theme(
      # reinstate x-axis ticks
      axis.ticks.x = element_line(colour = "grey50"),
      # remove y-axis ticks
      axis.ticks.y = element_blank())
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_utility_over_time_plot: EQ-5D values by timepoints: mean values and 95\% confidence intervals
#' 
#' @param df Data frame with the VAS columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column. 
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_utility_over_time_plot(
#'  example_data,
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  name_fu = "time",
#'  levels_fu = c('Pre-op', 'Post-op'),
#'  eq5d_version = "3L",
#'  country = "UK"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_utility_over_time_plot <- function(df,
                       names_eq5d = NULL,
                       name_fu = NULL,
                       levels_fu = NULL,
                       eq5d_version = NULL,
                       country) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_fu)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  df <- df[, c("fu", "utility"), drop = FALSE]
  
  ### analysis ###
  
  # prepare data for plotting
  plot_data <- .summary_mean_ci(df = df, group_by = "fu")
  
  # plot
  p <- ggplot(data = plot_data, aes(x = .data$fu, 
                                    y = .data$mean, ymin = .data$ci_lb, ymax = .data$ci_ub)) +
    # connect means with a line
    # create a dummy group variable as geom_line only connects points that belong to the same group
    geom_line(aes(group = 1), colour = "red") +
    # plot means
    geom_point(colour = "green") + 
    # add error bars
    geom_errorbar(width = 0.1) +
    # plot title
    ggtitle(paste0("EQ-5D values by ", name_fu, ": mean and 95% confidence intervals")) +
    # manipulate x-axis
    scale_x_discrete(name = "time") +
    # manipulate y-axis
    scale_y_continuous(name = "EQ-5D value")
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_utility_by_group_plot: Mean EQ-5D values and 95\% confidence intervals: all vs by groupvar
#' 
#' @param df Data frame with the EQ-5D and grouping columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country.
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_utility_by_group_plot(
#'  example_data,
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  name_groupvar = "procedure",
#'  eq5d_version = "3L",
#'  country = "UK"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_utility_by_group_plot <- function(df,
                       names_eq5d = NULL,
                       name_groupvar = NULL,
                       eq5d_version = NULL,
                       country) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(name_groupvar, names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  if (is.null(name_groupvar)) {
    df$groupvar <- "All"
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
  }
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("groupvar", "utility"), drop = FALSE]

  ### analysis ###

  # prepare data for plotting
  plot_data <- .summary_mean_ci(df = df, group_by = "groupvar")

  # add totals
  plot_data_total <- .summary_mean_ci(df = df, group_by = NULL)
  plot_data_total$groupvar <- "All groups"
  plot_data_total <- plot_data_total[, c("groupvar", setdiff(names(plot_data_total), "groupvar")), drop = FALSE]

  # combine
  groupvar_levels <- c(plot_data$groupvar, "All groups")
  plot_data <- rbind(plot_data, plot_data_total)
  plot_data$groupvar <- factor(plot_data$groupvar, levels = groupvar_levels)
  plot_data <- plot_data[order(plot_data$groupvar), , drop = FALSE]

  groupvar_label <- if (is.null(name_groupvar)) "Group" else name_groupvar
  # plot
  p <- ggplot(data = plot_data, aes(x = .data$groupvar, y = .data$mean, ymin = .data$ci_lb, ymax = .data$ci_ub)) +
    # plot means
    geom_bar(colour = "blue", fill = "blue", stat = "identity", width = 0.5) + 
    geom_point() +
    # add error bars
    geom_errorbar(width = 0.1, size = 1) +
    # plot title
    ggtitle(paste0("Mean EQ-5D values 95% confidence intervals: all vs by ", groupvar_label)) +
    # manipulate x-axis
    scale_x_discrete(name = groupvar_label) +
    # manipulate y-axis
    scale_y_continuous(name = "EQ-5D value",
                       limits = c(0, 1),
                       breaks = seq(from = 0, to = 1, by = 0.1),
                       expand = c(0, 0))
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_utility_change_by_group_plot: EQ-5D values: smoothed lines and confidence intervals by groupvar
#' 
#' @param df Data frame with the EQ-5D, follow-up and grouping columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_fu Character string for the follow-up column
#' @param levels_fu Character vector containing the order of the values in the follow-up column.
#' If NULL (default value), the levels will be ordered in the order of appearance in df.
#' @param name_groupvar Character string for the grouping column. If NULL (default), the analysis is performed on the full population.
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country.
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_utility_change_by_group_plot(
#'  example_data,
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  name_fu = "time",
#'  levels_fu = c('Pre-op', 'Post-op'),
#'  name_groupvar = "procedure",
#'  eq5d_version = "3L",
#'  country = "UK"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_utility_change_by_group_plot <- function(df,
                       names_eq5d = NULL,
                       name_fu = NULL,
                       levels_fu = NULL,
                       name_groupvar = NULL,
                       eq5d_version = NULL,
                       country) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(df = df, names_eq5d = names_eq5d, 
                     name_fu = name_fu, levels_fu = levels_fu,
                     eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_fu <- temp$name_fu
  levels_fu <- temp$levels_fu
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_fu, name_groupvar)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  if (is.null(name_groupvar)) {
    df$groupvar <- factor("All")
  } else {
    names(df)[names(df) == name_groupvar] <- "groupvar"
    df$groupvar <- factor(df$groupvar)
  }
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- .prep_fu(df = df, name = name_fu, levels = levels_fu)
  df <- df[, c("fu", "groupvar", "utility"), drop = FALSE]
  
  ### analysis ###
  
  # prepare data for plotting
  plot_data <- .summary_mean_ci(df = df, group_by = c("fu", "groupvar"))

  groupvar_label <- if (is.null(name_groupvar)) "Group" else name_groupvar
  # plot
  p <- ggplot(data = plot_data, aes(x = .data$fu,
                                    y = .data$mean, ymin = .data$ci_lb, ymax = .data$ci_ub,
                                    group = .data$groupvar, colour = .data$groupvar)) +
    # plot means
    geom_point() +
    # add error bars
    geom_errorbar(width = 0.1, alpha = 0.5) +
    # connect means with a smooth line
    geom_line() +
    # plot title
    ggtitle(paste0("Longitudinal EQ-5D values by ", name_fu, " and ", groupvar_label, ":\nmeans and 95% confidence intervals")) +
    # manipulate x-axis
    scale_x_discrete(name = name_fu) +
    # manipulate y-axis
    scale_y_continuous(name = "EQ-5D value")
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_utility_distribution_plot: EQ-5D values: smoothed lines and confidence intervals by groupvar
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_utility_distribution_plot(
#'  example_data,
#'  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
#'  eq5d_version = "3L",
#'  country = "UK"
#' )
#' tmp$p
#' tmp$plot_data

eq5d_utility_distribution_plot <- function(df, 
                       names_eq5d = NULL,
                       eq5d_version = NULL,
                       country) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, "utility", drop = FALSE]

  ### analysis ###

  # prepare data for plotting
  util_tbl <- table(df$utility)
  plot_data <- data.frame(
    utility = as.numeric(names(util_tbl)),
    count   = as.integer(util_tbl),
    stringsAsFactors = FALSE
  )
  plot_data <- plot_data[order(plot_data$utility), , drop = FALSE]
  
  # plot
  p <- ggplot(plot_data, aes(x = .data$utility, y = .data$count)) +
    geom_bar(fill = "blue", stat = "identity") + 
    scale_x_continuous(name = "EQ-5D value") +
    scale_y_continuous(name = "Frequency") + 
    ggtitle("Distribution of EQ-5D values") + 
    theme(
      # reinstate axis.ticks
      axis.ticks = element_line(colour = "grey50"))
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

#' eq5d_utility_vas_scatter_plot: EQ-5D values: smoothed lines and confidence intervals by groupvar
#' 
#' @param df Data frame with the EQ-5D columns
#' @param names_eq5d Character vector of column names for the EQ-5D dimensions
#' @param name_vas Character string for the VAS column
#' @param eq5d_version Version of the EQ-5D instrument
#' @param country A character string representing the name of the country. 
#' This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.
#' @return Summary plot and data used for plotting
#' @export
#' @examples
#' tmp <- eq5d_utility_vas_scatter_plot(
#'    example_data,
#'    names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
#'    name_vas = "vas",
#'    eq5d_version = "3L",
#'    country = "UK"
#'  )
#' tmp$p
#' tmp$plot_data

eq5d_utility_vas_scatter_plot <- function(df,
                        names_eq5d = NULL,
                        name_vas = NULL,
                        eq5d_version = NULL,
                        country) {
  
  ### data preparation ###
  
  # replace NULL names with defaults
  temp <- .get_names(names_eq5d = names_eq5d, name_vas = name_vas, eq5d_version = eq5d_version)
  names_eq5d <- temp$names_eq5d
  name_vas <- temp$name_vas
  eq5d_version <- temp$eq5d_version
  # check existence of columns 
  names_all <- c(names_eq5d, name_vas)
  if (!all(names_all %in% colnames(df)))
    stop("Provided column names not in dataframe. Stopping.")
  # all columns defined and exist; only leave relevant columns now
  df <- df[, names_all, drop = FALSE]
  # further checks and data preparation
  df <- .prep_vas(df = df, name = name_vas)
  df <- .prep_eq5d(df = df, names = names_eq5d,
                   add_state = TRUE,
                   add_utility = TRUE, eq5d_version = eq5d_version, country = country)
  df <- df[, c("vas", "utility"), drop = FALSE]

  ### analysis ###

  # prepare data for plotting
  plot_data <- df[order(df$vas, df$utility), , drop = FALSE]
  
  # plot
  p <- ggplot(plot_data, aes(x = .data$vas, y = .data$utility)) +
    geom_point(colour = "blue") + 
    scale_x_continuous(name = "EQ VAS") +
    scale_y_continuous(name = "EQ-5D value") + 
    ggtitle("EQ-5D values plotted against EQ VAS")
  
  return(list(plot_data = plot_data, p = .modify_ggplot_theme(p = p)))
}

# Base URL for the value sets repository
.vs_base_url <- paste0(
  "https://raw.githubusercontent.com/",
  "MathsInHealth/eq5dsuite-value-sets/main/"
)

# Map instrument version to folder name
.vs_folder <- function(version) {
  switch(version,
    "3L"  = "EQ-5D-3L",
    "5L"  = "EQ-5D-5L",
    "Y3L" = "EQ-5D-Y-3L",
    stop("Unknown version: ", version)
  )
}

#' Fetch the index of available value sets from GitHub
#'
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @return A data frame with columns Version, Name, Name_short,
#'   Country_code, VS_code, doi. Returns NULL if fetch fails.
#' @keywords internal
fetch_available_value_sets <- function(version) {
  url <- paste0(
    .vs_base_url,
    .vs_folder(version),
    "/value_sets.csv"
  )
  tryCatch({
    utils::read.csv(
      curl::curl(url),
      stringsAsFactors = FALSE
    )
  },
  error = function(e) {
    message("eq5dsuite: Could not fetch value set index for ",
            version, ": ", conditionMessage(e))
    NULL
  })
}

#' Get the VS_codes currently installed for a given instrument
#'
#' Returns all installed VS_codes for the specified instrument
#' version, including both built-in value sets shipped with the
#' package and any user-added value sets installed via
#' \code{eqvs_add()} or \code{update_value_sets()}.
#'
#' Reads directly from the runtime package environment
#' (\code{getOption("eq.env")}) to include user-added value sets
#' without triggering the display side-effects of
#' \code{eqvs_display()}.
#'
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @return A character vector of VS_codes.
#' @keywords internal
get_installed_vs_codes <- function(version) {
  tryCatch({
    pkgenv <- getOption("eq.env")
    version_upper    <- toupper(version)
    user_defined_str <- paste0("user_defined_", version_upper)

    built_in     <- pkgenv$country_codes[[version_upper]]
    user_defined <- pkgenv[[user_defined_str]]

    built_in_codes <- if (!is.null(built_in) && nrow(built_in) > 0)
      built_in$VS_code else character(0)
    user_codes <- if (NROW(user_defined) > 0)
      user_defined$VS_code else character(0)

    c(built_in_codes, user_codes)
  },
  error = function(e) {
    # Fall back to namespace if pkgenv is unavailable
    cc <- get(".cntrcodes", envir = asNamespace("eq5dsuite"))
    cc$VS_code[cc$Version == version]
  })
}

#' Download and install a single value set
#'
#' @param vs_code Character. The VS_code to download.
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @param meta_row A single-row data frame from the index CSV.
#' @return Logical. TRUE if successful, FALSE otherwise.
#' @keywords internal
install_value_set <- function(vs_code, version, meta_row) {
  url <- paste0(
    .vs_base_url,
    .vs_folder(version), "/",
    vs_code, ".csv"
  )
  vs_data <- tryCatch({
    utils::read.csv(
      curl::curl(url),
      stringsAsFactors = FALSE
    )
  },
  error = function(e) {
    message("eq5dsuite: Could not download ", vs_code,
            ": ", conditionMessage(e))
    NULL
  })
  
  if (is.null(vs_data)) return(FALSE)
  
  if (!all(c("state", "value") %in% colnames(vs_data))) {
    message("eq5dsuite: Invalid format for ", vs_code,
            " \u2014 expected columns 'state' and 'value'")
    return(FALSE)
  }
  
  expected_rows <- switch(version,
                          "3L"  = 243,
                          "5L"  = 3125,
                          "Y3L" = 243
  )
  
  if (nrow(vs_data) != expected_rows) {
    message("eq5dsuite: ", vs_code, " has ", nrow(vs_data),
            " rows (expected ", expected_rows, ")")
    return(FALSE)
  }
  
  tryCatch({
    eqvs_add(
      vs_data,
      version     = version,
      country     = meta_row$Name,
      countryCode = meta_row$Country_code,
      VSCode      = vs_code,
      description = paste0("doi:", meta_row$doi),
      saveOption  = 2
    )
    TRUE
  },
  error = function(e) {
    message("eq5dsuite: Could not install ", vs_code,
            ": ", conditionMessage(e))
    FALSE
  })
}

#' Drop a value set from the installed library
#'
#' Removes a value set from the installed library using
#' \code{eqvs_drop()}. Only user-added value sets can be
#' removed — built-in value sets are protected.
#'
#' @param vs_code Character. The VS_code to remove.
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @param ask Logical. Whether to ask for confirmation.
#'   Defaults to TRUE.
#' @return Logical. TRUE if successful, FALSE otherwise.
#' @keywords internal
drop_value_set <- function(vs_code, version, ask = TRUE) {

  installed <- get_installed_vs_codes(version)
  if (!vs_code %in% installed) {
    message("eq5dsuite: ", vs_code, " is not installed.")
    return(FALSE)
  }

  if (ask && interactive()) {
    message("Remove ", vs_code, " (EQ-5D-", version, ")?")
    response <- readline(
      "Enter [y] to confirm or [n] to cancel: "
    )
    if (tolower(trimws(response)) != "y") {
      message("eq5dsuite: Removal of ", vs_code, " cancelled.")
      return(FALSE)
    }
  }

  tryCatch({
    suppressMessages(
      eqvs_drop(
        country    = vs_code,
        version    = version,
        saveOption = 2
      )
    )
    message("\u2705 ", vs_code, " removed successfully.")
    TRUE
  },
  error = function(e) {
    message("\u274c Could not remove ", vs_code,
            ": ", conditionMessage(e))
    FALSE
  })
}

#' Rename a value set
#'
#' Renames a value set by dropping the old VS_code and
#' reinstalling under the new VS_code. The value data is
#' preserved — only the identifier changes.
#'
#' @param old_vs_code Character. The current VS_code.
#' @param new_vs_code Character. The new VS_code to use.
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @param ask Logical. Whether to ask for confirmation.
#'   Defaults to TRUE.
#' @return Logical. TRUE if successful, FALSE otherwise.
#' @keywords internal
rename_value_set <- function(old_vs_code,
                             new_vs_code,
                             version,
                             ask = TRUE) {

  installed <- get_installed_vs_codes(version)

  if (!old_vs_code %in% installed) {
    message("eq5dsuite: ", old_vs_code, " is not installed.")
    return(FALSE)
  }

  if (new_vs_code %in% installed) {
    message("eq5dsuite: ", new_vs_code,
            " already exists. Drop it first.")
    return(FALSE)
  }

  if (ask && interactive()) {
    message("Rename ", old_vs_code, " to ", new_vs_code,
            " (EQ-5D-", version, ")?")
    response <- readline(
      "Enter [y] to confirm or [n] to cancel: "
    )
    if (tolower(trimws(response)) != "y") {
      message("eq5dsuite: Rename cancelled.")
      return(FALSE)
    }
  }

  vsets_obj <- switch(version,
    "3L"  = ".vsets3L",
    "5L"  = ".vsets5L",
    "Y3L" = ".vsetsY3L"
  )

  vsets <- tryCatch(
    get(vsets_obj, envir = asNamespace("eq5dsuite")),
    error = function(e) NULL
  )

  if (is.null(vsets) || !old_vs_code %in% colnames(vsets)) {
    message("eq5dsuite: Could not extract data for ",
            old_vs_code)
    return(FALSE)
  }

  cc <- get(".cntrcodes", envir = asNamespace("eq5dsuite"))
  old_meta <- cc[cc$VS_code == old_vs_code &
                 cc$Version  == version, ]

  if (nrow(old_meta) == 0) {
    message("eq5dsuite: No metadata found for ", old_vs_code)
    return(FALSE)
  }

  vs_data <- data.frame(
    state = vsets$state,
    value = vsets[[old_vs_code]],
    stringsAsFactors = FALSE
  )
  colnames(vs_data)[2] <- new_vs_code

  message("Removing ", old_vs_code, "...")
  dropped <- drop_value_set(old_vs_code, version, ask = FALSE)
  if (!dropped) return(FALSE)

  message("Installing as ", new_vs_code, "...")
  new_meta            <- old_meta
  new_meta$VS_code    <- new_vs_code
  new_meta$Name       <- gsub(old_vs_code, new_vs_code,
                               old_meta$Name)
  new_meta$Name_short <- gsub(old_vs_code, new_vs_code,
                               old_meta$Name_short)

  tryCatch({
    eqvs_add(
      vs_data,
      version     = version,
      country     = new_meta$Name,
      countryCode = new_meta$Country_code,
      VSCode      = new_vs_code,
      description = paste0("doi:", new_meta$doi),
      saveOption  = 2
    )
    TRUE
  },
  error = function(e) {
    message("\u274c Could not install ", new_vs_code,
            ": ", conditionMessage(e))
    FALSE
  })
}

#' Check for conflicts between user-defined and built-in value sets
#'
#' After applying migrations or installing new value sets, checks
#' whether any built-in value sets share a VS_code with a migration
#' that has been applied. If conflicts are found, warns the user
#' that the built-in value set still exists and will be removed
#' in the next package release.
#'
#' @param version Character. One of "3L", "5L", or "Y3L".
#' @return Invisibly returns a character vector of conflicting
#'   VS_codes, or character(0) if none found.
#' @keywords internal
check_builtin_conflicts <- function(version) {

  # Get built-in VS_codes from package namespace
  cc <- get(".cntrcodes", envir = asNamespace("eq5dsuite"))
  builtin_codes <- cc$VS_code[cc$Version == version]

  # Get all installed codes including user-defined
  all_codes <- get_installed_vs_codes(version)

  # User-defined codes are those not in the built-in list
  user_codes <- setdiff(all_codes, builtin_codes)

  # Fetch migrations to find shadowed built-in codes
  # A built-in is shadowed when:
  # - old_VS_code still exists as a built-in
  # - new_VS_code exists as a user-defined set
  migrations <- tryCatch(
    fetch_migrations(),
    error = function(e) NULL
  )

  shadowed <- character(0)

  if (!is.null(migrations)) {
    version_migrations <- migrations[
      migrations$version == version, ,
      drop = FALSE
    ]
    for (i in seq_len(nrow(version_migrations))) {
      old_code <- trimws(version_migrations$old_VS_code[i])
      new_code <- trimws(version_migrations$new_VS_code[i])
      if (old_code %in% builtin_codes &&
          new_code %in% user_codes) {
        shadowed <- c(shadowed, old_code)
      }
    }
  }

  if (length(shadowed) > 0) {
    message(
      "\n\u26a0\ufe0f  Warning: The following EQ-5D-", version,
      " value set(s) have been superseded by a renamed version:\n",
      paste0("  - ", shadowed, collapse = "\n"), "\n",
      "  The old codes above are still present in the package ",
      "but will be removed in the next release.\n",
      "  To avoid this warning, switch to the new VS_code ",
      "when calculating values."
    )
  }

  invisible(shadowed)
}

#' Update eq5dsuite value sets
#'
#' Connects to the eq5dsuite value sets repository and checks
#' whether new value sets are available for any of the three
#' EQ-5D instruments. Optionally drop or rename existing value
#' sets before checking for updates.
#'
#' This function requires an internet connection for checking
#' and installing new value sets. Drop and rename operations
#' work offline.
#'
#' @param versions Character vector. Which instruments to check.
#'   Defaults to c("3L", "5L", "Y3L").
#' @param ask Logical. Whether to ask for confirmation before
#'   installing, dropping, or renaming. Defaults to TRUE.
#' @param drop Character vector of VS_codes to remove, in the
#'   format "VS_code:version" e.g. c("NL:3L", "US:5L").
#'   Defaults to NULL (no removals).
#' @param rename Named character vector of renames in the format
#'   c("old_VS_code:version" = "new_VS_code") e.g.
#'   c("NL:3L" = "NL_2006"). Defaults to NULL (no renames).
#' @return Invisibly returns a list with elements
#'   \code{checked}, \code{new}, \code{installed},
#'   \code{dropped}, and \code{renamed}.
#' @export
#' @examples
#' \dontrun{
#' # Check for new value sets interactively
#' update_value_sets()
#'
#' # Check specific instrument only
#' update_value_sets(versions = "5L")
#'
#' # Drop a value set
#' update_value_sets(drop = "NL:3L")
#'
#' # Rename a value set
#' update_value_sets(rename = c("NL:3L" = "NL_2006"))
#'
#' # Drop and rename in one call
#' update_value_sets(
#'   drop   = "DE:3L",
#'   rename = c("NL:3L" = "NL_2006", "SI:3L" = "SI_TTO")
#' )
#'
#' # Non-interactive update
#' update_value_sets(ask = FALSE)
#' }
update_value_sets <- function(versions = c("3L", "5L", "Y3L"),
                              ask      = TRUE,
                              drop     = NULL,
                              rename   = NULL) {

  dropped  <- character(0)
  renamed  <- character(0)

  # --------------------------------------------------------
  # 0. Apply any pending migrations from the repository
  # --------------------------------------------------------
  if (curl::has_internet()) {
    apply_pending_migrations(ask = ask)
  }

  # --------------------------------------------------------
  # 1. Handle drops first
  # --------------------------------------------------------
  if (!is.null(drop)) {
    message("eq5dsuite: Processing removals...\n")
    for (entry in drop) {
      parts <- strsplit(entry, ":")[[1]]
      if (length(parts) != 2) {
        message("\u26a0\ufe0f Invalid drop format: '", entry,
                "' \u2014 expected 'VS_code:version'")
        next
      }
      vs_code <- trimws(parts[1])
      version <- trimws(parts[2])
      success <- drop_value_set(vs_code, version, ask = ask)
      if (success) dropped <- c(dropped, entry)
    }
  }

  # --------------------------------------------------------
  # 2. Handle renames
  # --------------------------------------------------------
  if (!is.null(rename)) {
    message("eq5dsuite: Processing renames...\n")
    for (old_entry in names(rename)) {
      new_vs_code <- rename[[old_entry]]
      parts <- strsplit(old_entry, ":")[[1]]
      if (length(parts) != 2) {
        message("\u26a0\ufe0f Invalid rename format: '", old_entry,
                "' \u2014 expected 'VS_code:version'")
        next
      }
      old_vs_code <- trimws(parts[1])
      version     <- trimws(parts[2])
      success <- rename_value_set(
        old_vs_code, new_vs_code, version, ask = ask
      )
      if (success) renamed <- c(renamed, old_entry)
    }
  }

  # --------------------------------------------------------
  # 3. Check for new value sets
  # --------------------------------------------------------
  if (!curl::has_internet()) {
    message("eq5dsuite: No internet connection available.")
    return(invisible(list(
      checked   = versions,
      new       = character(0),
      installed = character(0),
      dropped   = dropped,
      renamed   = renamed
    )))
  }

  all_new       <- list()
  all_installed <- character(0)

  for (version in versions) {
    message("eq5dsuite: Checking EQ-5D-", version,
            " value sets...")

    available <- fetch_available_value_sets(version)
    if (is.null(available)) next

    installed_codes <- get_installed_vs_codes(version)
    new_codes       <- setdiff(available$VS_code,
                               installed_codes)

    if (length(new_codes) == 0) {
      message("eq5dsuite: EQ-5D-", version,
              " value sets are up to date.")
      next
    }

    message("\neq5dsuite: New EQ-5D-", version,
            " value sets available:")
    new_rows <- available[available$VS_code %in% new_codes, ]
    for (i in seq_len(nrow(new_rows))) {
      message("  - ", new_rows$Name[i],
              " (", new_rows$VS_code[i], ")",
              " doi:", new_rows$doi[i])
    }
    all_new[[version]] <- new_rows
  }

  if (length(all_new) == 0) {
    message("\neq5dsuite: All value sets are up to date.")
    set_last_checked()
    return(invisible(list(
      checked   = versions,
      new       = character(0),
      installed = character(0),
      dropped   = dropped,
      renamed   = renamed
    )))
  }

  # Ask for confirmation
  if (ask && interactive()) {
    total_new <- sum(sapply(all_new, nrow))
    message("\n", total_new,
            " new value set(s) available. Install now?")
    response <- readline(
      "Enter [y] to install or [n] to cancel: "
    )
    if (tolower(trimws(response)) != "y") {
      message("eq5dsuite: Update cancelled.")
      return(invisible(list(
        checked   = versions,
        new       = unlist(lapply(all_new, `[[`, "VS_code")),
        installed = character(0),
        dropped   = dropped,
        renamed   = renamed
      )))
    }
  }

  # Install new value sets
  for (version in names(all_new)) {
    new_rows <- all_new[[version]]
    for (i in seq_len(nrow(new_rows))) {
      vs_code  <- new_rows$VS_code[i]
      meta_row <- new_rows[i, ]
      message("Installing ", vs_code, "...")
      success <- install_value_set(vs_code, version, meta_row)
      if (success) {
        all_installed <- c(all_installed, vs_code)
        message("\u2705 ", vs_code, " installed successfully")
      } else {
        message("\u274c ", vs_code, " could not be installed")
      }
    }
  }

  set_last_checked()

  # Check for conflicts between built-in and user-defined
  for (v in versions) {
    check_builtin_conflicts(v)
  }

  message("\neq5dsuite: Update complete. ",
          length(all_installed), " value set(s) installed.")

  invisible(list(
    checked   = versions,
    new       = unlist(lapply(all_new, `[[`, "VS_code")),
    installed = all_installed,
    dropped   = dropped,
    renamed   = renamed
  ))
}

#' Convert EQ-5D dimension scores to a five-digit profile index
#' @param x A data.frame, matrix, or named numeric vector of EQ-5D dimension
#'   scores. Each dimension must contain integer values (typically 1–3 or 1–5).
#' @param dim.names Character vector of length 5 giving the dimension names, in
#'   the conventional MO, SC, UA, PD, AD order.
#' @param na.rm Logical. If \code{FALSE} (default), any \code{NA} in a row
#'   produces \code{NA} in the output. If \code{TRUE}, \code{NA} dimensions are
#'   treated as 0 — use with care, as this silently changes the index value.
#' @param quiet Logical. Suppress informational messages about assumed column /
#'   name order. Default \code{FALSE} so existing scripts see the same messages;
#'   set \code{TRUE} inside pipelines.
#' @return An integer vector with one element per row (data.frame/matrix input)
#'   or a single integer (vector input).  Works inside \code{dplyr::mutate()}
#'   without \code{rowwise()}.
#' @examples
#' # Named vector — scalar usage unchanged
#' toEQ5Dindex(c(mo=1, sc=2, ua=3, pd=1, ad=2))
#' @export

toEQ5Dindex <- function(
    x,
    dim.names = c("mo", "sc", "ua", "pd", "ad"),
    na.rm     = FALSE,
    quiet     = FALSE
) {
  
  # ── 1. Validate dim.names ─────────────────────────────────────────────────
  if (length(dim.names) != 5L)
    stop("'dim.names' must be a character vector of exactly 5 names.")
  if (anyDuplicated(dim.names))
    stop("'dim.names' contains duplicate entries: ",
         paste(dim.names[duplicated(dim.names)], collapse = ", "), ".")
  
  msg <- function(...) if (!quiet) message(...)
  
  # ── 2. Weights: 10^4, 10^3, 10^2, 10^1, 10^0 ────────────────────────────
  weights <- 10L ^ (4L:0L)
  
  # ── 3. Matrix / data.frame branch (vectorised) ───────────────────────────
  if (length(dim(x)) == 2L) {
    
    # Normalise column names to lower-case
    dim.names <- tolower(dim.names)
    if (!is.null(colnames(x))) colnames(x) <- tolower(colnames(x))
    
    # Assign default names if none present
    if (is.null(colnames(x))) {
      if (ncol(x) == 5L) {
        msg("No column names found; assuming conventional order: ",
            paste(toupper(dim.names), collapse = ", "), ".")
        colnames(x) <- dim.names
      } else {
        stop("'x' has no column names and ", ncol(x),
             " columns (expected 5). ",
             "Provide a named data.frame/matrix or set 'dim.names'.")
      }
    }
    
    missing_cols <- setdiff(dim.names, colnames(x))
    if (length(missing_cols))
      stop("Required dimension column(s) not found in 'x': ",
           paste(missing_cols, collapse = ", "), ".")
    
    m <- as.matrix(x[, dim.names, drop = FALSE])
    mode(m) <- "integer"
    
    # NA handling
    if (!na.rm && anyNA(m)) {
      # Propagate NA row-wise without changing non-NA rows
      row_has_na <- rowSums(is.na(m)) > 0L
      result <- integer(nrow(m))
      result[!row_has_na] <- as.integer(
        m[!row_has_na, , drop = FALSE] %*% weights
      )
      result[row_has_na] <- NA_integer_
      return(result)
    }
    
    if (na.rm) m[is.na(m)] <- 0L
    return(as.integer(m %*% weights))
  }
  
  # ── 4. Named-vector branch (scalar, backward-compatible) ─────────────────
  if (length(x) < 5L)
    stop("'x' has ", length(x), " element(s); at least 5 required.")
  
  if (is.null(names(x))) {
    if (length(x) == 5L) {
      msg("No names found in vector; assuming conventional order: ",
          paste(toupper(dim.names), collapse = ", "), ".")
      names(x) <- dim.names
    } else {
      stop("'x' has no names and length != 5. ",
           "Provide a named vector or a data.frame/matrix.")
    }
  } else {
    names(x) <- tolower(names(x))
  }
  
  missing_names <- setdiff(dim.names, names(x))
  if (length(missing_names))
    stop("Required dimension name(s) not found in 'x': ",
         paste(missing_names, collapse = ", "), ".")
  
  vals <- x[dim.names]
  
  if (!na.rm && anyNA(vals))
    return(NA_integer_)
  
  if (na.rm) vals[is.na(vals)] <- 0L
  as.integer(vals %*% weights)
}

#' @title toEQ5Ddims
#' @description Generate dimension vectors based on state index
#' @param x A vector of 5-digit EQ-5D state indexes.
#' @param dim.names A vector of dimension names to be used as names for output columns.
#' @return A data.frame with 5 columns, one for each EQ-5D dimension, with names from dim.names argument.
#' @examples 
#' toEQ5Ddims(c(12345, 54321, 12321))
#' @export
toEQ5Ddims <- function(x, dim.names = c("mo", "sc", "ua", "pd", "ad")) {
  if(!length(dim.names) == 5) stop("Argument dim.names not of length 5.")
  # coerce to integer
  x <- as.integer(x)
  # Remove items outside of bounds
  x[!regexpr("^[1-5]{5}$", x)==1] <- NA
  if(sum(is.na(x))) warning(paste0("Provided vector contained ", sum(is.na(x)), " items not conforming to 5-digit numbers with exclusively digits in the 1-5 range."))
  as.data.frame(outer(X = x, Y = structure(.Data = 10^(4:0), .Names = dim.names), FUN = "%/%") %% 10)
}

#' @title make_all_EQ_states
#' @description Make a data.frame with all health states defined by dimensions
#' @param version Either "3L" or "5L", to signify whether 243 or 3125 states should be generated
#' @param dim.names A vector of dimension names to be used as names for output columns.
#' @param append_index Boolean to indicate whether a column of 5-digit EQ-5D health state indexes should be added to output.
#' @return A data.frame with 5 columns and 243 (-3L) or 3125 (-5L) health states
#' @examples 
#' make_all_EQ_states('3L')
#' @export
make_all_EQ_states <- function(version = "5L", dim.names = c("mo", "sc", "ua", "pd", "ad"), append_index = FALSE) {
  if(!length(dim.names) == 5) stop("Argument dim.names not of length 5.")
  if(!is.vector(version)) stop("version argument is not a vector of length 1")
  if(length(version)>1) {
      message("version argument provided length of more than 1, first element is used.")
      version <- version[[1]]
    }
  if(!toupper(version) %in% c("5L", "3L")) stop('version argument should be either "3L" or "5L"')
  xout <- do.call(expand.grid, structure(rep(list(1:ifelse(version == "5L", 5, 3)), 5),.Names = dim.names[5:1]))[,5:1]
  if(append_index) xout$state <- toEQ5Dindex(xout, dim.names)
  xout
}

#' @title make_all_EQ_indexes
#' @description Make a vector containing all 5-digit EQ-5D indexes for -3L or -5L version.
#' @param version Either "3L" or "5L", to signify whether 243 or 3125 states should be generated
#' @param dim.names A vector of dimension names to be used as names for output columns.
#' @return A vector with 5-digit state indexes for all 243 (-3L) or 3125 (-5L) EQ-5D health states
#' @examples 
#' make_all_EQ_indexes('3L')
#' @export
make_all_EQ_indexes <- function(version = "5L", dim.names = c("mo", "sc", "ua", "pd", "ad")) {
  if(!length(dim.names) == 5) stop("Argument dim.names not of length 5.")
  states <- do.call(make_all_EQ_states, list(version = version, dim.names = dim.names))
  toEQ5Dindex(states, dim.names = dim.names)
  #toEQ5Dindex(do.call(make_all_EQ_states, as.list(match.call()[-1])))
}

#' @title EQ_dummies
#' @description Make a data.frame of all EQ-5D dummies relevant for e.g. regression modeling. 
#' @param df data.frame containing EQ-5D health states.
#' @param version Either "3L" or "5L", to signify EQ-5D instrument version
#' @param dim.names A vector of dimension names to be used as names for output columns.
#' @param drop_level_1 If set to FALSE, dummies for level 1 will be included. Defaults to TRUE.
#' @param add_intercept If set to TRUE, a column containing 1s will be appended. Defaults to FALSE.
#' @param incremental If set to TRUE, incremental dummies will be produced (e.g. MO = 3 will give mo2 = 1, mo3 = 1). Defaults to FALSE.
#' @param append Optional string to be appended to column names.
#' @param prepend Optional string to be prepended to column names.
#' @param return_df If set to TRUE, data.frame is returned, otherwise matrix. Defaults to TRUE.
#' @return A data.frame of dummy variables 
#' @examples 
#' make_dummies(make_all_EQ_states('3L'), '3L')
#' 
#' make_dummies(df = make_all_EQ_states('3L'), 
#'              version =  '3L', 
#'              incremental = TRUE, 
#'              add_intercept = TRUE, 
#'              prepend = "d_")
#' @export
make_dummies <- function(df, 
                         version      = "5L", 
                         dim.names    = c("mo", "sc", "ua", "pd", "ad"), 
                         drop_level_1  = TRUE, 
                         add_intercept = FALSE, 
                         incremental   = FALSE, 
                         prepend       = NULL, 
                         append        = NULL, 
                         return_df     = TRUE) {
  
  if (!length(dim.names) == 5) 
    stop("Argument dim.names not of length 5.")
  if (!length(dim(df) == 2)) 
    stop("Need to provide matrix or data.frame of 2 dimensions.")
  
  # Case-insensitive column matching without renaming
  col_lower    <- tolower(colnames(df))
  dim_lower    <- tolower(dim.names)
  matched_cols <- col_lower %in% dim_lower
  
  if (!all(dim_lower %in% col_lower)) {
    if (NCOL(df) == 5) {
      # Assume conventional order — map actual column names
      # to dim.names without renaming
      message(
        "Column names do not match dim.names. ",
        "Assuming dimensions are in conventional order: ",
        paste(dim.names, collapse = ", "), "."
      )
      # Create a named mapping: dim.name -> actual column name
      col_map <- setNames(colnames(df), dim.names)
    } else {
      stop(
        "Provided dimension names (",
        paste(dim.names, collapse = ", "),
        ") not found in column names (",
        paste(colnames(df), collapse = ", "),
        "). Please rename your columns or update dim.names."
      )
    }
  } else {
    # Match dim.names to actual column names case-insensitively
    col_map <- setNames(
      colnames(df)[match(dim_lower, col_lower)],
      dim.names
    )
  }
  
  vers       <- ifelse(version == "5L", 5, 3)
  startlevel <- ifelse(drop_level_1, 2, 1)
  
  if (incremental) {
    tmp <- 1 * lower.tri(diag(vers), TRUE)[, startlevel:vers]
  } else {
    tmp <- diag(vers)[, startlevel:vers]
  }
  
  # Use actual column names via col_map
  tmpout <- do.call(cbind, lapply(col_map, function(col) {
    tmp[df[[col]], ]
  }))
  
  colnames(tmpout) <- as.vector(
    t(outer(paste0(prepend, dim.names),
            paste0(startlevel:vers, append),
            FUN = paste0))
  )
  
  if (add_intercept) tmpout <- cbind(intercept = 1, tmpout)
  if (return_df)     tmpout <- as.data.frame(tmpout)
  
  tmpout
}


#' @title eqvs_add
#' @description Add user-defined EQ-5D value set and corresponding crosswalk option.
#' @param df A data.frame or file name pointing to csv file. The contents of the data.frame or csv file should be exactly two columns: state, containing a list of all 3125 (for 5L) or 243 (for 3L) EQ-5D health state vectors, and a column of corresponding utility values, with a suitable name.
#' @param version Version of the EQ-5D instrument. Can take values 5L (default) or 3L.
#' @param country Optional string. If not NULL, will be used as a country description for the user-defined value set.
#' @param countryCode Optional string. If not NULL, will be used as the two-digit code for the value set. Must be different from any existing national value set code.
#' @param VSCode Optional string. If not NULL, will be used as the three-digit code for the value set. Must be different from any existing national value set code.
#' @param description Optional string. If not NULL, will be used as a descriptive text for the user-defined value set. 
#' @param saveOption Integer indicating how the cache data should be saved. 1: Do not save (default), 2: Save in package folder, 3: Save in another path.
#' @param savePath A path where the cache data should be saved when `saveOption` is 3. Please use `eqvs_load` to load it in your next session.
#' @return True/False, indicating success or error.
#' @examples 
#' # make nonsense value set
#' new_df <- data.frame(state = make_all_EQ_indexes(), TEST = runif(3125))
#' # Add as value set for Fantasia
#' eqvs_add(
#'    new_df,
#'    version = "5L",
#'    country = 'Fantasia',
#'    countryCode = "MyCountry",
#'    VSCode = "FAN",
#'    saveOption = 1
#' )
#' eq5d5l(55555,country = "FAN")
#' @importFrom utils read.csv
#' @export

eqvs_add <- function(df, version = "5L", country = NULL, countryCode = NULL, VSCode = NULL, description = NULL, saveOption = 1, savePath = NULL) {
  # Ensure saveOption is either 1, 2, or 3
  if (!saveOption %in% c(1, 2, 3)) {
    stop("Invalid 'saveOption'. It must be 1, 2, or 3.")
  }
  
  pkgenv <- getOption("eq.env")
  if(inherits(df, 'character')) {
    if(!file.exists(df)) stop('File named ', df, ' does not appear to exist. Exiting.')
    df <- utils::read.csv(file = df, stringsAsFactors = F)
  } 
  if(!NCOL(df) == 2) stop('df should have exactly two columns.')
  class(df[, 1]) <- 'integer'
  
  # read off version-dependent parameters
  j <- if (version == "5L") 5 else 3
  states_str <- paste0("states_", version)
  eq5d_str <- paste0("EQ-5D-", version)
  uservsets_str <- paste0("uservsets", version)
  user_defined_str <- paste0("user_defined_", version)
  
  # check correct number of rows
  n <- j^5
  if(!NROW(df) == n) stop(paste0("df should have exactly ", n, " rows."))
  # check all states present
  if(!all(df[,1] %in% pkgenv[[states_str]]$state)) stop(paste0("First column of df should contain all ", eq5d_str, " health state indexes exactly once."))
  df <- df[match(pkgenv[[states_str]]$state, df[,1]),]
  # check country
  if(!is.null(country)) {
    if(length(country)>1) {
      warning('Length of country argument > 1, first item used.')
      country <- country[1]
    }
  }
  if(!is.null(description)) {
    if(length(description)>1) {
      warning('Length of description argument > 1, first item used.')
      description <- description[1]
    }
  }
  thisName <- ifelse(is.null(VSCode), colnames(df)[2], VSCode)
  if(thisName %in% colnames(pkgenv[[uservsets_str]])) {
    warning(paste0("New country name already in user-defined ", eq5d_str, " value set list."))
    return(0)
  }
  
  if(any(is.na(df[,2]*-1.1))) stop("Non-numeric values in second column of df.")
  
  # No problems
  tmp <- pkgenv[[uservsets_str]]
  tmp[, thisName] <- df[, 2]
  assign(x = uservsets_str, value = tmp, envir = pkgenv)
  
  tmp <- data.frame(Version = version,
                    Name = ifelse(is.null(country), NA, country), 
                    Name_short = ifelse(is.null(country), NA, country), 
                    Country_code = ifelse(is.null(countryCode), colnames(df)[2], countryCode), 
                    VS_code = ifelse(is.null(VSCode), colnames(df)[2], VSCode),
                    doi = ifelse(is.null(description), NA, description))
  
  if(user_defined_str %in% names(pkgenv)) tmp <- rbind(pkgenv[[user_defined_str]], tmp)
  
  assign(x = user_defined_str, value = tmp, envir = pkgenv)
  
  # Handle save options
  if (saveOption == 2) {
    path <- pkgenv$cache_path
    if (!dir.exists(path)) {
      dir.create(path, recursive = TRUE)
    }
  } else if (saveOption == 3) {
    if (is.null(savePath) || !nzchar(savePath)) {
      stop("Option 3 requires a valid 'savePath'.")
    } else {
      if (!dir.exists(savePath)) {
        stop("The specified 'savePath' does not exist.")
      } else {
        path <- savePath
      }
    }
  }
  
  if(saveOption == 1){
    .fixPkgEnv(saveCache = FALSE)
  }
  if (saveOption == 2 || saveOption == 3) {
    filePath <- file.path(path, 'cache.Rdta.')
    .fixPkgEnv(saveCache = TRUE, filePath = filePath)
    message(paste0('Cache data saved to ', file.path(path, 'cache.Rdta.')))
  }

  message(paste("The value set", country, "was added."))
}

#' @title eqvs_load
#' @description Load cache data from a specified path.
#' @param loadPath The path from which to load the cache data.
#' @return TRUE if loading is successful, FALSE otherwise.
#' @export
eqvs_load <- function(loadPath) {
  pkgenv <- getOption("eq.env")
  if (is.null(loadPath) || !nzchar(loadPath)) {
    stop("A valid 'loadPath' is required.")
  }
  cacheFile <- file.path(loadPath, 'cache.Rdta')
  if (!file.exists(cacheFile)) {
    stop("Cache file not found at specified 'loadPath'.")
  }
  load(cacheFile, envir = pkgenv)
  message(paste0('Cache data loaded from ', cacheFile, '.'))
  return(TRUE)
}


#' @title eqvs_drop
#' @description Drop user-defined EQ-5D value set to reverse crosswalk options.
#' @param version Version of the EQ-5D instrument. Can take values 5L (default) or 3L.
#' @param country Optional string. If NULL, a list of current user-defined value sets will be provided for selection. If set, and matching an existing user-defined value set, a prompt will be given as to whether the value set should be deleted. 
#' @param saveOption Integer indicating how the cache data should be saved. 1: Do not save (default), 2: Save in package folder, 3: Save in another path.
#' @param savePath A path where the cache data should be saved when `saveOption` is 3. Please use `eqvs_load` to load it in your next session.
#' @return True/False, indicating success or error.
#' @examples 
#' \donttest{
#'   # make nonsense value set
#'   new_df <- data.frame(state = make_all_EQ_indexes(), TEST = runif(3125))
#'   # Add as value set for Fantasia
#'   eqvs_add(
#'    new_df,
#'    version = "5L",
#'    country = 'Fantasia',
#'    countryCode = "MyCountry",
#'    VSCode = "FAN",
#'    saveOption = 1
#'   )
#'   # Test the new value set
#'   eq5d5l(55555,country = "FAN")
#'   # Drop value set for Fantasia
#'   eqvs_drop(country = 'FAN', saveOption = 1)
#' }
#' @export
eqvs_drop <- function(country = NULL, version = "5L", saveOption = 1, savePath = NULL) {
  
  # Ensure saveOption is valid
  if (!saveOption %in% c(1, 2, 3)) {
    stop("Invalid 'saveOption'. It must be 1, 2, or 3.")
  }
  
  pkgenv <- getOption("eq.env")
  version <- toupper(version)
  
  user_defined_str <- paste0("user_defined_", version)
  uservsets_str <- paste0("uservsets", version)
  
  if (!user_defined_str %in% names(pkgenv)) {
    message(paste0("No user-defined value sets exist for ", version, ". Exiting."))
    return(FALSE)
  }
  
  udc <- pkgenv[[user_defined_str]]
  
  # Ensure `udc` exists and is not empty
  if (is.null(udc) || nrow(udc) == 0) {
    message("No user-defined value sets found. Exiting.")
    return(FALSE)
  }
  
  # Handle case where multiple value sets exist for the same country
  matched_rows <- which(toupper(udc$Country_code) == toupper(country) | toupper(udc$VS_code) == toupper(country))
  
  if (length(matched_rows) == 0) {
    message("No matching user-defined value set found for country: ", country, ". Exiting.")
    return(FALSE)
  }
  
  if (length(matched_rows) > 1) {
    message(paste0("There are ", length(matched_rows), " value sets available for country code '", country, "'."))
    options_table <- udc[matched_rows, c("Version", "Name", "Country_code", "VS_code", "doi")]
    print(options_table)
    
    # Ask user which value set to delete
    repeat {
      user_input <- readline(prompt = "Please enter the VS_code you want to delete: ")
      if (user_input %in% udc$VS_code[matched_rows]) {
        country <- user_input
        break
      } else {
        message("Invalid VS_code. Please enter one from the table above.")
      }
    }
  } else {
    country <- udc$VS_code[matched_rows]  # If only one match, proceed
  }
  
  # Ask for confirmation
  yesno <- readline(prompt = paste0('Are you sure you want to delete value set "', country, '" for ', version, '? ([Y]es/[N]o) : '))
  if (tolower(yesno) %in% c("yes", "y")) {
    
    message('Removing ', country, ' from user-defined value sets.')
    
    # Remove from user-defined dataset
    udc <- udc[udc$VS_code != country, ]
    if (nrow(udc) > 0) {
      assign(x = user_defined_str, value = udc, envir = pkgenv)
    } else {
      rm(list = user_defined_str, envir = pkgenv)
    }
    
    # Remove from value set matrix
    tmp <- pkgenv[[uservsets_str]]
    if (country %in% colnames(tmp)) {
      tmp <- tmp[, !colnames(tmp) %in% country, drop = FALSE]
      assign(x = uservsets_str, value = tmp, envir = pkgenv)
    }
    
    # Handle save options
    if (saveOption == 2) {
      path <- pkgenv$cache_path
      if (!dir.exists(path)) {
        dir.create(path, recursive = TRUE)
      }
    } else if (saveOption == 3) {
      if (is.null(savePath) || !nzchar(savePath)) {
        stop("Option 3 requires a valid 'savePath'.")
      } else if (!dir.exists(savePath)) {
        stop("The specified 'savePath' does not exist.")
      } else {
        path <- savePath
      }
    }
    
    if (saveOption == 1) {
      .fixPkgEnv(saveCache = FALSE)
    }
    
    if (saveOption == 2 || saveOption == 3) {
      filePath <- file.path(path, 'cache.Rdta.')
      .fixPkgEnv(saveCache = TRUE, filePath = filePath)
      message(paste0('Cache data saved to ', filePath))
    }
    message(paste("The value set", country, "was deleted."))
  } else {
    message("OK. Exiting without deletion.")
  }
}


#' @title eqvs_display
#' @description Display available value sets, which can also be used as
#'   (reverse) crosswalks. Built-in value sets are shown first, followed
#'   by any user-defined value sets added via \code{eqvs_add()}.
#' @param version Version of the EQ-5D instrument. One of \code{"3L"},
#'   \code{"5L"} (default), or \code{"Y3L"}.
#' @param return_df Logical. If \code{TRUE}, returns a data.frame with
#'   all columns (including \code{citation} if present). Defaults to
#'   \code{FALSE}.
#' @param show_citation Logical. If \code{TRUE}, prints the full AMA
#'   citation for each value set after the summary table. Defaults to
#'   \code{FALSE}.
#' @return \code{NULL} invisibly (default), or a data.frame when
#'   \code{return_df = TRUE}.
#' @examples
#' # Display available EQ-5D-5L value sets.
#' eqvs_display(version = "5L")
#' @export
eqvs_display <- function(version       = "5L",
                          return_df     = FALSE,
                          show_citation = FALSE) {

  # Helper: print a compact table (no citation column)
  print_vs_table <- function(df) {
    display_cols <- c("Version", "Name_short", "Country_code", "VS_code", "doi")
    display_cols <- display_cols[display_cols %in% colnames(df)]
    display_df   <- df[, display_cols, drop = FALSE]
    if ("Name_short" %in% colnames(display_df))
      display_df$Name_short <- strtrim(display_df$Name_short, 20)
    if ("doi" %in% colnames(display_df))
      display_df$doi <- strtrim(display_df$doi, 35)
    print(display_df, row.names = FALSE)
  }

  # Helper: align columns of two data frames before rbind
  align_df_cols <- function(df1, df2) {
    for (col in setdiff(colnames(df1), colnames(df2)))
      df2[[col]] <- NA
    for (col in setdiff(colnames(df2), colnames(df1)))
      df1[[col]] <- NA
    df2 <- df2[, colnames(df1), drop = FALSE]
    rbind(df1, df2)
  }

  pkgenv          <- getOption("eq.env")
  version         <- toupper(version)
  user_defined_str <- paste0("user_defined_", version)

  builtin <- pkgenv$country_codes[[version]]
  ud      <- pkgenv[[user_defined_str]]

  # --- Print built-in sets ---
  message("Available national value sets for ", version, " version:")
  print_vs_table(builtin)

  # --- Print user-defined sets ---
  if (NROW(ud) > 0) {
    message("User-defined value sets:")
    print_vs_table(ud)
  } else {
    message("No user-defined value sets available.")
  }

  # --- Optionally print citations ---
  if (show_citation) {
    combined_for_cit <- if (NROW(ud) > 0) align_df_cols(builtin, ud) else builtin
    if ("citation" %in% colnames(combined_for_cit)) {
      message("\nCitations:")
      for (i in seq_len(nrow(combined_for_cit))) {
        cit <- combined_for_cit$citation[i]
        vs  <- combined_for_cit$VS_code[i]
        if (!is.na(cit) && nchar(trimws(cit)) > 0)
          message("[", vs, "] ", cit)
      }
    }
  }

  # --- Return data frame if requested ---
  if (return_df) {
    if (NROW(ud) > 0) {
      combined <- align_df_cols(
        cbind(Type = "Value set",    builtin),
        cbind(Type = "User-defined", ud)
      )
    } else {
      combined <- cbind(Type = "Value set", builtin)
    }
    return(combined)
  }

  invisible(NULL)
}


#' @title eq5d
#' @description Get EQ-5D index values for the -3L, -5L, crosswalk (-3L value set applied to -5L health states),  reverse crosswealk (-5L value set applied to -3L health states), and -Y-3L
#' @param x A vector of 5-digit EQ-5D-3L state indexes or a matrix/data.frame with columns corresponding to EQ-5D state dimensions
#' @param version String indicating which version to use. Options are '5L'  (default), '3L', 'xw', 'xwr', and 'Y3L'.
#' @param country String vector indicating country names or  ISO3166 Alpha 2 / 3 country codes.
#' @param dim.names A vector of dimension names to identify dimension columns.
#' @return A vector of values or data.frame with one column for each value set requested.
#' @examples 
#' # US -3L value set
#' eq5d(c(11111, 12321, 32123, 33333), 'US', '3L') 
#' # Danish and US -5L value sets applied to -3L descriptives, i.e. reverse crosswalk
#' eq5d(make_all_EQ_states('3L'), c('DK', 'US'), 'XWR') 
#' # US -5L value set
#' eq5d(c(11111, 12321, 32153, 55555), 'US', '5L') 
#' @export
eq5d <- function(x, country = NULL, version = '5L', dim.names = c("mo", "sc", "ua", "pd", "ad")) {
  pkgenv <- getOption("eq.env")
  version <- toupper(version)

  valid_versions <- c('3L', '5L', 'Y3L', 'XW', 'RXW', 'XWR', 'CW', 'CWR', 'RCW')
  if (!version %in% valid_versions) stop("No valid argument for EQ-5D version.")
  
  # Normalize version mapping
  version <- c('3L', '5L', 'Y3L', 'XW', 'XWR', 'XWR', 'XW', 'XWR', 'XWR')[match(version, valid_versions)]
  vers <- c('3L', '5L', 'Y3L', '3L', '5L')[match(version, c('3L', '5L', 'Y3L', 'XW', 'XWR'))]
  
  if (length(dim.names) != 5) stop("Argument dim.names must be of length 5.")
  
  if (is.matrix(x) || is.data.frame(x)) {
    if (is.null(colnames(x))) {
      message("No column names detected.")
      if (NCOL(x) == 5) {
        message("Assuming dimensions in order: MO, SC, UA, PD, AD.")
        colnames(x) <- dim.names
      }
    }
    if (!all(dim.names %in% colnames(x))) stop("Provided dimension names not found in input matrix/data.frame.")
    x <- toEQ5Dindex(x = x, dim.names = dim.names)
  }
  
  country <- .fixCountries(country, EQvariant = vers)
  
  # Handle cases where the country input is invalid
  if (any(is.na(country))) {
    invalid_countries <- names(country)[is.na(country)]
    warning("The following countries were not found and will be ignored: ", paste(invalid_countries, collapse = ", "))
    country <- country[!is.na(country)]
  }
  
  if (length(country) == 0) {
    message("No valid countries listed. Available value sets are:")
    eqvs_display(version = vers)
    stop("No valid countries listed.")
  }
  
  # If multiple countries, apply the function iteratively
  if (length(country) > 1) {
    names(country) <- country
    return(do.call(cbind, lapply(country, function(count) eq5d(x, count, version, dim.names))))
  }
  
  # Validate and match states
  xorig <- x
  x <- as.integer(x)
  pattern <- if (version %in% c('3L', 'Y3L', 'XWR')) "^[1-3]{5}$" else "^[1-5]{5}$"
  x[!grepl(pattern, x)] <- NA
  
  # Select the appropriate value set and state mapping
  vset <- switch(version,
                 '3L' = pkgenv$vsets3L_combined,
                 '5L' = pkgenv$vsets5L_combined,
                 'XW' = pkgenv$xwsets,
                 'XWR' = pkgenv$xwrsets,
                 'Y3L' = pkgenv$vsetsY3L_combined)
  svec <- switch(version,
                 '3L' = pkgenv$states_3L,
                 '5L' = pkgenv$states_5L,
                 'XW' = pkgenv$states_5L,
                 'XWR' = pkgenv$states_3L,
                 'Y3L' = pkgenv$states_3L)
  
  # Compute EQ-5D index values
  xout <- rep(NA, length(x))
  xout[!is.na(x)] <- vset[match(x[!is.na(x)], svec$state), country]
  names(xout) <- xorig
  xout
}

#' @title eq5d3l
#' @description
#' Get EQ-5D-3L index values from individual responses to the five
#' dimensions of the EQ-5D-3L.
#' @param x A vector of 5-digit EQ-5D-3L state indexes, or a matrix/data.frame
#'   with columns corresponding to the EQ-5D-3L dimensions.
#' @param country String vector indicating country names or ISO3166 Alpha 2 / 3
#'   country codes.
#' @param dim.names A character vector specifying the names of the EQ-5D-3L
#'   dimensions. Default is `c("mo", "sc", "ua", "pd", "ad")`.
#' @return A numeric vector of EQ-5D-3L values, or a data.frame with one column
#'   for each requested value set.
#' @examples
#' # Example 1: utility values from EQ-5D-3L profile codes
#' eq5d3l(c(11111, 12321, 32123, 33333), country = "US")
#'
#' # Example 2: request multiple value sets
#' eq5d3l(make_all_EQ_states("3L"), country = c("DK", "CA"))
#'
#' # Example 3: use a data.frame with dimension columns
#' df3l <- data.frame(
#'   mo = c(1, 2, 3),
#'   sc = c(1, 2, 2),
#'   ua = c(1, 3, 1),
#'   pd = c(2, 2, 3),
#'   ad = c(1, 1, 2)
#' )
#' eq5d3l(df3l, country = "US")
#'
#' # Example 4: use custom dimension column names
#' df3l_named <- data.frame(
#'   mobility = c(1, 2, 3),
#'   self_care = c(1, 2, 2),
#'   usual_activities = c(1, 3, 1),
#'   pain_discomfort = c(2, 2, 3),
#'   anxiety_depression = c(1, 1, 2)
#' )
#' eq5d3l(
#'   df3l_named,
#'   country = "US",
#'   dim.names = c(
#'     "mobility", "self_care", "usual_activities",
#'     "pain_discomfort", "anxiety_depression"
#'   )
#' )
#' @export
eq5d3l <- function(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad")){
  eq5d(x = x, country = country, version = "3L", dim.names = dim.names)
}

#' @title eq5d5l
#' @description
#' Get EQ-5D-5L index values from individual responses to the five
#' dimensions of the EQ-5D-5L.
#'
#' @param x A vector of 5-digit EQ-5D-5L state indexes, or a matrix/data.frame
#'   with columns corresponding to the EQ-5D-5L dimensions.
#' @param country String vector indicating country names or ISO3166 Alpha 2 / 3
#'   country codes.
#' @param dim.names A character vector specifying the names of the EQ-5D-5L
#'   dimensions. Default is `c("mo", "sc", "ua", "pd", "ad")`.
#'
#' @return A numeric vector of EQ-5D-5L values, or a data.frame with one column
#'   for each requested value set.
#'
#' @examples
#' # Example 1: utility values from EQ-5D-5L profile codes
#' eq5d5l(c(11111, 12321, 32423, 55555), country = "IT")
#'
#' # Example 2: request multiple value sets
#' eq5d5l(make_all_EQ_states("5L"), country = c("ES", "DE"))
#'
#' # Example 3: use a data.frame with dimension columns
#' df5l <- data.frame(
#'   mo = c(1, 2, 5),
#'   sc = c(1, 2, 4),
#'   ua = c(1, 3, 3),
#'   pd = c(2, 4, 2),
#'   ad = c(1, 5, 1)
#' )
#' eq5d5l(df5l, country = "ES")
#'
#' # Example 4: use custom dimension column names from a real-world style dataset
#' df5l_named <- data.frame(
#'   mobility = c(1, 5, 3),
#'   self_care = c(2, 4, 2),
#'   usual_activities = c(3, 3, 1),
#'   pain_discomfort = c(4, 2, 2),
#'   anxiety_depression = c(5, 1, 3)
#' )
#' eq5d5l(
#'   df5l_named,
#'   country = "ES",
#'   dim.names = c(
#'     "mobility", "self_care", "usual_activities",
#'     "pain_discomfort", "anxiety_depression"
#'   )
#' )
#'
#' @export
eq5d5l <- function(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad")){
  eq5d(x = x, country = country, version = "5L", dim.names = dim.names)
}

#' @title eq5dy3l
#' @description
#' Get EQ-5D-Y-3L index values from individual responses to the five
#' dimensions of the EQ-5D-Y-3L.
#' @param x A vector of 5-digit EQ-5D-Y-3L state indexes, or a matrix/data.frame
#'   with columns corresponding to the EQ-5D-Y-3L dimensions.
#' @param country String vector indicating country names or ISO3166 Alpha 2 / 3
#'   country codes.
#' @param dim.names A character vector specifying the names of the EQ-5D-Y-3L
#'   dimensions. Default is `c("mo", "sc", "ua", "pd", "ad")`.
#' @return A numeric vector of EQ-5D-Y-3L values, or a data.frame with one
#'   column for each requested value set.
#' @examples
#' # Example 1: utility values from EQ-5D-Y-3L profile codes
#' eq5dy3l(x = c(11111, 12321, 33333), country = "SI")
#' 
#' # Example 2: request multiple value sets
#' eq5dy3l(make_all_EQ_states("3L"), country = c("ES", "DE"))
#'
#' # Example 3: use a data.frame with dimension columns
#' dfy3l <- data.frame(
#'   mo = c(1, 2, 3),
#'   sc = c(1, 1, 2),
#'   ua = c(1, 2, 3),
#'   pd = c(2, 2, 3),
#'   ad = c(1, 3, 2)
#' )
#' eq5dy3l(dfy3l, country = "SI")
#'
#' # Example 4: use custom dimension column names
#' dfy3l_named <- data.frame(
#'   mobility = c(1, 2, 3),
#'   self_care = c(1, 1, 2),
#'   usual_activities = c(1, 2, 3),
#'   pain_discomfort = c(2, 2, 3),
#'   anxiety_depression = c(1, 3, 2)
#' )
#' eq5dy3l(
#'   dfy3l_named,
#'   country = "SI",
#'   dim.names = c(
#'     "mobility", "self_care", "usual_activities",
#'     "pain_discomfort", "anxiety_depression"
#'   )
#' )
#' @export
eq5dy3l <- function(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad")){
  eq5d(x = x, country = country, version = "Y3L", dim.names = dim.names)
}

##  Reverse crosswalk for R. Using EQ-5D-5L value sets for EQ-5D-3L data.
## 
##  This file is part of eqxwr.
##
##  eqxwr is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 2 of the License, or
##  (at your option) any later version.
##
##  eqxwr is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with eqxwr If not, see <http://www.gnu.org/licenses/>.

.onLoad <- function(libname, pkgname) {
  # Check if 'eq.env' is already an option
  if('eq.env' %in% names(.Options)) {
    pkgenv <- getOption('eq.env')
  } else {
    options("eq.env" = (pkgenv <- new.env(parent=emptyenv())))
  }
  
  # Assign cache path
  cache_path <- find_cache_dir('eq5dsuite')
  assign(x = "cache_path", value = cache_path, envir = pkgenv)

  # Check if cache directory and file exist, then load
  fexist <- FALSE
  cache_file <- file.path(cache_path, "cache.Rdta")
  if (dir.exists(cache_path) && file.exists(cache_file)) {
      fexist <- TRUE
      load(cache_file, envir = pkgenv)
  } 

  .fixPkgEnv(saveCache = FALSE)
}

.fixPkgEnv <- function(saveCache = FALSE, filePath = NULL) {
  pkgenv <- getOption('eq.env')
  
  # if(!'EQrxwmod7' %in% names(pkgenv)) assign(x = "EQrxwmod7", .EQrxwmod7, envir = pkgenv)
  # if(!'vsets5L' %in% names(pkgenv)) assign(x = "vsets5L", .vsets5L, envir = pkgenv)
  # if(!'cntrcodes' %in% names(pkgenv)) assign(x = "cntrcodes", .cntrcodes, envir = pkgenv)
  
  # possible 3L/5L/Y3L states
  if(!'states_3L' %in% names(pkgenv)) assign(x = "states_3L", value = make_all_EQ_states(version = '3L', append_index = T), envir = pkgenv)
  if(!'states_5L' %in% names(pkgenv)) assign(x = "states_5L", value =  make_all_EQ_states(version = '5L', append_index = T), envir = pkgenv)
  
  # generate empty uservsets3L/5L
  if(!'uservsets3L' %in% names(pkgenv)) assign(x = "uservsets3L", value =  pkgenv$states_3L[,"state", drop = F], envir = pkgenv)
  if(!'uservsets5L' %in% names(pkgenv)) assign(x = "uservsets5L", value =  pkgenv$states_5L[,"state", drop = F], envir = pkgenv)
  if(!'uservsetsY3L' %in% names(pkgenv)) assign(x = "uservsetsY3L", value =  pkgenv$states_3L[,"state", drop = F], envir = pkgenv)
  
  # combined core and user-defined sets
  assign(x = "vsets3L_combined", value = merge(.vsets3L, pkgenv$uservsets3L), envir = pkgenv)
  assign(x = "vsets5L_combined", value = merge(.vsets5L, pkgenv$uservsets5L), envir = pkgenv)
  assign(x = "vsetsY3L_combined", value = merge(.vsetsY3L, pkgenv$uservsetsY3L), envir = pkgenv)
  
  # variables used for reverse crosswalk
  if(!'PPP' %in% names(pkgenv)) assign(x = "PPP", value = .EQxwrprob(par = .EQrxwmod7), envir = pkgenv)
  if(!'probs' %in% names(pkgenv)) assign(x = "probs", value = .pstate3t5(pkgenv$PPP), envir = pkgenv)
  if('probs' %in% names(pkgenv)) assign(x = 'xwrsets', value = pkgenv$probs %*% cbind(as.matrix(.vsets5L[, -1, drop = F]), if("uservsets5L" %in% names(pkgenv)) as.matrix(pkgenv$uservsets5L[,-1, drop = F]) else NULL), envir = pkgenv)
  
  # variables used for crosswalk
  if (!'probs5t3' %in% names(pkgenv)) assign(x = "probs5t3", value = .pstate5t3(.EQxwprob), envir = pkgenv)
  if('probs5t3' %in% names(pkgenv)) assign(x = 'xwsets', value = pkgenv$probs5t3 %*% cbind(as.matrix(.vsets3L[, -1, drop = F]), if("uservsets3L" %in% names(pkgenv)) as.matrix(pkgenv$uservsets3L[,-1, drop = F]) else NULL), envir = pkgenv)
  
  # variables used for NICE crosswalk
  if (!'.crosswalk_NICE' %in% names(pkgenv)) {
    assign(x = "crosswalk_NICE", value = .crosswalk_NICE, envir = pkgenv)
  }  
  
  EQvariants <- c('5L' = '5L', '3L' = '3L', "Y3L" = "Y3L")
  assign(x = 'country_codes', envir = pkgenv, value = lapply(EQvariants, function(EQvariant) {
    tmp <- .cntrcodes[.cntrcodes$Version == EQvariant & 
                      .cntrcodes$VS_code %in% colnames(get(paste0('.vsets', EQvariant)))[-1],]
    rownames(tmp) <- NULL
    tmp
  }))
  
  if(saveCache){
    save(list = ls(envir = pkgenv), envir = pkgenv, file = filePath)
  }
  
}

---
title: "Analysing EQ-5D data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analysing EQ-5D data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4
)
library(eq5dsuite)
```

## Introduction

This vignette demonstrates how `eq5dsuite` supports the
full EQ-5D analytical workflow recommended by Devlin et al.
(2020), using the NHS Patient Reported Outcome Measures
(PROMs) dataset bundled with the package. The dataset
includes patients undergoing hip replacement, knee
replacement, groin hernia repair, and varicose vein
surgery, who completed the EQ-5D-3L before and after
their procedure.

```{r load-data}
# The example dataset is bundled with the package
data(example_data)
head(example_data)
```

## Worked example 1: Hip replacement outcomes

This example illustrates a typical single-group longitudinal
workflow, addressing three practical questions: which problems
do patients report before surgery, how does health change
following the procedure, and how large is the overall
health gain?

### Data preparation

```{r wex1-prep}
dim_names <- c("mo", "sc", "ua", "pd", "ad")

# Subset to hip replacement patients
hip_data <- example_data[example_data$procedure == "Hip Replacement", ]

# Add profile code and EQ-5D value
hip_data$profile_code <- toEQ5Dindex(
  x         = hip_data,
  dim.names = dim_names
)
hip_data$value <- eq5d3l(
  hip_data[, dim_names],
  country   = "UK",
  dim.names = dim_names
)

# Pre-operative subset
hip_preop <- hip_data[hip_data$time == "Pre-op", ]
```

### Pre-operative profile distribution

```{r wex1-profile}
eq5d_profile_level_summary(
  df           = hip_preop,
  names_eq5d   = dim_names,
  eq5d_version = "3L"
)
```

Most patients report problems with pain/discomfort,
usual activities, and mobility, consistent with the
profile expected in patients awaiting hip replacement.

### Most common health states

```{r wex1-top-states}
eq5d_profile_top_states(
  df           = hip_preop,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  n            = 5
)
```

### Change between time points: PCHC

The Paretian Classification of Health Change (PCHC)
classifies each patient as Better, Worse, Mixed, or
No Change between time points. All plot functions in
`eq5dsuite` return ggplot2 objects that can be customised
directly:

```{r wex1-pchc, fig.cap = "PCHC classification for hip replacement patients."}
eq5d_profile_pchc_by_group_plot(
  df         = hip_data,
  name_id    = "id",
  names_eq5d = dim_names,
  name_fu    = "time",
  levels_fu  = c("Pre-op", "Post-op")
)$p +
  ggplot2::labs(
    title = "PCHC: hip replacement patients",
    x     = NULL,
    y     = "Percentage of respondents"
  ) +
  ggplot2::theme_minimal()
```

### Dimensions driving improvement

```{r wex1-better, fig.cap = "Dimensions improved among patients classified as Better."}
eq5d_profile_better_dimensions_by_group_plot(
  df         = hip_data,
  name_id    = "id",
  names_eq5d = dim_names,
  name_fu    = "time",
  levels_fu  = c("Pre-op", "Post-op")
)$p +
  ggplot2::theme_minimal()
```

### EQ-5D value summary

```{r wex1-utility}
eq5d_utility_summary(
  df           = hip_data,
  name_fu      = "time",
  levels_fu    = c("Pre-op", "Post-op"),
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)
```

### EQ-VAS summary

```{r wex1-vas}
eq5d_vas_summary(
  df        = hip_data,
  name_vas  = "vas",
  name_fu   = "time",
  levels_fu = c("Pre-op", "Post-op")
)
```

---

## Worked example 2: Comparing two procedures

This example demonstrates a cross-sectional group
comparison using pre-operative data from knee replacement
and groin hernia patients — two groups with very different
levels of pre-operative health burden.

### Data preparation

```{r wex2-prep}
# Subset to two procedures, pre-operative only
procs <- c("Knee Replacement", "Groin Hernia")
comparison_data <- example_data[
  example_data$procedure %in% procs &
  example_data$time == "Pre-op", ]

# Add profile code and EQ-5D value
comparison_data$profile_code <- toEQ5Dindex(
  x         = comparison_data,
  dim.names = dim_names
)
comparison_data$value <- eq5d3l(
  comparison_data[, dim_names],
  country   = "UK",
  dim.names = dim_names
)
```

### Profile comparison by group

```{r wex2-profile}
eq5d_profile_level_summary_by_group(
  df           = comparison_data,
  names_eq5d   = dim_names,
  name_cat     = "procedure",
  eq5d_version = "3L"
)
```

### EQ-5D value comparison

```{r wex2-utility-summary}
eq5d_utility_summary_by_group(
  df            = comparison_data,
  names_eq5d    = dim_names,
  name_groupvar = "procedure",
  eq5d_version  = "3L",
  country       = "UK"
)
```

```{r wex2-utility-plot, fig.cap = "Mean pre-operative EQ-5D values by procedure group."}
eq5d_utility_by_group_plot(
  df            = comparison_data,
  names_eq5d    = dim_names,
  name_groupvar = "procedure",
  eq5d_version  = "3L",
  country       = "UK"
)$p +
  ggplot2::theme_minimal()
```

### LSS analysis by group

The Level Sum Score (LSS) summarises overall severity as
the sum of level scores across the five dimensions (range
5–15 for 3L, 5–25 for 5L). The plot below shows how
EQ-5D values relate to LSS within each procedure group:

```{r wex2-lss-hernia, fig.cap = "EQ-5D values by LSS — Groin Hernia."}
hernia_data <- comparison_data[
  comparison_data$procedure == "Groin Hernia", ]

eq5d_profile_lss_utility_plot(
  hernia_data,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)$p +
  ggplot2::labs(
    title = "Groin Hernia",
    x     = "Level Sum Score (LSS)",
    y     = "EQ-5D value"
  ) +
  ggplot2::scale_x_continuous(limits = c(5, 15),
                               breaks = seq(5, 15, 2)) +
  ggplot2::theme_minimal()
```

```{r wex2-lss-knee, fig.cap = "EQ-5D values by LSS — Knee Replacement."}
knee_data <- comparison_data[
  comparison_data$procedure == "Knee Replacement", ]

eq5d_profile_lss_utility_plot(
  knee_data,
  names_eq5d   = dim_names,
  eq5d_version = "3L",
  country      = "UK"
)$p +
  ggplot2::labs(
    title = "Knee Replacement",
    x     = "Level Sum Score (LSS)",
    y     = "EQ-5D value"
  ) +
  ggplot2::scale_x_continuous(limits = c(5, 15),
                               breaks = seq(5, 15, 2)) +
  ggplot2::theme_minimal()
```

## Further reading

- `vignette("crosswalk-methods")` — when and how to use
  crosswalk methods
- `vignette("custom-value-sets")` — adding and managing
  custom value sets
- `vignette("update-value-sets")` — keeping value sets
  up to date

## References

Devlin N, Parkin D, Janssen B (2020). *Methods for
Analysing and Reporting EQ-5D Data*. Springer, Cham.
<https://doi.org/10.1007/978-3-030-47622-9>

---
title: "Keeping value sets up to date"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Keeping value sets up to date}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)
```

## Introduction

New EQ-5D value sets are published regularly. The
`eq5dsuite` package includes a built-in update system
that allows you to install new value sets without
waiting for a CRAN package update.

## How the update system works

When you load the package, it checks the date of the
last value set update. If more than two months have
passed, you will see a reminder:

```
eq5dsuite: Value sets were last checked more than
2 months ago.
Run eq5dsuite::update_value_sets() to check for
new value sets.
```

## Checking for updates

Run `update_value_sets()` to check for new value sets:

```{r update-basic, eval = FALSE}
update_value_sets()
```

If new value sets are available, you will see:

```
eq5dsuite: Checking EQ-5D-3L value sets...

eq5dsuite: New EQ-5D-3L value sets available:
  - China 2026 (CN_2026) doi:10.xxxx/xxxxx

1 new value set(s) available. Install now?
Enter [y] to install or [n] to cancel:
```

## Checking specific instruments

```{r update-specific, eval = FALSE}
# Check only EQ-5D-5L value sets
update_value_sets(versions = "5L")

# Check 3L and Y3L only
update_value_sets(versions = c("3L", "Y3L"))
```

## Non-interactive updates

For use in scripts or automated workflows:

```{r update-noninteractive, eval = FALSE}
# Install without asking for confirmation
update_value_sets(ask = FALSE)
```

## Automatic migrations

When value set codes change across the package ecosystem
(for example, when a country publishes a second value set
and the original code is disambiguated), `update_value_sets()`
automatically applies the necessary renames before checking
for new value sets. This ensures your installed value sets
remain consistent with the online repository without
requiring manual intervention.

## Value sets repository

All available value sets are maintained in the
[eq5dsuite-value-sets](https://github.com/MathsInHealth/eq5dsuite-value-sets)
repository. You can browse the available value sets and
their metadata there directly.

---
title: "Custom value sets"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Custom value sets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)
```

## Introduction

New EQ-5D value sets are published regularly. Rather than
waiting for a package update, `eq5dsuite` allows you to
add your own value sets using `eqvs_add()`. Custom value
sets work with all scoring and analysis functions in the
package.

## Adding a custom value set

A custom value set must be a data frame with two columns:

- `state` — the EQ-5D health state code (integer)
- A column named after the VS_code containing utility values

```{r add-custom-vs}
# Generate all valid 3L health state codes
states <- make_all_EQ_indexes(version = "3L")

# Create a custom value set (random values for illustration)
set.seed(42)
custom_vs <- data.frame(
  state  = states,
  MY_VS  = runif(243, min = -0.5, max = 1.0)
)

head(custom_vs)
```

```{r register-vs, eval = FALSE}
# Register for current session only (saveOption = 1)
eqvs_add(
  custom_vs,
  version     = "3L",
  country     = "My Country",
  countryCode = "MC",
  VSCode      = "MY_VS",
  description = "Custom value set — demonstration",
  saveOption  = 1
)

# Verify it appears in the list
eqvs_display(version = "3L")

# Use it for value calculation
eq5d3l(c(11111, 12321), country = "MY_VS")
```

## Saving a custom value set permanently

To persist a custom value set across R sessions, use
`saveOption = 2`:

```{r save-vs, eval = FALSE}
eqvs_add(
  custom_vs,
  version     = "3L",
  country     = "My Country",
  countryCode = "MC",
  VSCode      = "MY_VS",
  description = "Custom value set — permanent",
  saveOption  = 2  # saves to user cache
)
```

Load it in a new session with:

```{r load-vs, eval = FALSE}
eqvs_load(country = "MY_VS", version = "3L")
```

## Removing a custom value set

```{r drop-vs, eval = FALSE}
eqvs_drop(
  country    = "MY_VS",
  version    = "3L",
  saveOption = 1  # remove from current session only
)
```

## Validation

`eqvs_add()` validates the value set before registration:

- For 3L: checks all 243 health states are present
- For 5L: checks all 3125 health states are present
- Checks values are numeric with no duplicates

## Using custom value sets with crosswalk methods

Custom value sets are fully compatible with all crosswalk
functions:

```{r custom-xw, eval = FALSE}
# Use custom 3L value set with original crosswalk
eqxw(
  eq5d5l_data,
  country   = "MY_VS",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)
```

## Keeping value sets up to date

Use `update_value_sets()` to automatically check for
and install newly published value sets from the
eq5dsuite online repository:

```{r update-vs, eval = FALSE}
update_value_sets()
```

See `vignette("update-value-sets")` for full details.

---
title: "Crosswalk methods"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Crosswalk methods}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)
```

## Introduction

Crosswalk methods allow EQ-5D values to be estimated when
a country-specific value set exists for one instrument
version but not another. `eq5dsuite` implements three
widely used approaches:

- **Original crosswalk** (`eqxw`) — maps EQ-5D-5L responses
  to EQ-5D-3L values (van Hout et al., 2012)
- **Reverse crosswalk** (`eqxwr`) — maps EQ-5D-3L responses
  to EQ-5D-5L values (van Hout and Shaw, 2021)
- **UK-specific crosswalk** (`eqxw_UK`) — NICE-recommended
  mapping for UK analyses (Hernandez Alava et al., 2023)

## When to use crosswalk methods

Use crosswalk methods when:

- You have EQ-5D-5L data but only a 3L value set exists
  for your country → use `eqxw()`
- You have EQ-5D-3L data but only a 5L value set exists
  → use `eqxwr()`
- You are conducting a NICE technology appraisal and have
  EQ-5D-5L data → use `eqxw_UK()` until a NICE-approved
  5L value set is available

## Original crosswalk: eqxw()

```{r eqxw-example}
eq5d5l_data <- data.frame(
  id = 1:5,
  mo = c(1, 2, 3, 1, 2),
  sc = c(1, 1, 2, 1, 3),
  ua = c(2, 1, 3, 1, 2),
  pd = c(2, 3, 2, 1, 4),
  ad = c(1, 2, 1, 3, 2)
)

# Apply crosswalk using UK 3L value set
eq5d5l_data$value_xw <- eqxw(
  eq5d5l_data,
  country   = "UK",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

eq5d5l_data
```

## Reverse crosswalk: eqxwr()

```{r eqxwr-example}
eq5d3l_data <- data.frame(
  id = 1:5,
  mo = c(1, 2, 1, 3, 2),
  sc = c(1, 1, 2, 2, 1),
  ua = c(1, 2, 1, 3, 2),
  pd = c(2, 2, 1, 3, 3),
  ad = c(1, 1, 2, 2, 1)
)

# Apply reverse crosswalk using Spain 5L value set
eq5d3l_data$value_rxw <- eqxwr(
  eq5d3l_data,
  country   = "ES",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

eq5d3l_data
```

## UK-specific crosswalk: eqxw_UK()

The UK-specific crosswalk requires age and sex in addition
to the five EQ-5D dimensions:

```{r eqxw-uk-example}
uk_data <- data.frame(
  id   = 1:5,
  mo   = c(1, 2, 1, 3, 2),
  sc   = c(1, 1, 2, 2, 1),
  ua   = c(1, 2, 1, 3, 2),
  pd   = c(2, 2, 1, 3, 3),
  ad   = c(1, 1, 2, 2, 1),
  age  = c(45, 62, 38, 71, 55),
  male = c(1,  0,  1,  0,  1)
)

uk_data$value_uk <- eqxw_UK(
  uk_data,
  age  = "age",
  male = "male"
)

uk_data
```

## Crosswalk compatibility with custom value sets

All crosswalk functions are compatible with custom value
sets added via `eqvs_add()`. The mapping and valuation
steps are independent, so any compatible value set can
be used without modification.

## References

van Hout B, et al. (2012). Interim scoring for the
EQ-5D-5L. *Value in Health*, 15(5), 708--715.

van Hout B, Shaw JW (2021). Mapping EQ-5D-3L to EQ-5D-5L.
*Value in Health*, 24(9), 1285--1293.

Hernandez Alava M, et al. (2023). Estimating the
relationship between EQ-5D-5L and EQ-5D-3L.
*PharmacoEconomics*, 41(2), 199--207.

---
title: "Getting started with eq5dsuite"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with eq5dsuite}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
library(eq5dsuite)
```

## Introduction

The `eq5dsuite` package provides standardised tools for
calculating EQ-5D preference-based values and analysing
EQ-5D data following the recommendations of Devlin et al.
(2020). This vignette introduces the core workflow:
loading the package, exploring available value sets, and
calculating EQ-5D values from profile data.

## Installation

Install the released version from CRAN:

```{r install-cran, eval = FALSE}
install.packages("eq5dsuite")
```

Install the development version from GitHub:

```{r install-github, eval = FALSE}
remotes::install_github("MathsInHealth/eq5dsuite-r")
```

## Available value sets

The package includes country-specific value sets for
the EQ-5D-3L, EQ-5D-5L, and EQ-5D-Y-3L instruments.
Use `eqvs_display()` to see all available sets:

```{r display-value-sets}
# List all available EQ-5D-3L value sets
eqvs_display(version = "3L")
```

## Calculating EQ-5D values

### EQ-5D-3L

The most common workflow uses a data frame with one column
per EQ-5D dimension. The `dim.names` argument maps your
column names to the five dimensions in the standard order
(mobility, self-care, usual activities, pain/discomfort,
anxiety/depression):

```{r eq5d3l-example}
# Example data with EQ-5D-3L responses
eq5d3l_data <- data.frame(
  id = 1:5,
  mo = c(1, 2, 1, 3, 2),
  sc = c(1, 1, 2, 2, 1),
  ua = c(1, 2, 1, 3, 2),
  pd = c(2, 2, 1, 3, 3),
  ad = c(1, 1, 2, 2, 1)
)

# Calculate EQ-5D-3L values using the UK value set
eq5d3l_data$value <- eq5d3l(
  eq5d3l_data,
  country   = "UK",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

eq5d3l_data
```

### EQ-5D-5L

```{r eq5d5l-example}
eq5d5l_data <- data.frame(
  id = 1:5,
  mo = c(1, 2, 3, 1, 2),
  sc = c(1, 1, 2, 1, 3),
  ua = c(2, 1, 3, 1, 2),
  pd = c(2, 3, 2, 1, 4),
  ad = c(1, 2, 1, 3, 2)
)

eq5d5l_data$value <- eq5d5l(
  eq5d5l_data,
  country   = "IT",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

eq5d5l_data
```

### EQ-5D-Y-3L

```{r eq5dy3l-example}
eq5dy3l_data <- data.frame(
  id = 1:5,
  mo = c(1, 2, 1, 2, 3),
  sc = c(1, 1, 2, 1, 2),
  ua = c(2, 1, 1, 3, 2),
  pd = c(1, 2, 3, 2, 1),
  ad = c(2, 1, 2, 1, 3)
)

eq5dy3l_data$value <- eq5dy3l(
  eq5dy3l_data,
  country   = "SI",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)

eq5dy3l_data
```

## Custom value sets

If a value set is not yet included in the package, you
can add your own using `eqvs_add()`:

```{r custom-vs, eval = FALSE}
# Create a custom value set data frame
custom_vs <- data.frame(
  state = make_all_EQ_indexes(version = "3L"),
  MY_VS = runif(243)
)

# Register it temporarily for this session
eqvs_add(
  custom_vs,
  version     = "3L",
  country     = "My Country",
  countryCode = "MC",
  VSCode      = "MC",
  description = "Custom value set for demonstration",
  saveOption  = 1
)

# Use the custom value set
eq5d3l(c(11111, 12321), country = "MC")
```

## Keeping value sets up to date

New EQ-5D value sets are published regularly. Use
`update_value_sets()` to check for and install new
value sets from the online repository:

```{r update-vs, eval = FALSE}
update_value_sets()
```

## Next steps

- `vignette("analysing-eq5d-data")` — complete analytical
  workflow using NHS PROMs data
- `vignette("crosswalk-methods")` — when and how to use
  crosswalk methods
- `vignette("custom-value-sets")` — adding and managing
  custom value sets
- `?eq5dsuite` — full package documentation

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{toEQ5Ddims}
\alias{toEQ5Ddims}
\title{toEQ5Ddims}
\usage{
toEQ5Ddims(x, dim.names = c("mo", "sc", "ua", "pd", "ad"))
}
\arguments{
\item{x}{A vector of 5-digit EQ-5D state indexes.}

\item{dim.names}{A vector of dimension names to be used as names for output columns.}
}
\value{
A data.frame with 5 columns, one for each EQ-5D dimension, with names from dim.names argument.
}
\description{
Generate dimension vectors based on state index
}
\examples{
toEQ5Ddims(c(12345, 54321, 12321))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.get_names}
\alias{.get_names}
\title{Replace NULL names with default values}
\usage{
.get_names(df = NULL, ...)
}
\arguments{
\item{df}{a data frame; only used/supplied if levels_fu needs to be defined}

\item{...}{a list of parameters consisting of any/all of `names_eq5d`, `name_fu`, `levels_fu`, `eq5d_version`, and `name_vas`.}
}
\value{
a list of parameters with null entries replaced with default values.
}
\description{
This function takes in a list of parameters, which would be column names of the input data frame, and checks if they are null. Any nulls are replaced with default values, and the updated list of parameters is returned.
}
\examples{
.get_names(names_eq5d = c("mo", "sc", "ua", "pd", "ad"))
.get_names(names_eq5d = NULL, eq5d_version = NULL, name_vas = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{drop_value_set}
\alias{drop_value_set}
\title{Drop a value set from the installed library}
\usage{
drop_value_set(vs_code, version, ask = TRUE)
}
\arguments{
\item{vs_code}{Character. The VS_code to remove.}

\item{version}{Character. One of "3L", "5L", or "Y3L".}

\item{ask}{Logical. Whether to ask for confirmation.
Defaults to TRUE.}
}
\value{
Logical. TRUE if successful, FALSE otherwise.
}
\description{
Removes a value set from the installed library using
\code{eqvs_drop()}. Only user-added value sets can be
removed — built-in value sets are protected.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{set_last_checked}
\alias{set_last_checked}
\title{Set the date of the last value set check}
\usage{
set_last_checked(date = Sys.Date())
}
\arguments{
\item{date}{A \code{Date} object. Defaults to \code{Sys.Date()}.}
}
\value{
Invisibly returns \code{date}.
}
\description{
Writes the given date to the user cache directory so that future
calls to \code{get_last_checked()} and \code{is_update_due()} can
determine how long ago the check was performed.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{figure_1_3_1}
\alias{figure_1_3_1}
\title{Figure 1.3.1: EQ-5D values plotted against LSS}
\usage{
figure_1_3_1(df, names_eq5d = NULL, eq5d_version = NULL, country)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country. 
This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
Summary plot and data used for plotting
}
\description{
Figure 1.3.1: EQ-5D values plotted against LSS
}
\examples{
df <- data.frame(make_all_EQ_states(version = "5L"))
tmp <- figure_1_3_1(
 df, 
 names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
 eq5d_version = "5L", 
 country = "US"
)
tmp$p
tmp$plot_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{table_3_2}
\alias{table_3_2}
\title{Table 3.2 EQ-5D values: by groupvar}
\usage{
table_3_2(df, names_eq5d = NULL, name_groupvar, eq5d_version = NULL, country)
}
\arguments{
\item{df}{Data frame with the EQ-5D, follow-up and grouping columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_groupvar}{Character string for the grouping column}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country.}
}
\value{
Summary data frame
}
\description{
Table 3.2 EQ-5D values: by groupvar
}
\examples{
table_3_2(
  example_data,
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_groupvar = "procedure",
  eq5d_version = "3L",
  country = "UK"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_pchc_with_no_problems_table}
\alias{eq5d_profile_pchc_with_no_problems_table}
\title{eq5d_profile_pchc_with_no_problems_table: Changes in health according to the PCHC, taking account of those with no problems}
\usage{
eq5d_profile_pchc_with_no_problems_table(
  df,
  name_id,
  name_groupvar = NULL,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{Data frame with the EQ-5D, grouping, id and follow-up columns}

\item{name_id}{Character string for the patient id column}

\item{name_groupvar}{Character string for the grouping column. If NULL (default), the analysis is performed on the full population.}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column.
If NULL (default value), the levels will be ordered in the order of appearance in df.}
}
\value{
Summary data frame
}
\description{
eq5d_profile_pchc_with_no_problems_table: Changes in health according to the PCHC, taking account of those with no problems
}
\examples{
eq5d_profile_pchc_with_no_problems_table(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c("Pre-op" , "Post-op")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.check_uniqueness}
\alias{.check_uniqueness}
\title{Check the uniqueness of groups
This function takes a data frame `df` and a vector of columns `group_by`, and checks whether the combinations of values in the columns specified by `group_by` are unique. If the combinations are not unique, a warning message is printed.}
\usage{
.check_uniqueness(df, group_by)
}
\arguments{
\item{df}{A data frame.}

\item{group_by}{A character vector of column names in `df` that specify the groups to check for uniqueness.}
}
\value{
No return value, called for side effects: it will stop with an error if any group combinations are not unique.
}
\description{
Check the uniqueness of groups
This function takes a data frame `df` and a vector of columns `group_by`, and checks whether the combinations of values in the columns specified by `group_by` are unique. If the combinations are not unique, a warning message is printed.
}
\examples{
df <- data.frame(id = c(1, 1, 1, 1, 2, 2),
                 fu = rep(c("baseline", "follow-up"), 3),
                 value = rnorm(6))
.check_uniqueness(df, c("id", "fu"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_density_curve}
\alias{eq5d_profile_density_curve}
\title{eq5d_profile_density_curve: Generate a Health State Density Curve (HSDC) for EQ-5D Data}
\usage{
eq5d_profile_density_curve(df, names_eq5d, eq5d_version)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}
}
\value{
A list containing:
  \item{plot_data}{A data frame with the cumulative distribution of profiles}
  \item{p}{A ggplot2 object showing the Health State Density Index}
}
\description{
This function calculates and plots the Health State Density Curve (HSDC) for a given
EQ-5D dataset. It concatenates dimension values to form health state profiles, filters
out invalid states based on the specified EQ-5D version, then computes the cumulative
distribution of profiles (profiles vs. observations). A diagonal reference line
indicates a perfectly even distribution. The function also calculates the Health State
Density Index (HSDI), representing how sharply the observed distribution deviates from
the diagonal.
}
\examples{
figure <- eq5d_profile_density_curve(
            df = example_data, 
            names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
            eq5d_version = "3L"
          )
figure$plot_data
figure$p
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_utility_norms_comparison}
\alias{eq5d_utility_norms_comparison}
\title{eq5d_utility_norms_comparison:EQ-5D values: by age and groupvar}
\usage{
eq5d_utility_norms_comparison(
  df,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL,
  name_groupvar = NULL,
  name_age,
  eq5d_version = NULL,
  country
)
}
\arguments{
\item{df}{Data frame with the EQ-5D, age, follow-up and grouping columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column.
If NULL (default value), the levels will be ordered in the order of appearance in df.}

\item{name_groupvar}{Character string for the grouping column. If NULL (default), the analysis is performed on the full population.}

\item{name_age}{Character string for the age column}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country. 
This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
Summary data frame
}
\description{
eq5d_utility_norms_comparison:EQ-5D values: by age and groupvar
}
\examples{
example_data$ageband <- factor(
  example_data$ageband,
  levels = c("20 to 29", "30 to 39", "40 to 49", "50 to 59", "60 to 69", "70 to 79", "80 to 89")
)
example_data <- example_data[example_data$gender \%in\% c("Male", "Female"),]
eq5d_utility_norms_comparison(
  example_data,
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c('Pre-op', 'Post-op'),
  name_groupvar = "gender",
  name_age = "ageband",
  eq5d_version = "3L",
  country = "UK"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_vas_summary}
\alias{eq5d_vas_summary}
\title{eq5d_vas_summary: EQ VAS Score by timepoints}
\usage{
eq5d_vas_summary(df, name_vas = NULL, name_fu = NULL, levels_fu = NULL)
}
\arguments{
\item{df}{Data frame with the VAS and the follow-up columns}

\item{name_vas}{Character string for the VAS column}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column.}
}
\value{
Summary data frame
}
\description{
eq5d_vas_summary: EQ VAS Score by timepoints
}
\examples{
eq5d_vas_summary(
  example_data,
  name_vas = 'vas', 
  name_fu = 'time', 
  levels_fu = c('Pre-op', 'Post-op')
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_data}
\alias{example_data}
\title{example_data}
\format{
A data frame with 10000 rows and 14 variables:
\describe{
  \item{\code{id}}{double Patient id}
  \item{\code{time}}{character Follow-up (Pre-op / Post-op)}
  \item{\code{mo}}{double EQ-5D-5L Mobility dimension}
  \item{\code{sc}}{double EQ-5D-5L Self-care dimension}
  \item{\code{ua}}{double EQ-5D-5L Usual activities dimension}
  \item{\code{pd}}{double EQ-5D-5L Pain / discomfort dimension}
  \item{\code{ad}}{double EQ-5D-5L Anxiety/depression dimension}
  \item{\code{vas}}{double Value of the VAS scale measurememnt}
  \item{\code{providercode}}{character Provider code}
  \item{\code{procedure}}{character Type of surgery}
  \item{\code{year}}{character Year of intervention}
  \item{\code{ageband}}{character Age in pre-defined ranges} 
  \item{\code{gender}}{character Patient's gender (Female / Male)}
}
}
\usage{
data(example_data)
}
\description{
A dataset containing patient-level data in a long format.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqxwr.R
\name{.EQxwrprob}
\alias{.EQxwrprob}
\title{.EQxwrprob}
\usage{
.EQxwrprob(par = NULL)
}
\arguments{
\item{par}{Matrix of model parameters}
}
\value{
An 243 * 25 matrix with probabilities for state level transitions.
}
\description{
Takes a matrix of parameters for reverse crosswalk model, returns 243 x 25 matrix of state/level transition probabilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eqvs_load}
\alias{eqvs_load}
\title{eqvs_load}
\usage{
eqvs_load(loadPath)
}
\arguments{
\item{loadPath}{The path from which to load the cache data.}
}
\value{
TRUE if loading is successful, FALSE otherwise.
}
\description{
Load cache data from a specified path.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqxw.R
\name{eqxw_UK}
\alias{eqxw_UK}
\title{eqxw_UK}
\usage{
eqxw_UK(x, age, male, dim.names = c("mo", "sc", "ua", "pd", "ad"), bwidth = 0)
}
\arguments{
\item{x}{A vector of 5-digit EQ-5D-5L states (domain scores) or a summary score.}

\item{age}{A numeric vector or column name (if `x` is a data frame). Can be either:
(1) a numeric age between 18 and 100, which will be automatically grouped into NICE-defined age bands (18-35, 35-45, 45-55, 55-65, +65), or
(2) a factor/character/numeric vector already representing the NICE age bands with values 1-5 indicating age bands (18-35, 35-45, 45-55, 55-65, +65).}

\item{male}{A numeric vector (1=male, 0=female) or column name indicating gender.}

\item{dim.names}{A vector of dimension names for EQ-5D states (default: c("mo", "sc", "ua", "pd", "ad")).}

\item{bwidth}{Numeric. Bandwidth for kernel smoothing when using summary scores.}
}
\value{
A vector or data frame with crosswalked EQ-5D-3L utilities.
}
\description{
Crosswalks EQ-5D-5L responses to EQ-5D-3L utilities using NICE's mapping.
}
\examples{
eqxw_UK(c(11111, 12345, 32423, 55555), age = c(30, 40, 55, 70), male = c(1, 0, 1, 0))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{table_2_2}
\alias{table_2_2}
\title{Table 2.2: EQ VAS Scores frequency of mid-points}
\usage{
table_2_2(df, name_vas = NULL, add_na_total = TRUE)
}
\arguments{
\item{df}{Data frame with the VAS column}

\item{name_vas}{Character string for the VAS column}

\item{add_na_total}{Logical, whether to add summary of the missing, and across the Total, data}
}
\value{
Summary data frame
}
\description{
Table 2.2: EQ VAS Scores frequency of mid-points
}
\examples{
table_2_2(
  example_data,
  name_vas = 'vas', 
  add_na_total =  TRUE
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{is_update_due}
\alias{is_update_due}
\title{Check whether a value set update check is due}
\usage{
is_update_due(threshold_days = 60)
}
\arguments{
\item{threshold_days}{A single positive integer giving the number of
days after which an update check is considered overdue. Defaults
to \code{60}.}
}
\value{
A single logical value.
}
\description{
Compares the date of the last value set check (from the user cache)
against \code{Sys.Date()}. Returns \code{TRUE} if more than
\code{threshold_days} days have elapsed, indicating that the user
should be prompted to run \code{update_value_sets()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{make_dummies}
\alias{make_dummies}
\title{EQ_dummies}
\usage{
make_dummies(
  df,
  version = "5L",
  dim.names = c("mo", "sc", "ua", "pd", "ad"),
  drop_level_1 = TRUE,
  add_intercept = FALSE,
  incremental = FALSE,
  prepend = NULL,
  append = NULL,
  return_df = TRUE
)
}
\arguments{
\item{df}{data.frame containing EQ-5D health states.}

\item{version}{Either "3L" or "5L", to signify EQ-5D instrument version}

\item{dim.names}{A vector of dimension names to be used as names for output columns.}

\item{drop_level_1}{If set to FALSE, dummies for level 1 will be included. Defaults to TRUE.}

\item{add_intercept}{If set to TRUE, a column containing 1s will be appended. Defaults to FALSE.}

\item{incremental}{If set to TRUE, incremental dummies will be produced (e.g. MO = 3 will give mo2 = 1, mo3 = 1). Defaults to FALSE.}

\item{prepend}{Optional string to be prepended to column names.}

\item{append}{Optional string to be appended to column names.}

\item{return_df}{If set to TRUE, data.frame is returned, otherwise matrix. Defaults to TRUE.}
}
\value{
A data.frame of dummy variables
}
\description{
Make a data.frame of all EQ-5D dummies relevant for e.g. regression modeling.
}
\examples{
make_dummies(make_all_EQ_states('3L'), '3L')

make_dummies(df = make_all_EQ_states('3L'), 
             version =  '3L', 
             incremental = TRUE, 
             add_intercept = TRUE, 
             prepend = "d_")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{make_all_EQ_states}
\alias{make_all_EQ_states}
\title{make_all_EQ_states}
\usage{
make_all_EQ_states(
  version = "5L",
  dim.names = c("mo", "sc", "ua", "pd", "ad"),
  append_index = FALSE
)
}
\arguments{
\item{version}{Either "3L" or "5L", to signify whether 243 or 3125 states should be generated}

\item{dim.names}{A vector of dimension names to be used as names for output columns.}

\item{append_index}{Boolean to indicate whether a column of 5-digit EQ-5D health state indexes should be added to output.}
}
\value{
A data.frame with 5 columns and 243 (-3L) or 3125 (-5L) health states
}
\description{
Make a data.frame with all health states defined by dimensions
}
\examples{
make_all_EQ_states('3L')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{toEQ5Dindex}
\alias{toEQ5Dindex}
\title{Convert EQ-5D dimension scores to a five-digit profile index}
\usage{
toEQ5Dindex(
  x,
  dim.names = c("mo", "sc", "ua", "pd", "ad"),
  na.rm = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{A data.frame, matrix, or named numeric vector of EQ-5D dimension
scores. Each dimension must contain integer values (typically 1–3 or 1–5).}

\item{dim.names}{Character vector of length 5 giving the dimension names, in
the conventional MO, SC, UA, PD, AD order.}

\item{na.rm}{Logical. If \code{FALSE} (default), any \code{NA} in a row
produces \code{NA} in the output. If \code{TRUE}, \code{NA} dimensions are
treated as 0 — use with care, as this silently changes the index value.}

\item{quiet}{Logical. Suppress informational messages about assumed column /
name order. Default \code{FALSE} so existing scripts see the same messages;
set \code{TRUE} inside pipelines.}
}
\value{
An integer vector with one element per row (data.frame/matrix input)
  or a single integer (vector input).  Works inside \code{dplyr::mutate()}
  without \code{rowwise()}.
}
\description{
Convert EQ-5D dimension scores to a five-digit profile index
}
\examples{
# Named vector — scalar usage unchanged
toEQ5Dindex(c(mo=1, sc=2, ua=3, pd=1, ad=2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_change_summary}
\alias{eq5d_profile_change_summary}
\title{eq5d_profile_change_summary: Frequency of levels by dimensions, by follow-up}
\usage{
eq5d_profile_change_summary(
  df,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL,
  eq5d_version = NULL
)
}
\arguments{
\item{df}{Data frame with the EQ-5D and follow-up columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column. If NULL, the function will check if there is a column named "follow-up" or "fu", in which case the first of those will be used.}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column. 
If NULL (default value), the levels will be ordered in the order of appearance in df.}

\item{eq5d_version}{Version of the EQ-5D instrument}
}
\value{
Summary data frame.
}
\description{
eq5d_profile_change_summary: Frequency of levels by dimensions, by follow-up
}
\examples{
eq5d_profile_change_summary(
  df = example_data,
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c("Pre-op" , "Post-op"),
  eq5d_version = "3L"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.summary_table_4_4}
\alias{.summary_table_4_4}
\title{Summary wrapper for Table 4.4}
\usage{
.summary_table_4_4(df, group_by)
}
\arguments{
\item{df}{A data frame.}

\item{group_by}{A character vector of names of variables by which to group the data.}
}
\value{
A data frame with the summary statistics.
}
\description{
This internal function creates a summary of the data frame for Table 4.4. 
It groups the data by the variables specified in `group_by` and calculates various summary statistics.
}
\examples{
df <- data.frame(group = c("A", "A", "B", "B"), 
                 utility = c(0.5, 0.7, 0.8, 0.9))
.summary_table_4_4(df, group_by = "group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{get_last_checked}
\alias{get_last_checked}
\title{Get the date of the last value set check}
\usage{
get_last_checked()
}
\value{
A \code{Date} object.
}
\description{
Reads the cached date of the last value set update check from the
user cache directory. Returns \code{1970-01-01} if no cache file
exists (i.e., the check has never been run).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.prep_eq5d}
\alias{.prep_eq5d}
\title{Data checking/preparation: EQ-5D variables}
\usage{
.prep_eq5d(
  df,
  names,
  add_state = FALSE,
  add_lss = FALSE,
  add_lfs = FALSE,
  add_utility = FALSE,
  eq5d_version = NULL,
  country = NULL
)
}
\arguments{
\item{df}{a data frame of EQ-5D scores}

\item{names}{character vector of length 5 with names of EQ-5D variables in the data frame. The variables should be in an integer format.}

\item{add_state}{logical indicating whether the EQ-5D state should be added}

\item{add_lss}{logical indicating whether the LSS (Level Sum Score) should be added}

\item{add_lfs}{logical indicating whether the LFS (Level Frequency Score) should be added}

\item{add_utility}{logical indicating whether the utility should be added}

\item{eq5d_version}{character indicating the version of the EQ-5D questionnaire to use (either "3L" or "5L")}

\item{country}{character indicating the country to retrieve the quality of life score for}
}
\value{
a modified data frame with EQ-5D domain columns renamed to default names, and, if necessary, with added columns for state, LSS, LFS, and/or utility. If any of the checks fail (e.g. EQ-5D columns are not in an integer format), an error message is displayed and the function is stopping.
}
\description{
This function prepares a data frame for analysis by extracting, processing, and adding columns for EQ-5D variables, including state, LSS (Level Sum Score), LFS (Level Frequency Score) and utility.
}
\examples{
set.seed(1234)
df <- data.frame(mo = sample(1:5, 3), sc = sample(1:5, 3), 
  ua = sample(1:5, 3), pd = sample(1:5, 3), ad = sample(1:5, 3))
.prep_eq5d(df, names = c("mo", "sc", "ua", "pd", "ad"), 
  add_state = TRUE, add_lss = TRUE)
.prep_eq5d(df, names = c("mo", "sc", "ua", "pd", "ad"),
  add_state = TRUE, add_lss = TRUE, add_lfs = TRUE, add_utility = TRUE,
  eq5d_version = "5L", country = "ES")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.modify_ggplot_theme}
\alias{.modify_ggplot_theme}
\title{Modify ggplot2 theme}
\usage{
.modify_ggplot_theme(p)
}
\arguments{
\item{p}{ggplot2 plot}
}
\value{
ggplot2 plot with modified theme
}
\description{
Modify ggplot2 theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_better_dimensions_by_group_plot}
\alias{eq5d_profile_better_dimensions_by_group_plot}
\title{eq5d_profile_better_dimensions_by_group_plot: Percentage of Respondents Who Improved in Each EQ-5D Dimension, by Group
This function calculates how many respondents improved in each dimension between
two time points and summarizes the results for each group. The, it prodcuces 
a dimension-focused chart illustrating improvement percentages by dimension.}
\usage{
eq5d_profile_better_dimensions_by_group_plot(
  df,
  name_id,
  name_groupvar = NULL,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{Data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column}

\item{name_id}{Character string for the patient ID column}

\item{name_groupvar}{Character string for the grouping column (e.g. procedure). If NULL (default), the analysis is performed on the full population.}

\item{names_eq5d}{Character vector of EQ-5D dimension names}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector of length 2, specifying the order of the follow-up levels (e.g. c("Pre-op","Post-op"))}
}
\value{
A list containing:
  \item{plot_data}{A data frame of improvements by group and dimension}
  \item{p}{A ggplot2 object produced by `.pchc_plot_by_dim()`}
}
\description{
eq5d_profile_better_dimensions_by_group_plot: Percentage of Respondents Who Improved in Each EQ-5D Dimension, by Group
This function calculates how many respondents improved in each dimension between
two time points and summarizes the results for each group. The, it prodcuces 
a dimension-focused chart illustrating improvement percentages by dimension.
}
\examples{
result <- eq5d_profile_better_dimensions_by_group_plot(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c("Pre-op","Post-op")
)
result$p
result$plot_data

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/migrations.R
\name{apply_pending_migrations}
\alias{apply_pending_migrations}
\title{Apply pending value set migrations}
\usage{
apply_pending_migrations(ask = TRUE)
}
\arguments{
\item{ask}{Logical. Whether to ask for confirmation before
applying migrations. Defaults to TRUE.}
}
\value{
Invisibly returns a character vector of migration IDs
  that were applied in this session.
}
\description{
Fetches the migrations index from the repository and applies
any renames that have not yet been applied locally. This
ensures that value set codes remain consistent when a country
publishes a second value set and the original code needs to
be disambiguated with a year suffix.
}
\details{
Migrations are applied before checking for new value sets in
\code{update_value_sets()}, ensuring the local state is
consistent before any new installations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqxw.R
\name{.pstate5t3}
\alias{.pstate5t3}
\title{.pstate5t3}
\usage{
.pstate5t3(probs = .EQxwprob)
}
\arguments{
\item{probs}{15 x 5 matrix with probabilities per level/dimension, typically saved in .EQxwprob}
}
\value{
An 3125x243 matrix with probabilities per state
}
\description{
Takes a 15 x 5 matrix with probabilities per level/dimension, and creates an 3125x243 matrix with probabilities per state
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_lfs_distribution}
\alias{eq5d_profile_lfs_distribution}
\title{eq5d_profile_lfs_distribution: Distribution of the EQ-5D states by LFS (Level Frequency Score)}
\usage{
eq5d_profile_lfs_distribution(df, names_eq5d = NULL, eq5d_version = NULL)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}
}
\value{
Summary data frame
}
\description{
eq5d_profile_lfs_distribution: Distribution of the EQ-5D states by LFS (Level Frequency Score)
}
\examples{
eq5d_profile_lfs_distribution(
  example_data, 
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
  eq5d_version = "3L"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.summary_table_2_1}
\alias{.summary_table_2_1}
\title{Wrapper for the repetitive code in function_table_2_1. Data frame summary}
\usage{
.summary_table_2_1(df, group_by)
}
\arguments{
\item{df}{A data frame}

\item{group_by}{A character vector of variables in `df` to group by. Should contain 'eq5d' and 'fu'.}
}
\value{
A summarised data frame with groups defined by `eq5d` and `fu` variables, the count of observations in each group, and the frequency of each group.
}
\description{
This internal function summarises a data frame by grouping it based on the variables specified in the 'group_by' argument and calculates the frequency of each group. The output is used in Table 2.1
}
\examples{
set.seed(1234)
df <- data.frame(eq5d = rep(rnorm(5), 2),
                 fu = rep(c(1, 0, 1, 0, 1), 2))
.summary_table_2_1(df, c("eq5d", "fu"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{figure_3_3}
\alias{figure_3_3}
\title{Figure 3.3: EQ-5D values: smoothed lines and confidence intervals by groupvar}
\usage{
figure_3_3(
  df,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL,
  name_groupvar,
  eq5d_version = NULL,
  country
)
}
\arguments{
\item{df}{Data frame with the EQ-5D, follow-up and grouping columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column. 
If NULL (default value), the levels will be ordered in the order of appearance in df.}

\item{name_groupvar}{Character string for the grouping column}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country. 
This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
Summary plot and data used for plotting
}
\description{
Figure 3.3: EQ-5D values: smoothed lines and confidence intervals by groupvar
}
\examples{
tmp <- figure_3_3(
 example_data,
 names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
 name_fu = "time",
 levels_fu = c('Pre-op', 'Post-op'),
 name_groupvar = "procedure",
 eq5d_version = "3L",
 country = "UK"
)
tmp$p
tmp$plot_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{get_installed_vs_codes}
\alias{get_installed_vs_codes}
\title{Get the VS_codes currently installed for a given instrument}
\usage{
get_installed_vs_codes(version)
}
\arguments{
\item{version}{Character. One of "3L", "5L", or "Y3L".}
}
\value{
A character vector of VS_codes.
}
\description{
Returns all installed VS_codes for the specified instrument
version, including both built-in value sets shipped with the
package and any user-added value sets installed via
\code{eqvs_add()} or \code{update_value_sets()}.
}
\details{
Reads directly from the runtime package environment
(\code{getOption("eq.env")}) to include user-added value sets
without triggering the display side-effects of
\code{eqvs_display()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{install_value_set}
\alias{install_value_set}
\title{Download and install a single value set}
\usage{
install_value_set(vs_code, version, meta_row)
}
\arguments{
\item{vs_code}{Character. The VS_code to download.}

\item{version}{Character. One of "3L", "5L", or "Y3L".}

\item{meta_row}{A single-row data frame from the index CSV.}
}
\value{
Logical. TRUE if successful, FALSE otherwise.
}
\description{
Download and install a single value set
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{get_cache_dir}
\alias{get_cache_dir}
\title{Get the eq5dsuite cache directory}
\usage{
get_cache_dir()
}
\value{
A character string giving the path to the cache directory.
}
\description{
Returns the path to the user-specific cache directory used by
eq5dsuite to store the date of the last value set check.
The directory is created if it does not exist.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.summary_cts_by_fu}
\alias{.summary_cts_by_fu}
\title{Wrapper to summarise a continuous variable by follow-up (FU)}
\usage{
.summary_cts_by_fu(df, name_v)
}
\arguments{
\item{df}{A data frame containing the FU and continuous variable of interest. The dataset must contain an ordered `fu` variable.}

\item{name_v}{A character string with the name of the continuous variable in `df` to be summarised.}
}
\value{
Data frame with one row for each statistic and one column for each FU.
}
\description{
This function summarizes a continuous variable for each follow-up (FU) and calculates various statistics such as mean, standard deviation, median, mode, kurtosis, skewness, minimum, maximum, range, and number of observations. It also reports the total sample size and the number (and proportion) of missing values for each FU. 
The input `df` must contain an ordered FU variable and the continuous variable of interest. 
The name of the continuous variable must be specified using `name_v`. 
The wrapper is used in Table 3.1 (for VAS) or Table 4.2 (for EQ-5D utility)
}
\examples{
df <- data.frame(fu = c(1,1,2,2,3,3), 
                 vas = c(7,8,9,NA,7,6))
.summary_cts_by_fu(df, name_v = "vas")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{update_value_sets}
\alias{update_value_sets}
\title{Update eq5dsuite value sets}
\usage{
update_value_sets(
  versions = c("3L", "5L", "Y3L"),
  ask = TRUE,
  drop = NULL,
  rename = NULL
)
}
\arguments{
\item{versions}{Character vector. Which instruments to check.
Defaults to c("3L", "5L", "Y3L").}

\item{ask}{Logical. Whether to ask for confirmation before
installing, dropping, or renaming. Defaults to TRUE.}

\item{drop}{Character vector of VS_codes to remove, in the
format "VS_code:version" e.g. c("NL:3L", "US:5L").
Defaults to NULL (no removals).}

\item{rename}{Named character vector of renames in the format
c("old_VS_code:version" = "new_VS_code") e.g.
c("NL:3L" = "NL_2006"). Defaults to NULL (no renames).}
}
\value{
Invisibly returns a list with elements
  \code{checked}, \code{new}, \code{installed},
  \code{dropped}, and \code{renamed}.
}
\description{
Connects to the eq5dsuite value sets repository and checks
whether new value sets are available for any of the three
EQ-5D instruments. Optionally drop or rename existing value
sets before checking for updates.
}
\details{
This function requires an internet connection for checking
and installing new value sets. Drop and rename operations
work offline.
}
\examples{
\dontrun{
# Check for new value sets interactively
update_value_sets()

# Check specific instrument only
update_value_sets(versions = "5L")

# Drop a value set
update_value_sets(drop = "NL:3L")

# Rename a value set
update_value_sets(rename = c("NL:3L" = "NL_2006"))

# Drop and rename in one call
update_value_sets(
  drop   = "DE:3L",
  rename = c("NL:3L" = "NL_2006", "SI:3L" = "SI_TTO")
)

# Non-interactive update
update_value_sets(ask = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_top_states}
\alias{eq5d_profile_top_states}
\title{eq5d_profile_top_states: Prevalence of the 10 most frequently observed self-reported health states}
\usage{
eq5d_profile_top_states(df, names_eq5d = NULL, eq5d_version = NULL, n = 10)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{n}{Number of most frequently observed states to display (default 10)}
}
\value{
Summary data frame
}
\description{
eq5d_profile_top_states: Prevalence of the 10 most frequently observed self-reported health states
}
\examples{
eq5d_profile_top_states(
  df = example_data[example_data$time == "Pre-op",],
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  eq5d_version = "3L",
  n = 10
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_pchc_by_group_plot}
\alias{eq5d_profile_pchc_by_group_plot}
\title{eq5d_profile_pchc_by_group_plot: Paretian Classification of Health Change (PCHC) by Group
This function computes PCHC categories between two time points for each subject, 
stratifies them by a grouping variable, and produces
a single bar chart with side-by-side bars showing the distribution of PCHC categories.}
\usage{
eq5d_profile_pchc_by_group_plot(
  df,
  name_id,
  name_groupvar = NULL,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{Data frame containing EQ-5D dimensions, a grouping variable, patient ID, and follow-up columns}

\item{name_id}{Character string for the patient ID column}

\item{name_groupvar}{Character string for the grouping column (e.g., procedure). If NULL (default), the analysis is performed on the full population.}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector of length 2 indicating the order of follow-up time points (e.g., c("Pre-op", "Post-op"))}
}
\value{
A list with two elements:
  \item{plot_data}{A tibble of PCHC percentages by group}
  \item{p}{A ggplot2 object showing a bar chart with side-by-side bars for each PCHC category}
}
\description{
eq5d_profile_pchc_by_group_plot: Paretian Classification of Health Change (PCHC) by Group
This function computes PCHC categories between two time points for each subject, 
stratifies them by a grouping variable, and produces
a single bar chart with side-by-side bars showing the distribution of PCHC categories.
}
\examples{
result <- eq5d_profile_pchc_by_group_plot(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c("Pre-op", "Post-op")
)
result$p        # shows the plot
result$plot_data  # shows the summary table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.pchc}
\alias{.pchc}
\title{Wrapper to determine Paretian Classification of Health Change}
\usage{
.pchc(df, level_fu_1, add_noprobs = FALSE)
}
\arguments{
\item{df}{A data frame with EQ-5D states and follow-up variable. The dataset is assumed to be have been ordered correctly.}

\item{level_fu_1}{Value of the first (i.e. earliest) follow-up. Would normally be defined as levels_fu[1].}

\item{add_noprobs}{Logical value indicating whether to include a separate classification for those without problems (default is FALSE)}
}
\value{
A data frame with PCHC value for each combination of the grouping variables. 
If 'add_noprobs' is TRUE, a separate classification for those without problems is also included.
}
\description{
This internal function determines Paretian Classification of Health Change (PCHC) for each combination of the variables specified in the `group_by` argument. 
It is used in the code for eq5d_profile_pchc_table, eq5d_profile_pchc_with_no_problems_table, eq5d_profile_dimension_change_table, and the eq5d_profile_*_by_group_plot functions.
An EQ-5D health state is deemed to be `better` than another if it is better on at least one dimension and is no worse on any other dimension.
An EQ-5D health state is deemed to be `worse` than another if it is worse in at least one dimension and is no better in any other dimension.
}
\examples{
df <- data.frame(id = c(1, 1, 2, 2),
                 fu = c(1, 2, 1, 2),
                 mo = c(1, 1, 1, 1),
                 sc = c(1, 1, 5, 1),
                 ua = c(1, 1, 4, 3),
                 pd = c(1, 1, 1, 3),
                 ad = c(1, 1, 1, 1))
.pchc(df, level_fu_1 = 1, add_noprobs = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{check_builtin_conflicts}
\alias{check_builtin_conflicts}
\title{Check for conflicts between user-defined and built-in value sets}
\usage{
check_builtin_conflicts(version)
}
\arguments{
\item{version}{Character. One of "3L", "5L", or "Y3L".}
}
\value{
Invisibly returns a character vector of conflicting
  VS_codes, or character(0) if none found.
}
\description{
After applying migrations or installing new value sets, checks
whether any built-in value sets share a VS_code with a migration
that has been applied. If conflicts are found, warns the user
that the built-in value set still exists and will be removed
in the next package release.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.summary_mean_ci}
\alias{.summary_mean_ci}
\title{Wrapper to calculate summary mean with 95\% confidence interval}
\usage{
.summary_mean_ci(df, group_by)
}
\arguments{
\item{df}{A data frame containing a `utility` column.}

\item{group_by}{A character vector of column names to group by.}
}
\value{
A data frame with the mean, lower bound, and upper bound of the 95% confidence interval of `utility` grouped by the `group_by` variables.
}
\description{
This internal function calculates summary mean and 95\% confidence interval of the utility variable, which can also be grouped.
The function is used in Figures 4.2-4.4.
}
\examples{
df <- data.frame(group = c("A", "A", "B", "B"), 
                 utility = c(0.5, 0.7, 0.8, 0.9))
.summary_mean_ci(df, group_by = "group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{fetch_available_value_sets}
\alias{fetch_available_value_sets}
\title{Fetch the index of available value sets from GitHub}
\usage{
fetch_available_value_sets(version)
}
\arguments{
\item{version}{Character. One of "3L", "5L", or "Y3L".}
}
\value{
A data frame with columns Version, Name, Name_short,
  Country_code, VS_code, doi. Returns NULL if fetch fails.
}
\description{
Fetch the index of available value sets from GitHub
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.prep_fu}
\alias{.prep_fu}
\title{Data checking/preparation: follow-up variable}
\usage{
.prep_fu(df, name = NULL, levels = NULL)
}
\arguments{
\item{df}{A data frame.}

\item{name}{Column name in the data frame that contains follow-up information.}

\item{levels}{Levels to factorise the FU variable into.}
}
\value{
A data frame with the follow-up variable renamed as "fu" and factorised.
}
\description{
This function prepares the follow-up (FU) variable for analysis by giving it a default name (`fu`) and factorising
}
\examples{
df <- data.frame(id = c(1, 1, 2, 2),
  visit = c("baseline", "follow-up", "baseline", "follow-up"))
.prep_fu(df = df, name = "visit", levels = c("baseline", "follow-up"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/migrations.R
\name{set_applied_migrations}
\alias{set_applied_migrations}
\title{Save the list of applied migrations to cache}
\usage{
set_applied_migrations(migrations)
}
\arguments{
\item{migrations}{Character vector of migration IDs to save.}
}
\value{
Invisibly returns the saved migrations.
}
\description{
Save the list of applied migrations to cache
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{make_all_EQ_indexes}
\alias{make_all_EQ_indexes}
\title{make_all_EQ_indexes}
\usage{
make_all_EQ_indexes(
  version = "5L",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)
}
\arguments{
\item{version}{Either "3L" or "5L", to signify whether 243 or 3125 states should be generated}

\item{dim.names}{A vector of dimension names to be used as names for output columns.}
}
\value{
A vector with 5-digit state indexes for all 243 (-3L) or 3125 (-5L) EQ-5D health states
}
\description{
Make a vector containing all 5-digit EQ-5D indexes for -3L or -5L version.
}
\examples{
make_all_EQ_indexes('3L')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{figure_2_1}
\alias{figure_2_1}
\title{Figure 2.1: EQ VAS scores}
\usage{
figure_2_1(df, name_vas = NULL)
}
\arguments{
\item{df}{Data frame with the VAS column}

\item{name_vas}{Character string for the VAS column}
}
\value{
Summary plot and data used for plotting
}
\description{
Figure 2.1: EQ VAS scores
}
\examples{
tmp <- figure_2_1(example_data, name_vas = 'vas')
tmp$p
tmp$plot_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eqvs_drop}
\alias{eqvs_drop}
\title{eqvs_drop}
\usage{
eqvs_drop(country = NULL, version = "5L", saveOption = 1, savePath = NULL)
}
\arguments{
\item{country}{Optional string. If NULL, a list of current user-defined value sets will be provided for selection. If set, and matching an existing user-defined value set, a prompt will be given as to whether the value set should be deleted.}

\item{version}{Version of the EQ-5D instrument. Can take values 5L (default) or 3L.}

\item{saveOption}{Integer indicating how the cache data should be saved. 1: Do not save (default), 2: Save in package folder, 3: Save in another path.}

\item{savePath}{A path where the cache data should be saved when `saveOption` is 3. Please use `eqvs_load` to load it in your next session.}
}
\value{
True/False, indicating success or error.
}
\description{
Drop user-defined EQ-5D value set to reverse crosswalk options.
}
\examples{
\donttest{
  # make nonsense value set
  new_df <- data.frame(state = make_all_EQ_indexes(), TEST = runif(3125))
  # Add as value set for Fantasia
  eqvs_add(
   new_df,
   version = "5L",
   country = 'Fantasia',
   countryCode = "MyCountry",
   VSCode = "FAN",
   saveOption = 1
  )
  # Test the new value set
  eq5d5l(55555,country = "FAN")
  # Drop value set for Fantasia
  eqvs_drop(country = 'FAN', saveOption = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_value_sets.R
\name{rename_value_set}
\alias{rename_value_set}
\title{Rename a value set}
\usage{
rename_value_set(old_vs_code, new_vs_code, version, ask = TRUE)
}
\arguments{
\item{old_vs_code}{Character. The current VS_code.}

\item{new_vs_code}{Character. The new VS_code to use.}

\item{version}{Character. One of "3L", "5L", or "Y3L".}

\item{ask}{Logical. Whether to ask for confirmation.
Defaults to TRUE.}
}
\value{
Logical. TRUE if successful, FALSE otherwise.
}
\description{
Renames a value set by dropping the old VS_code and
reinstalling under the new VS_code. The value data is
preserved — only the identifier changes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.get_lfs}
\alias{.get_lfs}
\title{Calculate the Level Frequency Score (LFS)}
\usage{
.get_lfs(s, eq5d_version)
}
\arguments{
\item{s}{A character vector representing the EQ-5D state, e.g. 11123.}

\item{eq5d_version}{A character string specifying the version of EQ-5D, i.e. 3L or 5L.}
}
\value{
A character vector representing the calculated LFS.
}
\description{
This function calculates the Level Frequency Score (LFS) for a given EQ-5D state and a specified version of EQ-5D.
If at least one domain contains a missing entry, the whole LFS is set to be NA.
}
\examples{
.get_lfs("333", "3L") # returns 003
.get_lfs("333", "5L") # returns 00300
.get_lfs("12345", "5L") # returns 11111
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_vas_distribution_table}
\alias{eq5d_vas_distribution_table}
\title{eq5d_vas_distribution_table: EQ VAS Scores frequency of mid-points}
\usage{
eq5d_vas_distribution_table(df, name_vas = NULL, add_na_total = TRUE)
}
\arguments{
\item{df}{Data frame with the VAS column}

\item{name_vas}{Character string for the VAS column}

\item{add_na_total}{Logical, whether to add summary of the missing, and across the Total, data}
}
\value{
Summary data frame
}
\description{
eq5d_vas_distribution_table: EQ VAS Scores frequency of mid-points
}
\examples{
eq5d_vas_distribution_table(
  example_data,
  name_vas = 'vas', 
  add_na_total =  TRUE
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_utility_distribution_plot}
\alias{eq5d_utility_distribution_plot}
\title{eq5d_utility_distribution_plot: EQ-5D values: smoothed lines and confidence intervals by groupvar}
\usage{
eq5d_utility_distribution_plot(
  df,
  names_eq5d = NULL,
  eq5d_version = NULL,
  country
)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country. 
This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
Summary plot and data used for plotting
}
\description{
eq5d_utility_distribution_plot: EQ-5D values: smoothed lines and confidence intervals by groupvar
}
\examples{
tmp <- eq5d_utility_distribution_plot(
 example_data,
 names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
 eq5d_version = "3L",
 country = "UK"
)
tmp$p
tmp$plot_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Launch the eq5dsuite Shiny Application}
\usage{
run_app(...)
}
\arguments{
\item{...}{Additional arguments passed to \code{\link[shiny]{runApp}},
such as \code{port} or \code{launch.browser}.}
}
\value{
Called for its side effect of launching a Shiny application.
  Returns invisibly.
}
\description{
Opens an interactive Shiny application for uploading, processing,
analysing, and exporting EQ-5D data using the eq5dsuite package.
}
\examples{
\dontrun{
  eq5dsuite::run_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.add_utility}
\alias{.add_utility}
\title{Add utility values to a data frame}
\usage{
.add_utility(df, eq5d_version, country)
}
\arguments{
\item{df}{A data frame containing the state data. The state must be included in the data frame as a character vector under the column named `state`.}

\item{eq5d_version}{A character string specifying the version of EQ-5D, i.e. 3L or 5L.}

\item{country}{A character string representing the name of the country. This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
A data frame with an additional column named `utility` containing the calculated utility values. If the input country name is not found in the country_codes dataset, a list of available codes is printed, and subsequentyl an error message is displayed and the function stops.
}
\description{
This function adds utility values to a data frame based on a specified version of EQ-5D and a country name.
}
\examples{
df <- data.frame(state = c("11111", "11123", "32541"))
.add_utility(df, "5L", "DK")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{table_1_3_1}
\alias{table_1_3_1}
\title{Table 1.3.1: Summary statistics for the EQ-5D values by all the different LSSs (Level Sum Scores)}
\usage{
table_1_3_1(df, names_eq5d = NULL, eq5d_version = NULL, country)
}
\arguments{
\item{df}{Data frame with the EQ-5D columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country. 
This could be in a 2-letter format, full name or short name, as specified in the country_codes datasets.}
}
\value{
Summary data frame
}
\description{
Table 1.3.1: Summary statistics for the EQ-5D values by all the different LSSs (Level Sum Scores)
}
\examples{
df <- data.frame(make_all_EQ_states(version = "5L"))
table_1_3_1(
  df, 
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
  eq5d_version = "3L", 
  country = "US"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eqvs_display}
\alias{eqvs_display}
\title{eqvs_display}
\usage{
eqvs_display(version = "5L", return_df = FALSE, show_citation = FALSE)
}
\arguments{
\item{version}{Version of the EQ-5D instrument. One of \code{"3L"},
\code{"5L"} (default), or \code{"Y3L"}.}

\item{return_df}{Logical. If \code{TRUE}, returns a data.frame with
all columns (including \code{citation} if present). Defaults to
\code{FALSE}.}

\item{show_citation}{Logical. If \code{TRUE}, prints the full AMA
citation for each value set after the summary table. Defaults to
\code{FALSE}.}
}
\value{
\code{NULL} invisibly (default), or a data.frame when
  \code{return_df = TRUE}.
}
\description{
Display available value sets, which can also be used as
  (reverse) crosswalks. Built-in value sets are shown first, followed
  by any user-defined value sets added via \code{eqvs_add()}.
}
\examples{
# Display available EQ-5D-5L value sets.
eqvs_display(version = "5L")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.pchctab}
\alias{.pchctab}
\title{.pchctab: Changes in health according to the PCHC (Paretian Classification of Health Change)}
\usage{
.pchctab(
  df,
  name_id,
  name_groupvar,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL,
  add_noprobs = FALSE
)
}
\arguments{
\item{df}{Data frame with the EQ-5D, grouping, id and follow-up columns}

\item{name_id}{Character string for the patient id column}

\item{name_groupvar}{Character string for the grouping column}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column. 
If NULL (default value), the levels will be ordered in the order of appearance in df.}

\item{add_noprobs}{if set to TRUE, level corresponding to "no problems" will be added to the table}
}
\value{
Summary data frame
}
\description{
.pchctab: Changes in health according to the PCHC (Paretian Classification of Health Change)
}
\examples{
.pchctab(df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  name_fu = "time",
  levels_fu = c('Pre-op', 'Post-op')
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{figure_1_2_3}
\alias{figure_1_2_3}
\title{This function identifies respondents with a "Worsen" PCHC state (i.e., overall
health state got worse between levels_fu[1] and levels_fu[2]), checks
dimension-specific changes (e.g., mo_diff < 0), and summarizes by a grouping variable
(e.g., procedure) and time points. It returns a data table and a ggplot object.}
\usage{
figure_1_2_3(
  df,
  name_id,
  name_groupvar,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{A data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column}

\item{name_id}{A character string for the patient ID column}

\item{name_groupvar}{A character string for the grouping column (e.g., procedure)}

\item{names_eq5d}{A character vector of EQ-5D dimension names}

\item{name_fu}{A character string for the follow-up column}

\item{levels_fu}{A character vector of length 2, specifying the order of the follow-up levels (e.g., c("Pre-op","Post-op"))}
}
\value{
A list containing:
  \item{plot_data}{A data frame of "Worsen" percentages by group and dimension}
  \item{p}{A ggplot2 object produced by `.pchc_plot_by_dim()`}
}
\description{
This function identifies respondents with a "Worsen" PCHC state (i.e., overall
health state got worse between levels_fu[1] and levels_fu[2]), checks
dimension-specific changes (e.g., mo_diff < 0), and summarizes by a grouping variable
(e.g., procedure) and time points. It returns a data table and a ggplot object.
}
\examples{
result <- figure_1_2_3(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
  name_fu = "time",
  levels_fu = c("Pre-op","Post-op")
)
result$p        # shows the plot
result$plot_data  # shows the summary table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.getmode}
\alias{.getmode}
\title{Get the mode of a vector.}
\usage{
.getmode(v)
}
\arguments{
\item{v}{A numeric or character vector.}
}
\value{
The mode of `v`.
}
\description{
This function calculates the mode of a numeric or character vector. 
If there are multiple modes, the first one is returned. 
The code is taken from an \href{https://www.tutorialspoint.com/r/r_mean_median_mode.htm}{R help page}.
}
\examples{
.getmode(c(1, 2, 3, 3))
.getmode(c("a", "b", "b", "c"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/migrations.R
\name{get_applied_migrations}
\alias{get_applied_migrations}
\title{Get the list of already-applied migrations}
\usage{
get_applied_migrations()
}
\value{
A character vector of migration IDs.
}
\description{
Reads the locally cached list of migration IDs that have
already been applied. Returns an empty character vector if
no migrations have been applied yet.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{figure_1_2_4}
\alias{figure_1_2_4}
\title{Figure 1.2.4: Percentage of Respondents Who Had a Mixed Change Overall,
by Dimension Improved or Worsened, Grouped by Procedure (or Other Grouping)}
\usage{
figure_1_2_4(
  df,
  name_id,
  name_groupvar,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{Data frame containing columns for EQ-5D dimensions, a grouping variable,
a patient ID, and a follow-up variable}

\item{name_id}{Character string indicating the patient ID column}

\item{name_groupvar}{Character string for the grouping column (e.g. "procedure")}

\item{names_eq5d}{Character vector naming the EQ-5D dimensions (e.g. c("mo","sc","ua","pd","ad"))}

\item{name_fu}{Character string for the follow-up column (e.g. "time")}

\item{levels_fu}{Character vector of length 2 specifying the time order (e.g. c("Pre-op","Post-op"))}
}
\value{
A list with two elements:
  \item{plot_data}{A wide-format data frame of dimension-specific improvements/worsenings for "Mixed change"}
  \item{p}{A ggplot2 object showing a dimension-level bar chart from .pchc_plot_by_dim}
}
\description{
This function focuses on patients classified as having "Mixed change" overall
(i.e., some dimensions improved, others worsened). It then examines which dimensions
improved vs. worsened for each subject. Results are summarized by a grouping variable
(e.g., procedure) and time points. The final output is a table plus a ggplot object.
}
\examples{
result <- figure_1_2_4(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo","sc","ua","pd","ad"),
  name_fu = "time",
  levels_fu = c("Pre-op","Post-op")
)
result$plot_data
result$p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_level_summary}
\alias{eq5d_profile_level_summary}
\title{eq5d_profile_level_summary: Frequency of levels by dimensions, cross-sectional}
\usage{
eq5d_profile_level_summary(df, names_eq5d = NULL, eq5d_version = NULL)
}
\arguments{
\item{df}{Data frame with the EQ-5D and follow-up columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{eq5d_version}{Version of the EQ-5D instrument}
}
\value{
Summary data frame.
}
\description{
eq5d_profile_level_summary: Frequency of levels by dimensions, cross-sectional
}
\examples{
eq5d_profile_level_summary(
 df = example_data[example_data$time == "Pre-op",],
 names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
 eq5d_version = "3L"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/migrations.R
\name{fetch_migrations}
\alias{fetch_migrations}
\title{Fetch the migrations index from the repository}
\usage{
fetch_migrations()
}
\value{
A data frame with columns version, old_VS_code,
  new_VS_code, reason, date. Returns NULL if fetch fails.
}
\description{
Downloads the migrations.csv file from the eq5dsuite-value-sets
repository, which records all historical value set renames.
Returns NULL silently if the fetch fails.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.summary_table_4_3}
\alias{.summary_table_4_3}
\title{Summary wrapper for Table 4.3}
\usage{
.summary_table_4_3(df, group_by)
}
\arguments{
\item{df}{A data frame.}

\item{group_by}{A character vector of names of variables by which to group the data.}
}
\value{
A data frame with the summary statistics.
}
\description{
This internal function creates a summary of the data frame for Table 4.3. 
It groups the data by the variables specified in `group_by` and calculates various summary statistics.
}
\examples{
df <- data.frame(group = c("A", "A", "B", "B"), 
                 utility = c(0.5, 0.7, 0.8, 0.9))
.summary_table_4_3(df, group_by = "group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eq5d}
\alias{eq5d}
\title{eq5d}
\usage{
eq5d(
  x,
  country = NULL,
  version = "5L",
  dim.names = c("mo", "sc", "ua", "pd", "ad")
)
}
\arguments{
\item{x}{A vector of 5-digit EQ-5D-3L state indexes or a matrix/data.frame with columns corresponding to EQ-5D state dimensions}

\item{country}{String vector indicating country names or  ISO3166 Alpha 2 / 3 country codes.}

\item{version}{String indicating which version to use. Options are '5L'  (default), '3L', 'xw', 'xwr', and 'Y3L'.}

\item{dim.names}{A vector of dimension names to identify dimension columns.}
}
\value{
A vector of values or data.frame with one column for each value set requested.
}
\description{
Get EQ-5D index values for the -3L, -5L, crosswalk (-3L value set applied to -5L health states),  reverse crosswealk (-5L value set applied to -3L health states), and -Y-3L
}
\examples{
# US -3L value set
eq5d(c(11111, 12321, 32123, 33333), 'US', '3L') 
# Danish and US -5L value sets applied to -3L descriptives, i.e. reverse crosswalk
eq5d(make_all_EQ_states('3L'), c('DK', 'US'), 'XWR') 
# US -5L value set
eq5d(c(11111, 12321, 32153, 55555), 'US', '5L') 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eq5d3l}
\alias{eq5d3l}
\title{eq5d3l}
\usage{
eq5d3l(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad"))
}
\arguments{
\item{x}{A vector of 5-digit EQ-5D-3L state indexes, or a matrix/data.frame
with columns corresponding to the EQ-5D-3L dimensions.}

\item{country}{String vector indicating country names or ISO3166 Alpha 2 / 3
country codes.}

\item{dim.names}{A character vector specifying the names of the EQ-5D-3L
dimensions. Default is `c("mo", "sc", "ua", "pd", "ad")`.}
}
\value{
A numeric vector of EQ-5D-3L values, or a data.frame with one column
  for each requested value set.
}
\description{
Get EQ-5D-3L index values from individual responses to the five
dimensions of the EQ-5D-3L.
}
\examples{
# Example 1: utility values from EQ-5D-3L profile codes
eq5d3l(c(11111, 12321, 32123, 33333), country = "US")

# Example 2: request multiple value sets
eq5d3l(make_all_EQ_states("3L"), country = c("DK", "CA"))

# Example 3: use a data.frame with dimension columns
df3l <- data.frame(
  mo = c(1, 2, 3),
  sc = c(1, 2, 2),
  ua = c(1, 3, 1),
  pd = c(2, 2, 3),
  ad = c(1, 1, 2)
)
eq5d3l(df3l, country = "US")

# Example 4: use custom dimension column names
df3l_named <- data.frame(
  mobility = c(1, 2, 3),
  self_care = c(1, 2, 2),
  usual_activities = c(1, 3, 1),
  pain_discomfort = c(2, 2, 3),
  anxiety_depression = c(1, 1, 2)
)
eq5d3l(
  df3l_named,
  country = "US",
  dim.names = c(
    "mobility", "self_care", "usual_activities",
    "pain_discomfort", "anxiety_depression"
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_mixed_dimensions_by_group_plot}
\alias{eq5d_profile_mixed_dimensions_by_group_plot}
\title{eq5d_profile_mixed_dimensions_by_group_plot: Percentage of Respondents Who Had a Mixed Change Overall,
by Dimension Improved or Worsened, Grouped by Procedure (or Other Grouping)}
\usage{
eq5d_profile_mixed_dimensions_by_group_plot(
  df,
  name_id,
  name_groupvar = NULL,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL
)
}
\arguments{
\item{df}{Data frame containing columns for EQ-5D dimensions, a grouping variable,
a patient ID, and a follow-up variable}

\item{name_id}{Character string indicating the patient ID column}

\item{name_groupvar}{Character string for the grouping column (e.g. "procedure"). If NULL (default), the analysis is performed on the full population.}

\item{names_eq5d}{Character vector naming the EQ-5D dimensions (e.g. c("mo","sc","ua","pd","ad"))}

\item{name_fu}{Character string for the follow-up column (e.g. "time")}

\item{levels_fu}{Character vector of length 2 specifying the time order (e.g. c("Pre-op","Post-op"))}
}
\value{
A list with two elements:
  \item{plot_data}{A wide-format data frame of dimension-specific improvements/worsenings for "Mixed change"}
  \item{p}{A ggplot2 object showing a dimension-level bar chart from .pchc_plot_by_dim}
}
\description{
This function focuses on patients classified as having "Mixed change" overall
(i.e., some dimensions improved, others worsened). It then examines which dimensions
improved vs. worsened for each subject. Results are summarized by a grouping variable
(e.g., procedure) and time points. The final output is a table plus a ggplot object.
}
\examples{
result <- eq5d_profile_mixed_dimensions_by_group_plot(
  df = example_data,
  name_id = "id",
  name_groupvar = "procedure",
  names_eq5d = c("mo","sc","ua","pd","ad"),
  name_fu = "time",
  levels_fu = c("Pre-op","Post-op")
)
result$plot_data
result$p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.pchc_plot_by_dim}
\alias{.pchc_plot_by_dim}
\title{Wrapper to generate Paretian Classification of Health Change plot by dimension}
\usage{
.pchc_plot_by_dim(plot_data, ylab, title, cols, text_rotate = FALSE)
}
\arguments{
\item{plot_data}{A data frame containing information to plot, with columns for name (the dimensions to plot), p (the proportion of the total data falling into each dimension), and fu (the follow-up).}

\item{ylab}{The label for the y-axis.}

\item{title}{The plot title.}

\item{cols}{A vector of colors to use for the bars.}

\item{text_rotate}{A logical indicating whether to rotate the text labels for the bars.}
}
\value{
A ggplot object containing the PCHC plot.
}
\description{
This internal function plots Paretian Classification of Health Change (PCHC) by dimension. 
The input is a data frame containing the information to plot, and the plot will contain bars representing 
the proportion of the total data that falls into each dimension, stacked by covariate.
The wrapper is used in Figures 2.2-2.4.
}
\examples{
df <- data.frame(
  name = rep(c("Dim1", "Dim2"), each = 2),
  p = c(0.6, 0.4, 0.7, 0.3),
  groupvar = rep(c("Group A", "Group B"), 2)
)
colors <- c("Group A" = "#1b9e77", "Group B" = "#d95f02")
.pchc_plot_by_dim(df, ylab = "Proportion", title = "Example Plot", cols = colors)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqxw.R
\name{eqxw}
\alias{eqxw}
\title{eqxw}
\usage{
eqxw(x, country = NULL, dim.names = c("mo", "sc", "ua", "pd", "ad"))
}
\arguments{
\item{x}{A vector of 5-digit EQ-5D-5L state indexes or a matrix/data.frame with columns corresponding to EQ-5D state dimensions}

\item{country}{String vector indicating country names or  ISO3166 Alpha 2 / 3 country codes.}

\item{dim.names}{A vector of dimension names to identify dimension columns}
}
\value{
A vector of reverse crosswalk values or data.frame with one column per reverse crosswalk set requested.
}
\description{
Get crosswalk values
}
\examples{
eqxw(c(11111, 12521, 32123, 55555), 'US')
eqxw(make_all_EQ_states('5L'), c('DK', 'US'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.prep_vas}
\alias{.prep_vas}
\title{Data checking/preparation: VAS variable}
\usage{
.prep_vas(df, name)
}
\arguments{
\item{df}{A data frame.}

\item{name}{Column name in the data frame that holds the VAS score. The column can only contain integers or NAs}
}
\value{
A modified data frame with the VAS score renamed to "vas". If any checks fail (e.g. column is not numeric), an error message is displayed and the function is stopping.
}
\description{
The function prepares the data for VAS (Visual Analogue Scale) analyses.
}
\examples{
df <- data.frame(vas_score = c(20, 50, 80, NA, 100))
.prep_vas(df = df, name = "vas_score")
df <- data.frame(vas_score = c(20.5, 50, 80, NA, 100))
.prep_vas(df = df, name = "vas_score")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5dsuite-package.R
\docType{package}
\name{eq5dsuite-package}
\alias{eq5dsuite-package}
\alias{eq5dsuite}
\title{eq5dsuite: Standardised Tools for EQ-5D Analysis}
\description{
The \pkg{eq5dsuite} package provides a comprehensive set of
functions for calculating EQ-5D preference-based values and
analysing EQ-5D data following the recommendations of
Devlin et al. (2020).

The package supports three EQ-5D instruments:
\itemize{
  \item EQ-5D-3L
  \item EQ-5D-5L
  \item EQ-5D-Y-3L
}
}
\section{Value calculation}{

Functions for calculating EQ-5D preference-based values:
\itemize{
  \item \code{\link{eq5d}} — Generic calculator for all
    instruments
  \item \code{\link{eq5d3l}} — EQ-5D-3L value calculation
  \item \code{\link{eq5d5l}} — EQ-5D-5L value calculation
  \item \code{\link{eq5dy3l}} — EQ-5D-Y-3L value calculation
  \item \code{\link{eqxw}} — Original crosswalk method
    (van Hout et al., 2012)
  \item \code{\link{eqxwr}} — Reverse crosswalk method
    (van Hout and Shaw, 2021)
  \item \code{\link{eqxw_UK}} — UK-specific NICE-recommended
    mapping (Hernandez Alava et al., 2023)
}
}

\section{Value set management}{

Functions for managing country-specific value sets:
\itemize{
  \item \code{\link{eqvs_display}} — List all available
    value sets
  \item \code{\link{eqvs_add}} — Add a custom value set
  \item \code{\link{eqvs_drop}} — Remove a custom value set
  \item \code{\link{eqvs_load}} — Load a saved custom value
    set into the current session
  \item \code{\link{update_value_sets}} — Check for and
    install new value sets from the online repository
}
}

\section{Profile analysis}{

Functions for analysing EQ-5D health state profiles,
including cross-sectional summaries, longitudinal change
analyses, and severity metrics:
\itemize{
  \item \code{\link{eq5d_profile_level_summary}} —
    Cross-sectional frequency summary by dimension
  \item \code{\link{eq5d_profile_level_summary_by_group}} —
    Stratified frequency summary by subgroup
  \item \code{\link{eq5d_profile_top_states}} —
    Most frequently observed health state profiles
  \item \code{\link{eq5d_profile_change_summary}} —
    Dimension-level frequencies at two time points
  \item \code{\link{eq5d_profile_dimension_change_table}} —
    Changes in levels per dimension as percentages of total
    and of type of change
  \item \code{\link{eq5d_profile_pchc_table}} —
    Paretian Classification of Health Change table
  \item \code{\link{eq5d_profile_pchc_with_no_problems_table}}
    — PCHC table with no-problems category
  \item \code{\link{eq5d_profile_pchc_by_group_plot}} —
    PCHC bar chart by subgroup
  \item \code{\link{eq5d_profile_better_dimensions_by_group_plot}}
    — Dimensions improved among Better patients
  \item \code{\link{eq5d_profile_worse_dimensions_by_group_plot}}
    — Dimensions worsened among Worse patients
  \item \code{\link{eq5d_profile_mixed_dimensions_by_group_plot}}
    — Dimensions changed among Mixed patients
  \item \code{\link{eq5d_profile_health_profile_grid}} —
    Health Profile Grid of individual transitions
  \item \code{\link{eq5d_profile_lss_utility_summary}} —
    Descriptive statistics by Level Sum Score
  \item \code{\link{eq5d_profile_lss_utility_plot}} —
    EQ-5D values plotted against LSS
  \item \code{\link{eq5d_profile_lfs_distribution}} —
    Level Frequency Score distribution table
  \item \code{\link{eq5d_profile_lfs_mean_utility}} —
    Mean EQ-5D values by LFS category
  \item \code{\link{eq5d_profile_lfs_utility_summary}} —
    Descriptive statistics by Level Frequency Score
  \item \code{\link{eq5d_profile_lfs_utility_plot}} —
    EQ-5D values plotted against LFS
  \item \code{\link{eq5d_profile_density_curve}} —
    Health State Density Curve
}
}

\section{EQ-5D value analysis}{

Functions for analysing EQ-5D preference-based values
across time points, subgroups, and population norms:
\itemize{
  \item \code{\link{eq5d_utility_summary}} —
    Descriptive statistics at each time point
  \item \code{\link{eq5d_utility_summary_by_group}} —
    Descriptive statistics stratified by subgroup
  \item \code{\link{eq5d_utility_norms_comparison}} —
    Comparison with population norms by age and sex
  \item \code{\link{eq5d_utility_over_time_plot}} —
    Mean values with confidence intervals over time
  \item \code{\link{eq5d_utility_by_group_plot}} —
    Mean values with confidence intervals by subgroup
  \item \code{\link{eq5d_utility_change_by_group_plot}} —
    Longitudinal change by subgroup
  \item \code{\link{eq5d_utility_distribution_plot}} —
    Histogram of EQ-5D value distribution
  \item \code{\link{eq5d_utility_vas_scatter_plot}} —
    Scatter plot of EQ-5D values against EQ-VAS
}
}

\section{EQ-VAS analysis}{

Functions for analysing EQ-VAS self-rated health scores:
\itemize{
  \item \code{\link{eq5d_vas_summary}} —
    Descriptive statistics at each time point
  \item \code{\link{eq5d_vas_distribution_table}} —
    Frequency distribution in pre-defined ranges
  \item \code{\link{eq5d_vas_histogram}} —
    Histogram of EQ-VAS scores
  \item \code{\link{eq5d_vas_grouped_distribution_plot}} —
    Bar chart of grouped EQ-VAS frequencies
}
}

\section{Helper functions}{

Utility functions for preparing EQ-5D data:
\itemize{
  \item \code{\link{toEQ5Dindex}} — Combine dimension
    columns into a 5-digit profile code
  \item \code{\link{toEQ5Ddims}} — Split a 5-digit profile
    code into separate dimension columns
  \item \code{\link{make_all_EQ_indexes}} — Generate all
    valid EQ-5D state codes as a vector
  \item \code{\link{make_all_EQ_states}} — Generate all
    valid EQ-5D states as a data frame
  \item \code{\link{make_dummies}} — Create dummy variables
    from EQ-5D dimension columns
}
}

\section{Interactive application}{

A Shiny application providing point-and-click access to
the same analytical workflow without requiring R coding:
\itemize{
  \item \code{\link{run_app}} — Launch the Shiny application
}
}

\section{Cross-platform suite}{

eq5dsuite is part of a cross-platform suite also available
in Stata and Excel:
\itemize{
  \item Full suite:
    \url{https://github.com/MathsInHealth/eq5dsuite}
  \item R package:
    \url{https://github.com/MathsInHealth/eq5dsuite-r}
  \item Value sets repository:
    \url{https://github.com/MathsInHealth/eq5dsuite-value-sets}
}
}

\references{
Devlin N, Parkin D, Janssen B (2020).
\emph{Methods for Analysing and Reporting EQ-5D Data}.
Springer, Cham.
\doi{10.1007/978-3-030-47622-9}

van Hout B, Janssen MF, Feng YS, et al. (2012).
Interim scoring for the EQ-5D-5L: mapping the EQ-5D-5L
to EQ-5D-3L value sets.
\emph{Value in Health}, 15(5), 708--715.
\doi{10.1016/j.jval.2012.02.008}
}
\author{
Anabel Estévez-Carrillo, Oliver Rivero-Arias,
Iryna Schlackow, Kim Rand

Maintainer: Anabel Estévez-Carrillo
\email{aestevez@mathsinhealth.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_vas_histogram}
\alias{eq5d_vas_histogram}
\title{eq5d_vas_histogram: EQ VAS scores}
\usage{
eq5d_vas_histogram(df, name_vas = NULL)
}
\arguments{
\item{df}{Data frame with the VAS column}

\item{name_vas}{Character string for the VAS column}
}
\value{
Summary plot and data used for plotting
}
\description{
eq5d_vas_histogram: EQ VAS scores
}
\examples{
tmp <- eq5d_vas_histogram(example_data, name_vas = 'vas')
tmp$p
tmp$plot_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EQ_functions.R
\name{eqvs_add}
\alias{eqvs_add}
\title{eqvs_add}
\usage{
eqvs_add(
  df,
  version = "5L",
  country = NULL,
  countryCode = NULL,
  VSCode = NULL,
  description = NULL,
  saveOption = 1,
  savePath = NULL
)
}
\arguments{
\item{df}{A data.frame or file name pointing to csv file. The contents of the data.frame or csv file should be exactly two columns: state, containing a list of all 3125 (for 5L) or 243 (for 3L) EQ-5D health state vectors, and a column of corresponding utility values, with a suitable name.}

\item{version}{Version of the EQ-5D instrument. Can take values 5L (default) or 3L.}

\item{country}{Optional string. If not NULL, will be used as a country description for the user-defined value set.}

\item{countryCode}{Optional string. If not NULL, will be used as the two-digit code for the value set. Must be different from any existing national value set code.}

\item{VSCode}{Optional string. If not NULL, will be used as the three-digit code for the value set. Must be different from any existing national value set code.}

\item{description}{Optional string. If not NULL, will be used as a descriptive text for the user-defined value set.}

\item{saveOption}{Integer indicating how the cache data should be saved. 1: Do not save (default), 2: Save in package folder, 3: Save in another path.}

\item{savePath}{A path where the cache data should be saved when `saveOption` is 3. Please use `eqvs_load` to load it in your next session.}
}
\value{
True/False, indicating success or error.
}
\description{
Add user-defined EQ-5D value set and corresponding crosswalk option.
}
\examples{
# make nonsense value set
new_df <- data.frame(state = make_all_EQ_indexes(), TEST = runif(3125))
# Add as value set for Fantasia
eqvs_add(
   new_df,
   version = "5L",
   country = 'Fantasia',
   countryCode = "MyCountry",
   VSCode = "FAN",
   saveOption = 1
)
eq5d5l(55555,country = "FAN")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqxwr.R
\name{.pstate3t5}
\alias{.pstate3t5}
\title{.pstate3t5}
\usage{
.pstate3t5(PPP)
}
\arguments{
\item{PPP}{N x 25 matrix with probabilities per level/dimension created by EQrxwprobs}
}
\value{
An N * 3125 matrix with probabilities per state
}
\description{
Takes a N x 25 matrix with probabilities per level/dimension, and creates an N * 3125 matrix with probabilities per state
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.freqtab}
\alias{.freqtab}
\title{Helper function for frequency of levels by dimensions tables}
\usage{
.freqtab(
  df,
  names_eq5d = NULL,
  name_fu = NULL,
  levels_fu = NULL,
  eq5d_version = NULL,
  add_summary_problems_change = TRUE
)
}
\arguments{
\item{df}{Data frame with the EQ-5D and follow-up columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_fu}{Character string for the follow-up column. If NULL, no grouping is used, and the table reports for the total population.}

\item{levels_fu}{Character vector containing the order of the values in the follow-up column. 
If NULL (default value), the levels will be ordered in the order of appearance in df.}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{add_summary_problems_change}{If set to false, the resulting dataframe does not include a row on problems change.}
}
\value{
Summary data frame.
}
\description{
Helper function for frequency of levels by dimensions tables
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_aux.R
\name{.gen_colours}
\alias{.gen_colours}
\title{Generate colours for PCHC figures}
\usage{
.gen_colours(col, n)
}
\arguments{
\item{col}{A character string specifying the base colour. Only "green" or "orange" is accepted.}

\item{n}{A positive integer specifying the number of colours to generate.}
}
\value{
A vector of colours generated based on the specified base colour and number of colours.
}
\description{
This internal function generates a vector of colours based on the specified base colour. 
Currently only green and orange colours are implemented. 
The wrapper is used in Figures 2.2-2.4.
}
\examples{
# generate 10 colours for base colour "green"
.gen_colours("green", 10)
# generate 7 colours for base colour "orange"
.gen_colours("orange", 7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_health_profile_grid}
\alias{eq5d_profile_health_profile_grid}
\title{eq5d_profile_health_profile_grid: Health Profile Grid (HPG) for Two Time Points}
\usage{
eq5d_profile_health_profile_grid(
  df,
  names_eq5d,
  name_fu,
  levels_fu = NULL,
  name_id,
  eq5d_version,
  country
)
}
\arguments{
\item{df}{A data frame containing EQ-5D columns, a grouping variable, an ID column, and a follow-up column}

\item{names_eq5d}{A character vector of EQ-5D dimension names}

\item{name_fu}{A character string for the follow-up column}

\item{levels_fu}{A character vector of length 2, specifying the order of the follow-up levels (e.g., c("Pre-op","Post-op"))}

\item{name_id}{A character string for the patient ID column}

\item{eq5d_version}{Version of the EQ-5D instrument}

\item{country}{A character string representing the name of the country.}
}
\value{
A list with components:
  \item{plot_data}{The plot data with ranks and classification.}
  \item{p}{A \code{ggplot2} object displaying the HPG scatter plot.}
}
\description{
This function creates a Health Profile Grid (HPG) for EQ-5D data, plotting each
individual's change in health states (ranked from best to worst) between two time
points. A diagonal reference line indicates no change; points above the line reflect
improvement, and points below indicate deterioration.
}
\examples{
tmp <- eq5d_profile_health_profile_grid(
           df = example_data, 
           names_eq5d = c("mo", "sc", "ua", "pd", "ad"), 
           name_fu = "time", 
           levels_fu = c("Pre-op", "Post-op"), 
           name_id = "id", 
           eq5d_version = "3L", 
           country = "UK"
       )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq5d_devlin.R
\name{eq5d_profile_level_summary_by_group}
\alias{eq5d_profile_level_summary_by_group}
\title{eq5d_profile_level_summary_by_group: Frequency of levels by dimensions, separated by category}
\usage{
eq5d_profile_level_summary_by_group(
  df,
  names_eq5d = NULL,
  name_cat = NULL,
  levels_cat = NULL,
  eq5d_version = NULL
)
}
\arguments{
\item{df}{Data frame with the EQ-5D and follow-up columns}

\item{names_eq5d}{Character vector of column names for the EQ-5D dimensions}

\item{name_cat}{Character string for the category  column. If NULL, no grouping is used, and the table reports for the total population, i.e. equal to table 1.1.1.}

\item{levels_cat}{Character vector containing the order of the values in the category column, if the wish is to have these presented in a particular order. 
If NULL (default value), unless the variable is a factor, the levels will be ordered in the order of appearance in df.}

\item{eq5d_version}{Version of the EQ-5D instrument}
}
\value{
Summary data frame.
}
\description{
eq5d_profile_level_summary_by_group: Frequency of levels by dimensions, separated by category
}
\examples{
eq5d_profile_level_summary_by_group(
 df = example_data[example_data$time == "Pre-op",],
 names_eq5d = c("mo", "sc", "ua", "pd", "ad"),
 name_cat = "procedure",
 levels_cat = c("Hip Replacement", "Knee Replacement"),
 eq5d_version = "3L"
)
}
